function Y_hat = f_RFregPredict(X,model)
% - predict fitted values via a Random Forest regression
% 
% USAGE: Y_hat = f_RFregPredict(X,model);
% 
% X     = predictors (row = obs, col = variables)
% model = Random Forest model generated by f_RFregTrain
% 
% Y_hat = predicted values
% 
% SEE ALSO: f_RFreg, f_RFclass

% ----- Author(s): -----
%**************************************************************
%* mex interface to Andy Liaw et al.'s C code (used in R package randomForest)
%* Added by Abhishek Jaiantilal ( abhishek.jaiantilal@colorado.edu )
%* License: GPLv2
%* Version: 0.02
%
% Calls Regression Random Forest
% A wrapper matlab file that calls the mex file
% This does prediction given the data and the model file
%**************************************************************
% 
% modified by David L. Jones, Feb-2010
% This file is part of the 'FATHOM TOOLBOX FOR MATLAB'.

% -----Check input:-----
if (nargin~=2)
   error('Two inputs are required: X & MODEL!');
end

% Check for compatible sizes:
if (size(X,2) ~= size(model.X,2)), error('Size mismatch b/n X & MODEL.X!'); end;

% -----Standardize/Center data:-----
switch model.stnd
   case 'raw'  % use raw data for X
   case 'stnd' % standardize variables in X
      X = f_stnd(X);
   case 'ctr'  % center variables in X
      X = f_center(X);
   otherwise
      error('Invalid option for STND!');
end

% =================================
%     Call *.mexmaci64 MEX file:
% =================================
Y_hat = mexRF_predict(X', model.lDau, model.rDau, model.nodestatus,...
   model.nrnodes, model.upper, model.avnode, model.mbest, model.treeSize, model.nTree);

% Clean up:
clear mexRF_predict

% -----Bias Correction:-----
if ~isnan(model.coef)
   Y_hat = model.coef(1) + model.coef(2)*Y_hat;
end
