function Tcell_images_downloader

curr_path = which('Tcell_images_downloader.m');

curr_path = fileparts(curr_path);
cd(curr_path);
base_dir = which(mfilename);
base_dir = strsplit(base_dir, '/');
base_dir = strjoin(base_dir(1 : end - 2), '/');

url = 'http://murphylab.web.cmu.edu/data/TcellModels/';
str = urlread(url);

patterns = {'"([^"]+Full.zip)"', '"([^"]+Block.zip)"', '"([^"]+Rescue.zip)"'};
filenames = cellfun(@(x) regexp(str, x, 'tokens'), patterns, 'Uniformoutput', false);

sensor_names = cellfun(@(x) x{1}(1 : end - 8), filenames{1}, 'Uniformoutput', false);
sensor_rescure_names = cellfun(@(x) x{1}(1 : end - 10), filenames{3}, 'Uniformoutput', false);

conditions_folder_names = {'Actin Regulator - Full Stimulus', 'Actin Regulators - B7 Blockade', 'Actin Regulators - B7 Blockade active Rac Cofilin'};
conditions_short_names = {'Full', 'Block', 'Rescue'};
sensors_to_use = sensor_names;

image_location = [base_dir, '/Tcell_images/'];
if ~exist(image_location, 'dir')
    mkdir(image_location);
end

if false
    conditions_short_names = conditions_short_names(3);
    conditions_folder_names = conditions_folder_names(3);
    sensor_names = sensor_names(end);
end

empty_prefix_group = {'LATRescue', 'WAVE2Rescue', 'LATBlock', 'WAVE2Block', 'WAVE2Full'};

for i = 1 : numel(conditions_short_names)
    curr_condition_folder = conditions_folder_names{i};
    curr_condition = conditions_short_names{i};
    curr_image_dir = [image_location, curr_condition_folder, '/']; 
    if ~exist(curr_image_dir, 'dir')
        mkdir(curr_image_dir);
    end    
   
    for j = 1 : numel(sensor_names)
        folder_prefix = 'projects/cellorganizer/xruan/tcell_project/images_online/';
        current_sensor = sensor_names{j};
        if strcmp(curr_condition, 'Rescue')
            if ~any(strcmp(sensor_rescure_names, current_sensor))
                continue;
            end
        end
        cur_sensor_dir = [curr_image_dir, current_sensor];
        if ~exist(cur_sensor_dir, 'dir')
            mkdir(cur_sensor_dir);
        end
        cur_combine_name = [current_sensor, curr_condition];
        
        if any(strcmp(empty_prefix_group, cur_combine_name))
            folder_prefix = '';
        end           
        
        zip_filename = [current_sensor, curr_condition, '.zip'];
        zip_fullname = [image_location  zip_filename];
        done_flag = [image_location  cur_combine_name, '.done'];
        disp(sprintf('current dataset: %s', zip_filename));
        if exist(done_flag, 'file')
            disp(sprintf('%s exists, skip it!', cur_combine_name));
            continue;
        end
        if ~exist(zip_fullname )
            disp(sprintf('Downloading dataset: %s', zip_filename));
            urlwrite([url  zip_filename], zip_fullname);
            disp('Extracting files');
            system(['unzip -o "' zip_fullname '" -d ' image_location]);
            system(['rsync -avxHl --delete ', image_location, folder_prefix, cur_combine_name, '/* "', [cur_sensor_dir], '"']);
            system(['rm -rf "', image_location, folder_prefix, cur_combine_name, '"']);
            fclose(fopen(done_flag, 'w'));
        else
            disp('Pattern found on disk. Extracting files from existing zip file');
            disp('Extracting files');
            try 
                unzip(zip_fullname, image_location)
            catch
                disp('unable to extract zip file')
                delete(zip_fullname);
                continue
            end
            system(['rsync -avxHl --delete ', image_location, folder_prefix, cur_combine_name, '/* "', [cur_sensor_dir], '"']);
            system(['rm -rf "', image_location, folder_prefix, cur_combine_name, '"']);
            fclose(fopen(done_flag, 'w'));
        end
        % synapse_location{end + 1} = [image_location, zip_filename(1 : end - 4), filesep];
        system(['rm -rf ',  zip_fullname]);
    end
end
% remove the zip files and the intermediate file. 
system(['rm -rf ', image_location, 'projects']);
system(['rm -rf ',  image_location, '*.zip']);

end
