function [current_window_image] = WAVE2_adapt_pipeline(current_frame_info)
    % this function is for adaptation of manual segmentation of WAVE2
    % active condition. 
    % Author: Xiongtao Ruan
    % Date: Aug. 2015

%
% Author: Taraz E. Buck and Xiongtao Ruan
%
% Copyright (C) 2012-2016 Murphy Lab
% Computational Biology Department
% School of Computer Science
% Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu





    debug = false;
    if nargin == 1
        window_size_2d = current_frame_info.window_size_2d;
        current_z_desired_scale = current_frame_info.current_z_desired_scale;
        current_room = current_frame_info.current_room;
        current_synapse_center_rounded = current_frame_info.current_synapse_center_rounded;
        position = current_synapse_center_rounded;
        run_num = current_frame_info.run_num;
        frame_num = current_frame_info.frame_num;
        all_images_dir = current_frame_info.WAVE2_active_root_dir;
    else
        debug = true;
    end
    
    if debug
        clear;
        close all;
        position = [83, 190]; 
        run_num = 1;
        frame_num = 21;

        window_size_2d = [71, 71];
        current_synapse_center_rounded = position;
        current_room = 35;
        current_z_desired_scale = 1;
        all_images_dir = '/projects/cellorganizer/xruan/tcell_project/images/';
        addpath(genpath(pwd));
    end

    filename_list = dir_recursive(all_images_dir);
    pattern = '/WAVE2/';
    file_inds = find(~cellfun(@isempty, regexp(filename_list, pattern)) == 1);
    WAVE2_dir = regexp(filename_list{file_inds(1)}, pattern, 'split');
    root_dir = [WAVE2_dir{1}, pattern];
    
    % root_dir = '/projects/cellorganizer/xruan/tcell_project/images/tcell_images/Actin Regulators - B7 Blockade active Rac Cofilin/WAVE2/';
    % run_num = [ 1, 2]
    
    image_dir = [root_dir, sprintf('030512WAVE2B7Block1uMRac250nmCofRun%s/EGFP/', num2str(run_num))];
    manual_seg_dir = [root_dir, 'manual_segmentation/'];
    mask_dir = [manual_seg_dir, sprintf('masks/run %s/', num2str(run_num))];
    segmented_image_dir = [manual_seg_dir, 'segmented_images/'];
    run_result_dir = [segmented_image_dir, sprintf('run%s/', num2str(run_num))];

    mask_cell_dir_name = dir_recursive(mask_dir);

    regex_pattern = [mask_dir, 'cell\d/EGFP'];

    iscell = cellfun(@isempty, regexp(mask_cell_dir_name, regex_pattern));
    mask_cell_dir_name = mask_cell_dir_name(~iscell);


    regex_pattern = ['T', num2str(frame_num, '%02d')];

    cell_frame_names = mask_cell_dir_name(~cellfun(@isempty, regexp(mask_cell_dir_name, regex_pattern)));

    max_overlap = -1;
    max_ind = 1;


    for i = 1 : length(cell_frame_names)

        current_cell_frame_name = cell_frame_names{i};
        part_names = strsplit(current_cell_frame_name, filesep);

        current_cell_name = part_names{end - 1};

        current_frame_name = part_names{end};

        segmented_image_name = sprintf('%s%s/%s', run_result_dir, current_cell_name, current_frame_name);


        current_frame_info = imfinfo([image_dir, current_frame_name]);
        image_slice_num = length({current_frame_info.FileSize});
        a = imread([image_dir, current_frame_name], 1);
        [m, n] = size(a);



        I = zeros([m, n, image_slice_num]);

        % read image 
        for k = 1 : image_slice_num
            I(:, :, k) = imread([image_dir, current_frame_name], k);

        end

        % read mask
        I_mask = zeros([m, n, image_slice_num]);
        for k = 1 : image_slice_num
            I_mask(:, :, k) = imread([mask_dir, current_cell_name, filesep,  current_frame_name], k);
        end

        I_mask = I_mask > 0;            

        [sub1, sub2, sub3] = ind2sub([m, n, image_slice_num], find(I_mask == 1));
        I_crop = I([max(min(sub1) -10, 1) : min(max(sub1) + 10, m)], [max(min(sub2) - 10, 1) : min(max(sub2) + 10, n)], :);
        I_mask_crop = I_mask([max(min(sub1) -10, 1) : min(max(sub1) + 10, m)], [max(min(sub2) - 10, 1) : min(max(sub2) + 10, n)], :);

        % read segmented image
        I_seg = zeros(size(I_mask_crop));

        for k = 1 : image_slice_num
            I_seg(:, :, k) = imread([run_result_dir, current_cell_name, filesep,  current_frame_name], k);
        end                                

        I_seg = I_seg > 0;
        
        if false
            close all;
            figure, imshow(reshape_2d(I_seg), []);
            title([run_result_dir, current_cell_name, filesep,  current_frame_name]);
            pause(1);
        end

        I_seg_large = zeros([m, n, image_slice_num]);


        I_seg_large([max(min(sub1) -10, 1) : min(max(sub1) + 10, m)], [max(min(sub2) - 10, 1) : min(max(sub2) + 10, n)], :) = I_seg;

        I_position = zeros([m, n, image_slice_num]);

        I_position(position(2), position(1), :) = 1;
        se = fspecial('gaussian', 10, 3);
        se = strel('disk', 5);
        I_position_extend = imdilate(I_position, se);
        overlap = sum(I_position_extend(:) .* I_seg_large(:));

        if ~exist('max_seg_img', 'var')
            max_seg_img = zeros([m, n, image_slice_num]);
        end

        if overlap > max_overlap
            max_overlap = overlap;
            max_ind = i;
            max_seg_img = I_seg_large;    
        end   

    end
    
    current_frame_image = max_seg_img;
    debug = ~true;
    if debug
        close all;
        imshow(reshape_2d([max_seg_img; current_frame_image; max_seg_img .* current_frame_image], 5), []);
        pause(1);
    end
    
    current_window_image = imtransform(current_frame_image, maketform('affine', eye(3)), 'bilinear', 'UData', ...
        [1, size(current_frame_image, 2)], 'VData', [1, size(current_frame_image, 1)], 'Size', ...
        [window_size_2d, size(current_frame_image, 3)], 'XData', current_synapse_center_rounded(1) + ...
        current_room * [-1, 1], 'YData', current_synapse_center_rounded(2) + current_room * [-1, 1], 'FillValues', nan);

    current_window_image = image_resize_nd(current_window_image, [1, 1, current_z_desired_scale], 'bilinear', true);
    current_window_image = extrapolate_nans(current_window_image);
    
    if debug
        close all;
        im_center = 0 * current_window_image;
        im_center(36, 36, :) = 1;
        im_center = imdilate(im_center, ones(3, 3, 3));
        imshow(reshape_2d([current_window_image; im_center; current_window_image .* im_center], 5), []);
        pause(1);
    end
end







