function [I_out, I_sum, img_num] = binary_image_average(input_image_names, input_images_func, t_cell_info)
% the method is based on the overlapping of all images, and used a guassian
% filter to smooth the image, and then take the 0.5 quantile. This is the
% approximate average shape.
%
% Author: Xiongtao Ruan 
% Date: Oct. 29, 2015

%
% Author: Taraz E. Buck and Xiongtao Ruan
%
% Copyright (C) 2012-2016 Murphy Lab
% Computational Biology Department
% School of Computer Science
% Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu



img_num = numel(input_image_names);
[l, m, n] = size(input_images_func(1));
I_sum = zeros([l, m, n]);

for i = 1 : img_num
    img_i = input_images_func(i);
    I_sum = I_sum + img_i;
end

I_sum_smooth = smooth3(I_sum, 'gaussian', 3, 1);

thresh = prctile(unique(I_sum_smooth(I_sum > 1)), 50);

I_sum_thresh = I_sum_smooth >= thresh;
I_out = I_sum_thresh > 0;
I_out(I_sum == 0) = 0;

end
