function [can_start, final_name, final_exists, temp_exists] = chunk_start_clean(path, fname, final_extension, check_loadable, should_block)
  %[can_start, final_name, final_exists] = chunk_start_clean(path, fname, final_extension, should_block)
  %chunk_finish(path, fname, unique_filename)
  %  chunk_start checks if a certain unit of work has been done or is being 
  %  computed. The path is the location of both the final and temporary/lock
  %  files. Final files are assumed to be named [fname, '.mat'] by default,
  %  but the extension can be changed with the optional argument
  %  final_extension. Temporary files are named [fname, '.tmp']. Save to
  %  final_name and then call chunk_finish to delete the temporary file.
  %
  %
  %
  % 2016-02-24 xruan: copied from chunk_start.m
  % the purpose of this file is to make a clean mechanism, we store the job
  % id in the tem file, if the temp file exist but the job isn't running,
  % then assign the job for this job. Also simplify the temp file creat
  % procedure. That is directly make the file. 
  % 2016-03-02 xruan: add check if the mat file is loadable. 
  % 2016-03-10: change to use hardlink number as in chunk_start

%
% Author: Taraz E. Buck and Xiongtao Ruan
%
% Copyright (C) 2012-2016 Murphy Lab
% Computational Biology Department
% School of Computer Science
% Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu


  
    % Process options
    if nargin < 3 || isempty(final_extension)
      final_extension = '.mat';
    end
    if nargin < 4 || isempty(check_loadable)
      check_loadable = false;
    end
    if nargin < 5 || isempty(should_block)
      should_block = false;
    end
    % 2016-03-10 check if use ~ to represent home directory
    if strcmp(path(1), '~')
        home = getenv('HOME');
        path = [home, path(2 : end)];
    end
    if regexp(path, '//')
        strrep(path, '//', '/');
    end
    if strcmp(path(end), '/')
        path(end) = [];
    end
    temp_name = [path, '/', fname, '.tmp'];
    final_name = [path, '/', fname, final_extension];

    if check_loadable && ~exist(temp_name, 'file') && exist(final_name, 'file') 
        % 03/03/2016 check if the file is being written
        disp(final_name);
        try load(final_name)
        catch ME
            disp(sprintf('%s is not loadable, delete it and regenerate it!', final_name));
            delete(final_name);
        end
    end

    if ~exist(temp_name, 'file') && exist(final_name, 'file')
        can_start = false;
        final_exists = true;
        temp_exists = false;
        return;
    end
    hostname = regexprep(getenv('HOSTNAME'), '[\r\n]', '');
    if isempty(hostname)
        hostname = char(getHostName(java.net.InetAddress.getLocalHost()));
    end
    %xruan 02/21/2016
    % change to slurm job id setting
    % job_id = regexprep(getenv('PBS_JOBID'), '[\r\n]', '');
    job_id = regexprep(getenv('SLURM_JOB_ID'), '[\r\n]', '');
    jid_word = 'jid';

    if isempty(job_id)
        jid_word = 'pid';
        job_id = char(num2str(feature('getpid')));
    end

    unique_name = [path, '/', fname, '_', hostname, '_', job_id, '.tmp'];
    % current_command = ['touch "', unique_name, '"'];
    current_command = sprintf('printf "%s\\n%s" > "%s"', jid_word, job_id, unique_name);
    [status, output] = system(current_command);
    stat_command = ['stat --format="%h" "', unique_name, '"'];
    [status, stat_output] = system(stat_command);
    current_command = ['link "', unique_name, '" "', temp_name, '"'];
    [status, output] = system(current_command);
    [status, stat_output2] = system(stat_command);

    work_being_done = false;
    if length(stat_output) > 2 || length(stat_output2) > 2 || length(stat_output) == 0 || length(stat_output2) == 0
      pause(1);
      if length(stat_output) > 2 || length(stat_output) == 0
          if ~exist(unique_name, 'file')
            current_command = sprintf('printf "%s\\n%s" > "%s"', jid_word, job_id, unique_name);
            [status, output] = system(current_command);
          end
          stat_output = '1';
      end

      if length(stat_output2) > 2 || length(stat_output2) == 0
          for i = 1 : 5
              current_command2 = ['link "', unique_name, '" "', temp_name, '"'];
              [status, output] = system(current_command2);
              [status, stat_output2] = system(stat_command);
              if length(stat_output2) == 2 &&  length(stat_output2) > 0 && strcmp(stat_output2(1), '2')
                  break;
              end
          end
          if length(stat_output2) > 2 || length(stat_output2) == 0
              stat_output2 = '0';
          end              
      end
    end
    % hard_link_count = str2num(stat_output)
    % hard_link_count2 = str2num(stat_output2)
    % When code is pasted into the command line, it is taken up by Matlab as if it were part of the stdout of system, so just read the first integer instead of the above two lines:
    hard_link_count = strread(stat_output, '%d', 1);
    hard_link_count2 = strread(stat_output2, '%d', 1);
    if hard_link_count2 == 2 && hard_link_count == 1
        % Go ahead with work, we successfully created the first .tmp file:
        can_start = true;
    else
        % Someone else, possibly another matlabpool worker on the same machine and job, has already linked this or another file, do not start:
        can_start = false;
        work_being_done = true;
    end

    % Unlink the unique file, stat_output2 should be 1 if creating a new link fails:
    current_command = ['unlink "', unique_name, '"'];
    [status, output] = system(current_command);

    % final_exists, can_start, pause

    while true
        % If another job is doing this work but is unfinished and we should wait until the work finishes, just pause and keep checking again if work and writing are both finished:
        temp_exists = exist(temp_name, 'file') > 0;
        if work_being_done && should_block && temp_exists
          pause(1);
          continue;
        else
          final_exists = exist(final_name, 'file') > 0;
          break;
        end
    end

end

