function [result] = get_synapse_file_data(filename, include_cell_index)
% Returns a cell array where index is timeframe, and each element contains an array where a row is a cell (not necessarily the same one at the same index across timeframes) and columns are X, Y, and T (timeframe relative to synapse formation (T=0)).
% Just read the whole synapse file and return its parsed contents.
% 2012-09-02 tebuck: Copied from BK's read_synapse_file_new127.m.

%
% Author: Taraz E. Buck and Xiongtao Ruan
%
% Copyright (C) 2012-2016 Murphy Lab
% Computational Biology Department
% School of Computer Science
% Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu


  
  % error('Implementation yet unfinished below this line!')
  
if ~exist('include_cell_index', 'var')
  include_cell_index = false;
end

% fname = filename{1};
fname = filename;
%fname, pause
fid=fopen(fname);

slash_ = find(fname == '/');

Cell_index = 0;
Flag_new_cell = false;

cell_fname = '';
final_exists = false;

synapses_per_timeframe = {};

while ~feof(fid)
    tline = fgetl(fid);
    if isempty(tline)
        continue;
    end
    if tline(1) == '#'  % ignore this line & recognize a new cell starts from the next line
        Cell_index = Cell_index + 1;
        Flag_new_cell = true;
        Cell_volume_first = 0;
        Cell_fluo_first = 0;
        
    else
        [T, X, Y, timeframe, Manual, opt_seg_1, opt_seg_2, opt_cutoff, opt_fg_seed, opt_fg_seed2]=strread(tline,'%d %d %d %d %c %f %d %f %d %d');
        if isempty(X) || isempty(Y) || isempty(timeframe)
            continue;
        else
            % Check if this work has been done:
            
            while length(synapses_per_timeframe) < timeframe
              if include_cell_index
                synapses_per_timeframe{end + 1} = zeros(0, 4);
              else
                synapses_per_timeframe{end + 1} = zeros(0, 3);
              end
            end
            
            % The index into synapses_per_timeframe gives timeframe:
            if include_cell_index
              synapses_per_timeframe{timeframe} = [synapses_per_timeframe{timeframe}; ...
                X, Y, T, Cell_index];
            else
              synapses_per_timeframe{timeframe} = [synapses_per_timeframe{timeframe}; ...
                X, Y, T];
            end
            % synapses_per_timeframe{timeframe} = [synapses_per_timeframe{timeframe}; ...
              % X, Y, timeframe];

          % st=['T: ' num2str(T) ' X: ' num2str(X) ' Y: ' num2str(Y) ' t: ' num2str(timeframe)];
          % disp(st);
        end
    end
end


fclose('all');


result = synapses_per_timeframe;

