function [slice_highest_orders] = get_template_highest_orders(highest_order, options)
  % Process options:

%
% Author: Taraz E. Buck and Xiongtao Ruan
%
% Copyright (C) 2012-2016 Murphy Lab
% Computational Biology Department
% School of Computer Science
% Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu



  default_options = get_default_template_options(); 
  default_options.highest_order_same_across_slices = false;
  if ~exist('options', 'var') || isempty(options)
    options = default_options;
  else
    option_names = fieldnames(default_options); 
    for index = 1:length(option_names)
      current_option = option_names{index}; 
      if ~isfield(options, current_option)
        options = setfield(options, current_option, getfield(default_options, current_option)); 
      end
    end
  end
  %options
  
  [templ, cross_section_radii] = get_template(options);
  % BHC's convention is row is x, column is y:
  occupied_slices = find(isfinite(cross_section_radii(:, 1)'));
  if options.highest_order_same_across_slices
    % slice_highest_orders = highest_order .* ones(length(cross_section_radii), 1);
    slice_highest_orders = highest_order .* ones(length(occupied_slices), 1);
  else
    % slice_highest_orders = ceil(highest_order .* min(cross_section_radii(occupied_slices, :) ./ repmat([options.yr, options.zr], length(occupied_slices), 1), [], 2));
    slice_highest_orders = round(highest_order .* min(cross_section_radii(occupied_slices, :) ./ repmat([options.yr, options.zr], length(occupied_slices), 1), [], 2));
    if false
      % Debug:
      slice_highest_orders
      pause
    end
  end
  
