function [centroid] = image_centroid(given_image)
  % Centroid returned in XYZ order.
  %
  % Tests:
  % image_centroid(ones(5, 10, 15))
  % image_centroid(rand(5, 10, 15))

%
% Author: Taraz E. Buck and Xiongtao Ruan
%
% Copyright (C) 2012-2016 Murphy Lab
% Computational Biology Department
% School of Computer Science
% Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu


  
  number_dimensions = ndims(given_image);
  centroid = zeros(1, number_dimensions);
  total_intensity = sum(given_image(:));
  given_image = given_image ./ total_intensity;
  for dimension_index = 1:number_dimensions
    dimension_range = (1:size(given_image, dimension_index))';
    dimension_range = permute(dimension_range, circshift((1:number_dimensions)', dimension_index - 1));
    repeat_size = size(given_image);
    repeat_size(dimension_index) = 1;
    a = repmat(dimension_range, repeat_size);
    centroid(dimension_index) = sum(given_image(:) .* a(:));
  end
  
  centroid = centroid([2, 1, 3:end]);
  
end
