function [result_image] = image_resize_nd(given_image, scale, method, use_scale_not_size)

%
% Author: Taraz E. Buck and Xiongtao Ruan
%
% Copyright (C) 2012-2016 Murphy Lab
% Computational Biology Department
% School of Computer Science
% Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu




  image_dimensionality = max(ndims(given_image), numel(scale)); 
  if isscalar(scale)
    scale = scale * ones(1, image_dimensionality); 
  end
  if ~exist('method', 'var')
    method = 'bicubic';
  end
  if ~exist('use_scale_not_size', 'var')
    use_scale_not_size = false;
  end
  
  [discard, dimension_order] = sort(scale);
  if use_scale_not_size
    for dimension_index = dimension_order
      % Reorder the dimensions so that scaling along the first
      % dimension/vertical axis of the given_image is resizing along that
      % dimension after returning the given_image to the original order:
      current_dimension_order = [dimension_index, 1:(dimension_index - 1), (dimension_index + 1):image_dimensionality]; 
      given_image = permute(given_image, current_dimension_order); 
      image_size = size(given_image);
      novel_image_sizes = num2cell(image_size);
      novel_image_sizes{1} = [];
      given_image = reshape(given_image, image_size(1), image_size(2), []);
      % given_image = imresize(given_image, scale(dimension_index), method); 
      % given_image = imresize(given_image, [size(given_image, 1), image_size(2)], method); 
      % given_image = imresize(given_image, scale(dimension_index), method, 'Scale', [scale(dimension_index), 1]); 
      given_image = imresize(given_image, method, 'Scale', [scale(dimension_index), 1]); 
      given_image = reshape(given_image, novel_image_sizes{:});
      given_image = ipermute(given_image, current_dimension_order); 
      if false
        % Debug info:
        fprintf('dimension_index %d, scale %f\n', dimension_index, scale(dimension_index))
      end
    end
  else
    for dimension_index = dimension_order
      % Reorder the dimensions so that scaling along the first
      % dimension/vertical axis of the given_image is resizing along that
      % dimension after returning the given_image to the original order:
      current_dimension_order = [dimension_index, 1:(dimension_index - 1), (dimension_index + 1):image_dimensionality]; 
      given_image = permute(given_image, current_dimension_order); 
      image_size = size(given_image);
      novel_image_size = image_size;
      novel_image_size(1:2) = ceil(novel_image_size(1:2) .* [scale(dimension_index), 1]);
      given_image = reshape(given_image, image_size(1), image_size(2), []);
      given_image = imresize(given_image, novel_image_size(1:2), method); 
      given_image = reshape(given_image, novel_image_size);
      given_image = ipermute(given_image, current_dimension_order); 
    end
  end
  
  result_image = given_image;
  
end
  
