function [t_cell_info] = master_script_average_morphed_maps_new(t_cell_info, options)
  % generate average distances to the template for each protein of each
  % condition. 
  % generate sum of all morphed cell shapes.  
  %
  % 2015-11-24: xruan@andrew.cmu.edu
  % copied from master_script_build_models.m
  % 
  % 
  % 
  % 
  % 
  % % Dependencies:
  % % From the File Exchange: export_fig, pmkmp, toolbox_graph, toolbox_fast_marching, affine, Snake3D, Mesh_voxelisation, dirr, inhull
  % % From tebuck: modified version of Jieyue Li's HPA_lib, rasterize_mesh

%
% Author: Taraz E. Buck and Xiongtao Ruan
%
% Copyright (C) 2012-2016 Murphy Lab
% Computational Biology Department
% School of Computer Science
% Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu




  
  
  
  % Variables referenced more than once can be copied to local variables:
  master_script_options = t_cell_info.options;
  fprintf_diary_cycle_function = t_cell_info.log_info.fprintf_diary_cycle_function;
  regions_location = t_cell_info.path_info.regions_location;
  segmentations_location = t_cell_info.path_info.segmentations_location;
  segmentations_filtered_location = t_cell_info.path_info.segmentations_filtered_location;
  alignments_location = t_cell_info.path_info.alignments_location;
  deformations_location = t_cell_info.path_info.deformations_location;
  models_location = t_cell_info.path_info.models_location;
  
  run_relative_path_list = t_cell_info.synapse_info.run_relative_path_list;
  run_key_list = t_cell_info.synapse_info.run_key_list;
  % run_data_list = t_cell_info.synapse_info.run_data_list;
  run_images_list = t_cell_info.synapse_info.run_images_list;
  run_brightfield_images_list = t_cell_info.synapse_info.run_brightfield_images_list;
  run_image_frame_indices_list = t_cell_info.synapse_info.run_image_frame_indices_list;
  run_number_cells = t_cell_info.synapse_info.run_number_cells;
  run_number_frames = t_cell_info.synapse_info.run_number_frames;
  run_point_tracks = t_cell_info.synapse_info.run_point_tracks;
  run_relative_time_tracks = t_cell_info.synapse_info.run_relative_time_tracks;
  run_relative_times = t_cell_info.synapse_info.run_relative_times;
  run_number_relative_times = t_cell_info.synapse_info.run_number_relative_times;
  all_relative_times = t_cell_info.synapse_info.all_relative_times;
  number_all_relative_times = t_cell_info.synapse_info.number_all_relative_times;
  all_relative_times_seconds_data = t_cell_info.synapse_info.all_relative_times_seconds_data;
  all_relative_times_seconds = t_cell_info.synapse_info.all_relative_times_seconds;
  number_runs = t_cell_info.synapse_info.number_runs;
  
  condition_sensor_combinations = t_cell_info.synapse_info.condition_sensor_combinations;
  number_condition_sensors = t_cell_info.synapse_info.number_condition_sensors;
  conditions = t_cell_info.synapse_info.conditions;
  sensors = t_cell_info.synapse_info.sensors;
  number_conditions = t_cell_info.synapse_info.number_conditions;
  number_sensors = t_cell_info.synapse_info.number_sensors;
  condition_sensor_combination_ismember = t_cell_info.synapse_info.condition_sensor_combination_ismember;
  
  model_types = t_cell_info.model_type_info.model_types;
  number_model_types = t_cell_info.model_type_info.number_model_types;
  model_representation_functions = t_cell_info.model_type_info.model_representation_functions;
  model_reconstruction_functions = t_cell_info.model_type_info.model_reconstruction_functions;
  model_dimensionality_reduction_methods = t_cell_info.model_type_info.model_dimensionality_reduction_methods;
  model_dimensionality_reduction_method_parameters = t_cell_info.model_type_info.model_dimensionality_reduction_method_parameters;
  model_dimensionality_reduction_method_transformations = t_cell_info.model_type_info.model_dimensionality_reduction_method_transformations;
  
  template_centroid = t_cell_info.template_info.template_centroid;
  template_synapse = t_cell_info.template_info.template_synapse;
  template_image = t_cell_info.template_info.template_image;

  window_size_2d = t_cell_info.preprocessing_info.window_size_2d;
  window_center_2d = t_cell_info.preprocessing_info.window_center_2d;
  cropped_size = t_cell_info.preprocessing_info.cropped_size;
  background_level_function = t_cell_info.preprocessing_info.background_level_function;

  segmentation_rasterization_function = t_cell_info.segmentation_info.segmentation_rasterization_function;
  
  template_volume = t_cell_info.alignment_info.template_volume;
  segmentation_volume_threshold = t_cell_info.alignment_info.segmentation_volume_threshold;
  
  illustration_colormap_to_use = t_cell_info.plotting_info.illustration_colormap_to_use;

  
  % debug_print_model_filenames = false;
  debug_print_model_filenames = true
  
  average_morph_dir = [alignments_location(1 : end - 11), 'average_morphing_maps/'];
  if ~exist(average_morph_dir, 'dir')
      mkdir(average_morph_dir);
  end
  
  condition_sensor_combinations, number_condition_sensors, conditions, sensors, number_conditions, number_sensors
  model_types, number_model_types
  number_all_relative_times
  
  % error('Implementation yet unfinished below this line!')
    
  
  Full_sum_maps = {};
  B7Block_sum_maps = {};
  

  skip_morphing_sum = false;
  
  if skip_morphing_sum
    
    warning('>>>> HACK, skip_morphing_sum true!')
    
  else
    
    
    warning('Need to ensure all above computations are finished somehow before building models... Maybe block on every single good deformation''s file?')
    warning('Nevermind, can always rebuild!')
    % % [~, ~, deformations_list] = dir(deformations_location, 'name', '');
    % deformations_list = dir(deformations_location);
    % deformations_list = {deformations_list.name}.';
    % % deformations_list = strcat(deformations_location, deformations_list);
    % relevant_pattern = ['\.tmp$'];
    % number_temporary_deformation_files = sum(~cellfun(@isempty, regexp(deformations_list, relevant_pattern, 'start')))
    % if number_temporary_deformation_files > 0
      % error('%d temporary deformation files remain!', number_temporary_deformation_files)
    % end
    
    % deformations_list = master_script_list_relevant_results(t_cell_info, struct('given_directory', deformations_location));
    % deformations_list, beep, keyboard
    
    
    for condition_sensor_combination_index = 1:number_condition_sensors
      current_condition = condition_sensor_combinations{condition_sensor_combination_index, 1};
      current_sensor = condition_sensor_combinations{condition_sensor_combination_index, 2};
      % current_condition, current_sensor
      % error
      
      fprintf('condition_sensor_combination_index %d, current_condition "%s", current_condition "%s"\n', condition_sensor_combination_index, current_condition, current_sensor)

      % Find relevant runs:
      % Should this just compare .mat files for speed?:
      run_list_relevant = true(size(run_key_list));
      % relevant_pattern = ['^[^-]+ - ', current_condition, '_', current_sensor, '\.xls'];
      relevant_pattern = ['^[^-]+ - ', current_condition, '_', current_sensor, '_'];
      run_list_relevant = run_list_relevant & ~cellfun(@isempty, regexp(run_key_list, relevant_pattern, 'start'));
      relevant_runs = run_key_list(run_list_relevant);
      relevant_run_indices = reshape(find(run_list_relevant), 1, []);
      
      % relevant_runs, beep, keyboard
      
      % We want separate model files for each relative time to keep down memory usage and sooner have intermediate results:
      
      for all_relative_time_index = 1:number_all_relative_times
        relative_time = all_relative_times(all_relative_time_index);
        
        fprintf(' all_relative_time_index %d, relative_time % d\n', all_relative_time_index, relative_time)
        
        % We want separate model files for each model type (makes adding new model types easier):
        
        for model_type_index = 1:number_model_types
          model_type = model_types{model_type_index};
          if ~strcmp(model_type, 'standardized_voxels')
              continue;
          end
          representation_function = model_representation_functions{model_type_index};
          reconstruction_function = model_reconstruction_functions{model_type_index};

          % This is the .mat file, not the figure:
          current_model_filename = sprintf('%s%s_%s_reltime% 3d_morphing_map_information', master_script_options.model_prefix, current_condition, current_sensor, relative_time);
          % current_model_filename = sprintf('%s_%s_frame%03d_reltime% 3d_%s', current_condition, current_sensor, all_relative_time_index, relative_time, model_type);
          [can_start, final_name, final_exists] = chunk_start(average_morph_dir, current_model_filename);
          
          if debug_print_model_filenames, fprintf_diary_cycle_function('Model file ''%s'': can_start %d, final_exists %d\n', current_model_filename, can_start, final_exists), end
          
          
          if false
            % Debugging:
            warning('>>>> HACK, always building models!')
            can_start = true;
            chunk_finish(average_morph_dir, current_model_filename);
          end
          
          if ~can_start
            continue
          end
          
          fprintf('  model_type_index %d, model_type "%s"\n', model_type_index, model_type)
          
          current_count = 0;
          
          current_morph_sum = zeros(71, 71, 35);
          current_vec_x = zeros(71, 71, 35);
          current_vec_y = zeros(71, 71, 35);
          current_vec_z = zeros(71, 71, 35);
          current_field_magnitude = zeros(71, 71, 35);
          
          current_sum_distance = 0;
          current_mean_distance = 0;
          
          % Running/accumulated model:
          current_mean = [];
          current_standard_deviation = [];
          current_covariance = [];
          
          should_compute_covariance = false;
          
          % Batch model:
          % Preallocate as much space as might be necessary:
          current_number_cells = sum(cellfun(@(x)size(x, 1), run_relative_time_tracks(relevant_run_indices)));
          current_number_features = numel(representation_function(double(template_image)));
          current_data = zeros(current_number_cells, current_number_features);
          
          current_data_relevant_run_keys = run_key_list(relevant_run_indices);
          current_data_relevant_run_key_indices = zeros(current_number_cells, 1);
          
          for run_index = relevant_run_indices
            % Prepare useful variables:
            run_relative_path = run_relative_path_list{run_index};
            run_key = run_key_list{run_index};
            run_image_path = run_images_list{run_index};
            number_cells = run_number_cells(run_index);
            number_frames = run_number_frames(run_index);
            cell_synapse_point_tracks = run_point_tracks{run_index};
            cell_relative_time_tracks = run_relative_time_tracks{run_index};
            
            cell_relative_time_matrix = cellfun(@(x)[x, nan(1, 1 - length(x))], cell_relative_time_tracks);
            
            [relative_time_cell_indices, relative_time_frame_indices] = find(cell_relative_time_matrix == relative_time);
            
            for cell_frame_pair_index = 1:length(relative_time_cell_indices)
              cell_index = relative_time_cell_indices(cell_frame_pair_index);
              frame_index = relative_time_frame_indices(cell_frame_pair_index);
              current_synapse_center_rounded = cell_synapse_point_tracks{cell_index, frame_index};
              if isempty(current_synapse_center_rounded)
                continue
              end
              current_synapse_center_rounded = current_synapse_center_rounded(1, 1:2);
              cell_deformation_filename = [run_key, sprintf('_frame%05d_synapse%05d,%05d', frame_index, current_synapse_center_rounded)];
              
              if false && all_relative_time_index == 1 && model_type_index == 1
                % Debug info:
                fprintf('   cell_frame_pair_index %d, cell_index %d, frame_index %d\n', cell_frame_pair_index, cell_index, frame_index)
                % keyboard
              end
              
              % Eventually, this should wait for all .mat files, and master_script_morph_segmentations.m should save .mat files with empty results if failed instead of leaving .tmp files. This way, running multiple jobs, others will quit when chunk_start here fails but the job that computes will get everything in the model files.
              
              if ~exist([deformations_location, cell_deformation_filename, '.mat'], 'file')
                if false && exist([segmentations_location, cell_deformation_filename, '.mat'], 'file')
                  fprintf('   "%s" does not exist, but segmentation does!\n', [deformations_location, cell_deformation_filename, '.mat'])
                  % beep, keyboard
                end
                if false && exist([segmentations_filtered_location, cell_deformation_filename, '.mat'], 'file')
                  fprintf('   "%s" does not exist, but filtered segmentation does!\n', [deformations_location, cell_deformation_filename, '.mat'])
                  % beep, keyboard
                end
                if false && all_relative_time_index == 1 && model_type_index == 1
                  % Debug info:
                  fprintf('   "%s" does not exist!\n', [deformations_location, cell_deformation_filename, '.mat'])
                  % run_existing_files = ls([deformations_location, run_key, '*.mat'])
                  % run_existing_files = arrayfun(@(x)x.name, dir([deformations_location, run_key, '*.mat']), 'UniformOutput', false)
                  % beep, keyboard
                  % pause
                end
                continue
              end

              % fprintf('   "%s" exists!\n', [deformations_location, cell_deformation_filename, '.mat'])

              a = load([regions_location, cell_deformation_filename, '.mat']);
              if isempty(fieldnames(a))
                continue
              end
              current_window_image = a.current_window_image;
              try 
                a = load([deformations_location, cell_deformation_filename, '.mat']);
              catch
                  continue;
              end
              if isempty(fieldnames(a))
                continue
              end
              cropped_raw_image_deformed = a.cropped_raw_image_deformed;
              cropped_segmentation_image_deformed = a.cropped_segmentation_image_deformed;
              
              current_volume = sum(cropped_segmentation_image_deformed(:));
              is_segmentation_low_volume = current_volume <= segmentation_volume_threshold;
              
              current_distance = a.r.total_distance;
              current_interpolated_image = contrast_stretch(round(a.r.interpolated_image.image{1}));
              
              % error('Correct if stmt')
              if is_segmentation_low_volume
                continue
              end
              
              % Subtract the original image's estimated background, then normalize total intensity to one:
              current_deformed_image = cropped_raw_image_deformed - background_level_function(current_window_image);
              current_deformed_image(current_deformed_image < 0) = 0;
              current_deformed_image(template_image < .5) = 0;
              current_deformed_image = current_deformed_image ./ sum(current_deformed_image(:));
              
              current_parameters = representation_function(current_deformed_image);
         
              
              if false
                current_parameters_norm = norm(current_parameters)
                imagesc(reshape_2d(current_deformed_image, -1)), axis image, colormap(illustration_colormap_to_use), pause
              end
              
              % Batch model:
              current_count = current_count + 1;
              % current_data(current_count, :) = reshape(current_parameters, 1, []);
              current_data(current_count, :) = current_parameters;
              % keyboard
              current_data_relevant_run_key_indices(current_count, :) = find(ismember(current_data_relevant_run_keys, run_key));
              
              current_morph_sum = current_morph_sum + current_interpolated_image;
          
              current_sum_distance = current_sum_distance + current_distance;
              current_mean_distance = (current_mean_distance * (current_count - 1) + current_distance) / current_count;
              current_vec_x = current_vec_x + a.r.source_deformation{1}.get_data();
              current_vec_y = current_vec_y + a.r.source_deformation{2}.get_data();
              current_vec_z = current_vec_z + a.r.source_deformation{3}.get_data();
              
            end

            if false && all_relative_time_index == 1 && model_type_index == 1
              % Debug info:
              keyboard
            end
            
          end
          
          % Batch model:
          current_data = current_data(1:current_count, :);
          current_mean = mean(current_data, 1);
          current_standard_deviation = std(current_data, [], 1);
          if should_compute_covariance
            current_covariance = cov(current_data);
          end
          current_data_relevant_run_key_indices = current_data_relevant_run_key_indices(1:current_count, :);

          % if any(isnan(current_mean))
            % beep, keyboard
          % end
          
          fprintf('   current_count %d\n', current_count)
          
          if false && current_count == 0
            % Debug info:
            beep, keyboard
          end
          
          if current_count > 0
            current_field_magnitude = sqrt(current_vec_x .^ 2 + current_vec_y .^ 2 + current_vec_z .^ 2);
            save(final_name, 'current_data_relevant_run_keys', 'current_data_relevant_run_key_indices', 'current_count', 'current_morph_sum', 'current_sum_distance', 'current_mean_distance', 'current_vec_x', 'current_vec_y', 'current_vec_z', 'current_field_magnitude');
            % Leave the .tmp lock file so this work isn't repeated:
            chunk_finish(average_morph_dir, current_model_filename);
          end
          
          % Leave the .tmp lock file so this work isn't repeated:
          % chunk_finish(models_location, current_model_filename);
          
          current_count = 0;
          % pause
        end
      end
      % pause
    end
    % error

  end
  
  visualization = true;
  
  if visualization
      used_sensors = sensors;
      used_conditions = {'Full Stimulus', 'B7 Blockade'};
      csv_mat = {};
      mean_all_morphing_maps = {};
      mean_all_vec_x = {};
      mean_all_vec_y = {};
      mean_all_vec_z = {};     
      
      for i = 1 : numel(used_conditions)   
          for all_relative_time_index = 1:number_all_relative_times
              sum_all_same_timepoint = zeros([71, 71, 35]);
              sum_vec_x = zeros([71, 71, 35]);
              sum_vec_y = zeros([71, 71, 35]);
              sum_vec_z = zeros([71, 71, 35]);
              sum_count = 0;
              for j = 1 : numel(used_sensors)
                current_condition = used_conditions{i};
                current_sensor = used_sensors{j};            
                relative_time = all_relative_times(all_relative_time_index);
                current_model_filename = sprintf('%s%s_%s_reltime% 3d_morphing_map_information', master_script_options.model_prefix, current_condition, current_sensor, relative_time);
                a = load([average_morph_dir, current_model_filename]);
                csv_mat{end + 1} = {current_condition, current_sensor, relative_time, a.current_mean_distance};
                % sum_all_same_timepoint = sum_all_same_timepoint + a.current_morph_sum;
                sum_vec_x = sum_vec_x + a.current_vec_x;
                sum_vec_y = sum_vec_y + a.current_vec_y;
                sum_vec_z = sum_vec_z + a.current_vec_z;
                sum_count = sum_count + a.current_count;
                
              end
              [x_ind, y_ind, z_ind] = meshgrid(1 : 71, 1 : 71, 1 : 35);
              men_vec_x = sum_vec_x / sum_count - x_ind;
              men_vec_y = sum_vec_y / sum_count - y_ind;
              men_vec_z = sum_vec_z / sum_count - z_ind;
              
              mean_all_morphing_maps{i, all_relative_time_index } = sqrt(men_vec_x .^ 2 + men_vec_y .^ 2 + men_vec_z .^ 2);
              mean_all_vec_x{i, all_relative_time_index} = men_vec_x;
              mean_all_vec_y{i, all_relative_time_index} = men_vec_y;
              mean_all_vec_z{i, all_relative_time_index} = men_vec_z;
          end
      end
      
      % write csv files for mean distances
        csv_field_matrix = reshape(reshape(csv_mat, numel(used_sensors), [])', [], 1);
        
        current_csv_filename = sprintf('%saverage_diffeomorphic_distances.csv', master_script_options.model_prefix);
        
        fid = fopen([average_morph_dir, current_csv_filename], 'wt');
        write_title = true;
        if write_title
            title = {'Condition', 'Protein', 'Relative time', 'Average distance'};
            csv_field_matrix = [{title}; csv_field_matrix];
        end
        for i = 1:size(csv_field_matrix, 1)
            csv_field_matrix_i = csv_field_matrix{i}
            for j = 1:size(csv_field_matrix_i, 2)
                if j > 1
                    fprintf(fid, ',');
                end
                if ischar(csv_field_matrix{i}{j})
                  fwrite(fid, csv_field_matrix{i}{j});
                else
                  fprintf(fid, '%f', csv_field_matrix{i}{j});
                end        
            end
            fprintf(fid, '\n');
        end
        fclose(fid);
        
        % xruan 03/19/2016 change the format of the csv files. 
        csv_field_matrix = reshape(reshape(csv_mat, numel(used_sensors), [])', [], 1);
        csv_field_matrix_dist = reshape(cellfun(@(x) x{4}, csv_field_matrix), [], numel(used_sensors));
        csv_field_matrix_time = reshape(cellfun(@(x) x{3}, csv_field_matrix), [], numel(used_sensors));
        csv_field_matrix_condition = reshape(cellfun(@(x) x{1}, csv_field_matrix, 'UniformOutput', false), [], numel(used_sensors));
        csv_field_matrix_time_col = csv_field_matrix_time(:, 1);
        csv_field_matrix_condition_col = csv_field_matrix_condition(:, 1);
        csv_field_matrix_dist = mat2cell(csv_field_matrix_dist, ones(size(csv_field_matrix_dist, 1), 1), ones(size(csv_field_matrix_dist, 2), 1));
        current_csv_filename = sprintf('%saverage_diffeomorphic_distances_reformat.csv', master_script_options.model_prefix);
        % csv_field_matrix_new = {used_sensors'; csv_field_matrix_dist}
        fid = fopen([average_morph_dir, current_csv_filename], 'wt');        
        
        for i = 1 : size(csv_field_matrix_dist) + 1
            if i == 1
                fprintf(fid, 'Condition,Relative time');
                for j = 1 : numel(used_sensors)
                    fprintf(fid, ',');
                    fwrite(fid, used_sensors{j});
                end
                fprintf(fid, '\n');
            else
                fwrite(fid, csv_field_matrix_condition_col{i - 1});
                fprintf(fid, ',');
                fprintf(fid, '%f', csv_field_matrix_time_col(i - 1));
                fprintf(fid, ',');
                for j = 1:size(csv_field_matrix_dist, 2)
                    if j > 1
                        fprintf(fid, ',');
                    end
                    if ischar(csv_field_matrix_dist{i-1, j})
                      fwrite(fid, csv_field_matrix_dist{i-1, j});
                    else
                      fprintf(fid, '%0.3f', csv_field_matrix_dist{i-1, j});
                    end        
                end
                fprintf(fid, '\n');
            end
        end            
        fclose(fid);    
        
        % visualize mean of morphed images. 
        for i = 1 : numel(used_conditions)
            current_condition = used_conditions{i};
            mean_group = mean_all_morphing_maps(i, :);
            %plot the mean of all images
            plot_color_bar = true;
            output_filename = sprintf('%s%smean_of_morphing_field_center_slices_normalized_%s.png', average_morph_dir, master_script_options.model_prefix, current_condition);
            max_int = max(cellfun(@(x) max(x(:)), mean_group));
            min_int = min(cellfun(@(x) min(x(:)), mean_group));
            normalized_mean_group = cellfun(@(x) (x - min_int) ./ (max_int - min_int), mean_group, 'UniformOutput', false)
            illustration_colormap_to_use_1 = [1, 1, 1; illustration_colormap_to_use];
            plot_center_slides_of_all_images(normalized_mean_group, 'Center slide of the average shapes', illustration_colormap_to_use_1, output_filename, plot_color_bar)
        end
        
        for i = 1 : numel(used_conditions)
            current_condition = used_conditions{i};
            mean_group = mean_all_morphing_maps(i, :);
            %plot the mean of all images
            plot_color_bar = true;
            output_filename = sprintf('%s%smean_of_morphing_field_center_slices_%s.png', average_morph_dir, master_script_options.model_prefix, current_condition);
            max_int = max(cellfun(@(x) max(x(:)), mean_group));
            min_int = min(cellfun(@(x) min(x(:)), mean_group));
            % normalized_mean_group = cellfun(@(x) (x - min_int) ./ (max_int - min_int), mean_group, 'UniformOutput', false)
            axis_scale = [round(min_int, 2), round(max_int, 2)];

            illustration_colormap_to_use_1 = [1, 1, 1; illustration_colormap_to_use];
            plot_center_slides_of_all_images(mean_group, 'Center slide of the average shapes', illustration_colormap_to_use_1, output_filename, plot_color_bar, axis_scale)
            save([output_filename(1 : end - 3), 'mat'], 'max_int', 'min_int');
        end
                
        % log ratio of magnitude of mean vectors
        mean_log_ratio = {};
        mean_log_ratio_raw = cellfun(@(x, y) log(x ./ y), mean_all_morphing_maps(2, :), mean_all_morphing_maps(1, :), 'Uniformoutput', false);
        for i = 1 : numel(mean_log_ratio_raw)
            current_mean_log_ratio = mean_log_ratio_raw{i};
            current_mean_log_ratio(isnan(current_mean_log_ratio)) = 0;
            mean_log_ratio{end + 1} = current_mean_log_ratio;
        end
        
        plot_color_bar = true;
        output_filename = sprintf('%s%sratio_of_mean_morphing_field_center_slices_%s.png', average_morph_dir, master_script_options.model_prefix, current_condition);
        max_log_ratio = max(cellfun(@(x) max(x(:)), mean_log_ratio));
        min_log_ratio = min(cellfun(@(x) min(x(:)), mean_log_ratio));
        axis_scale = [round(min_log_ratio, 2), round(max_log_ratio, 2)];
        plot_center_slides_of_all_images(mean_log_ratio, '', illustration_colormap_to_use, output_filename, plot_color_bar, axis_scale)
        
        % dot product of mean vectors. 
        dot_prod_maps_raw = cellfun(@(x1, x2, y1, y2, z1, z2) x1 .* x2 + y1 .* y2 + z1 .* z2, mean_all_vec_x(1, :), mean_all_vec_x(2, :), mean_all_vec_y(1, :), mean_all_vec_y(2, :), mean_all_vec_z(1, :), mean_all_vec_z(2, :), 'Uniformoutput', false);
        dot_prod_maps = {};
        output_filename = sprintf('%s%sdot_product_of_mean_morphing_field_center_slices_%s-%s.png', average_morph_dir, master_script_options.model_prefix, used_conditions{1}, used_conditions{2});
        for i = 1 : numel(mean_log_ratio_raw)
            background_inds = ((mean_all_morphing_maps{1, i} .^ 2 + mean_all_morphing_maps{2, i} .^ 2) == 0);
            current_dot_prod_map = dot_prod_maps_raw{i};
            % current_dot_prod_map(background_inds) = -1;
            current_dot_prod_map(current_dot_prod_map == 0) = -1; 
            dot_prod_maps{end + 1} = current_dot_prod_map;
        end
        
        max_dot_prod = max(cellfun(@(x) max(x(:)), dot_prod_maps));
        min_dot_prod = min(cellfun(@(x) min(x(:)), dot_prod_maps));
        axis_scale = [round(min_dot_prod, 2), round(max_dot_prod, 2)];

        plot_center_slides_of_all_images(dot_prod_maps, '', illustration_colormap_to_use_1, output_filename, plot_color_bar, axis_scale)
        
        prctiles = [10, 20, 30, 50, 70, 80, 90, 95, 99];
        
        for i = 1 : numel(prctiles)
            a = cell2mat(dot_prod_maps_raw);
            prctile_i = prctiles(i);
            threshold = prctile(unique(a(:)), prctile_i);
            thrd_dot_prod_maps = cellfun(@(x) x .* (x >= threshold), dot_prod_maps_raw, 'UniformOutput', false);
            max_dot_prod = max(cellfun(@(x) max(x(:)), thrd_dot_prod_maps));
            min_dot_prod = min(cellfun(@(x) min(x(:)), thrd_dot_prod_maps));
            axis_scale = [round(min_dot_prod, 2), round(max_dot_prod, 2)];
            output_filename = sprintf('%s%sdot_product_of_mean_morphing_field_center_slices_%s-%s_precentile-%d.png', average_morph_dir, master_script_options.model_prefix, used_conditions{1}, used_conditions{2}, prctile_i);

            plot_center_slides_of_all_images(thrd_dot_prod_maps, '', illustration_colormap_to_use_1, output_filename, plot_color_bar, axis_scale)

        end
        
        
end        
      
  
  
 
  
  
end  



function plot_center_slides_of_all_images(img_group, title_string, illustration_colormap_to_use, output_filename, plot_color_bar, axis_scale)
% here we have 12 relative time points, so we plot as 3 X 4. 

if nargin < 5
    plot_color_bar = false;
end
if nargin < 6
    axis_scale = [0, 1];
end

close all; 
illustration_colormap_to_use = [ illustration_colormap_to_use];
% figure('Color', illustration_colormap_to_use(1, :))
figure('Color', [1,1,1])

set(gcf, 'Visible', 'off')
set(gcf, 'Visible', 'on')
set(gcf, 'Renderer', 'opengl');
dpi = 300;
figure_height = 600; %round(master_script_options.figure_maximum_width / aspect_ratio * figure_height_scale);
figure_width = 800; %round(master_script_options.figure_maximum_width * figure_height_scale);
set(gcf, 'Position', [1920, 1, figure_width, figure_height])

x = 0.01 : 0.22 : 0.88;
y = 0.06 : 0.30 : 0.8;

[L, B] = meshgrid(x, y);
L = L';
B = flipud(B)';

W = 0.22;
H = 0.30;
reltimes = -2 : 9;
% title(title_string);
crop_img = true;
size_crop = 1;
axis off

for i = 1 : 12 
    image_i = img_group{i};
    image_to_show = squeeze(image_i(:, :, round(size(image_i, 3) / 2)));
    image_to_show = image_to_show;
    if crop_img 
        if any(reshape(image_to_show([ 1 : size_crop, end - size_crop + 1 : end], [ 1 : size_crop, end - size_crop + 1 : end]), 1, []) > 0)
            warning('some voxels have been cropped, use smaller crop size');
        end
        image_to_show = image_to_show(1 + size_crop : end - size_crop, 1 + size_crop : end - size_crop);
    end
    %matrix_plot_handle = gca;
    axes('Position', [L(i), B(i), W, H])
    %iptsetpref('ImshowBorder','tight')
    imagesc(image_to_show);
    % freezeColors;
    % text(14, 80, sprintf('Relative time %d', reltimes(i))) 
    axis off, colormap(illustration_colormap_to_use);
    
    caxis(axis_scale);
    hold on    
end


if plot_color_bar
    h = colorbar;
    set(h, 'Position', [.89 .06 .04 0.82]);
    % set(h, 'Position', [.89 .06 .04 0.87]);
    % ylabel(h, 'Relative probability (per model)');
    if nargin == 6
        set(h, 'Fontsize', 12, 'fontweight', 'bold');
    else
        set(h, 'Ticks', [0 : 0.2 : 1], 'Fontsize', 12, 'fontweight', 'bold');
    end
    axis off
end


hold off;


export_fig(output_filename, '-opengl', '-png', '-a1', ['-r', num2str(dpi)])


end
  
