function [t_cell_info] = master_script_average_shapes(t_cell_info, options)
  % compute the average shape in the same condition/sensor. 
  %
  % 2015-10-29: xruan 
  % copied from master_script_morph_segmentations.m
  % 2016-03-11: change to only save one time point 
  % 
  % 
  % 
  % 
  % % Dependencies:
  % % From the File Exchange: export_fig, pmkmp, toolbox_graph, toolbox_fast_marching, affine, Snake3D, Mesh_voxelisation, dirr, inhull
  % % From tebuck: modified version of Jieyue Li's HPA_lib, rasterize_mesh

%
% Author: Taraz E. Buck and Xiongtao Ruan
%
% Copyright (C) 2012-2016 Murphy Lab
% Computational Biology Department
% School of Computer Science
% Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu



  
  
  
  % Variables referenced more than once can be copied to local variables:
  master_script_options = t_cell_info.options;
  fprintf_diary_cycle_function = t_cell_info.log_info.fprintf_diary_cycle_function;
  regions_location = t_cell_info.path_info.regions_location;
  segmentations_filtered_location = t_cell_info.path_info.segmentations_filtered_location;
  alignments_location = t_cell_info.path_info.alignments_location;
  deformations_location = t_cell_info.path_info.deformations_location;
  
  run_relative_path_list = t_cell_info.synapse_info.run_relative_path_list;
  run_key_list = t_cell_info.synapse_info.run_key_list;
  % run_data_list = t_cell_info.synapse_info.run_data_list;
  run_images_list = t_cell_info.synapse_info.run_images_list;
  run_brightfield_images_list = t_cell_info.synapse_info.run_brightfield_images_list;
  run_image_frame_indices_list = t_cell_info.synapse_info.run_image_frame_indices_list;
  run_number_cells = t_cell_info.synapse_info.run_number_cells;
  run_number_frames = t_cell_info.synapse_info.run_number_frames;
  run_point_tracks = t_cell_info.synapse_info.run_point_tracks;
  run_relative_time_tracks = t_cell_info.synapse_info.run_relative_time_tracks;
  run_relative_times = t_cell_info.synapse_info.run_relative_times;
  run_number_relative_times = t_cell_info.synapse_info.run_number_relative_times;
  all_relative_times = t_cell_info.synapse_info.all_relative_times;
  number_all_relative_times = t_cell_info.synapse_info.number_all_relative_times;
  all_relative_times_seconds_data = t_cell_info.synapse_info.all_relative_times_seconds_data;
  all_relative_times_seconds = t_cell_info.synapse_info.all_relative_times_seconds;
  number_runs = t_cell_info.synapse_info.number_runs;
  
  template_centroid = t_cell_info.template_info.template_centroid;
  template_synapse = t_cell_info.template_info.template_synapse;
  template_image = t_cell_info.template_info.template_image;

  window_size_2d = t_cell_info.preprocessing_info.window_size_2d;
  window_center_2d = t_cell_info.preprocessing_info.window_center_2d;
  cropped_size = t_cell_info.preprocessing_info.cropped_size;

  segmentation_rasterization_function = t_cell_info.segmentation_info.segmentation_rasterization_function;
  
  template_volume = t_cell_info.alignment_info.template_volume;
  segmentation_volume_threshold = t_cell_info.alignment_info.segmentation_volume_threshold;
  
  % debug_print_deformation_skip_reasons = false;
  debug_print_deformation_skip_reasons = true;
  
  illustration_colormap_to_use = t_cell_info.plotting_info.illustration_colormap_to_use;


  % Set options for T cell image registration:
  registration_options = struct();
  
  % registration_options.filter_radius = 4;
  % registration_options.filter_radius = 8;
  % registration_options.filter_radius = 16;
  % registration_options.filter_radius = 32;
  % registration_options.filter_radius = ceil(master_script_options.cell_diameter / 4 / 2) * 2;
  registration_options.filter_radius = ceil(master_script_options.cell_diameter / 2 / 2) * 2;
  registration_options.kernel_z_radius = registration_options.filter_radius;
  registration_options.maximum_deformation_per_step = [1, 1, 1] * .5;
  registration_options.window_radius = t_cell_info.template_info.template_options.imx;
  % registration_options.convergence_absolute_error = prod(cropped_size) ./ 2e3;
  % registration_options.convergence_absolute_error = prod(cropped_size) ./ 1e3;
  % registration_options.convergence_absolute_error = prod(cropped_size) ./ 5e2;
  registration_options.convergence_absolute_error = prod(cropped_size) ./ 2e2;
  % registration_options.convergence_absolute_error = prod(cropped_size) ./ 1e2;
  % registration_options.convergence_absolute_error = prod(cropped_size) ./ 1e1;
  registration_options.single_sided = true;
  registration_options.maximum_iterations = max(t_cell_info.template_info.template_dimensions ./ registration_options.maximum_deformation_per_step);

  registration_options.save_intermediates = false;
  registration_options.save_stages = false;
  % registration_options.save_intermediate_images = true;
  % registration_options.always_save_full_intermediates = true;
  registration_options
  
  

  
  % % error('Update to scale voxels to cubical, use proper param adjustments to scale!')
  % warning('Update to scale voxels to cubical, use proper param adjustments to scale!')
  
  
  % Keep running until everything is finished:
  run_cell_index_frame_indices_uncomputed = cell(number_runs, 1);
  for run_index = 1:number_runs
    run_key = run_key_list{run_index};
    number_cells = t_cell_info.synapse_info.run_number_cells(run_index);
    number_frames = run_number_frames(run_index);
    cell_synapse_point_tracks = run_point_tracks{run_index};
    run_cell_index_frame_indices_uncomputed{run_index} = false(number_cells, number_frames);
    for current_cell_index = 1:number_cells
      for frame_index = 1:number_frames
        current_synapse_center_rounded = cell_synapse_point_tracks{current_cell_index, frame_index};
        if isempty(current_synapse_center_rounded)
          continue
        end
        current_synapse_center_rounded = current_synapse_center_rounded(1, 1:2);
        
        first_frame_index = find(~cellfun(@isempty, cell_synapse_point_tracks(current_cell_index, :)), 1);
        if isempty(first_frame_index)
          run_cell_indices_uncomputed{run_index}(current_cell_index) = false;
          continue
        end
        
        cell_deformation_filename = [run_key, sprintf('_frame%05d_synapse%05d,%05d', frame_index, current_synapse_center_rounded)];
        cell_alignment_filename = [alignments_location, cell_deformation_filename];
        
        if ~exist([cell_alignment_filename, '.mat'], 'file')
          % Given modifications to master_script_align_segmentations.m, all alignment should already be complete and thus this file will never be available (should look into why it needs to be checked, though):
          continue
        end
        
        run_cell_index_frame_indices_uncomputed{run_index}(current_cell_index, frame_index) = true;
      end
    end
  end
  
  recently_did_any_work = false;
  did_any_work = false;
  
  relative_time_index = run_relative_times;
  conditions = {'Full Stimulus'};
  image_group = {};
  image_name_group = {};
  average_group = {};
  sum_group = {};
  img_num_grop = {};
  img_sum_group = {};
  average_methods_set = {'sum_up', 'diffeomporphic'};
  average_method = average_methods_set{2};
  if strcmp(average_method, 'sum_up')
    average_img_dir = [alignments_location(1 : end - 11), 'average_shapes_by_time/'];
    if ~exist(average_img_dir, 'dir')
      mkdir(average_img_dir);
    end
  elseif strcmp(average_method, 'diffeomporphic')
    diff_dist_average_dir = [alignments_location(1 : end - 11), 'average_shapes_by_shapespace/'];
    average_img_dir = diff_dist_average_dir;
  end

  
  for i = 1 : numel(conditions)
      current_condition = conditions{i};
      
      used_run_list_ind = find(~cellfun(@isempty, regexp(run_key_list, current_condition)) == 1);      
      
      for j = 1 : numel(all_relative_times)
          current_reltime = all_relative_times(j);
          % image_group_j = {};
          image_name_group_j = {};
          image_fullname_group_j = {};
          I_sum = zeros(size(template_image));
          filename_reltime_j = sprintf('%s%s_reltime%2d_average_shape_%s', average_img_dir, current_condition, current_reltime, average_method);
          if exist(filename_reltime_j, 'file')
              % xruan 03/13/2016 because shapespace method compute partial columns one time
              % so do not load exisiting file. 
              if ~strcmp(average_method, 'diffeomporphic')
                load(filename_reltime_j);
              end
              
          else
              for k = 1 : numel(used_run_list_ind)
                  run_ind = used_run_list_ind(k);
                  run_key = run_key_list{run_ind};
                  run_reltime_track_k = run_relative_time_tracks{run_ind};
                  rel_time_ind = find(~cellfun(@isempty, cellfun(@(x) find(x == current_reltime), run_reltime_track_k, 'UniformOutput', false)) == 1);
                  [cell_inds, frame_inds] = ind2sub(size(run_reltime_track_k), rel_time_ind);
                  cell_synapse_point_tracks = run_point_tracks{run_ind};
                  for l = 1 : numel(rel_time_ind)
                      current_cell_index = cell_inds(l);
                      frame_index = frame_inds(l);
                      current_synapse_center_rounded = cell_synapse_point_tracks{current_cell_index, frame_index};
                      if isempty(current_synapse_center_rounded)
                          continue
                      end    
                      current_synapse_center_rounded = current_synapse_center_rounded(1, 1:2);
                    first_frame_index = find(~cellfun(@isempty, cell_synapse_point_tracks(current_cell_index, :)), 1);
                    if isempty(first_frame_index)
                      continue
                    end

                    cell_alignment_filename = [run_key, sprintf('_frame%05d_synapse%05d,%05d', frame_index, current_synapse_center_rounded)];
                    cell_alignment_fullname = [alignments_location, cell_alignment_filename];
                    if ~exist([cell_alignment_fullname, '.mat'], 'file');
                        continue;
                    end
                    a = load([cell_alignment_fullname, '.mat']);
                    if ~isfield(a, 'cropped_segmentation_image')
                        continue;
                    end
                    cropped_segmentation_image = a.cropped_segmentation_image;
                    current_volume = sum(cropped_segmentation_image(:));
                    is_segmentation_low_volume = current_volume <= segmentation_volume_threshold;

                    if is_segmentation_low_volume
                        if debug_print_deformation_skip_reasons, fprintf_diary_cycle_function('Skipping morph %s cell %d frame %d because cropped_segmentation_image blank\n', run_key, current_cell_index, frame_index), end
                        continue;
                    end

                    % image_group_j{end + 1} = cropped_segmentation_image;
                    I_sum = I_sum + cropped_segmentation_image;
                    image_name_group_j{end + 1} = cell_alignment_filename;
                    image_fullname_group_j{end + 1} = [cell_alignment_fullname, '.mat'];
                  end
              end
              % xruan 03/14/2016 in case of repeated images because some
              % files have both one point or two point annotations. 
              image_name_group_j = unique(image_name_group_j);
              image_fullname_group_j = unique(image_fullname_group_j);
              img_num = numel(image_name_group_j);
              image_group_j_func = @(x) cropped_segmentation_image_read(x, image_fullname_group_j);
              
              if strcmp(average_method, 'sum_up')
                 [I_mean] = binary_image_average(image_name_group_j, image_group_j_func, t_cell_info);
              elseif strcmp(average_method, 'diffeomporphic')
                  dist_param = struct();
                  dist_param.registration_options = registration_options;            
                  dist_param.working_dir = sprintf('%sreltime%02d/', diff_dist_average_dir, current_reltime);
                  % dist_param.compute_columns = round(img_num / 4);
                  %  xruan 03/13/2016 change to set fewer columns
                  column_to_compute = 100;
                  if strcmp(master_script_options.named_options_set, 'all_conditions_one_point')
                    column_to_compute = 80;
                  end
                  dist_param.compute_columns = min([img_num, column_to_compute]);
                  [I_mean] = shapespace_average(image_name_group_j, t_cell_info, dist_param);
              end             

          
              filename_reltime_j = sprintf('%s%s_reltime%2d_average_shape_%s', average_img_dir, current_condition, current_reltime, average_method);
              save([filename_reltime_j, '.mat'], 'I_mean', 'I_sum', 'img_num');
          end
          
          average_group{j} = I_mean;       
          sum_group{j} = I_sum;
          img_num_grop{j} = img_num;
          if false          
            image_to_show = [];   
            image_to_show = [image_to_show; I_mean];
            image_to_show = [image_to_show; I_sum > 0];
            image_to_show = [image_to_show; contrast_stretch(I_sum)];  
            image_to_show = reshape_2d(image_to_show, 1);
            imwrite(image_to_show, [filename_reltime_j, '.tif']);


            image_to_show = [];   
            image_to_show = [image_to_show; I_mean];
            image_to_show = [image_to_show; I_sum > 0];
            image_to_show = [image_to_show; contrast_stretch(I_sum)];  
            image_to_show = reshape_2d(image_to_show, 1);
            imwrite(image_to_show, [filename_reltime_j, '.tif']);
            close all;
            figure;
            set(gcf, 'Visible', 'off')
            %set(gcf, 'Visible', 'on')
            set(gcf, 'Renderer', 'opengl');

            dpi = master_script_options.figure_dpi;
            figure_height = 250; %round(master_script_options.figure_maximum_width / aspect_ratio * figure_height_scale);
            figure_width = 1000; %round(master_script_options.figure_maximum_width * figure_height_scale);
            set(gcf, 'Position', [1, 1, figure_width, figure_height])
            axes('Position',[0.05, 0.1, 0.9, 0.7]);
            imshow(image_to_show, []);        
            title(sprintf('%s reltime %2d image number %d', current_condition, current_reltime, img_num));
            export_fig([filename_reltime_j, '_show', '.png'], '-opengl', '-png', '-a1', ['-r', num2str(dpi)]);
          end           

      end
      
      all_times_filename = sprintf('%s%s_average_shape_%s', average_img_dir, current_condition, current_reltime, average_method);
      save([all_times_filename, '.mat'], 'average_group', 'sum_group', 'img_num_grop');
      
      if ~false
          % plot average image
          plot_color_bar = false;
          output_filename = sprintf('%saverage_shape_center_slices_%s.png', average_img_dir, average_method);
          plot_center_slides_of_all_images(average_group, 'Center slide of the average shapes', illustration_colormap_to_use, output_filename, plot_color_bar)
          
          %plot the sum of all images
          plot_color_bar = true;
          % output_filename = sprintf('%ssum_of_shapes_center_slices_%s.png', average_img_dir, average_method);
          % xruan 12/10/2015
          % to avoid confusion, change the name of sum of shapes. 
          output_filename = sprintf('%ssum_of_shapes_center_slices.png', average_img_dir);
          illustration_colormap_to_use_1 = [1, 1, 1; illustration_colormap_to_use];

          plot_center_slides_of_all_images(sum_group, 'Center slide of the average shapes', illustration_colormap_to_use_1, output_filename, plot_color_bar)
        
      end
      
      
      
      
  end
  
end

function plot_center_slides_of_all_images(img_group, title_string, illustration_colormap_to_use, output_filename, plot_color_bar, axis_scale)
% here we have 12 relative time points, so we plot as 3 X 4. 

if nargin < 5
    plot_color_bar = false;
end
if nargin < 6
    axis_scale = [0, 1];
end

close all; 
illustration_colormap_to_use = [ illustration_colormap_to_use];
% figure('Color', illustration_colormap_to_use(1, :))
figure('Color', [1,1,1])

set(gcf, 'Visible', 'off')
set(gcf, 'Visible', 'on')
set(gcf, 'Renderer', 'opengl');
dpi = 300;
figure_height = 600; %round(master_script_options.figure_maximum_width / aspect_ratio * figure_height_scale);
figure_width = 800; %round(master_script_options.figure_maximum_width * figure_height_scale);
set(gcf, 'Position', [1920, 1, figure_width, figure_height])

x = 0.01 : 0.22 : 0.88;
y = 0.06 : 0.30 : 0.8;

[L, B] = meshgrid(x, y);
L = L';
B = flipud(B)';

W = 0.22;
H = 0.30;
reltimes = -2 : 9;
% title(title_string);
crop_img = true;
size_crop = 1;
axis off

for i = 1 : 12 
    image_i = img_group{i};
    image_to_show = squeeze(image_i(:, :, round(size(image_i, 3) / 2)));
    image_to_show = contrast_stretch(image_to_show);
    
    if crop_img 
        if any(reshape(image_to_show([ 1 : size_crop, end - size_crop + 1 : end], [ 1 : size_crop, end - size_crop + 1 : end]), 1, []) > 0)
            warning('some voxels have been cropped, use smaller crop size');
        end
        image_to_show = image_to_show(1 + size_crop : end - size_crop, 1 + size_crop : end - size_crop);
    end
    %matrix_plot_handle = gca;
    axes('Position', [L(i), B(i), W, H])
    %iptsetpref('ImshowBorder','tight')
    imagesc(image_to_show);
    % freezeColors;
    % text(14, 80, sprintf('Relative time %d', reltimes(i))) 
    axis off, colormap(illustration_colormap_to_use);
    
    caxis(axis_scale);
    hold on    
end


if plot_color_bar
    h = colorbar;
    set(h, 'Position', [.89 .06 .04 0.82]);
    % set(h, 'Position', [.89 .06 .04 0.87]);
    % ylabel(h, 'Relative probability (per model)');
    if nargin == 6
        set(h, 'Fontsize', 12, 'fontweight', 'bold');
    else
        set(h, 'Ticks', [0 : 0.2 : 1], 'Fontsize', 12, 'fontweight', 'bold');
    end
    axis off
end


hold off;


export_fig(output_filename, '-opengl', '-png', '-a1', ['-r', num2str(dpi)])


end


  
  
function plot_center_slides_of_all_images_1(img_group, title_string, illustration_colormap_to_use, output_filename, plot_color_bar)
% here we have 12 relative time points, so we plot as 3 X 4. 

if nargin < 5
    plot_color_bar = false;
end

close all; 
illustration_colormap_to_use = [1, 1, 1; illustration_colormap_to_use];
% figure('Color', illustration_colormap_to_use(1, :))
figure('Color', [1,1,1])

set(gcf, 'Visible', 'off')
set(gcf, 'Visible', 'on')
set(gcf, 'Renderer', 'opengl');
dpi = 300;
figure_height = 600; %round(master_script_options.figure_maximum_width / aspect_ratio * figure_height_scale);
figure_width = 800; %round(master_script_options.figure_maximum_width * figure_height_scale);
set(gcf, 'Position', [1, 1, figure_width, figure_height])

x = 0.04 : 0.22 : 0.9;
y = 0.06 : 0.30 : 0.8;

[L, B] = meshgrid(x, y);
L = L';
B = flipud(B)';

W = 0.22;
H = 0.3;
reltimes = -2 : 9;
for i = 1 : 12 
    image_i = img_group{i};
    image_to_show = squeeze(image_i(:, :, round(size(image_i, 3) / 2)));
    image_to_show = contrast_stretch(image_to_show);
    %matrix_plot_handle = gca;
    axes('Position', [L(i), B(i), W, H])
    %iptsetpref('ImshowBorder','tight')
    imagesc(image_to_show);
    % freezeColors;
    % text(14, 80, sprintf('Relative time %d', reltimes(i))) 
    axis off, colormap(illustration_colormap_to_use);
    
    caxis([0, 1]);
    hold on    
end


if plot_color_bar
    h = colorbar;
    set(h, 'Position', [.92 .08 .02 0.72]);
    % ylabel(h, 'Relative probability (per model)');
    axis off
end


hold off;

export_fig(output_filename, '-opengl', '-png', '-a1', '-nocrop', ['-r', num2str(dpi)])


figure, 




end
  
  
  
