function [t_cell_info] = master_script_check_contributing_features_model_clustering(t_cell_info, options)
 % Author: Xiongtao Ruan 
 % Date: July, 2015
 % xruan 02/23/2016: change the location setting to be consistent with the
 % pipeline. 

%
% Author: Taraz E. Buck and Xiongtao Ruan
%
% Copyright (C) 2012-2016 Murphy Lab
% Computational Biology Department
% School of Computer Science
% Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu



  master_script_options = t_cell_info.options;
  fprintf_diary_cycle_function = t_cell_info.log_info.fprintf_diary_cycle_function;
  regions_location = t_cell_info.path_info.regions_location;
  segmentations_filtered_location = t_cell_info.path_info.segmentations_filtered_location;
  alignments_location = t_cell_info.path_info.alignments_location;
  deformations_location = t_cell_info.path_info.deformations_location;
  models_location = t_cell_info.path_info.models_location;
  figures_location = t_cell_info.path_info.figures_location;

  illustration_colormap_to_use = t_cell_info.plotting_info.illustration_colormap_to_use;
  
  condition_name_abbreviation_function = t_cell_info.abbreviation_info.condition_name_abbreviation_function;
  
  template_options = t_cell_info.template_info.template_options;
  template_centroid = t_cell_info.template_info.template_centroid;
  template_synapse = t_cell_info.template_info.template_synapse;
  template_image = t_cell_info.template_info.template_image;
  template_crop_function = t_cell_info.template_info.template_crop_function;
  template_crop_x = t_cell_info.template_info.template_crop_x;
  template_crop_y = t_cell_info.template_info.template_crop_y;
  template_crop_z = t_cell_info.template_info.template_crop_z;
  template_occupied_x_slices = t_cell_info.template_info.template_occupied_x_slices;
  template_occupied_x_slice_pad_function = t_cell_info.template_info.template_occupied_x_slice_pad_function;
  template_occupied_y_slices = t_cell_info.template_info.template_occupied_y_slices;
  template_occupied_y_slice_pad_function = t_cell_info.template_info.template_occupied_y_slice_pad_function;
  approximate_synapse_slice = t_cell_info.template_info.approximate_synapse_slice;
  approximate_synapse_slice_template_image = t_cell_info.template_info.approximate_synapse_slice_template_image;
  approximate_cytoplasm_slices = t_cell_info.template_info.approximate_cytoplasm_slices;
  approximate_cytoplasm_slice_template_image = t_cell_info.template_info.approximate_cytoplasm_slice_template_image;
  template_x = t_cell_info.template_info.template_x;
  template_y = t_cell_info.template_info.template_y;
  template_z = t_cell_info.template_info.template_z;
  template_cylindrical_r = t_cell_info.template_info.template_cylindrical_r;
  template_cylindrical_theta = t_cell_info.template_info.template_cylindrical_theta;
  

  run_relative_path_list = t_cell_info.synapse_info.run_relative_path_list;
  run_key_list = t_cell_info.synapse_info.run_key_list;
  % run_data_list = t_cell_info.synapse_info.run_data_list;
  run_images_list = t_cell_info.synapse_info.run_images_list;
  run_brightfield_images_list = t_cell_info.synapse_info.run_brightfield_images_list;
  run_image_frame_indices_list = t_cell_info.synapse_info.run_image_frame_indices_list;
  run_number_cells = t_cell_info.synapse_info.run_number_cells;
  run_number_frames = t_cell_info.synapse_info.run_number_frames;
  run_point_tracks = t_cell_info.synapse_info.run_point_tracks;
  run_relative_time_tracks = t_cell_info.synapse_info.run_relative_time_tracks;
  run_relative_times = t_cell_info.synapse_info.run_relative_times;
  run_number_relative_times = t_cell_info.synapse_info.run_number_relative_times;
  all_relative_times = t_cell_info.synapse_info.all_relative_times;
  number_all_relative_times = t_cell_info.synapse_info.number_all_relative_times;
  all_relative_times_seconds_data = t_cell_info.synapse_info.all_relative_times_seconds_data;
  all_relative_times_seconds = t_cell_info.synapse_info.all_relative_times_seconds;
  all_relative_times_seconds_rounded = t_cell_info.synapse_info.all_relative_times_seconds_rounded;
  number_runs = t_cell_info.synapse_info.number_runs;
  condition_sensor_combinations = t_cell_info.synapse_info.condition_sensor_combinations;
  number_condition_sensors = t_cell_info.synapse_info.number_condition_sensors;
  conditions = t_cell_info.synapse_info.conditions;
  sensors = t_cell_info.synapse_info.sensors;
  number_conditions = t_cell_info.synapse_info.number_conditions;
  number_sensors = t_cell_info.synapse_info.number_sensors;
  
  template_centroid = t_cell_info.template_info.template_centroid;
  template_synapse = t_cell_info.template_info.template_synapse;
  template_image = t_cell_info.template_info.template_image;
  template_crop_function = t_cell_info.template_info.template_crop_function;

  window_size_2d = t_cell_info.preprocessing_info.window_size_2d;
  window_center_2d = t_cell_info.preprocessing_info.window_center_2d;
  cropped_size = t_cell_info.preprocessing_info.cropped_size;

  segmentation_rasterization_function = t_cell_info.segmentation_info.segmentation_rasterization_function;

  
  
  % should_contrast_stretch_across_time = false
  should_contrast_stretch_across_time = true
  
  % debug_print_model_filenames = false;
  debug_print_model_filenames = true


% model_location = '/projects/cellorganizer/xruan/tcell_project/tcell_excel_results_2pt_only-lat_alignment_resampling/two-point_landmark-smoothing-method_none_intensity-morphing-method_none_models';

model_name = 'only-lat_hierarchical_clustering_All conditions_standardized_voxels.mat';

cluster_difference_location = [master_script_options.results_location, 'clustering_difference'];


if ~exist(cluster_difference_location, 'dir')
    mkdir(cluster_difference_location);
end
 
a = load([models_location, model_name]);

condition_sensor_combinations = a.condition_sensor_combinations;
clustering_distance_metrics = a.clustering_distance_metrics;
number_clustering_distance_metrics = a.number_clustering_distance_metrics;
condition_sensor_combination_features_subset = a.condition_sensor_combination_features_subset;
condition_sensor_combination_frame_counts = a.condition_sensor_combination_frame_counts;
condition_sensor_combinations_with_features = a.condition_sensor_combinations_with_features;
condition_sensor_combinations_with_features = a.condition_sensor_combinations_with_features;
empty_features_subset = a.empty_features_subset;
model_distances = a.model_distances;
model_trees = a.model_trees;
model_clusters = a.model_clusters;
close all;
figure,
%plot(condition_sensor_combination_features_subset')


% 1: coflin reactive
% 2: blocking
% 3: Full stimulus
dist_function = @(ind_1, ind_2) (condition_sensor_combination_features_subset(ind_1, : ) ...
    - condition_sensor_combination_features_subset(ind_2, :)) .^ 2;
dist_12 = dist_function(1, 2);
dist_23 = dist_function(2, 3);
dist_13 = dist_function(1, 3);

dist = [dist_12; dist_23; dist_13];

figure, 
plot([dist_12; dist_13]')
legend('12', '13');

figure, 
plot([dist_23; dist_13]')
legend('23', '13');

%% time point 

dist_t_12 = sum(reshape(dist_12, 6628, []));
dist_t_23 = sum(reshape(dist_23, 6628, []));
dist_t_13 = sum(reshape(dist_13, 6628, []));

figure,
plot([dist_t_12; dist_t_13]')
legend('12', '13');
figure, 
plot([dist_t_23; dist_t_13]')
legend('23', '13');
close all;
figure, 
% plot_parameter_1 = {'Color', [0, 0, 0], 'MarkerFaceColor', [0, 0, 0], 'LineStyle', ':', 'Marker', 'o'};
% plot_parameter_2 = {'Color', [0, 0, 0], 'MarkerFaceColor', [0, 0, 0], 'LineStyle', '--', 'Marker', 'o'};
% plot_parameter_3 = {'Color', [0, 0, 0], 'MarkerFaceColor', [0, 0, 0], 'LineStyle', '-', 'Marker', 'o'};
colors = distinguishable_colors(3, {'w','k'});
color_1 = colors(1, :);
color_2 = colors(2, :);
color_3 = colors(3, :);

plot_parameter_1 = {'Color', color_1, 'MarkerFaceColor', color_1, 'LineStyle', '-', 'Marker', 'o', 'MarkerSize', 12};
plot_parameter_2 = {'Color', color_2, 'MarkerFaceColor', color_2, 'LineStyle', '-', 'Marker', '^', 'MarkerSize', 12};
plot_parameter_3 = {'Color', color_3, 'MarkerFaceColor', color_3, 'LineStyle', '-', 'Marker', 's', 'MarkerSize', 12};

%plot([ -2 : 9]', [dist_t_12; dist_t_23; dist_t_13]')
hold on
plot(all_relative_times_seconds_rounded, dist_t_12', plot_parameter_1{:});
hold on
plot(all_relative_times_seconds_rounded,  dist_t_23', plot_parameter_2{:});
hold on
plot(all_relative_times_seconds_rounded, dist_t_13', plot_parameter_3{:});
hold off;
legend('Active - Blockade', 'Full Stimulus - Blockade', 'Active - Full Stimulus ','Location','northwest');

% figure_width = 750;
% figure_height = 600;

plot_aspect_ratio = 16 / 10;
aspect_ratio = plot_aspect_ratio;
figure_height_scale = 1;
figure_height_scale = figure_height_scale * 975 / 847;

figure_height = round(master_script_options.figure_maximum_width / aspect_ratio * figure_height_scale);
% figure_width = round(figure_height * aspect_ratio);
figure_width = round(master_script_options.figure_maximum_width * figure_height_scale);

% keyboard
set(gcf, 'Position', [1920, 1, figure_width, figure_height])
dpi = master_script_options.figure_dpi;
dpi = 300;
set(findobj(gca, '-property', 'LineWidth'), 'LineWidth', 3)
ylabel('sum of square of pixel difference', 'Interpreter', 'none', 'Fontsize', 18, 'Fontweight', 'bold');
ylabel('Distance', 'Interpreter', 'none', 'Fontsize', 18, 'Fontweight', 'bold');
xlabel('Time (seconds)', 'Interpreter', 'none', 'Fontsize', 18, 'Fontweight', 'bold');
% set(gca, 'XTick', all_relative_times_seconds_rounded, 'FontSize',8.5, 'Fontweight', 'bold') 
sparse_time_seconds_rounded = [-41 0 41 83 131 199 314 416];
xlim([sparse_time_seconds_rounded(1), sparse_time_seconds_rounded(end) + 4]);
set(gca, 'XTick', sparse_time_seconds_rounded, 'FontSize',15, 'Fontweight', 'bold') 

set(gcf, 'Color', 'w')
%dpi = 300;

image_name = [cluster_difference_location, filesep, 'standardized_voxel_euclidean_temporal_plots'];

export_fig(image_name, '-zbuffer', '-png', '-a1', '-nocrop', ['-r', num2str(dpi)])

%% total voxel
dist_p_12 = sum(reshape(dist_12, 6628, [])');
dist_p_23 = sum(reshape(dist_23, 6628, [])');
dist_p_13 = sum(reshape(dist_13, 6628, [])');

close all
figure,
plot([dist_p_12; dist_p_13]')
legend('p_12', 'p_13');
figure, 
plot([dist_p_23; dist_p_13]')
legend('p_23', 'p_13');
close all

figure, 
plot([dist_p_12; dist_p_23; dist_p_13]')
legend('p_12', 'p_23', 'p_13');

result_image_function = @(x)subsasgn(zeros(size(template_image)), ...
    struct('type', '()', 'subs', {{template_image}}), x);

dist_p_img_12 = result_image_function(dist_p_12);

dist_p_img_13 = result_image_function(dist_p_13);

dist_p_img_23 = result_image_function(dist_p_23);
close all

figure, 
imshow(reshape_2d(contrast_stretch(dist_p_img_12)), []);
title('p_image_12');
close all

figure,
imshow(reshape_2d(contrast_stretch(dist_p_img_13)), []);
title('p_image_13');
close all

figure,
imshow(reshape_2d(contrast_stretch(dist_p_img_23)), []);
title('p_image_23');


image_to_show = zeros(0, size(dist_p_img_23, 2), size(dist_p_img_23, 3));

image_to_show = [image_to_show;  dist_p_img_23];
image_to_show = [image_to_show;  dist_p_img_12];
image_to_show = [image_to_show;  dist_p_img_13];

image_to_show = reshape_2d(image_to_show);
close all

figure, 
imshow(contrast_stretch(image_to_show), []);

dpi = 300;

image_name = [cluster_difference_location, filesep, 'standardized_voxel_euclidean_square_difference_spatial_images'];


export_fig(image_name, '-opengl', '-png', '-a1', ['-r', num2str(dpi)])

% close, set(gcf, 'Visible', 'on'), imagesc(cell2mat(condition_sensor_combination_panels_subset)), ...
%     axis image, colormap(illustration_colormap_to_use), colorbar

dist_p_img_crop_12 = template_crop_function (dist_p_img_12);
dist_p_img_crop_13 = template_crop_function (dist_p_img_13);
dist_p_img_crop_23 = template_crop_function (dist_p_img_23);


image_to_show = zeros(0, size(dist_p_img_crop_23, 2), size(dist_p_img_crop_23, 3));

image_to_show = [image_to_show;  dist_p_img_crop_23];
image_to_show = [image_to_show;  dist_p_img_crop_12];
image_to_show = [image_to_show;  dist_p_img_crop_13];

image_to_show = reshape_2d(image_to_show);
close all;
figure, 
imagesc(contrast_stretch(image_to_show)), axis off, colormap(illustration_colormap_to_use), colorbar;
figure_width = 920;
figure_height = 120;
set(gcf, 'Position', [1, 1, figure_width, figure_height])

dpi = 300;

image_name = [cluster_difference_location, filesep, 'standardized_voxel_euclidean_square_difference_spatial_colormap_images'];

export_fig(image_name, '-opengl', '-png', '-a1', ['-r', num2str(dpi)])


% difference images

dist_diff_img_crop_23 = dist_p_img_crop_12 - dist_p_img_crop_13;

dist_diff_img_crop_12 = dist_p_img_crop_23 - dist_p_img_crop_13;

image_to_show = zeros(0, size(dist_diff_img_crop_23, 2), size(dist_diff_img_crop_23, 3));

image_to_show = [image_to_show; dist_diff_img_crop_12];

image_to_show = [image_to_show; dist_diff_img_crop_23];

image_to_show = reshape_2d(image_to_show);
close all

figure,

imagesc(contrast_stretch(image_to_show)), axis off, colormap(illustration_colormap_to_use), colorbar;
figure_width = 920;
figure_height = 80;

set(gcf, 'Position', [1, 1, figure_width, figure_height])

dpi = 300;

image_name = [cluster_difference_location, filesep, 'standardized_voxel_euclidean_square_difference_spatial_differences'];

export_fig(image_name, '-opengl', '-png', '-a1', ['-r', num2str(dpi)])


% spatiotemporal maps

dist_spatiotemporal_image_function = @(dist_ab) cellfun(@(x) subsasgn(zeros(size(template_image)), ...
    struct('type', '()', 'subs', {{template_image}}), x), mat2cell(dist_ab, 1, 6628 * ones(1, 12)), 'UniformOutput', false);

dist_sp_12 = dist_spatiotemporal_image_function(dist_12);
dist_sp_13 = dist_spatiotemporal_image_function(dist_13);
dist_sp_23 = dist_spatiotemporal_image_function(dist_23);

a = 1

dist_sp_12_2d = reshape_2d(cell2mat(dist_sp_12'));
dist_sp_13_2d = reshape_2d(cell2mat(dist_sp_13'));
dist_sp_23_2d = reshape_2d(cell2mat(dist_sp_23'));

close all

figure, 
imagesc(contrast_stretch(dist_sp_12_2d)), axis off, colormap(illustration_colormap_to_use), colorbar;
figure_width = 1050;
figure_height = 300;
set(gcf, 'Position', [1, 1, figure_width, figure_height])
title('Blockade - Active')
dpi = 500;

image_name = [cluster_difference_location, filesep, 'standardized_voxel_euclidean_square_difference_spatiotemporal_Blockade_Active_colormap_images'];

export_fig(image_name, '-opengl', '-png', '-a1', ['-r', num2str(dpi)])

close all

figure, 
imagesc(contrast_stretch(dist_sp_13_2d)), axis off, colormap(illustration_colormap_to_use), colorbar;
figure_width = 1050;
figure_height = 300;
set(gcf, 'Position', [1, 1, figure_width, figure_height])
title('Active - Full Stimulus ')
dpi = 500;

image_name = [cluster_difference_location, filesep, 'standardized_voxel_euclidean_square_difference_spatiotemporal_FullStimulus_Active_colormap_images'];

export_fig(image_name, '-opengl', '-png', '-a1', ['-r', num2str(dpi)])

close all

figure, 
imagesc(contrast_stretch(dist_sp_23_2d)), axis off, colormap(illustration_colormap_to_use), colorbar;
figure_width = 1050;
figure_height = 300;
set(gcf, 'Position', [1, 1, figure_width, figure_height])
title('Full Stimulus - Blockade')
dpi = 500;

image_name = [cluster_difference_location, filesep, 'standardized_voxel_euclidean_square_difference_spatiotemporal_FullStimulus_Blockade_colormap_images'];

export_fig(image_name, '-opengl', '-png', '-a1', ['-r', num2str(dpi)])



dist_spatiotemporal_image_crop_function = @(dist_sp_ab) cellfun(@(x)template_crop_function(x), dist_sp_ab, 'UniformOutput', false);

dist_sp_crop_12 = dist_spatiotemporal_image_crop_function(dist_sp_12);
dist_sp_crop_13 = dist_spatiotemporal_image_crop_function(dist_sp_13);
dist_sp_crop_23 = dist_spatiotemporal_image_crop_function(dist_sp_23);

a = 1

dist_sp_crop_12_2d = reshape_2d(cell2mat(dist_sp_crop_12'));
dist_sp_crop_13_2d = reshape_2d(cell2mat(dist_sp_crop_13'));
dist_sp_crop_23_2d = reshape_2d(cell2mat(dist_sp_crop_23'));

max_intensity = max([max(dist_sp_crop_12_2d(:)), max(dist_sp_crop_13_2d(:)), max(dist_sp_crop_23_2d(:))]);


dist_sp_crop_12_2d = dist_sp_crop_12_2d / max_intensity;

dist_sp_crop_13_2d = dist_sp_crop_13_2d / max_intensity;

dist_sp_crop_23_2d = dist_sp_crop_23_2d / max_intensity;

close all

figure, 
imagesc(dist_sp_crop_12_2d), axis off, colormap(illustration_colormap_to_use), colorbar;
figure_width = 750;
figure_height = 300;
set(gcf, 'Position', [1, 1, figure_width, figure_height])
title('Blockade - Active')
dpi = 500;
caxis([0, 1]);
image_name = [cluster_difference_location, filesep, 'standardized_voxel_euclidean_square_difference_spatiotemporal_Blockade_Active_cutoff_colormap_images'];

export_fig(image_name, '-opengl', '-png', '-a1', ['-r', num2str(dpi)])

close all

figure, 
imagesc(dist_sp_crop_13_2d), axis off, colormap(illustration_colormap_to_use), colorbar;
figure_width = 750;
figure_height = 300;
set(gcf, 'Position', [1, 1, figure_width, figure_height])
title('Full Stimulus - Active')
dpi = 500;
caxis([0, 1]);

image_name = [cluster_difference_location, filesep, 'standardized_voxel_euclidean_square_difference_spatiotemporal_FullStimulus_Active_cutoff_colormap_images'];

export_fig(image_name, '-opengl', '-png', '-a1', ['-r', num2str(dpi)])

close all

figure, 
imagesc(dist_sp_crop_23_2d), axis off, colormap(illustration_colormap_to_use), colorbar;
figure_width = 750;
figure_height = 300;
set(gcf, 'Position', [1, 1, figure_width, figure_height])
title('Full Stimulus - Blockade')
dpi = 500;
caxis([0, 1]);

image_name = [cluster_difference_location, filesep, 'standardized_voxel_euclidean_square_difference_spatiotemporal_FullStimulus_Blockade_cutoff_colormap_images'];

export_fig(image_name, '-opengl', '-png', '-a1', ['-r', num2str(dpi)])

close all

figure,

figure_width = 800;
figure_height = 2200;
set(gcf, 'Position', [1, 1, figure_width, figure_height])

subplot(3, 1, 1)
imagesc(dist_sp_crop_23_2d), axis off, colormap(illustration_colormap_to_use), colorbar;

title('Full Stimulus - Blockade')
dpi = 500;
caxis([0, 1]);


subplot(3, 1, 2)
imagesc(dist_sp_crop_12_2d), axis off, colormap(illustration_colormap_to_use), colorbar;
figure_width = 750;
figure_height = 300;
%set(gcf, 'Position', [1, 1, figure_width, figure_height])
title('Blockade - Active')
dpi = 500;
caxis([0, 1]);

image_name = [cluster_difference_location, filesep, 'standardized_voxel_euclidean_square_difference_spatiotemporal_Blockade_Active_cutoff_colormap_images'];



subplot(3, 1, 3)
imagesc(dist_sp_crop_13_2d), axis off, colormap(illustration_colormap_to_use), colorbar;
figure_width = 750;
figure_height = 300;
%set(gcf, 'Position', [1, 1, figure_width, figure_height])
title('Full Stimulus - Active')
dpi = 500;
caxis([0, 1]);

image_name = [cluster_difference_location, filesep, 'standardized_voxel_euclidean_square_difference_spatiotemporal_All_conditions_cutoff_colormap_images'];

export_fig(image_name, '-opengl', '-png', '-a1', ['-r', num2str(dpi)])


%% only difference without square
% 1: coflin reactive
% 2: blocking
% 3: Full stimulus

dist_function = @(ind_1, ind_2) (condition_sensor_combination_features_subset(ind_1, : ) ...
    - condition_sensor_combination_features_subset(ind_2, :));
dist_12 = dist_function(1, 2);
dist_32 = dist_function(3, 2);
dist_13 = dist_function(1, 3);

% spatiotemporal maps

dist_spatiotemporal_image_function = @(dist_ab) cellfun(@(x) subsasgn(zeros(size(template_image)), ...
    struct('type', '()', 'subs', {{template_image}}), x), mat2cell(dist_ab, 1, 6628 * ones(1, 12)), 'UniformOutput', false);

dist_sp_12 = dist_spatiotemporal_image_function(dist_12);
dist_sp_13 = dist_spatiotemporal_image_function(dist_13);
dist_sp_32 = dist_spatiotemporal_image_function(dist_32);

dist_sp_12_2d = reshape_2d(cell2mat(dist_sp_12'));
dist_sp_13_2d = reshape_2d(cell2mat(dist_sp_13'));
dist_sp_32_2d = reshape_2d(cell2mat(dist_sp_32'));
close all;
figure, 
imagesc(contrast_stretch(dist_sp_12_2d)), axis off, colormap(illustration_colormap_to_use), colorbar;
figure_width = 1050;
figure_height = 300;
set(gcf, 'Position', [1, 1, figure_width, figure_height])
title('Blockade - Active')
dpi = 500;

image_name = [cluster_difference_location, filesep, 'standardized_voxel_euclidean_difference_spatiotemporal_Blockade_Active_colormap_images'];

export_fig(image_name, '-opengl', '-png', '-a1', ['-r', num2str(dpi)])

close all

figure, 
imagesc(contrast_stretch(dist_sp_13_2d)), axis off, colormap(illustration_colormap_to_use), colorbar;
figure_width = 1050;
figure_height = 300;
set(gcf, 'Position', [1, 1, figure_width, figure_height])
title('Full Stimulus - Active')
dpi = 500;

image_name = [cluster_difference_location, filesep, 'standardized_voxel_euclidean_difference_spatiotemporal_FullStimulus_Active_colormap_images'];

export_fig(image_name, '-opengl', '-png', '-a1', ['-r', num2str(dpi)])

close all

figure, 
imagesc(contrast_stretch(dist_sp_23_2d)), axis off, colormap(illustration_colormap_to_use), colorbar;
figure_width = 1050;
figure_height = 300;
set(gcf, 'Position', [1, 1, figure_width, figure_height])
title('Full Stimulus - Blockade')
dpi = 500;

image_name = [cluster_difference_location, filesep, 'standardized_voxel_euclidean_difference_spatiotemporal_FullStimulus_Blockade_colormap_images'];

export_fig(image_name, '-opengl', '-png', '-a1', ['-r', num2str(dpi)])



dist_spatiotemporal_image_crop_function = @(dist_sp_ab) cellfun(@(x)template_crop_function(x), dist_sp_ab, 'UniformOutput', false);

dist_sp_crop_12 = dist_spatiotemporal_image_crop_function(dist_sp_12);
dist_sp_crop_13 = dist_spatiotemporal_image_crop_function(dist_sp_13);
dist_sp_crop_32 = dist_spatiotemporal_image_crop_function(dist_sp_32);


dist_sp_crop_12_2d = reshape_2d(cell2mat(dist_sp_crop_12'));
dist_sp_crop_13_2d = reshape_2d(cell2mat(dist_sp_crop_13'));
dist_sp_crop_32_2d = reshape_2d(cell2mat(dist_sp_crop_32'));

max_intensity = max([max(dist_sp_crop_12_2d(:)), max(dist_sp_crop_13_2d(:)), max(dist_sp_crop_32_2d(:))]);
min_intensity = min([min(dist_sp_crop_12_2d(:)), min(dist_sp_crop_13_2d(:)), min(dist_sp_crop_32_2d(:))]);

dist_sp_crop_12_2d = (dist_sp_crop_12_2d - min_intensity) / (max_intensity - min_intensity); 

dist_sp_crop_13_2d = (dist_sp_crop_13_2d - min_intensity) / (max_intensity - min_intensity);

dist_sp_crop_32_2d = (dist_sp_crop_32_2d - min_intensity) / (max_intensity - min_intensity);

close all
figure, 
imagesc(dist_sp_crop_12_2d), axis off, colormap(illustration_colormap_to_use), colorbar;
figure_width = 750;
figure_height = 300;
set(gcf, 'Position', [1, 1, figure_width, figure_height])
title('Active - Blockade')
dpi = 500;
caxis([0, 1]);
image_name = [cluster_difference_location, filesep, 'standardized_voxel_euclidean_difference_spatiotemporal_Blockade_Active_cutoff_colormap_images'];

export_fig(image_name, '-opengl', '-png', '-a1', ['-r', num2str(dpi)])

close all

figure, 
imagesc(dist_sp_crop_13_2d), axis off, colormap(illustration_colormap_to_use), colorbar;
figure_width = 750;
figure_height = 300;
set(gcf, 'Position', [1, 1, figure_width, figure_height])
title('Active - Full Stimus')
dpi = 500;
caxis([0, 1]);

image_name = [cluster_difference_location, filesep, 'standardized_voxel_euclidean_difference_spatiotemporal_FullStimulus_Active_cutoff_colormap_images'];

export_fig(image_name, '-opengl', '-png', '-a1', ['-r', num2str(dpi)])

close all

figure, 
imagesc(dist_sp_crop_32_2d), axis off, colormap(illustration_colormap_to_use), colorbar;
figure_width = 750;
figure_height = 300;
set(gcf, 'Position', [1, 1, figure_width, figure_height])
title('Full Stimulus - Blockade')
dpi = 500;
caxis([0, 1]);

image_name = [cluster_difference_location, filesep, 'standardized_voxel_euclidean_difference_spatiotemporal_FullStimulus_Blockade_cutoff_colormap_images'];

export_fig(image_name, '-opengl', '-png', '-a1', ['-r', num2str(dpi)])

close all

figure,

figure_width = 800;
figure_height = 2200;
set(gcf, 'Position', [1, 1, figure_width, figure_height])

subplot(3, 1, 1)
imagesc(dist_sp_crop_32_2d), axis off, colormap(illustration_colormap_to_use), colorbar;

title('Full Stimulus - Blockade')
dpi = 500;
caxis([0, 1]);


subplot(3, 1, 2)
imagesc(dist_sp_crop_12_2d), axis off, colormap(illustration_colormap_to_use), colorbar;
figure_width = 750;
figure_height = 300;
%set(gcf, 'Position', [1, 1, figure_width, figure_height])
title('Active - Blockade')
dpi = 500;
caxis([0, 1]);

image_name = [cluster_difference_location, filesep, 'standardized_voxel_euclidean_difference_spatiotemporal_Blockade_Active_cutoff_colormap_images'];



subplot(3, 1, 3)
imagesc(dist_sp_crop_13_2d), axis off, colormap(illustration_colormap_to_use), colorbar;
figure_width = 750;
figure_height = 300;
%set(gcf, 'Position', [1, 1, figure_width, figure_height])
title('Active - Full Stimulus')
dpi = 500;
caxis([0, 1]);

image_name = [cluster_difference_location, filesep, 'standardized_voxel_euclidean_difference_spatiotemporal_All_conditions_cutoff_colormap_images'];

export_fig(image_name, '-opengl', '-png', '-a1', ['-r', num2str(dpi)])


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
max_intensity = max([max(dist_12(:)), max(dist_13(:)), max(dist_32(:))]);
min_intensity = min([min(dist_12(:)), min(dist_13(:)), min(dist_32(:))]);

dist_12_norm = (dist_12 - min_intensity) / (max_intensity - min_intensity); 

dist_13_norm = (dist_13 - min_intensity) / (max_intensity - min_intensity);

dist_32_norm = (dist_32 - min_intensity) / (max_intensity - min_intensity);

dist_spatiotemporal_image_function = @(dist_ab) cellfun(@(x) subsasgn(zeros(size(template_image)), ...
    struct('type', '()', 'subs', {{template_image}}), x), mat2cell(dist_ab, 1, 6628 * ones(1, 12)), 'UniformOutput', false);

dist_12_norm_st = dist_spatiotemporal_image_function(dist_12_norm);

dist_13_norm_st = dist_spatiotemporal_image_function(dist_13_norm);

dist_32_norm_st = dist_spatiotemporal_image_function(dist_32_norm);

dist_spatiotemporal_image_crop_function = @(dist_sp_ab) cellfun(@(x)template_crop_function(x), dist_sp_ab, 'UniformOutput', false);

dist_st_crop_12 = dist_spatiotemporal_image_crop_function(dist_12_norm_st);
dist_st_crop_13 = dist_spatiotemporal_image_crop_function(dist_13_norm_st);
dist_st_crop_32 = dist_spatiotemporal_image_crop_function(dist_32_norm_st);

dist_st_crop_12_2d = reshape_2d(cell2mat(dist_st_crop_12'));
dist_st_crop_13_2d = reshape_2d(cell2mat(dist_st_crop_13'));
dist_st_crop_32_2d = reshape_2d(cell2mat(dist_st_crop_32'));



close all
figure, 
imagesc(dist_st_crop_12_2d), axis off, colormap(illustration_colormap_to_use), colorbar;
figure_width = 750;
figure_height = 300;
set(gcf, 'Position', [1, 1, figure_width, figure_height])
title('Active - Blockade')
dpi = 500;
caxis([0, 1]);
image_name = [cluster_difference_location, filesep, 'standardized_voxel_euclidean_norm_difference_spatiotemporal_Blockade_Active_cutoff_colormap_images'];

export_fig(image_name, '-opengl', '-png', '-a1', ['-r', num2str(dpi)])

close all

figure, 
imagesc(dist_st_crop_13_2d), axis off, colormap(illustration_colormap_to_use), colorbar;
figure_width = 750;
figure_height = 300;
set(gcf, 'Position', [1, 1, figure_width, figure_height])
title('Active - Full Stimus')
dpi = 500;
caxis([0, 1]);

image_name = [cluster_difference_location, filesep, 'standardized_voxel_euclidean_norm_difference_spatiotemporal_FullStimulus_Active_cutoff_colormap_images'];

export_fig(image_name, '-opengl', '-png', '-a1', ['-r', num2str(dpi)])

close all

figure, 
imagesc(dist_st_crop_32_2d), axis off, colormap(illustration_colormap_to_use), colorbar;
figure_width = 750;
figure_height = 300;
set(gcf, 'Position', [1, 1, figure_width, figure_height])
title('Full Stimulus - Blockade')
dpi = 500;
caxis([0, 1]);

image_name = [cluster_difference_location, filesep, 'standardized_voxel_euclidean_norm_difference_spatiotemporal_FullStimulus_Blockade_cutoff_colormap_images'];

export_fig(image_name, '-opengl', '-png', '-a1', ['-r', num2str(dpi)])

close all

figure,

figure_width = 800;
figure_height = 2200;
set(gcf, 'Position', [1, 1, figure_width, figure_height])

subplot(3, 1, 1)
imagesc(dist_st_crop_32_2d), axis off, colormap(illustration_colormap_to_use), colorbar;
title('Full Stimulus - Blockade')
dpi = 500;
caxis([0, 1]);


subplot(3, 1, 2)
imagesc(dist_st_crop_12_2d), axis off, colormap(illustration_colormap_to_use), colorbar;
%set(gcf, 'Position', [1, 1, figure_width, figure_height])
title('Active - Blockade')
dpi = 500;
caxis([0, 1]);


subplot(3, 1, 3)
imagesc(dist_st_crop_13_2d), axis off, colormap(illustration_colormap_to_use), colorbar;
%set(gcf, 'Position', [1, 1, figure_width, figure_height])
title('Active - Full Stimulus')
dpi = 300;
caxis([0, 1]);

image_name = [cluster_difference_location, filesep, 'standardized_voxel_euclidean_norm_difference_spatiotemporal_All_conditions_cutoff_colormap_images'];

export_fig(image_name, '-opengl', '-png', '-a1', ['-r', num2str(dpi)])









