function [t_cell_info] = master_script_cluster_models(t_cell_info, options)
  % Hierarchical clustering of condition-sensor combinations for each model type.
  %
  % 2014-01-12 tebuck: Copied from master_script_generate_model_figures.m.
  % 
  % 
  % 
  % 
  % 
  % % Dependencies:
  % % From the File Exchange: export_fig, pmkmp, toolbox_graph, toolbox_fast_marching, affine, Snake3D, Mesh_voxelisation, dirr, inhull
  % % From tebuck: modified version of Jieyue Li's HPA_lib, rasterize_mesh

%
% Author: Taraz E. Buck and Xiongtao Ruan
%
% Copyright (C) 2012-2016 Murphy Lab
% Computational Biology Department
% School of Computer Science
% Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu



  
  
  
  % Variables referenced more than once can be copied to local variables:
  master_script_options = t_cell_info.options;
  fprintf_diary_cycle_function = t_cell_info.log_info.fprintf_diary_cycle_function;
  regions_location = t_cell_info.path_info.regions_location;
  segmentations_filtered_location = t_cell_info.path_info.segmentations_filtered_location;
  alignments_location = t_cell_info.path_info.alignments_location;
  deformations_location = t_cell_info.path_info.deformations_location;
  models_location = t_cell_info.path_info.models_location;
  figures_location = t_cell_info.path_info.figures_location;

  illustration_colormap_to_use = t_cell_info.plotting_info.illustration_colormap_to_use;
  
  condition_name_abbreviation_function = t_cell_info.abbreviation_info.condition_name_abbreviation_function;
  
  template_options = t_cell_info.template_info.template_options;
  template_centroid = t_cell_info.template_info.template_centroid;
  template_synapse = t_cell_info.template_info.template_synapse;
  template_image = t_cell_info.template_info.template_image;
  template_crop_function = t_cell_info.template_info.template_crop_function;
  template_crop_x = t_cell_info.template_info.template_crop_x;
  template_crop_y = t_cell_info.template_info.template_crop_y;
  template_crop_z = t_cell_info.template_info.template_crop_z;
  template_occupied_x_slices = t_cell_info.template_info.template_occupied_x_slices;
  template_occupied_x_slice_pad_function = t_cell_info.template_info.template_occupied_x_slice_pad_function;
  template_occupied_y_slices = t_cell_info.template_info.template_occupied_y_slices;
  template_occupied_y_slice_pad_function = t_cell_info.template_info.template_occupied_y_slice_pad_function;
  approximate_synapse_slice = t_cell_info.template_info.approximate_synapse_slice;
  approximate_synapse_slice_template_image = t_cell_info.template_info.approximate_synapse_slice_template_image;
  approximate_cytoplasm_slices = t_cell_info.template_info.approximate_cytoplasm_slices;
  approximate_cytoplasm_slice_template_image = t_cell_info.template_info.approximate_cytoplasm_slice_template_image;
  template_x = t_cell_info.template_info.template_x;
  template_y = t_cell_info.template_info.template_y;
  template_z = t_cell_info.template_info.template_z;
  template_cylindrical_r = t_cell_info.template_info.template_cylindrical_r;
  template_cylindrical_theta = t_cell_info.template_info.template_cylindrical_theta;
  
  model_types = t_cell_info.model_type_info.model_types;
  number_model_types = t_cell_info.model_type_info.number_model_types;
  model_representation_functions = t_cell_info.model_type_info.model_representation_functions;
  model_reconstruction_functions = t_cell_info.model_type_info.model_reconstruction_functions;
  model_dimensionality_reduction_methods = t_cell_info.model_type_info.model_dimensionality_reduction_methods;
  model_dimensionality_reduction_method_parameters = t_cell_info.model_type_info.model_dimensionality_reduction_method_parameters;
  model_illustration_view = t_cell_info.model_type_info.model_illustration_view;

  run_relative_path_list = t_cell_info.synapse_info.run_relative_path_list;
  run_key_list = t_cell_info.synapse_info.run_key_list;
  % run_data_list = t_cell_info.synapse_info.run_data_list;
  run_images_list = t_cell_info.synapse_info.run_images_list;
  run_brightfield_images_list = t_cell_info.synapse_info.run_brightfield_images_list;
  run_image_frame_indices_list = t_cell_info.synapse_info.run_image_frame_indices_list;
  run_number_cells = t_cell_info.synapse_info.run_number_cells;
  run_number_frames = t_cell_info.synapse_info.run_number_frames;
  run_point_tracks = t_cell_info.synapse_info.run_point_tracks;
  run_relative_time_tracks = t_cell_info.synapse_info.run_relative_time_tracks;
  run_relative_times = t_cell_info.synapse_info.run_relative_times;
  run_number_relative_times = t_cell_info.synapse_info.run_number_relative_times;
  all_relative_times = t_cell_info.synapse_info.all_relative_times;
  number_all_relative_times = t_cell_info.synapse_info.number_all_relative_times;
  all_relative_times_seconds_data = t_cell_info.synapse_info.all_relative_times_seconds_data;
  all_relative_times_seconds = t_cell_info.synapse_info.all_relative_times_seconds;
  all_relative_times_seconds_rounded = t_cell_info.synapse_info.all_relative_times_seconds_rounded;
  number_runs = t_cell_info.synapse_info.number_runs;
  condition_sensor_combinations = t_cell_info.synapse_info.condition_sensor_combinations;
  number_condition_sensors = t_cell_info.synapse_info.number_condition_sensors;
  conditions = t_cell_info.synapse_info.conditions;
  sensors = t_cell_info.synapse_info.sensors;
  number_conditions = t_cell_info.synapse_info.number_conditions;
  number_sensors = t_cell_info.synapse_info.number_sensors;
  
  template_centroid = t_cell_info.template_info.template_centroid;
  template_synapse = t_cell_info.template_info.template_synapse;
  template_image = t_cell_info.template_info.template_image;
  template_crop_function = t_cell_info.template_info.template_crop_function;

  window_size_2d = t_cell_info.preprocessing_info.window_size_2d;
  window_center_2d = t_cell_info.preprocessing_info.window_center_2d;
  cropped_size = t_cell_info.preprocessing_info.cropped_size;

  segmentation_rasterization_function = t_cell_info.segmentation_info.segmentation_rasterization_function;
  
  model_original_representation_functions = t_cell_info.model_info.model_original_representation_functions;
  model_original_reconstruction_functions = t_cell_info.model_info.model_original_reconstruction_functions;
  model_reduction_representation_functions = t_cell_info.model_info.model_reduction_representation_functions;
  model_reduction_reconstruction_functions = t_cell_info.model_info.model_reduction_reconstruction_functions;
  
  
  % should_contrast_stretch_across_time = false
  should_contrast_stretch_across_time = true
  
  % debug_print_model_filenames = false;
  debug_print_model_filenames = true
  
  
  conditions = unique(condition_sensor_combinations(:, 1));
  sensors = unique(condition_sensor_combinations(:, 2));
  number_conditions = length(conditions)
  number_sensors = length(sensors)
  number_condition_pairs = (number_conditions - 1) * number_conditions / 2
  % condition_pair_sensor_statistics = cell(number_condition_pairs, number_sensors, number_all_relative_times, number_model_types);
  condition_pair_sensor_statistics = repmat(struct, [number_condition_pairs, number_sensors, number_all_relative_times, number_model_types]);
  
  
  warning('Should make it obvious in a figure that empty_features is true for a panel!')

  
  clustering_distance_metrics = {'euclidean', 'cityblock'};
  number_clustering_distance_metrics = length(clustering_distance_metrics);
  
  % all_model_distances = cell(number_model_types, number_clustering_distance_metrics);
  % all_model_trees = cell(number_model_types, number_clustering_distance_metrics);
  % all_model_clusters = cell(number_model_types, number_clustering_distance_metrics);
  
  % Hierarchically cluster multiple subsets of conditions (currently useful for everything and every individual condition as shown below):
  condition_subsets = num2cell(conditions);
  condition_subset_names = conditions;
  condition_subsets{end + 1} = conditions;
  condition_subset_names{end + 1} = 'All conditions';
  all_conditions_condition_subset_index = length(condition_subsets);
  number_condition_subsets = length(condition_subsets);
  all_condition_subset_model_distances = cell(number_condition_subsets, number_model_types, number_clustering_distance_metrics);
  all_condition_subset_model_trees = cell(number_condition_subsets, number_model_types, number_clustering_distance_metrics);
  all_condition_subset_model_clusters = cell(number_condition_subsets, number_model_types, number_clustering_distance_metrics);

  % model_clustering_filename = sprintf('hierarchical_clustering');
  % [can_start, final_name, final_exists] = chunk_start(models_location, model_clustering_filename);
  % % can_start, final_name, final_exists
  % % warning('>>>> HACK, starting "%s" anyway!', model_clustering_filename), can_start = true;
  % if can_start
  
  
  % For each model type, load the mean for each condition-sensor combination and each time point:
  
  condition_sensor_combinations_with_features = cell(number_model_types, 1);
  
  for model_type_index = 1:number_model_types
    
    model_type = model_types{model_type_index}

    representation_function = model_representation_functions{model_type_index};
    reconstruction_function = model_reconstruction_functions{model_type_index};
    original_representation_function = model_original_representation_functions{model_type_index};
    original_reconstruction_function = model_original_reconstruction_functions{model_type_index};
    reduction_representation_function = model_reduction_representation_functions{model_type_index};
    reduction_reconstruction_function = model_reduction_reconstruction_functions{model_type_index};
    
    condition_sensor_combination_features = cell(number_condition_sensors, number_all_relative_times);
    condition_sensor_combination_panels = cell(number_condition_sensors, number_all_relative_times);
    condition_sensor_combination_frame_counts = zeros(number_condition_sensors, number_all_relative_times);
    
    for all_relative_time_index = 1:number_all_relative_times
      relative_time = all_relative_times(all_relative_time_index);
      
      for condition_sensor_combination_index = 1:number_condition_sensors
        current_condition = condition_sensor_combinations{condition_sensor_combination_index, 1};
        current_sensor = condition_sensor_combinations{condition_sensor_combination_index, 2};

        % Get the model:
        current_model_filename = sprintf('%s%s_%s_reltime% 3d_%s', master_script_options.model_prefix, current_condition, current_sensor, relative_time, model_type);
        current_model_full_filename = [models_location, current_model_filename, '.mat'];
        % if ~exist(current_model_full_filename)
          % continue
        % end
        % current_mean = [];
        if exist(current_model_full_filename, 'file')
          if debug_print_model_filenames, fprintf_diary_cycle_function('Model exists: ''%s''\n', current_model_full_filename), end
          current_model = load(current_model_full_filename);
          current_mean = current_model.current_mean;
          condition_sensor_combination_frame_counts(condition_sensor_combination_index, all_relative_time_index) = size(current_model.current_data, 1);
          % current_mean_size = size(current_model.current_mean)
        else
          if debug_print_model_filenames, fprintf_diary_cycle_function('Model does not exist: ''%s''\n', current_model_full_filename), end
          continue
          % % current_mean = nan(size(representation_function(template_image)));
          % current_mean = nan(size(original_representation_function(template_image)));
        end
        
        % condition_sensor_combination_features{condition_sensor_combination_index, all_relative_time_index} = current_model.current_data;
        condition_sensor_combination_features{condition_sensor_combination_index, all_relative_time_index} = current_mean;
        
        % keyboard
        current_mean_reconstructed = original_reconstruction_function(current_mean);
        current_mean_reconstructed_cropped_slice = template_crop_function(current_mean_reconstructed);
        switch model_illustration_view{model_type_index}
          case 'top'
            current_mean_reconstructed_cropped_slice = current_mean_reconstructed_cropped_slice(:, :, round(end / 2));
          case 'front'
            % keyboard
            % current_mean_reconstructed_cropped_slice = permute(current_mean_reconstructed_cropped_slice(round(end / 2), :, :), [3, 2, 1]);
            % current_mean_reconstructed_cropped_slice = permute(current_mean_reconstructed_cropped_slice(template_crop_function(approximate_synapse_slice_template_image)), [3, 2, 1]);
            current_mean_reconstructed_cropped_slice = permute(current_mean_reconstructed_cropped_slice(approximate_synapse_slice - find(template_crop_y, 1) + 1, :, :), [3, 2, 1]);
          otherwise
            error('model_illustration_view{model_type_index} = ''%s'' is invalid', model_illustration_view{model_type_index})
        end
        % % Whole model (large!):
        % condition_sensor_combination_panels{condition_sensor_combination_index, all_relative_time_index} = reshape_contrast(current_mean_reconstructed);
        % Middle top-down slice (later add near-synapse slice?):
        % Also don't contrast stretch per model:
        condition_sensor_combination_panels{condition_sensor_combination_index, all_relative_time_index} = current_mean_reconstructed_cropped_slice;
        
        if ~should_contrast_stretch_across_time
          current_mean_reconstructed_cropped_slice = current_mean_reconstructed_cropped_slice ./ max(current_mean_reconstructed_cropped_slice(:));
        end
        
      end
    end
    
    % Fill in unobserved models (nans for features, zeros for figures):
    empty_features = cellfun(@isempty, condition_sensor_combination_features);
    if all(empty_features(:))
      continue
    end
    nan_features = nan(size(condition_sensor_combination_features{find(~empty_features, 1)}));
    blank_panel = zeros(size(condition_sensor_combination_panels{find(~empty_features, 1)}));
    should_fill_with_nan = false;
    % should_fill_with_nan = true;
    condition_sensor_combination_features_empty_proportion = mean(empty_features(:))
    if any(empty_features(:))
      if should_fill_with_nan
        % keyboard
        condition_sensor_combination_features(empty_features) = {nan_features};
        condition_sensor_combination_panels(empty_features) = {blank_panel};
      else
        should_fill_use_interpolation = false;
        % should_fill_use_interpolation = true;
        for condition_sensor_combination_index = 1:number_condition_sensors
          emptinesses = empty_features(condition_sensor_combination_index, :);
          naned_relative_time_indices = nan(size(emptinesses));
          naned_relative_time_indices(~emptinesses) = find(~emptinesses);
          if should_fill_use_interpolation
            extrapolated_relative_time_indices = extrapolate_nans(naned_relative_time_indices, struct('method', 'diffusion', 'diffusion_method', 4));
          else
            extrapolated_relative_time_indices = extrapolate_nans(naned_relative_time_indices, struct('method', 'nearest'));
          end
          
          % combination_maximum_intensity = 0;
          for all_relative_time_index = 1:number_all_relative_times
            if isempty(condition_sensor_combination_features{condition_sensor_combination_index, all_relative_time_index})
              % keyboard
              nearest_nonempty_time = round(extrapolated_relative_time_indices(all_relative_time_index));
              if ~isfinite(nearest_nonempty_time)
                continue
              end
              if should_fill_use_interpolation
                error
              else
                condition_sensor_combination_features{condition_sensor_combination_index, all_relative_time_index} = condition_sensor_combination_features{condition_sensor_combination_index, nearest_nonempty_time};
                condition_sensor_combination_panels{condition_sensor_combination_index, all_relative_time_index} = condition_sensor_combination_panels{condition_sensor_combination_index, nearest_nonempty_time};
              end
            end
          end
        end
      end
    end
          
    if should_contrast_stretch_across_time
      % Contrast stretch each protein model across all time points:
      for condition_sensor_combination_index = 1:number_condition_sensors
        % combination_maximum_intensity = max(cellfun(@(x)max(x(:)), condition_sensor_combination_panels(condition_sensor_combination_index, :)));
        % condition_sensor_combination_panels(condition_sensor_combination_index, :) = cellfun(@(x)x ./ combination_maximum_intensity, condition_sensor_combination_panels(condition_sensor_combination_index, :), 'UniformOutput', false);
        nonempty_current_condition_sensor_combination_panels = ~cellfun(@isempty, condition_sensor_combination_panels(condition_sensor_combination_index, :));
        if ~any(nonempty_current_condition_sensor_combination_panels)
          continue
        end
        combination_maximum_intensity = max(cellfun(@(x)max(x(:)), condition_sensor_combination_panels(condition_sensor_combination_index, nonempty_current_condition_sensor_combination_panels)));
        condition_sensor_combination_panels(condition_sensor_combination_index, nonempty_current_condition_sensor_combination_panels) = cellfun(@(x)x ./ combination_maximum_intensity, condition_sensor_combination_panels(condition_sensor_combination_index, nonempty_current_condition_sensor_combination_panels), 'UniformOutput', false);
        % for all_relative_time_index = 1:number_all_relative_times
        % end
      end
    end

    
    
    % condition_sensor_combination_features
    
    % Keep track of condition_sensor_combinations with features (just checking the first column would work, too, due to the filling in of blank features above):
    condition_sensor_combinations_with_features{model_type_index} = condition_sensor_combinations(~all(cellfun(@isempty, condition_sensor_combination_features), 2), :);
    % Combine features across time so that we are clustering spatiotemporal sensor distributions:
    condition_sensor_combination_features = cell2mat(condition_sensor_combination_features);
    % condition_sensor_combination_panels = cell2mat(condition_sensor_combination_panels);
    % whos condition_sensor_combination_features
    % condition_sensor_combination_features_nan_proportion = mean(isnan(condition_sensor_combination_features(:)))

      
    for condition_subset_index = 1:length(condition_subsets)
      
      current_condition_subset = condition_subsets{condition_subset_index};
      current_condition_subset_name = condition_subset_names{condition_subset_index};
      
      % condition_sensor_combination_features_subset_indices = ismember(condition_sensor_combinations(:, 1), current_condition_subset);
      condition_sensor_combination_features_subset_indices = ismember(condition_sensor_combinations_with_features{model_type_index}(:, 1), current_condition_subset);
      condition_sensor_combination_features_subset = condition_sensor_combination_features(condition_sensor_combination_features_subset_indices, :);
      condition_sensor_combination_panels_subset = condition_sensor_combination_panels(condition_sensor_combination_features_subset_indices, :);
      if false
        % Debug info:
        close, set(gcf, 'Visible', 'on'), imagesc(cell2mat(condition_sensor_combination_panels_subset)), axis image, colormap(illustration_colormap_to_use), colorbar
        beep, keyboard
      end
      empty_features_subset = empty_features(condition_sensor_combination_features_subset_indices, :);
      
      if false
        % Debug info:
        condition_subset_index, current_condition_subset, current_condition_subset_name, condition_sensor_combination_features_subset_indices
        beep, keyboard
      end
      
      model_clustering_filename = sprintf('%shierarchical_clustering_%s_%s', master_script_options.model_prefix, current_condition_subset_name, model_type);
      [can_start, final_name, final_exists] = chunk_start(models_location, model_clustering_filename);
      
      if debug_print_model_filenames, fprintf_diary_cycle_function('Model file ''%s'': can_start %d, final_exists %d\n', model_clustering_filename, can_start, final_exists), end

      % can_start, final_name, final_exists
      % warning('>>>> HACK, starting "%s" anyway!', model_clustering_filename), can_start = true;
      if can_start

        % Hierarchical clustering for multiple distance metrics:
        
        model_distances = cell(number_clustering_distance_metrics, 1);
        model_trees = cell(number_clustering_distance_metrics, 1);
        model_clusters = cell(number_clustering_distance_metrics, 1);
        
        for clustering_distance_metric_index = 1:number_clustering_distance_metrics
        
          current_clustering_distance_metric = clustering_distance_metrics{clustering_distance_metric_index};
          
          current_distances = pdist(condition_sensor_combination_features_subset, current_clustering_distance_metric);
          if isempty(current_distances)
            continue
          end
          current_tree = linkage(current_distances);
          current_distances = squareform(current_distances);
          
          % Inconsistency-based thresholding does not work here, it just gives many clusters or one, so hard coding three clusters:
          warning('Inconsistency-based thresholding does not work here, it just gives many clusters or one, so hard coding three clusters')
          current_clusters = cluster(current_tree, 'MaxClust', 3);
          
          model_distances{clustering_distance_metric_index} = current_distances;
          model_trees{clustering_distance_metric_index} = current_tree;
          model_clusters{clustering_distance_metric_index} = current_clusters;
            
        end
        
        
        % save(final_name, 'model_trees', 'model_distances', 'model_clusters', 'model_types', 'number_model_types', 'clustering_distance_metrics', 'number_clustering_distance_metrics', 'condition_sensor_combination_features_subset', 'condition_sensor_combinations', 'condition_sensor_combinations_with_features', 'condition_sensor_combination_panels_subset', 'condition_sensor_combination_frame_counts', 'empty_features_subset');
        save(final_name, 'model_trees', 'model_distances', 'model_clusters', 'clustering_distance_metrics', 'number_clustering_distance_metrics', 'condition_sensor_combination_features_subset', 'condition_sensor_combinations', 'condition_sensor_combinations_with_features', 'condition_sensor_combination_panels_subset', 'condition_sensor_combination_frame_counts', 'empty_features_subset');
        chunk_finish(models_location, model_clustering_filename);    
        
      else
        load(final_name)
      end
        
      % all_model_distances(model_type_index, :) = model_distances.';
      % all_model_trees(model_type_index, :) = model_trees.';
      % all_model_clusters(model_type_index, :) = model_clusters.';
      all_condition_subset_model_distances(condition_subset_index, model_type_index, :) = permute(model_distances, [2, 3, 1]);
      all_condition_subset_model_trees(condition_subset_index, model_type_index, :) = permute(model_trees, [2, 3, 1]);
      all_condition_subset_model_clusters(condition_subset_index, model_type_index, :) = permute(model_clusters, [2, 3, 1]);
        
    end
      
  end
  
    % save(final_name, 'model_trees', 'model_distances', 'model_types', 'number_model_types', 'clustering_distance_metrics', 'number_clustering_distance_metrics', '', '');
    % chunk_finish(models_location, model_clustering_filename);    
    
  % else
    
    % load(final_name);
    
  % end
  
  
  all_model_distances = squeeze(all_condition_subset_model_distances(all_conditions_condition_subset_index, :, :));
  all_model_trees = squeeze(all_condition_subset_model_trees(all_conditions_condition_subset_index, :, :));
  all_model_clusters = squeeze(all_condition_subset_model_clusters(all_conditions_condition_subset_index, :, :));

  
  % Create figures:
  
  warning('Currently there may be a bug due to completely blank models not showing up in these figures, replace with blanks instead!')
  
  
  % model_distances = {};
  % model_trees = {};
  % model_clustering_filename = sprintf('hierarchical_clustering');
  % load([models_location, model_clustering_filename, '.mat'])
  
  for model_type_index = 1:number_model_types
    
    model_type = model_types{model_type_index}
    
    representation_function = model_representation_functions{model_type_index};
    reconstruction_function = model_reconstruction_functions{model_type_index};
    original_representation_function = model_original_representation_functions{model_type_index};
    original_reconstruction_function = model_original_reconstruction_functions{model_type_index};
    reduction_representation_function = model_reduction_representation_functions{model_type_index};
    reduction_reconstruction_function = model_reduction_reconstruction_functions{model_type_index};
    
    if size(condition_sensor_combinations_with_features{model_type_index}, 1) == 0
      continue
    end
    
    for condition_subset_index = 1:length(condition_subsets)
      
      current_condition_subset = condition_subsets{condition_subset_index};
      current_condition_subset_name = condition_subset_names{condition_subset_index};
      
      % condition_sensor_combination_features_subset_indices = ismember(condition_sensor_combinations(:, 1), current_condition_subset);
      condition_sensor_combination_features_subset_indices = ismember(condition_sensor_combinations_with_features{model_type_index}(:, 1), current_condition_subset);
      % condition_sensor_combination_subset = condition_sensor_combinations(condition_sensor_combination_features_subset_indices, :);
      condition_sensor_combination_subset = condition_sensor_combinations_with_features{model_type_index}(condition_sensor_combination_features_subset_indices, :);
      condition_sensor_combination_features_subset = condition_sensor_combination_features(condition_sensor_combination_features_subset_indices, :);
      
      if false
        % Debug info:
        beep, condition_sensor_combination_features_subset_indices
      end
      
      model_clustering_filename = sprintf('%shierarchical_clustering_%s_%s', master_script_options.model_prefix, current_condition_subset_name, model_type);
      
      model_clustering_file_full_path = [models_location, model_clustering_filename, '.mat']
      if ~exist(model_clustering_file_full_path, 'file')
        continue
      end
      a = load(model_clustering_file_full_path);
      
      if false
        % Debug info:
        a
        beep, keyboard
      end
      
      model_trees = a.model_trees;
      model_distances = a.model_distances;
      clustering_distance_metrics = a.clustering_distance_metrics;
      number_clustering_distance_metrics = a.number_clustering_distance_metrics;
      % condition_sensor_combination_features = a.condition_sensor_combination_features;
      condition_sensor_combination_features_subset = a.condition_sensor_combination_features_subset;
      condition_sensor_combination_panels_subset = a.condition_sensor_combination_panels_subset;
      condition_sensor_combination_frame_counts = a.condition_sensor_combination_frame_counts;
      empty_features_subset = a.empty_features_subset;
      
      if all(empty_features_subset)
        continue
      end
      
      blank_panel = zeros(size(condition_sensor_combination_panels_subset{find(~empty_features_subset, 1)}));
      
      for clustering_distance_metric_index = 1:number_clustering_distance_metrics
      
        current_clustering_distance_metric = clustering_distance_metrics{clustering_distance_metric_index};
        
        
        % current_tree = model_trees{model_type_index, clustering_distance_metric_index};
        % current_distances = model_distances{model_type_index, clustering_distance_metric_index};
        
        current_tree = model_trees{clustering_distance_metric_index};
        current_distances = model_distances{clustering_distance_metric_index};
        % current_clusters = model_clusters{clustering_distance_metric_index};
        
        if isempty(current_tree)
          continue
        end
        
        close
        warning('off', 'MATLAB:CallbackError')
        [~, ~, current_tree_permutation] = dendrogram(current_tree);
        warning('on', 'MATLAB:CallbackError')
        close
        
        % current_cophenetic_coefficient = cophenet(current_tree, current_distances);
        current_cophenetic_coefficient = cophenet(current_tree, squareform(current_distances));
        current_inconsistencies = inconsistent(current_tree);
        %% Sensitive to threshold:
        %current_clusters = cluster(current_tree, 'cutoff', 1.2)
        %current_clusters = cluster(current_tree, 'cutoff', .8)
        %current_clusters = cluster(current_tree, 'criterion', 'distance', 'cutoff', 2000)

        condition_sensor_combination_figure = cell2mat(condition_sensor_combination_panels_subset(current_tree_permutation, :));
        if false
          % Debug info:
          % set(gcf, 'Visible', 'on'), imshow(cell2mat(condition_sensor_combination_panels_subset), [])
          close, set(gcf, 'Visible', 'on'), imagesc(cell2mat(condition_sensor_combination_panels_subset)), axis image, colormap(illustration_colormap_to_use), colorbar
          beep, keyboard
        end
        
        use_horizontal_colorbar = false;
        % use_horizontal_colorbar = true;
        
        % Loop to create figure using either distance matrix and model:
        for should_clustering_figure_use_models = [false, true]
        
          % Display:
          close all
          set(gcf, 'Visible', 'off')
          % set(gcf, 'Visible', 'on')
          figure_position = get(gcf, 'Position');
          if use_horizontal_colorbar
            % aspect_ratio = 16 / 10;
            aspect_ratio = 4 / 3;
          else
            aspect_ratio = 1 / 1;
            % aspect_ratio = 4 / 3;
            % aspect_ratio = 16 / 10;
            % aspect_ratio = 2 / 1;
          end
          % aspect_ratio = aspect_ratio * 1.2;
          % figure_height_scale = .75;
          figure_height_scale = 1.2;
          number_dendrogram_subplots = 1;
          dpi = master_script_options.figure_dpi;
          % if ~should_clustering_figure_use_models
          if true
            % Deal with longer condition names like reconstituted Rac and Cofilin:
            % aspect_ratio = aspect_ratio * 16 / 10;
            % aspect_ratio = aspect_ratio * 2;
            number_dendrogram_subplots = 2;
            % number_dendrogram_subplots = 3;
            % number_dendrogram_subplots = 6;
            % figure_height_scale = 2;
            % dpi = dpi / figure_height_scale;
          end
          figure_height = round(master_script_options.figure_maximum_width / aspect_ratio * figure_height_scale);
          % figure_width = round(figure_height * aspect_ratio);
          figure_width = round(master_script_options.figure_maximum_width / aspect_ratio * figure_height_scale);
          set(gcf, 'Position', [1920, 1, figure_width, figure_height])
        
          if use_horizontal_colorbar
            subplot(1, 4, 1);
          else
            % subplot(1, 5, 1);
            subplot(1, 4 + number_dendrogram_subplots, 1:number_dendrogram_subplots);
            % subplot(1, 5, 1:2);
          end
          dendrogram_plot_handle = gca;
          dendrogram(current_tree, 'orientation', 'left');
          % title(sprintf('Cophenetic coefficient: %.3f', current_cophenetic_coefficient), 'Interpreter', 'none')
          if use_horizontal_colorbar
            subplot(1, 4, 2:3);
          else
            % subplot(1, 5, 3:5);
            subplot(1, 4 + number_dendrogram_subplots, number_dendrogram_subplots + 2 + (0:2));
          end
          matrix_plot_handle = gca;
          tick_values = 1:number_condition_sensors;
          if should_clustering_figure_use_models
            % Template-space images:
            warning('Are these coordinates correct?')
            image_pixel_x_coordinates = [0, length(current_tree_permutation) * size(condition_sensor_combination_figure, 2) / size(condition_sensor_combination_figure, 1)] + 1/2 + 1 / (2 * size(blank_panel, 2));
            image_pixel_y_coordinates = [0, length(current_tree_permutation)] + 1/2 + 1 / (2 * size(blank_panel, 1));
            imagesc(image_pixel_x_coordinates, image_pixel_y_coordinates, condition_sensor_combination_figure)
            % xlabel('Frame number (relative to synapse formation)', 'Interpreter', 'none')
            xlabel('Time (seconds)', 'Interpreter', 'none')
            if use_horizontal_colorbar
              tick_values = contrast_stretch(1:number_condition_sensors) .* diff(image_pixel_x_coordinates) + image_pixel_x_coordinates(1);
            end
          else
            % Distance matrix:
            imagesc(current_distances(current_tree_permutation, current_tree_permutation))
          end
          tick_labels = arrayfun(@(x)[condition_sensor_combination_subset{x, 1}, ' - ', condition_sensor_combination_subset{x, 2}], current_tree_permutation, 'UniformOutput', false);
          if true
            tick_labels = condition_name_abbreviation_function(tick_labels);
          end
          
          debug_check_ytick_orientation = false;
          % debug_check_ytick_orientation = true;
          if debug_check_ytick_orientation
            warning('>>>> HACK, debug_check_ytick_orientation!')
            tick_labels = arrayfun(@(x)num2str(x), current_tree_permutation, 'UniformOutput', false);
          end
          
          axis image
          colormap(illustration_colormap_to_use)
          
          if should_clustering_figure_use_models
            % title(sprintf('Clustered models, model type: %s, cophenetic coefficient: %.3f', model_type, current_cophenetic_coefficient), 'Interpreter', 'none')
            % xruan 08/31/2015 show no titles
            % title({sprintf('Clustered models, model type: %s', model_type), sprintf('Cophenetic coefficient: %.3f', current_cophenetic_coefficient)}, 'Interpreter', 'none')
          else
            % title(sprintf('Distance matrix for model type %s', model_type), 'Interpreter', 'none')
            % title(sprintf('Distance matrix, model type: %s, cophenetic coefficient: %.3f', model_type, current_cophenetic_coefficient), 'Interpreter', 'none')
            title({sprintf('Distance matrix, model type: %s', model_type), sprintf('Cophenetic coefficient: %.3f', current_cophenetic_coefficient)}, 'Interpreter', 'none')
          end

          % ylabel(colorbar_handle, '-log(p\_value)')
          if should_clustering_figure_use_models
            % colorbar_label = 'Normalized intensity';
            % colorbar_label = 'Relative probability (within rows)';
            colorbar_label = {'Relative probability', '(within rows)'};
          else
            colorbar_label = 'Distance';
          end
          if use_horizontal_colorbar
            colorbar_handle = colorbar('SouthOutside');
            title(colorbar_handle, colorbar_label)
            % xlabel(colorbar_handle, colorbar_label)
            set(matrix_plot_handle, 'XTick', [], 'XTickLabel', [])
            set(matrix_plot_handle, 'YTick', tick_values, 'YTickLabel', tick_labels, 'YAxisLocation', 'right')
          else
            colorbar_handle = colorbar;
            ylabel(colorbar_handle, colorbar_label)
            % if should_clustering_figure_use_models
              % set(matrix_plot_handle, 'XTick', tick_values, 'XTickLabel', tick_labels)
              set(matrix_plot_handle, 'YTick', [], 'YTickLabel', [])
              set(dendrogram_plot_handle, 'YTick', tick_values, 'YTickLabel', tick_labels)
            % else
              % set(matrix_plot_handle, 'XTick', tick_values, 'XTickLabel', tick_labels)
            % end
            % rotateXLabels(matrix_plot_handle, 90)
            % set(matrix_plot_handle, 'YTick', [], 'YTickLabel', [])
            set(matrix_plot_handle, 'YTick', [], 'YTickLabel', [])
            if should_clustering_figure_use_models
              % beep, keyboard
              % set(matrix_plot_handle, 'XTick', ((1:number_all_relative_times) - .5) ./ number_all_relative_times .* diff(image_pixel_x_coordinates) + image_pixel_x_coordinates(1), 'XTickLabel', all_relative_times, 'TickDir', 'out')
              % set(matrix_plot_handle, 'XTick', ((1:number_all_relative_times) - .5) ./ number_all_relative_times .* diff(image_pixel_x_coordinates) + image_pixel_x_coordinates(1), 'XTickLabel', all_relative_times_seconds_rounded, 'TickDir', 'out')
              set(matrix_plot_handle, 'XTick', ((1:number_all_relative_times) - .5) ./ number_all_relative_times .* diff(image_pixel_x_coordinates) + image_pixel_x_coordinates(1), 'XTickLabel', arrayfun(@(x)[num2str(x), repmat(' ', 1, 1)], all_relative_times_seconds_rounded, 'UniformOutput', false))
              set(matrix_plot_handle, 'TickLength', [0, 0])
              rotateXLabels(matrix_plot_handle, 90)
            end
            if ~should_clustering_figure_use_models
              set(matrix_plot_handle, 'XTick', [], 'XTickLabel', [])
            end
          end
          if should_clustering_figure_use_models
            % Turn off colorbar ticks, the absolute values are unimportant:
            set(colorbar_handle, 'YTick', [], 'YTickLabel', [])
            
            % % Show condition_sensor_combination_frame_counts for each row (different for each column, nevermind):
            % ylabel(matrix_plot_handle, '')
            % condition_sensor_combination_frame_counts(condition_sensor_combination_index, all_relative_time_index)
            % set(matrix_plot_handle, 'YTick', [], 'YTickLabel', [])
          end
          
          % set(dendrogram_plot_handle, 'Position', get(dendrogram_plot_handle, 'Position') .* [1, 0, 1, 0] + min(get(matrix_plot_handle, 'Position') .* [inf, 1, inf, 1]) .* [0, 1, 0, 1])
          
          % Reposition the two subplots such that they align:
          % drawnow
          figure_position = get(gcf, 'Position');
          figure_width = figure_position(3);
          figure_height = figure_position(4);
          position_scales = [figure_width, figure_height, figure_width, figure_height];
          dendrogram_plot_position = get(dendrogram_plot_handle, 'Position') .* position_scales;
          dendrogram_plot_outer_position = get(dendrogram_plot_handle, 'OuterPosition') .* position_scales;
          % matrix_plot_position = get(matrix_plot_handle, 'Position') .* position_scales;
          matrix_plot_position = plotboxpos(matrix_plot_handle) .* position_scales;
          matrix_plot_height = matrix_plot_position(4);
          prior_dendrogram_plot_height = dendrogram_plot_position(4);
          % novel_dendrogram_plot_height = min(matrix_plot_position([3, 4]));
          novel_dendrogram_plot_height = matrix_plot_position(4);
          if use_horizontal_colorbar
            novel_dendrogram_plot_position = dendrogram_plot_position .* [1, 0, 1, 0] + [0, matrix_plot_position(2) + .5 * (matrix_plot_height - novel_dendrogram_plot_height), matrix_plot_position(1) - dendrogram_plot_position(1) - dendrogram_plot_position(3), novel_dendrogram_plot_height];
          else
            % novel_dendrogram_plot_position = dendrogram_plot_position .* [1, 0, 1, 0] + [0, matrix_plot_position(2) + .5 * (matrix_plot_height - novel_dendrogram_plot_height), 0, novel_dendrogram_plot_height];
            % novel_dendrogram_plot_x_offset = .05 * figure_width;
            novel_dendrogram_plot_x_offset = .1 * figure_width;
            % xruan 11/25/2015 change offset to allow less white space
            % between dendrogram to the clustering
            novel_dendrogram_plot_x_offset = -.01 * figure_width;
            % novel_dendrogram_plot_x_offset = .5 * figure_width;
            % Too low for reconstituted label:
            % novel_dendrogram_plot_x_buffer = .02 * figure_width;
            novel_dendrogram_plot_x_buffer = .01 * figure_width;
            
            novel_dendrogram_plot_position = dendrogram_plot_position .* [1, 0, 1, 0] + [-novel_dendrogram_plot_x_offset, matrix_plot_position(2) + .5 * (matrix_plot_height - novel_dendrogram_plot_height), matrix_plot_position(1) - (dendrogram_plot_outer_position(1) + dendrogram_plot_outer_position(3)) + novel_dendrogram_plot_x_offset - novel_dendrogram_plot_x_buffer, novel_dendrogram_plot_height];
          end
          ylim(dendrogram_plot_handle, ylim(matrix_plot_handle))
          set(matrix_plot_handle, 'YAxisLocation', 'right')
          
          % set(dendrogram_plot_handle, 'ActivePositionProperty', 'OuterPosition')
          set(dendrogram_plot_handle, 'YDir', 'reverse')
          % set(dendrogram_plot_handle, 'XTick', [])
          % set(dendrogram_plot_handle, 'XLabel', [])
          % set(dendrogram_plot_handle, 'XScale', 'log')
          
          if ~debug_check_ytick_orientation
            set(dendrogram_plot_handle, 'Position', novel_dendrogram_plot_position ./ position_scales)
            if use_horizontal_colorbar
              set(dendrogram_plot_handle, 'YTick', [])
              set(dendrogram_plot_handle, 'Visible', 'off')
            else
              set(dendrogram_plot_handle, 'XColor', get(gca, 'Color'))
            end
          end
          
          % beep, keyboard
          
          % subplot(1, 1, 1)
          % set(gca, 'Visible', 'off')
          % title(sprintf('', current_cophenetic_coefficient), 'Interpreter', 'none')
          
          
          % pause

          % keyboard
          
          set(gcf, 'Color', 'w')
          % set(gcf, 'Color', 'none')
          
          image_filename = sprintf('%shierarchical_clustering_%s_%s', master_script_options.model_prefix, current_condition_subset_name, model_type);
          % Filename contains distance metric name:
          current_clustering_distance_metric_name = current_clustering_distance_metric;
          if ~ischar(current_clustering_distance_metric_name)
            error('Need a set of names for each non-string distance metric...')
            % current_clustering_distance_metric_name = regexprep(func2str(current_clustering_distance_metric_name), '[^a-zA-Z0-9]', '');
          end
          image_filename = [image_filename, sprintf('_metric_%s', current_clustering_distance_metric_name)];
          % Filename changes if displaying models instead of distance matrix:
          if should_clustering_figure_use_models
            image_filename = [image_filename, '_mdl'];
          end
          % Filename changes if debugging to confirm proper orientation of dendrogram:
          if debug_check_ytick_orientation
            image_filename = [image_filename, '_check_ytick'];
          end
          image_full_filename = [figures_location, image_filename, '.png'];
          % export_fig(image_full_filename, '-zbuffer', '-png', '-a1', '-nocrop', ['-r', num2str(dpi)])
          export_fig(image_full_filename, '-zbuffer', '-png', '-a1', ['-r', num2str(dpi)])
          % export_fig(image_full_filename, '-zbuffer', '-png', '-a1', '-nocrop', ['-r', num2str(dpi)], 'bounds', 'tight')
        
        end

      end
      
    end
    
  end
  

  
  % Print the number of cells used for each model:
  fprintf('\n\n========================================\nStatistics after hierarchical clustering:\n')
  condition_sensor_combination_frame_counts
  condition_sensor_combination_counts = sum(condition_sensor_combination_frame_counts, 2)
  model_total_count = sum(condition_sensor_combination_frame_counts(:))
  frame_model_mean_count = mean(condition_sensor_combination_frame_counts(:))
  frame_model_minimum_count = min(condition_sensor_combination_frame_counts(:))
  frame_model_maximum_count = max(condition_sensor_combination_frame_counts(:))
  spatiotemporal_model_mean_count = mean(condition_sensor_combination_counts(:))
  spatiotemporal_model_minimum_count = min(condition_sensor_combination_counts(:))
  spatiotemporal_model_maximum_count = max(condition_sensor_combination_counts(:))
  save([models_location, sprintf('%scondition_sensor_combination_frame_counts.mat', master_script_options.model_prefix)], 'condition_sensor_combination_frame_counts', 'model_total_count', 'frame_model_mean_count', 'frame_model_minimum_count', 'frame_model_maximum_count', 'spatiotemporal_model_mean_count', 'spatiotemporal_model_minimum_count', 'spatiotemporal_model_maximum_count', 'condition_sensor_combinations', 'condition_sensor_combinations_with_features', 'all_relative_times')

  
  
  % beep, keyboard
  % error('Implementation yet unfinished below this line!')

  t_cell_info.clustering_info = struct();
  % t_cell_info.clustering_info.model_trees = model_trees;
  % t_cell_info.clustering_info.model_distances = model_distances;
  t_cell_info.clustering_info.clustering_distance_metrics = clustering_distance_metrics;
  t_cell_info.clustering_info.number_clustering_distance_metrics = number_clustering_distance_metrics;
  t_cell_info.clustering_info.all_model_distances = all_model_distances;
  t_cell_info.clustering_info.all_model_trees = all_model_trees;
  t_cell_info.clustering_info.all_model_clusters = all_model_clusters;
  
  
end  

