function [model_info] = master_script_collect_models(t_cell_info, options)
  % 2013-11-06 tebuck: Copied from master_script_create_blender_figures.m.
  % 
  % 
  % 
  % 
  % 
  % % Dependencies:
  % % From the File Exchange: export_fig, pmkmp, toolbox_graph, toolbox_fast_marching, affine, Snake3D, Mesh_voxelisation, dirr, inhull
  % % From tebuck: modified version of Jieyue Li's HPA_lib, rasterize_mesh

%
% Author: Taraz E. Buck and Xiongtao Ruan
%
% Copyright (C) 2012-2016 Murphy Lab
% Computational Biology Department
% School of Computer Science
% Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu



  
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % Set defaults and process options structure:
  
  
  default_options = struct();
  default_options.should_fill_use_interpolation = false;
  default_options.should_fill_with_nan = false;
  % default_options. = ;
  
  if ~exist('options', 'var')
    options = default_options;
  else
    options = process_options_structure(default_options, options);
  end

  
  % options
  % pause


  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % Hierarchical clustering of condition-sensor combinations for each model type:
  
  
  % Collect models:
  
  conditions = unique(condition_sensor_combinations(:, 1));
  sensors = unique(condition_sensor_combinations(:, 2));
  number_conditions = length(conditions)
  number_sensors = length(sensors)
  number_condition_pairs = (number_conditions - 1) * number_conditions / 2
  
  % should_contrast_stretch_across_time = false
  should_contrast_stretch_across_time = true
  
  warning('Should make it obvious in a figure that empty_features is true for a panel!')

  for model_type_index = 1:number_model_types
  
    model_type = model_types{model_type_index}

    representation_function = model_representation_functions{model_type_index};
    reconstruction_function = model_reconstruction_functions{model_type_index};
    original_representation_function = model_original_representation_functions{model_type_index};
    original_reconstruction_function = model_original_reconstruction_functions{model_type_index};
    reduction_representation_function = model_reduction_representation_functions{model_type_index};
    reduction_reconstruction_function = model_reduction_reconstruction_functions{model_type_index};
    
    
    condition_sensor_combination_features = cell(number_condition_sensors, number_all_relative_times);
    condition_sensor_combination_panels = cell(number_condition_sensors, number_all_relative_times);
    condition_sensor_combination_frame_counts = zeros(number_condition_sensors, number_all_relative_times);
    for all_relative_time_index = 1:number_all_relative_times
      relative_time = all_relative_times(all_relative_time_index);
      
      for condition_sensor_combination_index = 1:number_condition_sensors
        current_condition = condition_sensor_combinations{condition_sensor_combination_index, 1};
        current_sensor = condition_sensor_combinations{condition_sensor_combination_index, 2};

        % Get the model:
        current_model_filename = sprintf('%s%s_%s_reltime% 3d_%s', master_script_options.model_prefix, current_condition, current_sensor, relative_time, model_type);
        current_model_full_filename = [models_location, current_model_filename, '.mat'];
        if exist(current_model_full_filename)
          current_model = load(current_model_full_filename);
          current_mean = current_model.current_mean;
          condition_sensor_combination_frame_counts(condition_sensor_combination_index, all_relative_time_index) = size(current_model.current_data, 1);
        else
          continue
        end
        
        condition_sensor_combination_features{condition_sensor_combination_index, all_relative_time_index} = current_mean;
        
        current_mean_reconstructed = original_reconstruction_function(current_mean);
        current_mean_reconstructed_cropped_slice = template_crop_function(current_mean_reconstructed);
        switch model_illustration_view{model_type_index}
          case 'top'
            current_mean_reconstructed_cropped_slice = current_mean_reconstructed_cropped_slice(:, :, round(end / 2));
          case 'front'
            % keyboard
            % current_mean_reconstructed_cropped_slice = permute(current_mean_reconstructed_cropped_slice(round(end / 2), :, :), [3, 2, 1]);
            % current_mean_reconstructed_cropped_slice = permute(current_mean_reconstructed_cropped_slice(template_crop_function(approximate_synapse_slice_template_image)), [3, 2, 1]);
            current_mean_reconstructed_cropped_slice = permute(current_mean_reconstructed_cropped_slice(approximate_synapse_slice - find(template_crop_y, 1) + 1, :, :), [3, 2, 1]);
          otherwise
            error('model_illustration_view{model_type_index} = ''%s'' is invalid', model_illustration_view{model_type_index})
        end
        % % Whole model (large!):
        % condition_sensor_combination_panels{condition_sensor_combination_index, all_relative_time_index} = reshape_contrast(current_mean_reconstructed);
        % Middle top-down slice (later add near-synapse slice?):
        % Also don't contrast stretch per model:
        condition_sensor_combination_panels{condition_sensor_combination_index, all_relative_time_index} = current_mean_reconstructed_cropped_slice;
        
        if ~should_contrast_stretch_across_time
          current_mean_reconstructed_cropped_slice = current_mean_reconstructed_cropped_slice ./ max(current_mean_reconstructed_cropped_slice(:));
        end
        
      end
    end
    
    % Fill in unobserved models (nans for features, zeros for figures):
    empty_features = cellfun(@isempty, condition_sensor_combination_features);
    nan_features = nan(size(condition_sensor_combination_features{find(~empty_features, 1)}));
    blank_panel = zeros(size(condition_sensor_combination_panels{find(~empty_features, 1)}));
    condition_sensor_combination_features_empty_proportion = mean(empty_features(:))
    if any(empty_features(:))
      if options.should_fill_with_nan
        % keyboard
        condition_sensor_combination_features(empty_features) = {nan_features};
        condition_sensor_combination_panels(empty_features) = {blank_panel};
      else
        for condition_sensor_combination_index = 1:number_condition_sensors
          emptinesses = empty_features(condition_sensor_combination_index, :);
          naned_relative_time_indices = nan(size(emptinesses));
          naned_relative_time_indices(~emptinesses) = find(~emptinesses);
          if options.should_fill_use_interpolation
            extrapolated_relative_time_indices = extrapolate_nans(naned_relative_time_indices, struct('method', 'diffusion', 'diffusion_method', 4));
          else
            extrapolated_relative_time_indices = extrapolate_nans(naned_relative_time_indices, struct('method', 'nearest'));
          end
          
          % combination_maximum_intensity = 0;
          for all_relative_time_index = 1:number_all_relative_times
            if isempty(condition_sensor_combination_features{condition_sensor_combination_index, all_relative_time_index})
              % keyboard
              nearest_nonempty_time = round(extrapolated_relative_time_indices(all_relative_time_index));
              if options.should_fill_use_interpolation
                error
              else
                condition_sensor_combination_features{condition_sensor_combination_index, all_relative_time_index} = condition_sensor_combination_features{condition_sensor_combination_index, nearest_nonempty_time};
                condition_sensor_combination_panels{condition_sensor_combination_index, all_relative_time_index} = condition_sensor_combination_panels{condition_sensor_combination_index, nearest_nonempty_time};
              end
            end
          end
        end
      end
    end
          
    if should_contrast_stretch_across_time
      % Contrast stretch each protein model across all time points:
      for condition_sensor_combination_index = 1:number_condition_sensors
        combination_maximum_intensity = max(cellfun(@(x)max(x(:)), condition_sensor_combination_panels(condition_sensor_combination_index, :)));
        condition_sensor_combination_panels(condition_sensor_combination_index, :) = cellfun(@(x)x ./ combination_maximum_intensity, condition_sensor_combination_panels(condition_sensor_combination_index, :), 'UniformOutput', false);
      end
    end

    
    
    % condition_sensor_combination_features
    % Combine features across time so that we are clustering spatiotemporal sensor distributions:
    condition_sensor_combination_features = cell2mat(condition_sensor_combination_features);
    % condition_sensor_combination_panels = cell2mat(condition_sensor_combination_panels);
    % whos condition_sensor_combination_features
    % condition_sensor_combination_features_nan_proportion = mean(isnan(condition_sensor_combination_features(:)))
  
  end
  

end

