function [t_cell_info] = master_script_compare_models(t_cell_info, options)
  % Produce videos of constructed models for visual evaluation.
  %
  % 2014-06-25 tebuck: Copied from master_script_generate_model_videos.m.
  % 
  % 
  % 
  % 
  % 
  % % Dependencies:
  % % From the File Exchange: export_fig, pmkmp, toolbox_graph, toolbox_fast_marching, affine, Snake3D, Mesh_voxelisation, dirr, inhull
  % % From tebuck: modified version of Jieyue Li's HPA_lib, rasterize_mesh

%
% Author: Taraz E. Buck and Xiongtao Ruan
%
% Copyright (C) 2012-2016 Murphy Lab
% Computational Biology Department
% School of Computer Science
% Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu



  
  
  
  % Variables referenced more than once can be copied to local variables:
  master_script_options = t_cell_info.options;
  fprintf_diary_cycle_function = t_cell_info.log_info.fprintf_diary_cycle_function;
  regions_location = t_cell_info.path_info.regions_location;
  segmentations_filtered_location = t_cell_info.path_info.segmentations_filtered_location;
  alignments_location = t_cell_info.path_info.alignments_location;
  deformations_location = t_cell_info.path_info.deformations_location;
  models_location = t_cell_info.path_info.models_location;
  figures_location = t_cell_info.path_info.figures_location;
  videos_location = t_cell_info.path_info.videos_location;
  
  illustration_colormap_to_use = t_cell_info.plotting_info.illustration_colormap_to_use;
  
  condition_name_abbreviation_function = t_cell_info.abbreviation_info.condition_name_abbreviation_function;
  
  model_types = t_cell_info.model_type_info.model_types;
  number_model_types = t_cell_info.model_type_info.number_model_types;
  radial_distribution_types = t_cell_info.model_type_info.radial_distribution_types;
  radial_distribution_model_types = t_cell_info.model_type_info.radial_distribution_model_types;
  number_radial_distribution_types = t_cell_info.model_type_info.number_radial_distribution_types;
  model_representation_functions = t_cell_info.model_type_info.model_representation_functions;
  model_reconstruction_functions = t_cell_info.model_type_info.model_reconstruction_functions;
  model_dimensionality_reduction_methods = t_cell_info.model_type_info.model_dimensionality_reduction_methods;
  model_dimensionality_reduction_method_parameters = t_cell_info.model_type_info.model_dimensionality_reduction_method_parameters;

  run_relative_path_list = t_cell_info.synapse_info.run_relative_path_list;
  run_key_list = t_cell_info.synapse_info.run_key_list;
  % run_data_list = t_cell_info.synapse_info.run_data_list;
  run_images_list = t_cell_info.synapse_info.run_images_list;
  run_brightfield_images_list = t_cell_info.synapse_info.run_brightfield_images_list;
  run_image_frame_indices_list = t_cell_info.synapse_info.run_image_frame_indices_list;
  run_number_cells = t_cell_info.synapse_info.run_number_cells;
  run_number_frames = t_cell_info.synapse_info.run_number_frames;
  run_point_tracks = t_cell_info.synapse_info.run_point_tracks;
  run_relative_time_tracks = t_cell_info.synapse_info.run_relative_time_tracks;
  run_relative_times = t_cell_info.synapse_info.run_relative_times;
  run_number_relative_times = t_cell_info.synapse_info.run_number_relative_times;
  all_relative_times = t_cell_info.synapse_info.all_relative_times;
  number_all_relative_times = t_cell_info.synapse_info.number_all_relative_times;
  all_relative_times_seconds_data = t_cell_info.synapse_info.all_relative_times_seconds_data;
  all_relative_times_seconds = t_cell_info.synapse_info.all_relative_times_seconds;
  all_relative_times_seconds_rounded = t_cell_info.synapse_info.all_relative_times_seconds_rounded;
  number_runs = t_cell_info.synapse_info.number_runs;
  condition_sensor_combinations = t_cell_info.synapse_info.condition_sensor_combinations;
  number_condition_sensors = t_cell_info.synapse_info.number_condition_sensors;
  conditions = t_cell_info.synapse_info.conditions;
  sensors = t_cell_info.synapse_info.sensors;
  number_conditions = t_cell_info.synapse_info.number_conditions;
  number_sensors = t_cell_info.synapse_info.number_sensors;
  
  template_centroid = t_cell_info.template_info.template_centroid;
  template_synapse = t_cell_info.template_info.template_synapse;
  template_image = t_cell_info.template_info.template_image;
  template_crop_function = t_cell_info.template_info.template_crop_function;

  window_size_2d = t_cell_info.preprocessing_info.window_size_2d;
  window_center_2d = t_cell_info.preprocessing_info.window_center_2d;
  cropped_size = t_cell_info.preprocessing_info.cropped_size;

  segmentation_rasterization_function = t_cell_info.segmentation_info.segmentation_rasterization_function;
  
  model_original_representation_functions = t_cell_info.model_info.model_original_representation_functions;
  model_original_reconstruction_functions = t_cell_info.model_info.model_original_reconstruction_functions;
  model_reduction_representation_functions = t_cell_info.model_info.model_reduction_representation_functions;
  model_reduction_reconstruction_functions = t_cell_info.model_info.model_reduction_reconstruction_functions;
  
  confidence_level = master_script_options.confidence_level;
  nonparametric_manova_number_permutations = master_script_options.nonparametric_manova_number_permutations;
  rda_manova_number_permutations = master_script_options.rda_manova_number_permutations;
  
  
  % debug_print_statistic_computation_skip_reasons = false;
  debug_print_statistic_computation_skip_reasons = true
  
  
  function [result_p_value] = manova_p_value_computation_function(given_data1, given_data2)
    [~, result_p_value, ~] = manova1([given_data1; given_data2], [zeros(size(given_data1, 1), 1) + 1; zeros(size(given_data2, 1), 1) + 2], confidence_level);
    result_p_value = result_p_value(1);
  end
  
  
  % warning('Make this non-verbose!')
  warning('Choose an appropriate dissimilarity!')
  
  
  function [result_p_value] = nonparametric_manova_p_value_computation_function(given_data1, given_data2, metadata)
    if nargin < 3
      metadata = struct;
    end
    data_distances = squareform(pdist([given_data1; given_data2]));
    result = f_npManova(data_distances, [zeros(size(given_data1, 1), 1) + 1; zeros(size(given_data2, 1), 1) + 2], nonparametric_manova_number_permutations, 0, []);
    result_p_value = result(1).p;
  end
  
  
  function [result_p_value] = rda_manova_p_value_computation_function(given_data1, given_data2, metadata)
    if nargin < 3
      metadata = struct;
    end
    error('Not yet working!')
    whos given_data1 given_data2
    size([given_data1; given_data2])
    size([zeros(size(given_data1, 1), 1) + 1; zeros(size(given_data2, 1), 1) + 2])
    result = f_rdaAnova([given_data1; given_data2], [zeros(size(given_data1, 1), 1) + 1; zeros(size(given_data2, 1), 1) + 2], rda_manova_number_permutations, 0);
    result_p_value = result(1).p;
  end

  
  function [result_p_value] = cda_manova_p_value_computation_function(given_data1, given_data2, metadata)
    if nargin < 3
      metadata = struct;
    end
    % error('Not yet implemented!')
    combined_data = [given_data1; given_data2];
    combined_classes = [zeros(size(given_data1, 1), 1) + 1; zeros(size(given_data2, 1), 1) + 2];
    
    % beep, dbstack, keyboard
    
    % % Builds a number_features * number_features covariance matrix:
    % combined_data_cda = f_cda(combined_data, combined_classes, 2);
    
    % % Do our own CDA (hacky, not a fleshed-out idea):
    % % First apply PCA that works even with thousands of features:
    % combined_data_pca = pca(combined_data);
    
    % My implementation of Yu and Yang 2004, "A Direct LDA Algorithm for High-Dimensional Data--with Application to Face Recognition:"
    % combined_data_cda = direct_lda(struct('data', combined_data, 'class_assignments', combined_classes, 'minimum_dimensionality', ));
    combined_data_cda = direct_lda(struct('data', combined_data, 'class_assignments', combined_classes));
    combined_data_cda = combined_data_cda.data_sphered;
    if false
      % Debug info:
      a = combined_data_cda;
      a = [a, zeros(size(a, 1), max(3 - size(a, 2), 0))];
      scatter3(a(:, 1), a(:, 2), a(:, 3), [], combined_classes)
      axis equal
      title('combined_data_cda', 'Interpreter', 'none')
      
      dbstack, beep, keyboard
    end
    
    % % Use available implementation:
    % combined_data_cda = directlda(combined_data, combined_classes);
    
    % error('Not finished!')
    % whos combined_data_cda
    % scatter(combined_classes, combined_data_cda)
    % beep, dbstack, keyboard
    
    result_p_value = manova_p_value_computation_function(combined_data_cda(1:size(given_data1, 1), :), combined_data_cda(size(given_data1, 1) + 1:end, :));
    
    % whos combined_*
    % beep, dbstack, keyboard
  end
  
  
  function [result_p_value] = nm_dc_p_value_computation_function(given_data1, given_data2, metadata)
    % Deconfounds with respect to metadata, incomplete!
    if nargin < 3
      metadata = struct;
    end
    error('Not yet implemented!')
    combined_data = [given_data1; given_data2];
    combined_classes = [zeros(size(given_data1, 1), 1) + 1; zeros(size(given_data2, 1), 1) + 2];
    
    beep, dbstack, keyboard
    
    [result_p_value] = nonparametric_manova_p_value_computation_function(given_data1, given_data2)
    
  end
  
  
  % Initialization:
  
  
  % Versions using all parameter sets:
  % Assumes histogram-like, e.g., voxel or binned voxel intensity models:
  chi_square_p_value_function = @(given_data1, given_data2)pearson_chi_square_histograms(sum(given_data1, 1), sum(given_data2, 1));
  % For other types of model:
  manova_computation_function = @manova_p_value_computation_function;
  nonparametric_manova_computation_function = @nonparametric_manova_p_value_computation_function;
  rda_manova_computation_function = @rda_manova_p_value_computation_function;
  cda_manova_computation_function = @cda_manova_p_value_computation_function;
  nm_dc_computation_function = @nm_dc_p_value_computation_function;
  
  model_type_statistic_computation_functions = cell(number_model_types, 1);
  enabled_tests_with_whole_data_preprocessing = {};
  for model_type_index = 1:number_model_types
    
    model_type = model_types{model_type_index}
    statistic_computation_functions = struct;
    % switch model_type
      % case 'standardized_voxels'
      % case radial_distribution_model_types
        % statistic_computation_functions.manova = manova_computation_function;
    % end
    if ~false, warning('>>>> HACK, disabling statistic_computation_functions.manova for now for speed'), else, statistic_computation_functions.manova = manova_computation_function; end
    if ~false, warning('>>>> HACK, disabling statistic_computation_functions.nonparametric_manova for now for speed'), else, statistic_computation_functions.nonparametric_manova = nonparametric_manova_computation_function; end
    if ~false, warning('>>>> HACK, disabling statistic_computation_functions.rda_manova for now for speed'), else, statistic_computation_functions.rda_manova = rda_manova_computation_function; end
    if false, warning('>>>> HACK, disabling statistic_computation_functions.cda_manova for now for speed'), else, statistic_computation_functions.cda_manova = cda_manova_computation_function; end
    if ~false, warning('>>>> HACK, disabling statistic_computation_functions.nm_dc for now for speed'), else, statistic_computation_functions.nm_dc = nm_dc_computation_function; enabled_tests_with_whole_data_preprocessing{end + 1} = 'nm_dc'; end
    model_type_statistic_computation_functions{model_type_index} = statistic_computation_functions;
  end
  
  enabled_tests_with_whole_data_preprocessing = unique(enabled_tests_with_whole_data_preprocessing);
  
  statistic_names = cellfun(@(x)fieldnames(x).', model_type_statistic_computation_functions, 'UniformOutput', false).';
  statistic_names = unique([statistic_names{:}]);
  % blank_statistic_structure = struct;
  % blank_statistic_structure = cell2struct({repmat({}, [length(statistic_names), 1])}, statistic_names);
  blank_statistic_structure = cell2struct(repmat({[]}, [length(statistic_names), 1]), statistic_names);
  
  number_condition_pairs = (number_conditions - 1) * number_conditions / 2
  condition_pair_sensor_statistics = repmat(blank_statistic_structure, [number_condition_pairs, number_sensors, number_all_relative_times, number_model_types]);
  % indices_used = zeros(0, 4);
  
  number_condition_sensor_combination_pairs = (number_condition_sensors - 1) * number_condition_sensors / 2
  condition_sensor_combination_pair_statistics = repmat(blank_statistic_structure, [number_condition_sensor_combination_pairs, number_all_relative_times, number_model_types]);
  
  number_tests_performed = 0;
  
  
  % Collect models:
  
  work_uncomputed = true(number_model_types, number_all_relative_times);
  recently_did_any_work = false;
  did_any_work = false;
  
  while any(work_uncomputed(:))
    
    recently_did_any_work = did_any_work;
    did_any_work = false;
  
    for model_type_index = 1:number_model_types
      
      model_type = model_types{model_type_index}
      statistic_computation_functions = model_type_statistic_computation_functions{model_type_index};
      test_names = fieldnames(statistic_computation_functions);
      

      representation_function = model_representation_functions{model_type_index};
      reconstruction_function = model_reconstruction_functions{model_type_index};
      original_representation_function = model_original_representation_functions{model_type_index};
      original_reconstruction_function = model_original_reconstruction_functions{model_type_index};
      reduction_representation_function = model_reduction_representation_functions{model_type_index};
      reduction_reconstruction_function = model_reduction_reconstruction_functions{model_type_index};
      
      % First concatenate all data for producing information that can safely be extracted from everything (e.g., confounding, where the variable of interest isn't involved):
      model_reduced_data = [];
      model_data_condition_indices = [];
      model_data_sensor_indices = [];
      model_data_combination_indices = [];
      model_unique_run_ids = [];
      model_next_unique_run_id = 1;
      model_reduced_data_corrected = [];
      if ~isempty(enabled_tests_with_whole_data_preprocessing)
        for all_relative_time_index = 1:number_all_relative_times
          relative_time = all_relative_times(all_relative_time_index);
          for condition_sensor_combination_index = 1:number_condition_sensors
            current_condition = condition_sensor_combinations{condition_sensor_combination_index, 1};
            current_sensor = condition_sensor_combinations{condition_sensor_combination_index, 2};
            condition_index = find(strcmp(current_condition, conditions));
            sensor_index = find(strcmp(current_sensor, sensors));

            % Get the model:
            current_model_filename = sprintf('%s%s_%s_reltime% 3d_%s', master_script_options.model_prefix, current_condition, current_sensor, relative_time, model_type);
            current_model_full_filename = [models_location, current_model_filename, '.mat'];
            if ~exist(current_model_full_filename, 'file') || exist([current_model_full_filename(1:end-3), '.tmp'], 'file')
              continue
            end
            current_model = load(current_model_full_filename);
            current_model_reduced_data = reduction_representation_function(current_model.current_data);
            current_model_unique_run_ids = current_model.current_data_relevant_run_key_indices;
            
            model_reduced_data = [model_reduced_data; current_model_reduced_data];
            model_data_condition_indices = [model_data_condition_indices; zeros(size(current_model_reduced_data, 1), 1) + condition_index];
            model_data_sensor_indices = [model_data_sensor_indices; zeros(size(current_model_reduced_data, 1), 1) + sensor_index];
            model_data_combination_indices = [model_data_combination_indices; zeros(size(current_model_reduced_data, 1), 1) + condition_sensor_combination_index];
            model_unique_run_ids = [model_unique_run_ids; current_model_unique_run_ids + (model_next_unique_run_id - 1)];
            
            % model_next_unique_run_id = model_next_unique_run_id + length(unique(current_model_unique_run_ids));
            model_next_unique_run_id = model_next_unique_run_id + length(current_model.current_data_relevant_run_keys);
          end
        end
      end
      
      if false
        % Debug info:
        % Make sure this is strictly increasing:
        model_unique_run_ids_increasing = min(diff(model_unique_run_ids)) == 0
        number_negative_run_id_changes = sum(diff(model_unique_run_ids) < 0)
        plot(diff(model_unique_run_ids))
        plot(model_unique_run_ids)
        title('model_unique_run_ids', 'Interpreter', 'none')
        
        dbstack, beep, keyboard
      end
      
      
      if false
        % % % Save this work if remove_confounding_subspace starts getting expensive?
        % % current_models_statistics_filename = sprintf('%scondition_pair_statistics_reltime% 3d_%s', master_script_options.model_prefix, relative_time, model_type);
        % % [can_start, final_name, final_exists, temp_exists] = chunk_start(models_location, current_models_statistics_filename, []);
        % Compute the unconfounded subspace:
        addpath(genpath('distcorr'))
        deconfounding_options = struct;
        deconfounding_options.data = model_reduced_data;
        deconfounding_options.class_assignments = model_data_combination_indices;
        deconfounding_options.subclass_assignments = model_unique_run_ids;
        deconfounding_options.should_zscore = false;
        deconfounding_options.method = 'distance_correlation';
        deconfounding_options.verbose = 1;
        distance_correlation_options = struct;
        % distance_correlation_options.principal_component_variance = .95;
        deconfounding_options.distance_correlation_options = distance_correlation_options;
        clear distance_correlation_options
        deconfounding_result = remove_confounding_subspace(deconfounding_options)
        
        model_reduced_data_corrected = deconfounding_result.data_corrected;
        
        if ~false
          % Debug info:
          dbstack, beep, keyboard
        end
      end
      
      % Now run the tests:
      
      printed_run_work_message = false;
      
      for all_relative_time_index = 1:number_all_relative_times
        relative_time = all_relative_times(all_relative_time_index);
        
        if ~work_uncomputed(model_type_index, all_relative_time_index)
          continue
        end
        
        current_models_statistics_filename = sprintf('%scondition_pair_statistics_reltime% 3d_%s', master_script_options.model_prefix, relative_time, model_type);
        [can_start, final_name, final_exists, temp_exists] = chunk_start(models_location, current_models_statistics_filename, []);
        % warning('>>>> HACK, starting "%s" anyway!', current_models_statistics_filename), can_start = true;
        
        current_condition_sensor_combination_pair_statistics = repmat(blank_statistic_structure, [number_condition_sensor_combination_pairs, 1]);
        current_condition_pair_sensor_statistics = repmat(blank_statistic_structure, [number_condition_pairs, number_sensors]);
        current_number_tests_performed = 0;
        
        if can_start
          
          
          did_any_work = true;
          if ~printed_run_work_message
            fprintf_diary_cycle_function('Comparing models for model_type "%s"\n', model_type)
            printed_run_work_message = true;
          end
          
          % Test all pairs of condition-sensor combinations for each time and model type:
          
          for condition_sensor_combination_index1 = 1:number_condition_sensors
            current_condition1 = condition_sensor_combinations{condition_sensor_combination_index1, 1};
            current_sensor1 = condition_sensor_combinations{condition_sensor_combination_index1, 2};
            condition_index1 = find(strcmp(current_condition1, conditions));
            sensor_index1 = find(strcmp(current_sensor1, sensors));

            % Get the model:
            current_model1_filename = sprintf('%s%s_%s_reltime% 3d_%s', master_script_options.model_prefix, current_condition1, current_sensor1, relative_time, model_type);
            current_model1_full_filename = [models_location, current_model1_filename, '.mat'];
            if ~exist(current_model1_full_filename, 'file') || exist([current_model1_full_filename(1:end-3), '.tmp'], 'file')
              continue
            end
            current_model1 = load(current_model1_full_filename);
            current_model1_reduced_data = reduction_representation_function(current_model1.current_data);
            
            for condition_sensor_combination_index2 = condition_sensor_combination_index1 + 1:number_condition_sensors
              current_condition2 = condition_sensor_combinations{condition_sensor_combination_index2, 1};
              current_sensor2 = condition_sensor_combinations{condition_sensor_combination_index2, 2};
              condition_index2 = find(strcmp(current_condition2, conditions));
              sensor_index2 = find(strcmp(current_sensor2, sensors));
              
              % Number of combinations without replacement:
              condition_pair_index = sum(number_conditions - 1:-1:number_conditions - condition_index1 + 1) + condition_index2 - condition_index1;
              combination_pair_index = sum(number_condition_sensors - 1:-1:number_condition_sensors - condition_sensor_combination_index1 + 1) + condition_sensor_combination_index2 - condition_sensor_combination_index1;
                  
              % Get the model:
              current_model2_filename = sprintf('%s%s_%s_reltime% 3d_%s', master_script_options.model_prefix, current_condition2, current_sensor2, relative_time, model_type);
              current_model2_full_filename = [models_location, current_model2_filename, '.mat'];
              if ~exist(current_model2_full_filename, 'file') || exist([current_model2_full_filename(1:end-3), '.tmp'], 'file')
                continue
              end
              
              current_model2 = load(current_model2_full_filename);
              current_model2_reduced_data = reduction_representation_function(current_model2.current_data);
            
              % condition_pair_sensor_statistics{pair_index, sensor_index, all_relative_time_index, model_type_index} = struct;
              for test_index = 1:length(test_names)
                current_test_name = test_names{test_index};
                current_statistic_computation_function = statistic_computation_functions.(current_test_name);

                try
                  % current_statistic_p_value = current_statistic_computation_function(current_model1.current_data, current_model2.current_data);
                  % current_statistic_p_value = current_statistic_computation_function(current_model1_reduced_data, current_model2_reduced_data);
                  metadata = struct;
                  if false
                    % Debug info:
                    dbstack, beep, keyboard
                  end
                  % error('Define metadata.confounders!')
                  metadata.confounders = [];
                  
                  current_statistic_p_value = current_statistic_computation_function(current_model1_reduced_data, current_model2_reduced_data, metadata);
                  if length(current_statistic_p_value) == 1 && isfinite(current_statistic_p_value)
                    number_tests_performed = number_tests_performed + 1;
                    current_number_tests_performed = current_number_tests_performed + 1;
                  else
                    error('current_statistic_computation_function returns incorrect number of or nan p-values!')
                  end
                catch raised_error
                  % getReport(raised_error, 'extended')
                  fprintf('Error during statistic computation: %s\n', getReport(raised_error, 'basic'))
                  fprintf('Data sizes: %d, %d, %d, %d\n', size(current_model1_reduced_data), size(current_model2_reduced_data))
                  current_statistic_p_value = nan;
                  % continue
                end
                
                if sensor_index1 == sensor_index2
                  current_condition_pair_sensor_statistics(condition_pair_index, sensor_index1).(current_test_name) = current_statistic_p_value;
                end
                current_condition_sensor_combination_pair_statistics(combination_pair_index).(current_test_name) = current_statistic_p_value;
              end
            end
          end
          
          
          save(final_name, 'current_condition_sensor_combination_pair_statistics', 'current_condition_pair_sensor_statistics', 'conditions', 'sensors', 'condition_sensor_combinations', 'all_relative_times', 'model_types', 'confidence_level', 'current_number_tests_performed', 'model_type_index', 'all_relative_time_index', 'model_type', 'relative_time')
          chunk_finish(models_location, current_models_statistics_filename);
          work_uncomputed(model_type_index, all_relative_time_index) = false;
        else
          % If a .tmp file exists, work has been computed, is being computed, or was being computed and failed. In any case, we cannot continue computation so treat as computed:
          work_uncomputed(model_type_index, all_relative_time_index) = false;
          if final_exists && ~temp_exists
            load(final_name, 'current_condition_pair_sensor_statistics', 'current_condition_sensor_combination_pair_statistics', 'current_number_tests_performed')
          else
            continue
          end
        end
        
        condition_pair_sensor_statistics(:, :, all_relative_time_index, model_type_index) = current_condition_pair_sensor_statistics;
        condition_sensor_combination_pair_statistics(:, all_relative_time_index, model_type_index) = current_condition_sensor_combination_pair_statistics;
        number_tests_performed = number_tests_performed + current_number_tests_performed;
        
      end
    
      % number_indices_used = size(indices_used, 1)
      % number_unique_indices_used = size(unique(indices_used, 'rows'), 1)
      
      
    end

    pause(1)
    
    if recently_did_any_work && ~did_any_work
      fprintf_diary_cycle_function('Apparently no more work to do, wait for other jobs')
    end
    
  end
  
  
  
  
  % Output corrected p-values in an accessible format:
  
  fprintf_diary_cycle_function('\n\n')
  
  statistic_names = unique([fieldnames(condition_pair_sensor_statistics); fieldnames(condition_sensor_combination_pair_statistics)]);
  % uncorrected_p_values = [];
  uncorrected_p_values_cell = {};
  for statistic_name_index = 1:length(statistic_names)
    uncorrected_p_values_cell = [uncorrected_p_values_cell, {condition_pair_sensor_statistics.(statistic_names{statistic_name_index})}, {condition_sensor_combination_pair_statistics.(statistic_names{statistic_name_index})}];
  end
  number_tests_performed
  uncorrected_p_values_cell_occupied = ~cellfun(@isempty, uncorrected_p_values_cell);
  uncorrected_p_values_cell_valid = cellfun(@(x)~isempty(x) && isfinite(x), uncorrected_p_values_cell);
  number_uncorrected_p_values_cell = sum(uncorrected_p_values_cell_valid)
  % uncorrected_p_values = cell2mat(uncorrected_p_values_cell);
  uncorrected_p_values = [uncorrected_p_values_cell{:}];
  % uncorrected_p_values = [uncorrected_p_values{:}];
  uncorrected_p_value_deciles = prctile(uncorrected_p_values, 0:10:100)
  valid_test_indices = isfinite(uncorrected_p_values);
  proportion_valid_test_indices = mean(valid_test_indices)
  valid_uncorrected_p_values = uncorrected_p_values(valid_test_indices);
  % addpath(genpath('bonf_holm'))
  corrected_p_values = bonf_holm(valid_uncorrected_p_values);
  corrected_p_value_deciles = prctile(corrected_p_values, 0:10:100)
  % proportion_valid_test_indices = length(valid_uncorrected_p_values) / ((prod(size(a.condition_pair_sensor_statistics)) + prod(size(a.condition_sensor_combination_pair_statistics)))*2)
  
  corrected_p_values_cell = uncorrected_p_values_cell;
  whos corrected_p_values_cell corrected_p_values
  % sum(uncorrected_p_values_cell_occupied)
  sum(uncorrected_p_values_cell_valid)
  size(num2cell(corrected_p_values))
  % corrected_p_values_cell(uncorrected_p_values_cell_occupied) = num2cell(corrected_p_values);
  corrected_p_values_cell(uncorrected_p_values_cell_valid) = num2cell(corrected_p_values);
  corrected_condition_pair_sensor_statistics = condition_pair_sensor_statistics;
  corrected_condition_sensor_combination_pair_statistics = condition_sensor_combination_pair_statistics;
  for statistic_name_index = 1:length(statistic_names)
    % corrected_condition_pair_sensor_statistics.(statistic_names{statistic_name_index}) = corrected_p_values_cell{statistic_name_index * 2 - 1:length(statistic_names) * 2:end};
    % corrected_condition_sensor_combination_pair_statistics.(statistic_names{statistic_name_index}) = corrected_p_values_cell{statistic_name_index * 2 - 0:length(statistic_names) * 2:end};
    number_condition_pair_structures = numel(corrected_condition_pair_sensor_statistics);
    number_combination_pair_structures = numel(corrected_condition_sensor_combination_pair_statistics);
    for test_index = 1:number_condition_pair_structures
      corrected_condition_pair_sensor_statistics(test_index).(statistic_names{statistic_name_index}) = corrected_p_values_cell{(statistic_name_index - 1) * (number_condition_pair_structures + number_combination_pair_structures) + test_index};
    end
    for test_index = 1:number_combination_pair_structures
      corrected_condition_sensor_combination_pair_statistics(test_index).(statistic_names{statistic_name_index}) = corrected_p_values_cell{(statistic_name_index - 1) * (number_condition_pair_structures + number_combination_pair_structures) + number_condition_pair_structures + test_index};
    end
  end
  
  fprintf_diary_cycle_function('\n\n')
  
  % error('Implementation yet unfinished below this line!')
  
  
  
  % Now save some figures or HTML tables with stats:
  
  should_load_models = false;
  % should_load_models = true;
  for model_type_index = 1:number_model_types
    model_type = model_types{model_type_index}
    fprintf_diary_cycle_function('%s: here 000\n', mfilename)
    statistic_computation_functions = model_type_statistic_computation_functions{model_type_index};
    fprintf_diary_cycle_function('%s: here 001\n', mfilename)
    test_names = fieldnames(statistic_computation_functions);
    fprintf_diary_cycle_function('%s: here 002\n', mfilename)
    
    for test_index = 1:length(test_names)
      % % For each test, print statistics between combinations and then between conditions for each sensor:
      % For each test, print statistics between combinations:
      current_test_name = test_names{test_index};
      fprintf_diary_cycle_function('%s: here 003\n', mfilename)
      current_test_results = nan(number_condition_sensors, number_condition_sensors, number_all_relative_times);
      fprintf_diary_cycle_function('%s: here 004\n', mfilename)
      
      for condition_sensor_combination_index1 = 1:number_condition_sensors
        current_condition1 = condition_sensor_combinations{condition_sensor_combination_index1, 1};
        fprintf_diary_cycle_function('%s: here 005\n', mfilename)
        current_sensor1 = condition_sensor_combinations{condition_sensor_combination_index1, 2};
        fprintf_diary_cycle_function('%s: here 006\n', mfilename)
        condition_index1 = find(strcmp(current_condition1, conditions));
        sensor_index1 = find(strcmp(current_sensor1, sensors));
        fprintf_diary_cycle_function('%s: here 007\n', mfilename)
        
        if should_load_models
          % Get the model:
          fprintf_diary_cycle_function('%s: here 008\n', mfilename)
          current_model1_filename = sprintf('%s%s_%s_reltime% 3d_%s', master_script_options.model_prefix, current_condition1, current_sensor1, relative_time, model_type);
          current_model1_full_filename = [models_location, current_model1_filename, '.mat'];
          if ~exist(current_model1_full_filename, 'file') || exist([current_model1_full_filename(1:end-3), '.tmp'], 'file')
            continue
          end
          fprintf_diary_cycle_function('%s: here 009\n', mfilename)
          current_model1 = load(current_model1_full_filename);
          fprintf_diary_cycle_function('%s: here 010\n', mfilename)
          current_model1_reduced_data = reduction_representation_function(current_model1.current_data);
          fprintf_diary_cycle_function('%s: here 011\n', mfilename)
        end
        
        for condition_sensor_combination_index2 = condition_sensor_combination_index1 + 1:number_condition_sensors
          fprintf_diary_cycle_function('%s: here 012\n', mfilename)
          current_condition2 = condition_sensor_combinations{condition_sensor_combination_index2, 1};
          current_sensor2 = condition_sensor_combinations{condition_sensor_combination_index2, 2};
          condition_index2 = find(strcmp(current_condition2, conditions));
          sensor_index2 = find(strcmp(current_sensor2, sensors));
          
          % Number of combinations without replacement:
          fprintf_diary_cycle_function('%s: here 013\n', mfilename)
          condition_pair_index = sum(number_conditions - 1:-1:number_conditions - condition_index1 + 1) + condition_index2 - condition_index1;
          combination_pair_index = sum(number_condition_sensors - 1:-1:number_condition_sensors - condition_sensor_combination_index1 + 1) + condition_sensor_combination_index2 - condition_sensor_combination_index1;
              
          if should_load_models
            % Get the model:
            fprintf_diary_cycle_function('%s: here 014\n', mfilename)
            current_model2_filename = sprintf('%s%s_%s_reltime% 3d_%s', master_script_options.model_prefix, current_condition2, current_sensor2, relative_time, model_type);
            current_model2_full_filename = [models_location, current_model2_filename, '.mat'];
            if ~exist(current_model2_full_filename, 'file') || exist([current_model2_full_filename(1:end-3), '.tmp'], 'file')
              continue
            end
            
            fprintf_diary_cycle_function('%s: here 015\n', mfilename)
            current_model2 = load(current_model2_full_filename);
            fprintf_diary_cycle_function('%s: here 016\n', mfilename)
            current_model2_reduced_data = reduction_representation_function(current_model2.current_data);
            fprintf_diary_cycle_function('%s: here 017\n', mfilename)
          end
          
          current_pair_test_results = {corrected_condition_sensor_combination_pair_statistics(combination_pair_index, :, model_type_index).(current_test_name)};
          current_pair_test_results = cellfun(@(x)[x, nan(1 - length(x))], current_pair_test_results);
          current_test_results(condition_sensor_combination_index1, condition_sensor_combination_index2, :) = current_pair_test_results;
          current_test_results(condition_sensor_combination_index2, condition_sensor_combination_index1, :) = current_pair_test_results;
          fprintf_diary_cycle_function('%s: here 018\n', mfilename)
          
          % condition_pair_sensor_statistics{pair_index, sensor_index, all_relative_time_index, model_type_index} = struct;
        end
      end
      
      current_test_results = reshape_2d(permute(current_test_results, [1, 3, 2]));
      % imagesc(reshape_2d(current_test_results)), axis image, colormap(illustration_colormap_to_use), colorbar, pause
      current_test_results_significant = current_test_results < (1 - confidence_level);
      current_test_result_negative_log_p_values = -log10(current_test_results);
      current_test_result_negative_log_p_values(~current_test_results_significant) = nan;
      fprintf_diary_cycle_function('%s: here 019\n', mfilename)
      
      % Display:
      close all
      
      set(gcf, 'Visible', 'off')
      set(gcf, 'Visible', 'on')
      set(gcf, 'Renderer', 'opengl');
      figure_position = get(gcf, 'Position');
      % aspect_ratio_influence = 0.5;
      aspect_ratio_influence = 0.6;
      % aspect_ratio_influence = 0.75;
      % aspect_ratio_influence = 0.9;
      aspect_ratio = 4 / 3 * (1 - aspect_ratio_influence) + (size(current_test_results, 2) / size(current_test_results, 1)) * aspect_ratio_influence;
      % figure_height_scale = 1;
      % figure_height_scale = 2;
      % figure_height_scale = 3;
      figure_height_scale = 2.6 + number_condition_sensors * .4;
      % figure_height_scale = 3.5;
      % figure_height_scale = 5;
      % figure_height_scale = 10;
      dpi = master_script_options.figure_dpi;
      figure_height = round(master_script_options.figure_maximum_width / aspect_ratio * figure_height_scale);
      % figure_width = round(figure_height * aspect_ratio);
      % figure_width = round(master_script_options.figure_maximum_width / aspect_ratio * figure_height_scale);
      figure_width = round(master_script_options.figure_maximum_width * figure_height_scale);
      set(gcf, 'Position', [1, 1, figure_width, figure_height])
      
      fprintf_diary_cycle_function('%s: here 020\n', mfilename)
      matrix_plot_handle = gca;
      % error('Make this use imagesc w/3-channel imgs, setting invalid values to white iso using alpha...')
      imagesc(current_test_result_negative_log_p_values);
      % imagesc(current_test_result_negative_log_p_values, 'AlphaData', current_test_results_significant * 1.);
      alpha(double(current_test_results_significant))
      % hold on
      % [current_test_results_significant_y, current_test_results_significant_x] = find(current_test_results_significant);
      % significance_scatter_handle = scatter(current_test_results_significant_x(:), current_test_results_significant_y(:), 'k*');
      % hold off
      % legend(significance_scatter_handle, 'Corrected p-value < 0.05')
      % set(matrix_plot_handle, 'YDir', 'normal')
      image_pixel_x_coordinates = [0, number_condition_sensors * number_all_relative_times - 1] + 1/2 + 1 / (2 * 1);
      combination_names_abbreviated = condition_name_abbreviation_function(arrayfun(@(x)[condition_sensor_combinations{x, 1}, ' - ', condition_sensor_combinations{x, 2}], 1:number_condition_sensors, 'UniformOutput', false).');
      % x_tick_values = contrast_stretch(1:number_condition_sensors) .* diff(image_pixel_x_coordinates) + image_pixel_x_coordinates(1);
      % x_tick_values = contrast_stretch(1:number_condition_sensors * number_all_relative_times) .* diff(image_pixel_x_coordinates) + image_pixel_x_coordinates(1);
      x_tick_values = 1:number_condition_sensors * number_all_relative_times;
      % x_tick_labels = arrayfun(@(x)[condition_sensor_combinations{x, 1}, ' - ', condition_sensor_combinations{x, 2}], 1:number_condition_sensors, 'UniformOutput', false);
      x_tick_labels = combination_names_abbreviated;
      x_tick_labels = cellfun(@(x)strcat([[x, ' ']; cell(number_all_relative_times - 1, 1)], arrayfun(@(y)sprintf('% 3d', y), all_relative_times_seconds_rounded, 'UniformOutput', false)), x_tick_labels, 'UniformOutput', false);
      x_tick_labels = cellfun(@(x)x.', x_tick_labels, 'UniformOutput', false).';
      x_tick_labels = [x_tick_labels{:}].';
      set(matrix_plot_handle, 'XTick', x_tick_values, 'XTickLabel', x_tick_labels)
      y_tick_values = 1:number_condition_sensors;
      y_tick_labels = combination_names_abbreviated;
      set(matrix_plot_handle, 'YTick', y_tick_values, 'YTickLabel', y_tick_labels)
      set(matrix_plot_handle, 'TickDir', 'out')
      axis image
      colormap(illustration_colormap_to_use)
      % colorbar
      % pause
      % drawnow
      colorbar_axis = colorbar;
      % ylabel(colorbar_axis, 'Negative log p-values')
      ylabel(colorbar_axis, {'Negative log10 of', 'significant p-values'}, 'Interpreter', 'none')
      
      % rotateXLabels(matrix_plot_handle, 30)
      % % rotateXLabels(matrix_plot_handle, 45)
      % % rotateXLabels(matrix_plot_handle, 60)
      % XYrotalabel(30, 0, matrix_plot_handle);
      % xticklabel_rotate([], 30)
      % xticklabel_rotate([], 35)
      % xticklabel_rotate([], 40)
      xticklabel_rotate([], 45)
      
      % dbstack, beep, keyboard
      
      % xlabel('')
      % xruan 06/23/2015
      % change the position of y label
      ylabel_handle = ylabel('Model')
      set(ylabel_handle, 'position', get(ylabel_handle,'position')-[0.2,0,0])
      % xlabel('Model/seconds relative to synapse formation')
      xlabel('Model - seconds relative to synapse formation')
      
      % 01:11 2014-12-22: Error occurs after 021:
      fprintf_diary_cycle_function('%s: here 021\n', mfilename)
      
      set(gcf, 'Color', 'w')
      fprintf_diary_cycle_function('%s: here 021.1\n', mfilename)
      image_filename = sprintf('%sstatistical_comparison_model_%s_test_%s', master_script_options.model_prefix, model_type, current_test_name);
      fprintf_diary_cycle_function('%s: here 021.2\n', mfilename)
      image_full_filename = [figures_location, image_filename, '.png'];
      fprintf_diary_cycle_function('%s: here 021.3\n', mfilename)
      % export_fig(image_full_filename, '-zbuffer', '-png', '-a1', '-nocrop', ['-r', num2str(dpi)])
      fprintf_diary_cycle_function('%s: here 021.4\n', mfilename)
      image_full_filename
      if false
        beep, dbstack, keyboard
      end
      % export_fig(image_full_filename, '-zbuffer', '-png', '-a1', ['-r', num2str(dpi)])
      export_fig(image_full_filename, '-opengl', '-png', '-a1', ['-r', num2str(dpi)])
      
      fprintf_diary_cycle_function('%s: here 022\n', mfilename)
    end
  end
  
  
  
  if false
    
    error

    for condition_sensor_combination_index = 1:number_condition_sensors
      current_condition = condition_sensor_combinations{condition_sensor_combination_index, 1};
      current_sensor = condition_sensor_combinations{condition_sensor_combination_index, 2};
      
      for all_relative_time_index = 1:number_all_relative_times
        relative_time = all_relative_times(all_relative_time_index);
        
        current_count = condition_sensor_model_counts(condition_sensor_combination_index, all_relative_time_index);
        t_at_confidence_level = -tinv((1 - confidence_level) / 2, current_count - 1);
        
        for model_type_index = 1:number_model_types
          model_type = model_types{model_type_index};
          representation_function = model_representation_functions{model_type_index};
          reconstruction_function = model_reconstruction_functions{model_type_index};
          
          % Compute the mean and mean +/- confidence interval (assumes each voxel's intensity is independent and normally distributed?):
          
          current_mean = condition_sensor_model_template_means{condition_sensor_combination_index, all_relative_time_index, model_type_index};
          current_standard_deviation = condition_sensor_model_template_standard_deviations{condition_sensor_combination_index, all_relative_time_index, model_type_index};
          % current_standard_error = current_standard_deviation ./ sqrt(current_count);
          current_confidence_interval_lower_bound = current_mean - t_at_confidence_level .* current_standard_deviation ./ sqrt(current_count);
          current_confidence_interval_upper_bound = current_mean + t_at_confidence_level .* current_standard_deviation ./ sqrt(current_count);
          current_confidence_interval_range = t_at_confidence_level .* 2 .* current_standard_deviation ./ sqrt(current_count);
          % keyboard
          
          % Reconstruct images in the template:
          
          current_mean_reconstructed = reconstruction_function(current_mean);
          current_confidence_interval_lower_bound_reconstructed = reconstruction_function(current_confidence_interval_lower_bound);
          current_confidence_interval_upper_bound_reconstructed = reconstruction_function(current_confidence_interval_upper_bound);
          
          % error
          % keyboard
        end
      end
    end
  
  
  end
  
  
  
  
  % beep, keyboard
  % error('Implementation yet unfinished below this line!')
  
  % t_cell_info.figures_info = struct();
  % t_cell_info.figures_info.registration_options = registration_options;
  
  
  % error('Implementation yet unfinished below this line!')
  
  
end  

