function [t_cell_info] = master_script_count_cell_pairs(t_cell_info, options)
  % Fit parametric distributions to the protein intensities in the template space.
  %
  % 2015-08-23 xruan: Copied from master_script_morph_segmentations.m.
  % 2016-02-23 xruan: disable the print of current count and other
  % variables
  % 
  % 
  % 
  % 
  % % Dependencies:
  % % From the File Exchange: export_fig, pmkmp, toolbox_graph, toolbox_fast_marching, affine, Snake3D, Mesh_voxelisation, dirr, inhull
  % % From tebuck: modified version of Jieyue Li's HPA_lib, rasterize_mesh

%
% Author: Taraz E. Buck and Xiongtao Ruan
%
% Copyright (C) 2012-2016 Murphy Lab
% Computational Biology Department
% School of Computer Science
% Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu



  
  
  
  % Variables referenced more than once can be copied to local variables:
  master_script_options = t_cell_info.options;
  fprintf_diary_cycle_function = t_cell_info.log_info.fprintf_diary_cycle_function;
  regions_location = t_cell_info.path_info.regions_location;
  segmentations_location = t_cell_info.path_info.segmentations_location;
  segmentations_filtered_location = t_cell_info.path_info.segmentations_filtered_location;
  alignments_location = t_cell_info.path_info.alignments_location;
  deformations_location = t_cell_info.path_info.deformations_location;
  models_location = t_cell_info.path_info.models_location;
  
  run_relative_path_list = t_cell_info.synapse_info.run_relative_path_list;
  run_key_list = t_cell_info.synapse_info.run_key_list;
  % run_data_list = t_cell_info.synapse_info.run_data_list;
  run_images_list = t_cell_info.synapse_info.run_images_list;
  run_brightfield_images_list = t_cell_info.synapse_info.run_brightfield_images_list;
  run_image_frame_indices_list = t_cell_info.synapse_info.run_image_frame_indices_list;
  run_number_cells = t_cell_info.synapse_info.run_number_cells;
  run_number_frames = t_cell_info.synapse_info.run_number_frames;
  run_point_tracks = t_cell_info.synapse_info.run_point_tracks;
  run_relative_time_tracks = t_cell_info.synapse_info.run_relative_time_tracks;
  run_relative_times = t_cell_info.synapse_info.run_relative_times;
  run_number_relative_times = t_cell_info.synapse_info.run_number_relative_times;
  all_relative_times = t_cell_info.synapse_info.all_relative_times;
  number_all_relative_times = t_cell_info.synapse_info.number_all_relative_times;
  all_relative_times_seconds_data = t_cell_info.synapse_info.all_relative_times_seconds_data;
  all_relative_times_seconds = t_cell_info.synapse_info.all_relative_times_seconds;
  number_runs = t_cell_info.synapse_info.number_runs;
  
  condition_sensor_combinations = t_cell_info.synapse_info.condition_sensor_combinations;
  number_condition_sensors = t_cell_info.synapse_info.number_condition_sensors;
  conditions = t_cell_info.synapse_info.conditions;
  sensors = t_cell_info.synapse_info.sensors;
  number_conditions = t_cell_info.synapse_info.number_conditions;
  number_sensors = t_cell_info.synapse_info.number_sensors;
  condition_sensor_combination_ismember = t_cell_info.synapse_info.condition_sensor_combination_ismember;
  
  model_types = t_cell_info.model_type_info.model_types;
  number_model_types = t_cell_info.model_type_info.number_model_types;
  model_representation_functions = t_cell_info.model_type_info.model_representation_functions;
  model_reconstruction_functions = t_cell_info.model_type_info.model_reconstruction_functions;
  model_dimensionality_reduction_methods = t_cell_info.model_type_info.model_dimensionality_reduction_methods;
  model_dimensionality_reduction_method_parameters = t_cell_info.model_type_info.model_dimensionality_reduction_method_parameters;
  model_dimensionality_reduction_method_transformations = t_cell_info.model_type_info.model_dimensionality_reduction_method_transformations;
  
  template_centroid = t_cell_info.template_info.template_centroid;
  template_synapse = t_cell_info.template_info.template_synapse;
  template_image = t_cell_info.template_info.template_image;

  window_size_2d = t_cell_info.preprocessing_info.window_size_2d;
  window_center_2d = t_cell_info.preprocessing_info.window_center_2d;
  cropped_size = t_cell_info.preprocessing_info.cropped_size;
  background_level_function = t_cell_info.preprocessing_info.background_level_function;

  segmentation_rasterization_function = t_cell_info.segmentation_info.segmentation_rasterization_function;
  
  template_volume = t_cell_info.alignment_info.template_volume;
  segmentation_volume_threshold = t_cell_info.alignment_info.segmentation_volume_threshold;
  
  debug_print_model_filenames = true
  
  
  condition_sensor_combinations, number_condition_sensors, conditions, sensors, number_conditions, number_sensors
  model_types, number_model_types
  number_all_relative_times
   
    warning('Need to ensure all above computations are finished somehow before building models... Maybe block on every single good deformation''s file?')
    warning('Nevermind, can always rebuild!')

    cell_pair_count_cell = {};
    
    for condition_sensor_combination_index = 1:number_condition_sensors
      current_condition = condition_sensor_combinations{condition_sensor_combination_index, 1};
      current_sensor = condition_sensor_combinations{condition_sensor_combination_index, 2};
      current_count = {};
      current_count_cell{1} = current_condition;
      current_count_cell{2} = current_sensor;
      
      fprintf('condition_sensor_combination_index %d, current_condition "%s", current_condition "%s"\n', condition_sensor_combination_index, current_condition, current_sensor)

      % Find relevant runs:
      % Should this just compare .mat files for speed?:
      run_list_relevant = true(size(run_key_list));
      % relevant_pattern = ['^[^-]+ - ', current_condition, '_', current_sensor, '\.xls'];
      relevant_pattern = ['^[^-]+ - ', current_condition, '_', current_sensor, '_'];
      run_list_relevant = run_list_relevant & ~cellfun(@isempty, regexp(run_key_list, relevant_pattern, 'start'));
      relevant_runs = run_key_list(run_list_relevant);
      relevant_run_indices = reshape(find(run_list_relevant), 1, []);      
  
      current_count = 0;

      for run_index = relevant_run_indices
        % Prepare useful variables:
        run_relative_path = run_relative_path_list{run_index};
        run_key = run_key_list{run_index};
        run_image_path = run_images_list{run_index};
        number_cells = run_number_cells(run_index);
        number_frames = run_number_frames(run_index);
        cell_synapse_point_tracks = run_point_tracks{run_index};
        cell_relative_time_tracks = run_relative_time_tracks{run_index};

        cell_relative_time_matrix = cellfun(@(x)[x, nan(1, 1 - length(x))], cell_relative_time_tracks);

        cell_relative_time_matrix;

        for cell_index = 1: size(cell_relative_time_matrix, 1) 

          frame_indexes = find(~isnan(cell_relative_time_matrix(cell_index, :)));
          use_the_cell = 0;
          for frame_index_ind = 1 : length(frame_indexes)
              frame_index = frame_indexes(frame_index_ind);
              current_synapse_center_rounded = cell_synapse_point_tracks{cell_index, frame_index};
              if isempty(current_synapse_center_rounded)
                continue
              end
              current_synapse_center_rounded = current_synapse_center_rounded(1, 1:2);
              cell_deformation_filename = [run_key, sprintf('_frame%05d_synapse%05d,%05d', frame_index, current_synapse_center_rounded)];

              % Eventually, this should wait for all .mat files, and master_script_morph_segmentations.m should save .mat files with empty results if failed instead of leaving .tmp files. This way, running multiple jobs, others will quit when chunk_start here fails but the job that computes will get everything in the model files.

              if ~exist([deformations_location, cell_deformation_filename, '.mat'], 'file')
                if false && exist([segmentations_location, cell_deformation_filename, '.mat'], 'file')
                  fprintf('   "%s" does not exist, but segmentation does!\n', [deformations_location, cell_deformation_filename, '.mat'])
                  % beep, keyboard
                end
                if false && exist([segmentations_filtered_location, cell_deformation_filename, '.mat'], 'file')
                  fprintf('   "%s" does not exist, but filtered segmentation does!\n', [deformations_location, cell_deformation_filename, '.mat'])
                  % beep, keyboard
                end
                if false && all_relative_time_index == 1 && model_type_index == 1
                  % Debug info:
                  fprintf('   "%s" does not exist!\n', [deformations_location, cell_deformation_filename, '.mat'])
                  % run_existing_files = ls([deformations_location, run_key, '*.mat'])
                  % run_existing_files = arrayfun(@(x)x.name, dir([deformations_location, run_key, '*.mat']), 'UniformOutput', false)
                  % beep, keyboard
                  % pause
                end
                continue
              end

              % fprintf('   "%s" exists!\n', [deformations_location, cell_deformation_filename, '.mat'])

              a = load([regions_location, cell_deformation_filename, '.mat']);
              if isempty(fieldnames(a))
                continue
              end
              current_window_image = a.current_window_image;

              a = load([deformations_location, cell_deformation_filename, '.mat']);
              if isempty(fieldnames(a))
                continue
              end
              cropped_raw_image_deformed = a.cropped_raw_image_deformed;
              cropped_segmentation_image_deformed = a.cropped_segmentation_image_deformed;

              current_volume = sum(cropped_segmentation_image_deformed(:));
              is_segmentation_low_volume = current_volume <= segmentation_volume_threshold;

              % error('Correct if stmt')
              if is_segmentation_low_volume
                continue
              end

              current_deformed_image = cropped_raw_image_deformed - background_level_function(current_window_image);
              current_deformed_image(current_deformed_image < 0) = 0;
              current_deformed_image(template_image < .5) = 0;
              current_deformed_image = current_deformed_image ./ sum(current_deformed_image(:));


              if sum(current_deformed_image(:)) > 0
              % Batch model:
                use_the_cell = 1;
                break;
              end

         end
         if use_the_cell > 0
               current_count = current_count + 1;
         end

         % fprintf('   current_count %d\n', current_count)

        end 
     end
      current_count_cell{3} = current_count;
      cell_pair_count_cell{end + 1} = current_count_cell;
    end
    
    cell_pair_count_cell;
    csv_field_matrix = cell_pair_count_cell';
    fid = fopen([models_location, 'model_cell_couple_count.csv'], 'wt');
    for i = 1:size(csv_field_matrix, 1)
        csv_field_matrix_i = csv_field_matrix{i};
        for j = 1:size(csv_field_matrix_i, 2)
            if j > 1
                fprintf(fid, ',');
            end
            if ischar(csv_field_matrix{i}{j})
              fwrite(fid, csv_field_matrix{i}{j});
            else
              fprintf(fid, '%f', csv_field_matrix{i}{j});
            end        
        end
        fprintf(fid, '\n');
    end
    fclose(fid);
    save([models_location, 'model_cell_couple_count_cell.mat'], 'cell_pair_count_cell');

end



  