function [t_cell_info] = master_script_generate_fwd_cyto_radial_raw_plots(t_cell_info, options)
  % Produce spatiotemporal models of three sensors ( cofilin, MRLC, and
  % WAVE2) 
  % for figure S5 in the paper.
  % and standard deviation maps
  % all maps for all sensors, conditions and time points
  % 
  % xruan 07/29/2015 copied from master_script_generate_model_figures.m
  % 
  % 
  % 
  % % Dependencies:
  % % From the File Exchange: export_fig, pmkmp, toolbox_graph, toolbox_fast_marching, affine, Snake3D, Mesh_voxelisation, dirr, inhull
  % % From tebuck: modified version of Jieyue Li's HPA_lib, rasterize_mesh

%
% Author: Taraz E. Buck and Xiongtao Ruan
%
% Copyright (C) 2012-2016 Murphy Lab
% Computational Biology Department
% School of Computer Science
% Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu



  
  
  
  % Variables referenced more than once can be copied to local variables:
  
  debug = ~true
  if debug
    addpath(genpath(pwd));
  end
  
  
  master_script_options = t_cell_info.options;
  regions_location = t_cell_info.path_info.regions_location;
  segmentations_filtered_location = t_cell_info.path_info.segmentations_filtered_location;
  alignments_location = t_cell_info.path_info.alignments_location;
  deformations_location = t_cell_info.path_info.deformations_location;
  models_location = t_cell_info.path_info.models_location;
  figures_location = t_cell_info.path_info.figures_location;
  videos_location = t_cell_info.path_info.videos_location;
  
 fwd_cyto_radial_raw_directory = 'fwd_cyto_radial_raw_plots/'
  
  if master_script_options.use_two_point_synapses
      fwd_cyto_radial_raw_directory = ['two-point_', fwd_cyto_radial_raw_directory];
  end
  
  fwd_cyto_radial_raw_directory = [t_cell_info.path_info.figures_location, fwd_cyto_radial_raw_directory];
  
  if ~exist(fwd_cyto_radial_raw_directory, 'dir')
      mkdir(fwd_cyto_radial_raw_directory);
  end  
    
  figureS5_directory = 'figureS5/';
  
  figureS5_location = [t_cell_info.path_info.figures_for_paper_location, figureS5_directory];
  
  if ~exist(figureS5_location, 'dir')
      mkdir(figureS5_location);
       t_cell_info.path_info.figureS5_location = figureS5_location;
  end
  
  illustration_colormap_to_use = t_cell_info.plotting_info.illustration_colormap_to_use;
  
  illustration_colormap_to_use = pmkmp(1024, 'CubicL');
   
  condition_name_abbreviation_function = t_cell_info.abbreviation_info.condition_name_abbreviation_function;
  
  model_types = t_cell_info.model_type_info.model_types;
  number_model_types = t_cell_info.model_type_info.number_model_types;
  model_representation_functions = t_cell_info.model_type_info.model_representation_functions;
  model_reconstruction_functions = t_cell_info.model_type_info.model_reconstruction_functions;
  model_dimensionality_reduction_methods = t_cell_info.model_type_info.model_dimensionality_reduction_methods;
  model_dimensionality_reduction_method_parameters = t_cell_info.model_type_info.model_dimensionality_reduction_method_parameters;
  
  run_relative_path_list = t_cell_info.synapse_info.run_relative_path_list;
  run_key_list = t_cell_info.synapse_info.run_key_list;
  % run_data_list = t_cell_info.synapse_info.run_data_list;
  run_images_list = t_cell_info.synapse_info.run_images_list;
  run_brightfield_images_list = t_cell_info.synapse_info.run_brightfield_images_list;
  run_image_frame_indices_list = t_cell_info.synapse_info.run_image_frame_indices_list;
  run_number_cells = t_cell_info.synapse_info.run_number_cells;
  run_number_frames = t_cell_info.synapse_info.run_number_frames;
  run_point_tracks = t_cell_info.synapse_info.run_point_tracks;
  run_relative_time_tracks = t_cell_info.synapse_info.run_relative_time_tracks;
  run_relative_times = t_cell_info.synapse_info.run_relative_times;
  run_number_relative_times = t_cell_info.synapse_info.run_number_relative_times;
  all_relative_times = t_cell_info.synapse_info.all_relative_times;
  number_all_relative_times = t_cell_info.synapse_info.number_all_relative_times;
  all_relative_times_seconds_data = t_cell_info.synapse_info.all_relative_times_seconds_data;
  all_relative_times_seconds = t_cell_info.synapse_info.all_relative_times_seconds;
  all_relative_times_seconds_rounded = t_cell_info.synapse_info.all_relative_times_seconds_rounded;
  number_runs = t_cell_info.synapse_info.number_runs;
  condition_sensor_combinations = t_cell_info.synapse_info.condition_sensor_combinations;
  number_condition_sensors = t_cell_info.synapse_info.number_condition_sensors;
  conditions = t_cell_info.synapse_info.conditions;
  sensors = t_cell_info.synapse_info.sensors;
  number_conditions = t_cell_info.synapse_info.number_conditions;
  number_sensors = t_cell_info.synapse_info.number_sensors;
  unique_condition_sensor_combination_function = t_cell_info.synapse_info.unique_condition_sensor_combination_function;
  unique_condition_sensor_combination_pair_function = t_cell_info.synapse_info.unique_condition_sensor_combination_pair_function;
  condition_sensor_combination_ismember = t_cell_info.synapse_info.condition_sensor_combination_ismember;
  condition_sensor_combination_pair_ismember = t_cell_info.synapse_info.condition_sensor_combination_pair_ismember;
  
  template_centroid = t_cell_info.template_info.template_centroid;
  template_synapse = t_cell_info.template_info.template_synapse;
  template_image = t_cell_info.template_info.template_image;
  template_crop_function = t_cell_info.template_info.template_crop_function;

  window_size_2d = t_cell_info.preprocessing_info.window_size_2d;
  window_center_2d = t_cell_info.preprocessing_info.window_center_2d;
  cropped_size = t_cell_info.preprocessing_info.cropped_size;
 
  confidence_level = master_script_options.confidence_level;
  
  
  should_compute_individual_frame_illustrations = false;
  % should_compute_individual_frame_illustrations = true;
  
  should_skip_completed_images = false;
  % should_skip_completed_images = true;
  
  % warning('>>>> HACK, performing statistical tests of enrichment under master_script_options.skip_model_figure_generation!')
  
  current_mean_intensity_limit_excluded_size = 0;
  
  current_standard_deviation_intensity_limit_excluded_size = 0;
  
  % current_mean_intensity_limit_excluded_size = 1e-3;
  % current_mean_intensity_limit_excluded_size = 5e-3;
  intensity_limit_function = @(given_image, given_limits)given_image .* (given_image < given_limits(2)) .* (given_image > given_limits(1)) + given_limits(1) .* (given_image <= given_limits(1)) + given_limits(2) .* (given_image >= given_limits(2));
  
  if master_script_options.illustration_uses_histogram_equalization
    % histogram_equalization_function = @(given_image)masked_histogram_equalization(given_image, template_image > .5);
    histogram_equalization_function = @(given_image, given_limits)masked_histogram_equalization(given_image, (given_image > given_limits(1)) & (given_image < given_limits(2)));
  else
    histogram_equalization_function = @(given_image, given_limits)given_image;
  end
  
  if master_script_options.illustration_uses_logarithmic_scale
    intensity_warping_function = @(given_image, given_limits)log(given_image);
  else
    intensity_warping_function = @(given_image, given_limits)given_image;
  end

  

  % Plot formatting:
  use_fixed_safe_colors = false;
  % use_fixed_safe_colors = true;
  if use_fixed_safe_colors
    % http://safecolours.rigdenage.com/colourchoice.html#examples
    safe_colors = [.4, 1, .2; 1., .6, .2; 1., .2, .2; .2, .4, 0; .8, .6, .6; .8, .8, .8; 0, 0, 0; 0, .8, 1; .6, 0, .8; 0, .6, .6];
    Attempted modification, not good:
    % safe_colors = [.4, .8, .4; 1., .6, .2; 1., .2, .2; .2, .4, 0; .8, .6, .6; .7, .7, .7; 0, 0, 0; 0, .8, 1; .6, 0, .8; 0, .6, .6];
  else
    safe_colors = distinguishable_colors(number_sensors, {'w','k'});
  end
  % This should eventually cycle through combinations of markers and colors or plot things separately... Could also use filled vs. not markers.
  % marker_styles = '.o+xsd^v<>ph';
  % marker_styles = '.o+xsd^vph';
  marker_styles = 'o.x*sd^vph';
  % line_styles = {'-', '--'};
  % % Hack because B7 sorts before Full:
  % % line_styles = {'--', '-'};
  % line_styles = {':', '-'};
  % Now have three conditions:
  line_styles = {':', '--', '-'};
  

  
  
  % if strcmp(model_type, 'standardized_voxels')
  combination_enrichment_tests = [];
  % combination_enrichment_tests = struct(zeros(0, 1));
  % end
  
  for model_type_index = 1:number_model_types
    model_type = model_types{model_type_index};
    
    if ~strcmp(model_type, 'fwd_cyto_radial_raw')
        continue;
    end
    
   
    model_type
    
%     model_dimensionality_reduction_full_filename = [models_location, sprintf('%s%s_%s', master_script_options.model_prefix, model_type, dimensionality_reduction_method), '.mat'];
    
%     if ~exist(model_dimensionality_reduction_full_filename, 'file')
%       continue
%     end
    
    
 
    % Collect mean models:
    all_means= cell(number_condition_sensors, number_all_relative_times);
    all_means_cropped_nonempty = false(number_condition_sensors, number_all_relative_times);
    
    % collect standard deviration of models
    all_standard_deviation= cell(number_condition_sensors, number_all_relative_times);
    all_stds_cropped_nonempty = false(number_condition_sensors, number_all_relative_times);

    
    should_interpolate_empty_panels = false;
    % should_interpolate_empty_panels = true;
    
    for condition_sensor_combination_index = 1:number_condition_sensors
      current_condition = condition_sensor_combinations{condition_sensor_combination_index, 1};
      current_sensor = condition_sensor_combinations{condition_sensor_combination_index, 2};
      

      for all_relative_time_index = 1:number_all_relative_times
        relative_time = all_relative_times(all_relative_time_index);

        % Read all files sequentially:
        current_model_filename = [models_location, sprintf('%s%s_%s_reltime% 3d_%s.mat', master_script_options.model_prefix, current_condition, current_sensor, relative_time, model_type)];
        % current_model_filename = [models_location, sprintf('%s_%s_frame%03d_reltime% 3d_%s.mat', current_condition, current_sensor, all_relative_time_index, relative_time, model_type)];
        % current_model_filename
        if ~exist(current_model_filename, 'file')
          continue
        end
        % current_mean = [];
        % current_standard_deviation = [];
        % current_covariance = [];
        current_data = [];
        current_data_relevant_run_keys = [];
        current_data_relevant_run_key_indices = [];
        load(current_model_filename)
        % keyboard
        
        % current_data = reduction_representation_function(current_data);
                
        % Make model illustrations for this sensor:

        current_count = size(current_data, 1);
        
        % Used to be < 2:
        if current_count < 1
          continue
        end
        
        title_string = {sprintf('Condition: %s, sensor: %s, relative time: %d, number cells: %d', current_condition, current_sensor, relative_time, current_count), sprintf('model type: %s', model_type)};
        image_filename = sprintf('%s%s_%s_frame%03d_reltime% 3d_%s', master_script_options.model_prefix, condition_name_abbreviation_function(current_condition), current_sensor, all_relative_time_index, relative_time, model_type);
        
        image_full_filename = [figures_location, image_filename, '.png'];
        
        if should_skip_completed_images && exist(image_full_filename, 'file')
          continue
        end
        

        current_mean = mean(current_data, 1);
        current_standard_deviation = std(current_data, [], 1);
        all_means{condition_sensor_combination_index, all_relative_time_index} = current_mean;
        all_standard_deviation{condition_sensor_combination_index, all_relative_time_index} = current_standard_deviation;
      end   

  
    end
    
      %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      %% start here
      
      all_means
      all_standard_deviation
      condition_sensor_combinations    
  end
  legends_cell = mat2cell(all_relative_times_seconds_rounded, ones(12, 1), 1);
  % legends_cell = mat2cell(all_relative_times_seconds_rounded, 1, ones(1, 12));
  legends =  cellfun(@(x) num2str(x), legends_cell, 'UniformOutput', false);
  colors = illustration_colormap_to_use( 1 : 92 : 1024, 1 : 3);
  for i = 1 : condition_sensor_combination_index
      close all;
      figure,
      figure_width = 700;
      figure_height = 600;
      set(gcf, 'Position', [1, 1, figure_width, figure_height])
      set(groot,'defaultAxesColorOrder',[1 0 0; 0 1 0; 0 0 1; 0, 0, 0; 1, 0, 1; 0, 1, 1],...
      'defaultAxesLineStyleOrder','-|--')
      for j = 1 : all_relative_time_index
          
          h = plot(all_means{i, j});
          %set(h, 'Color', colors(j, :))
          hold on

      end
      legend(legends, 'Location', 'South');
      xlabel('Relative distance');
      ylabel('Probability density');
      title(sprintf('%s  %s', condition_sensor_combinations{i, 2}, condition_sensor_combinations{i, 1}))
      hold off
      dpi = 300;
      if master_script_options.use_two_point_synapses
        figure_name = sprintf('two-point_%s_%s_fwd_cyto_radial_raw', condition_sensor_combinations{i, 1}, condition_sensor_combinations{i, 2});
      else
        figure_name = sprintf('one-point_%s_%s_fwd_cyto_radial_raw', condition_sensor_combinations{i, 1}, condition_sensor_combinations{i, 2});
      end
      export_fig([fwd_cyto_radial_raw_directory, figure_name], '-opengl', '-png', '-a1', ['-r', num2str(dpi)]);

end
