function [t_cell_info] = master_script_generate_model_figures(t_cell_info, options)
  % Produce images of constructed models for visual evaluation.
  %
  % 2013-11-24 tebuck: Copied from master_script_morph_segmentations.m.
  % 
  % 
  % 
  % 
  % 
  % % Dependencies:
  % % From the File Exchange: export_fig, pmkmp, toolbox_graph, toolbox_fast_marching, affine, Snake3D, Mesh_voxelisation, dirr, inhull
  % % From tebuck: modified version of Jieyue Li's HPA_lib, rasterize_mesh

%
% Author: Taraz E. Buck and Xiongtao Ruan
%
% Copyright (C) 2012-2016 Murphy Lab
% Computational Biology Department
% School of Computer Science
% Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu



  
  
  
  % Variables referenced more than once can be copied to local variables:
  master_script_options = t_cell_info.options;
  regions_location = t_cell_info.path_info.regions_location;
  segmentations_filtered_location = t_cell_info.path_info.segmentations_filtered_location;
  alignments_location = t_cell_info.path_info.alignments_location;
  deformations_location = t_cell_info.path_info.deformations_location;
  models_location = t_cell_info.path_info.models_location;
  figures_location = t_cell_info.path_info.figures_location;
  videos_location = t_cell_info.path_info.videos_location;

  illustration_colormap_to_use = t_cell_info.plotting_info.illustration_colormap_to_use;
  
  condition_name_abbreviation_function = t_cell_info.abbreviation_info.condition_name_abbreviation_function;
  
  model_types = t_cell_info.model_type_info.model_types;
  number_model_types = t_cell_info.model_type_info.number_model_types;
  % model_representation_functions = t_cell_info.model_type_info.model_representation_functions;
  % model_reconstruction_functions = t_cell_info.model_type_info.model_reconstruction_functions;
  model_dimensionality_reduction_methods = t_cell_info.model_type_info.model_dimensionality_reduction_methods;
  model_dimensionality_reduction_method_parameters = t_cell_info.model_type_info.model_dimensionality_reduction_method_parameters;
  
  model_original_representation_functions = t_cell_info.model_info.model_original_representation_functions;
  model_original_reconstruction_functions = t_cell_info.model_info.model_original_reconstruction_functions;
  model_reduction_representation_functions = t_cell_info.model_info.model_reduction_representation_functions;
  model_reduction_reconstruction_functions = t_cell_info.model_info.model_reduction_reconstruction_functions;
  model_representation_functions = t_cell_info.model_info.model_representation_functions;
  model_reconstruction_functions = t_cell_info.model_info.model_reconstruction_functions;
  
  run_relative_path_list = t_cell_info.synapse_info.run_relative_path_list;
  run_key_list = t_cell_info.synapse_info.run_key_list;
  % run_data_list = t_cell_info.synapse_info.run_data_list;
  run_images_list = t_cell_info.synapse_info.run_images_list;
  run_brightfield_images_list = t_cell_info.synapse_info.run_brightfield_images_list;
  run_image_frame_indices_list = t_cell_info.synapse_info.run_image_frame_indices_list;
  run_number_cells = t_cell_info.synapse_info.run_number_cells;
  run_number_frames = t_cell_info.synapse_info.run_number_frames;
  run_point_tracks = t_cell_info.synapse_info.run_point_tracks;
  run_relative_time_tracks = t_cell_info.synapse_info.run_relative_time_tracks;
  run_relative_times = t_cell_info.synapse_info.run_relative_times;
  run_number_relative_times = t_cell_info.synapse_info.run_number_relative_times;
  all_relative_times = t_cell_info.synapse_info.all_relative_times;
  number_all_relative_times = t_cell_info.synapse_info.number_all_relative_times;
  all_relative_times_seconds_data = t_cell_info.synapse_info.all_relative_times_seconds_data;
  all_relative_times_seconds = t_cell_info.synapse_info.all_relative_times_seconds;
  all_relative_times_seconds_rounded = t_cell_info.synapse_info.all_relative_times_seconds_rounded;
  number_runs = t_cell_info.synapse_info.number_runs;
  condition_sensor_combinations = t_cell_info.synapse_info.condition_sensor_combinations;
  number_condition_sensors = t_cell_info.synapse_info.number_condition_sensors;
  conditions = t_cell_info.synapse_info.conditions;
  sensors = t_cell_info.synapse_info.sensors;
  number_conditions = t_cell_info.synapse_info.number_conditions;
  number_sensors = t_cell_info.synapse_info.number_sensors;
  unique_condition_sensor_combination_function = t_cell_info.synapse_info.unique_condition_sensor_combination_function;
  unique_condition_sensor_combination_pair_function = t_cell_info.synapse_info.unique_condition_sensor_combination_pair_function;
  condition_sensor_combination_ismember = t_cell_info.synapse_info.condition_sensor_combination_ismember;
  condition_sensor_combination_pair_ismember = t_cell_info.synapse_info.condition_sensor_combination_pair_ismember;
  
  template_centroid = t_cell_info.template_info.template_centroid;
  template_synapse = t_cell_info.template_info.template_synapse;
  template_image = t_cell_info.template_info.template_image;
  template_crop_function = t_cell_info.template_info.template_crop_function;

  window_size_2d = t_cell_info.preprocessing_info.window_size_2d;
  window_center_2d = t_cell_info.preprocessing_info.window_center_2d;
  cropped_size = t_cell_info.preprocessing_info.cropped_size;

  segmentation_rasterization_function = t_cell_info.segmentation_info.segmentation_rasterization_function;
  
  model_original_representation_functions = t_cell_info.model_info.model_original_representation_functions;
  model_original_reconstruction_functions = t_cell_info.model_info.model_original_reconstruction_functions;
  model_reduction_representation_functions = t_cell_info.model_info.model_reduction_representation_functions;
  model_reduction_reconstruction_functions = t_cell_info.model_info.model_reduction_reconstruction_functions;
  
  all_model_distances = t_cell_info.clustering_info.all_model_distances;
  all_model_trees = t_cell_info.clustering_info.all_model_trees;
  all_model_clusters = t_cell_info.clustering_info.all_model_clusters;
  clustering_distance_metrics = t_cell_info.clustering_info.clustering_distance_metrics;



  
  confidence_level = master_script_options.confidence_level;
  
  
  % warning('Hard-coding two major conditions!')
  % major_conditions = {'B7 Blockade', 'Full Stimulus'}.'
  
  
  should_compute_individual_frame_illustrations = false;
  % should_compute_individual_frame_illustrations = true;
  
  should_skip_completed_images = false;
  % should_skip_completed_images = true;
  
  % warning('>>>> HACK, performing statistical tests of enrichment under master_script_options.skip_model_figure_generation!')
  
  current_mean_intensity_limit_excluded_size = 0;
  % current_mean_intensity_limit_excluded_size = 1e-3;
  % current_mean_intensity_limit_excluded_size = 5e-3;
  intensity_limit_function = @(given_image, given_limits)given_image .* (given_image < given_limits(2)) .* (given_image > given_limits(1)) + given_limits(1) .* (given_image <= given_limits(1)) + given_limits(2) .* (given_image >= given_limits(2));
  
  if master_script_options.illustration_uses_histogram_equalization
    % histogram_equalization_function = @(given_image)masked_histogram_equalization(given_image, template_image > .5);
    histogram_equalization_function = @(given_image, given_limits)masked_histogram_equalization(given_image, (given_image > given_limits(1)) & (given_image < given_limits(2)));
  else
    histogram_equalization_function = @(given_image, given_limits)given_image;
  end
  
  if master_script_options.illustration_uses_logarithmic_scale
    intensity_warping_function = @(given_image, given_limits)log(given_image);
  else
    intensity_warping_function = @(given_image, given_limits)given_image;
  end

  

  % Plot formatting:
  use_fixed_safe_colors = false;
  % use_fixed_safe_colors = true;
  if use_fixed_safe_colors
    % http://safecolours.rigdenage.com/colourchoice.html#examples
    safe_colors = [.4, 1, .2; 1., .6, .2; 1., .2, .2; .2, .4, 0; .8, .6, .6; .8, .8, .8; 0, 0, 0; 0, .8, 1; .6, 0, .8; 0, .6, .6];
    Attempted modification, not good:
    % safe_colors = [.4, .8, .4; 1., .6, .2; 1., .2, .2; .2, .4, 0; .8, .6, .6; .7, .7, .7; 0, 0, 0; 0, .8, 1; .6, 0, .8; 0, .6, .6];
  else
    safe_colors = distinguishable_colors(number_sensors, {'w','k'});
  end
  % This should eventually cycle through combinations of markers and colors or plot things separately... Could also use filled vs. not markers.
  % marker_styles = '.o+xsd^v<>ph';
  % marker_styles = '.o+xsd^vph';
  marker_styles = 'o.x*sd^vph';
  % line_styles = {'-', '--'};
  % % Hack because B7 sorts before Full:
  % % line_styles = {'--', '-'};
  % line_styles = {':', '-'};
  % Now have three conditions:
  line_styles = {':', '--', '-'};
  
  
  % Enrichment computation parameters:
  % enrichment_region_percentiles = [98, 94, 90]
  enrichment_region_percentiles = [90]
  should_use_per_cell_max_enrichment = false
  % should_use_per_cell_max_enrichment = true
  % should_use_global_enrichment_region = false
  should_use_global_enrichment_region = true
  should_use_per_cell_enrichment_region = false
  % should_use_per_cell_enrichment_region = true
  
  % Create illustrations, compute enrichments, compute statistics:
  
  
  % if strcmp(model_type, 'standardized_voxels')
  combination_enrichment_tests = [];
  % combination_enrichment_tests = struct(zeros(0, 1));
  % end
  
  for model_type_index = 1:number_model_types
    model_type = model_types{model_type_index};
    representation_function = model_representation_functions{model_type_index};
    reconstruction_function = model_reconstruction_functions{model_type_index};
    original_representation_function = model_original_representation_functions{model_type_index};
    original_reconstruction_function = model_original_reconstruction_functions{model_type_index};
    reduction_representation_function = model_reduction_representation_functions{model_type_index};
    reduction_reconstruction_function = model_reduction_reconstruction_functions{model_type_index};
    dimensionality_reduction_method = model_dimensionality_reduction_methods{model_type_index};
    dimensionality_reduction_method_parameters = model_dimensionality_reduction_method_parameters{model_type_index};
    
    model_type
    
    model_dimensionality_reduction_full_filename = [models_location, sprintf('%s%s_%s', master_script_options.model_prefix, model_type, dimensionality_reduction_method), '.mat'];
    
    if ~exist(model_dimensionality_reduction_full_filename, 'file')
      continue
    end
    
    
    
    % Compute mean model for region enrichment computations further down:
  
    % model_mean = zeros(size(template_image));
    model_mean = [];
    model_count = 0;
    
    for condition_sensor_combination_index = 1:number_condition_sensors
      current_condition = condition_sensor_combinations{condition_sensor_combination_index, 1};
      current_sensor = condition_sensor_combinations{condition_sensor_combination_index, 2};
      
      all_means_cropped = cell(number_all_relative_times, 1);
      all_means_cropped_zy = cell(number_all_relative_times, 1);

      for all_relative_time_index = 1:number_all_relative_times
        relative_time = all_relative_times(all_relative_time_index);

        % Read all files sequentially:
        current_model_filename = [models_location, sprintf('%s%s_%s_reltime% 3d_%s.mat', master_script_options.model_prefix, current_condition, current_sensor, relative_time, model_type)];
        if ~exist(current_model_filename, 'file')
          continue
        end
        current_data = [];
        % current_data_relevant_run_keys = [];
        % current_data_relevant_run_key_indices = [];
        load(current_model_filename, 'current_data')
        
        current_data = reduction_representation_function(current_data);

        if isempty(model_mean)
          model_mean = zeros(size(current_data(1, :)));
        end
        for datum_index = 1:size(current_data, 1)
          model_mean = update_running_mean(model_mean, current_data(datum_index, :), model_count + 1);
          model_count = model_count + 1;
        end
        
      end
    end
    
    if isempty(model_mean)
      continue
    end
    
    % warning('Why doesn''t reconstruction_function get modified before this call???'), % model_mean_reconstructed = reconstruction_function(model_mean);
    model_mean_reconstructed = reconstruction_function(model_mean);
    % Take the top 2% of voxels from the overall mean image as the region of interest for any enrichment (looking at the data, assumes that all interesting enrichment is at the synapse and ):
    % enrichment_region_percentile = 98;
    % model_mean_enrichment_region = model_mean_reconstructed >= prctile(model_mean_reconstructed(template_image), enrichment_region_percentile);
    model_mean_enrichment_region_function = @(given_percentile)model_mean_reconstructed >= prctile(model_mean_reconstructed(template_image), given_percentile);
    model_mean_enrichment_regions = arrayfun(model_mean_enrichment_region_function, enrichment_region_percentiles, 'UniformOutput', false);
    if false
      imshow(reshape_contrast([model_mean_reconstructed; model_mean_reconstructed .* model_enrichment_region], -1))
      % pause
      whos model_mean model_mean_reconstructed, model_count
      keyboard
    end
    
    
    % Write the overall mean image and the above-threshold portion as an image:
    for enrichment_region_percentile_index = 1:length(enrichment_region_percentiles)
      enrichment_region_percentile = enrichment_region_percentiles(enrichment_region_percentile_index);
        model_mean_enrichment_region = model_mean_enrichment_regions{enrichment_region_percentile_index};
      image_to_show = [];
      % image_to_show = [contrast_stretch(model_mean_reconstructed); model_mean_enrichment_region];
      % image_to_show = [contrast_stretch(model_mean_reconstructed); contrast_stretch(model_mean_enrichment_region * 3 + template_image)];
      % image_to_show = [image_to_show; contrast_stretch(model_mean_reconstructed)];
      image_to_show = [image_to_show; contrast_stretch(template_crop_function(model_mean_reconstructed))];
      % image_to_show = [image_to_show; model_mean_enrichment_region_function(enrichment_region_percentile) * .675 + template_image * .2];
      % image_to_show = [image_to_show; model_mean_enrichment_region_function(enrichment_region_percentile) * .675 + template_image * .2];
      % image_to_show = [image_to_show; model_mean_enrichment_region * .675 + template_image * .2];
      image_to_show = [image_to_show; template_crop_function(model_mean_enrichment_region * .675 + template_image * .2)];
      % image_to_show = reshape_2d(image_to_show, -1);
      image_to_show = reshape_2d(image_to_show, 1);
      % Add color bar:
      image_to_show = [image_to_show, repmat(linspace(1, 0, size(image_to_show, 1)).', [1, size(template_crop_function(template_image), 2)])];
      % Recolor the image:
      image_to_show = ind2rgb(floor(contrast_stretch(image_to_show) .* (size(illustration_colormap_to_use, 1) - 1)) + 1, illustration_colormap_to_use);
      % Write:
      image_filename = [sprintf('%smodel_mean_enrichment_region_mdl-type_%s_enr-prct%d', master_script_options.model_prefix, model_type, enrichment_region_percentile)];
      image_full_filename = [figures_location, image_filename, '.png'];
      imwrite(image_to_show, image_full_filename)
      fprintf('Wrote figure %s.png\n', image_filename)
    end
    
    
    enrichment_types = {};
    enrichment_type_computation_functions = {};
    enrichment_type_mean_computation_functions = {};
    if should_use_per_cell_max_enrichment
      % Max / mean intensity:
      enrichment_types{end + 1} = 'max';
      enrichment_type_computation_functions{end + 1} = @(x)max(x(template_image >= .5)) ./ mean(x(template_image >= .5));
    end
    % for enrichment_region_percentile = enrichment_region_percentiles
    for enrichment_region_percentile_index = 1:length(enrichment_region_percentiles)
      enrichment_region_percentile = enrichment_region_percentiles(enrichment_region_percentile_index);
      model_mean_enrichment_region = model_mean_enrichment_regions{enrichment_region_percentile_index};
      if should_use_per_cell_enrichment_region
        % Mean intensity in per-cell enrichment region / mean intensity:
        enrichment_types{end + 1} = sprintf('celltop%03d', enrichment_region_percentile);
        enrichment_type_computation_functions{end + 1} = @(x)mean(x(x >= prctile(x(template_image >= .5), enrichment_region_percentile))) ./ mean(x(template_image >= .5));
      end
      if should_use_global_enrichment_region
        % Mean intensity in mean's enrichment region / mean intensity:
        enrichment_types{end + 1} = sprintf('meantop%03d', enrichment_region_percentile);
        % enrichment_type_computation_functions{end + 1} = @(x)mean(x(model_mean_enrichment_region_function(enrichment_region_percentile))) ./ mean(x(template_image >= .5));
        enrichment_type_computation_functions{end + 1} = @(x)mean(x(model_mean_enrichment_region)) ./ mean(x(template_image >= .5));
      end
    end
    
    number_enrichment_types = length(enrichment_types)
  
    all_enrichments = cell(number_condition_sensors, number_all_relative_times, number_enrichment_types);
    all_enrichment_counts = zeros(number_condition_sensors, number_all_relative_times, number_enrichment_types);
    % In linear space, not in log space:
    all_enrichment_means = zeros(number_condition_sensors, number_all_relative_times, number_enrichment_types);
    % In log space:
    all_log_enrichment_standard_deviations = zeros(number_condition_sensors, number_all_relative_times, number_enrichment_types);
    % In linear space, not in log space:
    all_enrichment_confidence_intervals = zeros(number_condition_sensors, number_all_relative_times, 2, number_enrichment_types);
    
    
    should_enrichments_be_larger_than_one = false
    % should_enrichments_be_larger_than_one = true
    
    number_tests_performed = 0;
    % % Just do for the standardized_voxels model:
    % % number_tests_expected = number_sensors * number_all_relative_times
    % number_tests_expected = number_sensors * number_all_relative_times * number_enrichment_types


    % Collect mean models:
    all_means_cropped = cell(number_condition_sensors, number_all_relative_times);
    all_means_cropped_zy = cell(number_condition_sensors, number_all_relative_times);
    all_means_cropped_nonempty = false(number_condition_sensors, number_all_relative_times);
    
    should_interpolate_empty_panels = false;
    % should_interpolate_empty_panels = true;
    
    for condition_sensor_combination_index = 1:number_condition_sensors
      current_condition = condition_sensor_combinations{condition_sensor_combination_index, 1};
      current_sensor = condition_sensor_combinations{condition_sensor_combination_index, 2};
      
      % all_means_cropped = cell(number_all_relative_times, 1);
      % all_means_cropped_zy = cell(number_all_relative_times, 1);

      for all_relative_time_index = 1:number_all_relative_times
        relative_time = all_relative_times(all_relative_time_index);

        % Read all files sequentially:
        current_model_filename = [models_location, sprintf('%s%s_%s_reltime% 3d_%s.mat', master_script_options.model_prefix, current_condition, current_sensor, relative_time, model_type)];
        % current_model_filename = [models_location, sprintf('%s_%s_frame%03d_reltime% 3d_%s.mat', current_condition, current_sensor, all_relative_time_index, relative_time, model_type)];
        % current_model_filename
        if ~exist(current_model_filename, 'file')
          continue
        end
        % current_mean = [];
        % current_standard_deviation = [];
        % current_covariance = [];
        current_data = [];
        current_data_relevant_run_keys = [];
        current_data_relevant_run_key_indices = [];
        load(current_model_filename)
        % keyboard
        
        current_data = reduction_representation_function(current_data);
        
        % Make model illustrations for this sensor:

        current_count = size(current_data, 1);
        
        % Used to be < 2:
        if current_count < 1
          continue
        end
        
        title_string = {sprintf('Condition: %s, sensor: %s, relative time: %d, number cells: %d', current_condition, current_sensor, relative_time, current_count), sprintf('model type: %s', model_type)};
        image_filename = sprintf('%s%s_%s_frame%03d_reltime% 3d_%s', master_script_options.model_prefix, condition_name_abbreviation_function(current_condition), current_sensor, all_relative_time_index, relative_time, model_type);
        
        image_full_filename = [figures_location, image_filename, '.png'];
        
        if should_skip_completed_images && exist(image_full_filename, 'file')
          continue
        end
        

        current_mean = mean(current_data, 1);
        current_standard_deviation = std(current_data, [], 1);
      
        % Reconstruct images in the template:
        
        current_mean_reconstructed = reconstruction_function(current_mean);

        % Produce the image for this condition-sensor combination:
        
        current_mean_intensity_limits = [0, quantile(current_mean_reconstructed(template_image > .5), 1 - current_mean_intensity_limit_excluded_size)];
        
        
        current_mean_cropped = template_crop_function(current_mean_reconstructed);
        
        
        % Compute and store the enrichments:
        
        if strcmp(model_type, 'standardized_voxels')
        % if true

          for enrichment_type_index = 1:number_enrichment_types
            current_enrichment_type = enrichment_types{enrichment_type_index};
            current_enrichment_type_computation_function = enrichment_type_computation_functions{enrichment_type_index};
            
            current_enrichments = [];
            for datum_index = 1:size(current_data, 1)
              datum_reconstruction = reconstruction_function(current_data(datum_index, :));
              % current_enrichments(end + 1) = max(datum_reconstruction(template_image >= .5)) ./ mean(datum_reconstruction(template_image >= .5));
              datum_enrichment = current_enrichment_type_computation_function(datum_reconstruction);
              if (should_enrichments_be_larger_than_one && datum_enrichment <= 1) || datum_enrichment <= 0
                fprintf('current_condition = %s, current_sensor = %s, datum_index = %d, current_enrichments(end) = %f, setting to nan\n', current_condition, current_sensor, datum_index, datum_enrichment)
                % beep, keyboard
                datum_enrichment = nan;
              end
              current_enrichments(end + 1) = datum_enrichment;
            end
            
            all_enrichments{condition_sensor_combination_index, all_relative_time_index, enrichment_type_index} = current_enrichments;
            % Lognormal mean:
            [current_lognormal_parameters, current_lognormal_confidence_intervals] = lognfit(current_enrichments, 1 - confidence_level);
            % whos current_enrichments
            all_enrichment_counts(condition_sensor_combination_index, all_relative_time_index, enrichment_type_index) = length(current_enrichments);
            all_enrichment_means(condition_sensor_combination_index, all_relative_time_index, enrichment_type_index) = exp(current_lognormal_parameters(1));
            all_log_enrichment_standard_deviations(condition_sensor_combination_index, all_relative_time_index, enrichment_type_index) = current_lognormal_parameters(2);
            all_enrichment_confidence_intervals(condition_sensor_combination_index, all_relative_time_index, :, enrichment_type_index) = permute(exp(current_lognormal_confidence_intervals(:, 1)), [2, 3, 1]);
          end
        end

        
        % keyboard
        current_mean_cropped = intensity_limit_function(current_mean_cropped, current_mean_intensity_limits);
        current_mean_cropped = histogram_equalization_function(current_mean_cropped, current_mean_intensity_limits);
        current_mean_cropped = intensity_warping_function(current_mean_cropped, current_mean_intensity_limits);
        
        
        % current_mean_cropped_zy = permute(current_mean_cropped, [2, 3, 1]);
        current_mean_cropped_zy = permute(current_mean_cropped, [3, 2, 1]);
        
        % Add to spatiotemporal illustration:
        % Do not contrast stretch:
        % all_means_cropped{all_relative_time_index} = reshape_2d(current_mean_cropped, 1);
        % all_means_cropped_zy{all_relative_time_index} = reshape_2d(current_mean_cropped_zy, 1);
        all_means_cropped{condition_sensor_combination_index, all_relative_time_index} = reshape_2d(current_mean_cropped, 1);
        all_means_cropped_zy{condition_sensor_combination_index, all_relative_time_index} = reshape_2d(current_mean_cropped_zy, 1);
        all_means_cropped_nonempty(condition_sensor_combination_index, all_relative_time_index) = true;
        

        if should_compute_individual_frame_illustrations
          close, set(gcf, 'Visible', 'off'), clf, set(gcf, 'Position', [1, 1, 3, 4] * 200)

          title_color = [0, 0, 0];
          set(gcf, 'Color', 'w')
          % title_color = [1, 1, 1];
          % set(gcf, 'Color', 'k')
          
          subplot(2, 1, 1)
          imagesc(reshape_contrast(current_mean_cropped, -1)), axis image, set(gca, 'XTick', [], 'YTick', [])
          colormap(illustration_colormap_to_use)
          title([title_string(1:end - 1), {[title_string{end}, ' (bottom to top slices)']}], 'Interpreter', 'none', 'Color', title_color)
          
          subplot(2, 1, 2)
          imagesc(reshape_contrast(current_mean_cropped_zy, -1)), axis image, set(gca, 'XTick', [], 'YTick', [])
          colormap(illustration_colormap_to_use)
          title([title_string(1:end - 1), {[title_string{end}, ' (synapse to end slices)']}], 'Interpreter', 'none', 'Color', title_color)
          
          dpi = master_script_options.figure_dpi;
          export_fig(image_full_filename, '-zbuffer', '-png', '-a1', '-nocrop', ['-r', num2str(dpi)])
        end
        
        % Now save grayscale images:
        image_to_show = [reshape_contrast(current_mean_cropped, -1); reshape_contrast(current_mean_cropped_zy, -1)];
        % grayscale_image_full_filename = [figures_location, image_filename, '_grayscale.png'];
        grayscale_image_full_filename = [videos_location, image_filename, '_grayscale.png'];
        imwrite(image_to_show, grayscale_image_full_filename)
        
        
      end
      
      
      if false
        % Debug info:
        
        beep, keyboard
      end
      % error('Remove slices in all dimensions to make same as smallest one!')
      
      % Fill in empty models:
      if ~any(all_means_cropped_nonempty(condition_sensor_combination_index, :))
        continue
      end
      all_means_cropped_nonempty_panel = all_means_cropped{condition_sensor_combination_index, find(all_means_cropped_nonempty(condition_sensor_combination_index, :), 1)};
      all_means_cropped_zy_nonempty_panel = all_means_cropped_zy{condition_sensor_combination_index, find(all_means_cropped_nonempty(condition_sensor_combination_index, :), 1)};
      % Set to nan:
      all_means_cropped_nonempty_panel = nan(size(all_means_cropped_nonempty_panel));
      all_means_cropped_zy_nonempty_panel = nan(size(all_means_cropped_zy_nonempty_panel));
      for nonempty_index = reshape(find(~all_means_cropped_nonempty(condition_sensor_combination_index, :)), 1, [])
        all_means_cropped{condition_sensor_combination_index, nonempty_index} = all_means_cropped_nonempty_panel;
        all_means_cropped_zy{condition_sensor_combination_index, nonempty_index} = all_means_cropped_zy_nonempty_panel;
      end
      
    end
    
    
    % Create and write the spatiotemporal model illustrations for each model and pair of models:
    
    
    [rows, columns] = find(triu(ones(number_conditions), 0));
    % [rows, columns] = find(triu(ones(number_conditions), 1));
    condition_pair_indices = [rows, columns];
    number_condition_pairs = (number_conditions + 1) * number_conditions / 2
    % number_condition_pairs = (number_conditions - 1) * number_conditions / 2
    number_condition_pairs = size(condition_pair_indices, 1)
    
    label_strings = arrayfun(@(x)num2str(x), all_relative_times_seconds_rounded, 'UniformOutput', false);
    
    for condition_pair_index = 1:number_condition_pairs
      condition_index1 = condition_pair_indices(condition_pair_index, 1);
      condition_index2 = condition_pair_indices(condition_pair_index, 2);
      current_condition1 = conditions{condition_index1, 1};
      current_condition2 = conditions{condition_index2, 1};
      condition_pair_distinct = condition_index1 ~= condition_index2;
      
      for sensor_index = 1:number_sensors
        current_sensor = sensors{sensor_index};
        
        condition_sensor_combination_index1 = find(ismember(condition_sensor_combinations(:, 1), current_condition1) & ismember(condition_sensor_combinations(:, 2), current_sensor));
        condition_sensor_combination_index2 = find(ismember(condition_sensor_combinations(:, 1), current_condition2) & ismember(condition_sensor_combinations(:, 2), current_sensor));
        
        if false
          % Debug info:
          fprintf('condition_index1 = %d, condition_index2 = %d\n', condition_index1, condition_index2)
          fprintf('current_condition1 = %s, current_condition2 = %s\n', current_condition1, current_condition2)
          fprintf('condition_sensor_combination_index1 = %d, condition_sensor_combination_index2 = %d\n', condition_sensor_combination_index1, condition_sensor_combination_index2)
          % beep, keyboard
        end
        
        % Create and write the spatiotemporal model illustrations:
        
        view_images = cell(2, 0);
        view_image_column_titles = cell(1, 0);
        view_image_colorbar_labels = cell(1, 0);
        
        % Use the same code for XY and ZY views:
        view_images(1:2, end + 1) = {cell2mat(all_means_cropped(condition_sensor_combination_index1, :).'); cell2mat(all_means_cropped_zy(condition_sensor_combination_index1, :).')};
        if should_interpolate_empty_panels
          view_images{1, end} = extrapolate_nans(view_images{1, end}, struct('method', 'diffusion', 'diffusion_method', 4));
          view_images{2, end} = extrapolate_nans(view_images{2, end}, struct('method', 'diffusion', 'diffusion_method', 4));
        end
        
        view_image_column_titles{1, end + 1} = sprintf('%s - %s', current_condition1, current_sensor);
        view_image_colorbar_labels{1, end + 1} = 'Relative probability';
        
        if condition_pair_distinct
          view_images(1:2, end + 1) = {cell2mat(all_means_cropped(condition_sensor_combination_index2, :).'); cell2mat(all_means_cropped_zy(condition_sensor_combination_index2, :).')};
          if should_interpolate_empty_panels
            view_images{1, end} = extrapolate_nans(view_images{1, end}, struct('method', 'diffusion', 'diffusion_method', 4));
            view_images{2, end} = extrapolate_nans(view_images{2, end}, struct('method', 'diffusion', 'diffusion_method', 4));
          end
          view_image_column_titles{1, end + 1} = sprintf('%s - %s', current_condition2, current_sensor);
          view_image_colorbar_labels{1, end + 1} = 'Relative probability';
        end
        
        if any(cellfun(@isempty, view_images(:)))
          continue
        end
        
        if condition_pair_distinct
          % Set the third image to be a difference image:
          view_images(1:2, end + 1) = {view_images{1, 2} - view_images{1, 1}; view_images{2, 2} - view_images{2, 1}};
          view_image_column_titles{1, end + 1} = sprintf('Difference (%s minus %s)', current_condition2, current_condition1);
          view_image_colorbar_labels{1, end + 1} = 'Relative probability difference';
        end
        
        view_image_suffixes = {'', '_zy'};
        
        label_indices = true(1, number_all_relative_times);
        
        % Plot each view and write to disk:
        
        for view_index = 1:length(view_image_suffixes)
          
          current_view_images = view_images(view_index, :);
          current_view_image_suffix = view_image_suffixes{view_index};
          number_current_view_images = length(current_view_images);
          
          % Views of models:
          
          % Display:
          close all
          set(gcf, 'Visible', 'off')
          % set(gcf, 'Visible', 'on')
          figure_position = get(gcf, 'Position');
          aspect_ratio = size(current_view_images{1}, 2) / size(current_view_images{1}, 1) * number_current_view_images;
          % aspect_ratio = aspect_ratio * 1.2;
          % figure_height_scale = .75;
          % figure_height_scale = 1;
          % figure_height_scale = 975 / 661;
          % % Nevermind, above good enough:
          % figure_height_scale = 975 / 661 * 975 / 1037;
          figure_height_scale = 975 / 703;
          current_figure_maximum_width = master_script_options.figure_maximum_width * number_current_view_images;
          figure_height = round(current_figure_maximum_width / aspect_ratio * figure_height_scale);
          % figure_width = round(figure_height * aspect_ratio);
          figure_width = round(current_figure_maximum_width * figure_height_scale);

          % keyboard
          set(gcf, 'Position', [1, 1, figure_width, figure_height])
          dpi = master_script_options.figure_dpi;
          
          for current_view_image_index = 1:number_current_view_images
            current_view_image = current_view_images{current_view_image_index};
            current_view_image_column_title = view_image_column_titles{current_view_image_index};
            current_view_image_colorbar_label = view_image_colorbar_labels{current_view_image_index};
            
            if false && current_view_image_index > 1
              % Debug info:
              beep, keyboard
            end
            
            % subplot()
            subplot(1, number_current_view_images, current_view_image_index)
            
            imagesc(current_view_image)
            % current_view_handle = imagesc(current_view_image);
            current_view_handle = gca;
            axis image
            ylabel('Time (seconds)', 'Interpreter', 'none')
            % y_tick_values = ((1:size(current_view_image, 1)) - 1/2) ./ (size(current_view_image, 1)) .* size(current_view_images{1}, 1);
            y_tick_values = ((1:number_all_relative_times) - 1/2) ./ (number_all_relative_times) .* (size(current_view_image, 1));
            y_tick_labels = cellfun(@(x)num2str(x), label_strings(label_indices), 'UniformOutput', false);
            set(current_view_handle, 'YTick', y_tick_values, 'YTickLabel', y_tick_labels)
            set(current_view_handle, 'XTick', [], 'XTickLabel', [])
            % set(current_view_handle, 'TickDir', 'out')

            colormap(illustration_colormap_to_use)
            
            % Add a colorbar:
            % use_horizontal_colorbar = false;
            use_horizontal_colorbar = true;
            if use_horizontal_colorbar
              colorbar_handle = colorbar('SouthOutside');
              xlabel(colorbar_handle, current_view_image_colorbar_label)
            else
              colorbar_handle = colorbar;
              ylabel(colorbar_handle, current_view_image_colorbar_label)
            end
            % set(colorbar_handle, 'TickDir', 'out')
            
            title(current_view_image_column_title, 'Interpreter', 'none')
            
          end
          
          % Write to disk:
          set(gcf, 'Color', 'w')
          if condition_pair_distinct
            image_filename = sprintf('%s%s_%s_%s_%s_spatiotemporal%s', master_script_options.model_prefix, condition_name_abbreviation_function(current_condition1), condition_name_abbreviation_function(current_condition2), current_sensor, model_type, current_view_image_suffix);
          else
            image_filename = sprintf('%s%s_%s_%s_spatiotemporal%s', master_script_options.model_prefix, condition_name_abbreviation_function(current_condition1), current_sensor, model_type, current_view_image_suffix);
          end
          image_full_filename = [figures_location, image_filename, '.png'];
          % imwrite(image_to_show, [figures_location, image_filename, '.png'])
          % export_fig(image_full_filename, '-zbuffer', '-png', '-a1', '-nocrop', ['-r', num2str(dpi)])
          export_fig(image_full_filename, '-opengl', '-png', '-a1', ['-r', num2str(dpi)])
          close gcf; clear gcf;
        end
        

        
        if strcmp(model_type, 'standardized_voxels') && condition_pair_distinct
          

        
          % Create enrichment plots:
          
          for enrichment_type_index = 1:number_enrichment_types
          
            current_enrichment_type = enrichment_types{enrichment_type_index};
            current_enrichment_type_computation_function = enrichment_type_computation_functions{enrichment_type_index};
            
            current_enrichments = all_enrichments(:, :, enrichment_type_index);
            current_enrichment_counts = all_enrichment_counts(:, :, enrichment_type_index);
            current_enrichment_means = all_enrichment_means(:, :, enrichment_type_index);
            current_enrichment_confidence_intervals = all_enrichment_confidence_intervals(:, :, :, enrichment_type_index);
            
            rounding_fraction_reciprocal = 2;
            enrichment_limits = [floor(min(current_enrichment_means(:) * rounding_fraction_reciprocal)), ceil(max(current_enrichment_means(:) * rounding_fraction_reciprocal))] ./ rounding_fraction_reciprocal;
            
            % Retrieve enrichments and related statistics:
            
            current_combinations_to_include = ismember(condition_sensor_combinations(:, 1), {current_condition1; current_condition2}) & ismember(condition_sensor_combinations(:, 2), current_sensor);
            current_combinations_to_plot = condition_sensor_combinations(current_combinations_to_include, :);
            current_plot_suffix = ['_', current_sensor];
            current_plot_enrichments = current_enrichments(current_combinations_to_include, :);
            current_plot_enrichment_means = current_enrichment_means(current_combinations_to_include, :);
            current_plot_enrichment_confidence_intervals = current_enrichment_confidence_intervals(current_combinations_to_include, :, :);
            
            % Test available enrichment distribution pairs:
            
            for all_relative_time_index = 1:number_all_relative_times
              % % Assumes just two conditions for now:
              % % (conditions 2, relative_time_seconds, p-value, Bonferroni corrected p-value, bonf_holm corrected p-value):
              % all_enrichments{current_combinations_to_include, all_relative_time_index, enrichment_type_index}
              if isempty(current_plot_enrichments{1, all_relative_time_index}) || isempty(current_plot_enrichments{2, all_relative_time_index})
                continue
              end
              
              % warning('Do we get here?')
              
              current_combination_plot_enrichments = current_plot_enrichments(1:2, all_relative_time_index);
              % whos current_combination_plot_enrichments
              % Uses (25) and (26) from Welch 1947, "The generalization of 'Student's' problem when several different population variances are involved:"
              [~, current_combination_enrichment_t_test_p_value] = ttest2(current_plot_enrichments{1, all_relative_time_index}, current_plot_enrichments{2, all_relative_time_index}, 1 - confidence_level, 'both', 'unequal');
              current_combination_enrichment_test = struct;
              current_combination_enrichment_test.enrichment_type = current_enrichment_type;
              current_combination_enrichment_test.combinations = current_combinations_to_plot;
              current_combination_enrichment_test.sensor = current_sensor;
              current_combination_enrichment_test.all_relative_time_index = all_relative_time_index;
              current_combination_enrichment_test.time_seconds_rounded = all_relative_times_seconds_rounded(all_relative_time_index);
              current_combination_enrichment_test.t_test_p_value = current_combination_enrichment_t_test_p_value;
              % current_combination_enrichment_test.combinations = current_combination_enrichment_t_test_p_value * number_tests_expected
              % current_combination_enrichment_test.combinations = nan;
              if length(combination_enrichment_tests) > 0
                combination_enrichment_tests(end + 1, 1) = current_combination_enrichment_test;
              else
                combination_enrichment_tests = current_combination_enrichment_test;
              end
              number_tests_performed = number_tests_performed + 1;
            end
            
            
            
          end
            
        end
        
      end
      
    end
    
  end
  
  
  % Bonferroni-Holm corrected p-values using bonf_holm:
  
  if ~isempty(combination_enrichment_tests)
    [corrected_p_values, corrected_p_value_significances] = bonf_holm([combination_enrichment_tests.t_test_p_value]);
    number_tests_performed = numel(corrected_p_values);
    number_tests_significant = sum(corrected_p_value_significances(:))
    % combination_enrichment_tests.t_test_p_value = num2cell(corrected_p_values);
    % combination_enrichment_tests.t_test_p_value_significant = num2cell(corrected_p_value_significances);
    for test_index = 1:length(combination_enrichment_tests)
      combination_enrichment_tests(test_index).t_test_p_value = corrected_p_values(test_index);
      combination_enrichment_tests(test_index).t_test_p_value_significant = corrected_p_value_significances(test_index);
    end
    save([models_location, sprintf('%senrichment_statistics.mat', master_script_options.model_prefix)], 'model_mean_reconstructed', 'enrichment_region_percentile', 'model_mean_enrichment_regions', 'enrichment_types', 'all_enrichments', 'all_enrichment_counts', 'all_enrichment_means', 'all_enrichment_confidence_intervals', 'combination_enrichment_tests', 'number_tests_performed', 'number_tests_significant', 'condition_sensor_combinations', 'conditions', 'sensors', 'all_relative_times', 'all_relative_times_seconds', 'all_relative_times_seconds_rounded')
    number_tests_performed
    
    if false
      % Debug info:
      % set(gcf, 'visible', 'on'), semilogy(sort([combination_enrichment_tests.t_test_p_value]))
      beep, keyboard
    end

    
    if strcmp(model_type, 'standardized_voxels')
      
      % Plot enrichments for every condition-sensor combination, each condition, and each sensor:
      
      for enrichment_type_index = 1:number_enrichment_types
      
        current_enrichment_type = enrichment_types{enrichment_type_index};
        current_enrichment_type_computation_function = enrichment_type_computation_functions{enrichment_type_index};

        current_enrichments = all_enrichments(:, :, enrichment_type_index);
        current_enrichment_counts = all_enrichment_counts(:, :, enrichment_type_index);
        current_enrichment_means = all_enrichment_means(:, :, enrichment_type_index);
        current_enrichment_confidence_intervals = all_enrichment_confidence_intervals(:, :, :, enrichment_type_index);
        
        rounding_fraction_reciprocal = 2;
        enrichment_limits = [floor(min(current_enrichment_means(:) * rounding_fraction_reciprocal)), ceil(max(current_enrichment_means(:) * rounding_fraction_reciprocal))] ./ rounding_fraction_reciprocal;
        
        % Convenience function to quickly check for significance between groups (UNTESTED!!!):
        current_enrichment_pair_tests_function = @(given_combination1, given_combination2)combination_enrichment_tests(arrayfun(@(given_test)strcmp(current_enrichment_type, given_test.enrichment_type) && any(ismember(given_test.combinations(:, 1), given_combination1(1)) & ismember(given_test.combinations(:, 2), given_combination1(2))) && any(ismember(given_test.combinations(:, 1), given_combination2(1)) & ismember(given_test.combinations(:, 2), given_combination2(2))), combination_enrichment_tests));
        % Assumes tests are sorted in ascending order with respect to all_relative_time_index and available at all all_relative_times:
        current_enrichment_all_significances_function = @(given_combination1, given_combination2)reshape(arrayfun(@(given_test)given_test.t_test_p_value_significant, current_enrichment_pair_tests_function(given_combination1, given_combination2)), 1, []);
        current_enrichment_significance_function = @(given_combination1, given_combination2)any(current_enrichment_all_significances_function(given_combination1, given_combination2));
        
        % significant_sensors = false(number_sensors, 1);
        % for test_index = 1:number_sensors
          % current_sensor = sensors{sensor_index};
          % significant_sensors(sensor_index)
        % end
        % for test_index = 1:length(combination_enrichment_tests)
          % current_test = combination_enrichment_tests(test_index);
          % if ~strcmp(current_test.enrichment_type, current_enrichment_type) || ~current_test.t_test_p_value_significant
            % continue
          % end
          % % significant_sensors(ismember(sensors, current_test.combinations(:, 2))) = true;
          % significant_sensors(ismember(sensors, current_test.sensor)) = true;
        % end
        
        % Significant differences:
        % significant_differences = strcmp(current_enrichment_type, {combination_enrichment_tests.enrichment_type}) & [combination_enrichment_tests.t_test_p_value_significant];
        % significant_difference_indices = find(significant_differences);
        significant_difference_indices = strcmp(current_enrichment_type, {combination_enrichment_tests.enrichment_type}) & [combination_enrichment_tests.t_test_p_value_significant];
        if ~any(significant_difference_indices)
          continue
        end
        
        % Find sensors that change between at least one condition pair for this enrichment type:
        % significant_sensors = unique({combination_enrichment_tests([combination_enrichment_tests.t_test_p_value_significant]).sensor});
        significant_sensors = unique({combination_enrichment_tests(significant_difference_indices).sensor}.');
        significant_sensor_indices = ismember(sensors, significant_sensors);
        significant_combinations = unique_condition_sensor_combination_function(vertcat(combination_enrichment_tests(significant_difference_indices).combinations));
        significant_combination_pairs = unique_condition_sensor_combination_pair_function({combination_enrichment_tests(significant_difference_indices).combinations}.');
        % % Find :
        % significant_condition_pair_sensor_combination_indices = 
        
        if false
          % Debug info for significant_*:
          significant_difference_tests = combination_enrichment_tests(significant_difference_indices)
          significant_difference_tests.combinations
          significant_sensors
          significant_combinations
          significant_combination_pairs
          significant_combination_pairs{:}
          beep, keyboard
        end
        
        % plot_group_index indicates which groups are plotted:
            
        for plot_group_index = 1:6
          
          combinations_to_plot = {};
          combinations_to_plot_groups = {};
          plot_suffixes = {};
          plot_enrichment_means = {};
          plot_enrichment_confidence_intervals = {};
          plot_show_significances_in_plot = false;
          % plot_significances = [];
          plot_aspect_ratio = [];
          plot_enrichment_significances = {};
          switch plot_group_index
            case 1
              % All plots:
              % combinations_to_plot = condition_sensor_combinations;
              combinations_to_plot = {condition_sensor_combinations};
              combinations_to_plot_groups = {1:size(combinations_to_plot{1}, 1)};
              plot_suffixes = {''};
              plot_enrichment_counts = {current_enrichment_counts};
              plot_enrichment_means = {current_enrichment_means};
              plot_enrichment_confidence_intervals = {current_enrichment_confidence_intervals};
              % plot_show_significances_in_plot = true;
              % error('Store significances here!')
              % plot_enrichment_significances = cellfun(@(x)ismember(x(:, 2), significant_sensors), combinations_to_plot, 'UniformOutput', false);
              % plot_enrichment_significances = cellfun(@(x)ismember(x, significant_combinations), combinations_to_plot, 'UniformOutput', false);
              plot_enrichment_significances = cellfun(@(x)condition_sensor_combination_ismember(x, significant_combinations), combinations_to_plot, 'UniformOutput', false);
              plot_aspect_ratio = 10 / 16;
            case 2
              % Per condition:
              for condition_index = 1:number_conditions
                current_combinations_to_include = ismember(condition_sensor_combinations(:, 1), conditions{condition_index});
                combinations_to_plot{condition_index} = condition_sensor_combinations(current_combinations_to_include, :);
                combinations_to_plot_groups{condition_index} = {1:size(combinations_to_plot{condition_index}, 1)};
                plot_suffixes{condition_index} = ['_', conditions{condition_index}];
                plot_enrichment_counts{condition_index} = current_enrichment_counts(current_combinations_to_include, :);
                plot_enrichment_means{condition_index} = current_enrichment_means(current_combinations_to_include, :);
                plot_enrichment_confidence_intervals{condition_index} = current_enrichment_confidence_intervals(current_combinations_to_include, :, :);
              end
              % plot_enrichment_significances = cellfun(@(x)ismember(x(:, 2), significant_sensors), combinations_to_plot, 'UniformOutput', false);
              % plot_enrichment_significances = cellfun(@(x)ismember(x, significant_combinations), combinations_to_plot, 'UniformOutput', false);
              plot_enrichment_significances = cellfun(@(x)condition_sensor_combination_ismember(x, significant_combinations), combinations_to_plot, 'UniformOutput', false);
              plot_aspect_ratio = 1 / 1;
            case 3
              % Per sensor:
              for sensor_index = 1:number_sensors
                current_combinations_to_include = ismember(condition_sensor_combinations(:, 2), sensors{sensor_index});
                combinations_to_plot{sensor_index} = condition_sensor_combinations(current_combinations_to_include, :);
                combinations_to_plot_groups{sensor_index} = {1:size(combinations_to_plot{sensor_index}, 1)};
                plot_suffixes{sensor_index} = ['_', sensors{sensor_index}];
                plot_enrichment_counts{sensor_index} = current_enrichment_counts(current_combinations_to_include, :);
                plot_enrichment_means{sensor_index} = current_enrichment_means(current_combinations_to_include, :);
                plot_enrichment_confidence_intervals{sensor_index} = current_enrichment_confidence_intervals(current_combinations_to_include, :, :);
              end
              % plot_show_significances_in_plot = true;
              % plot_enrichment_significances = cellfun(@(x)false(size(x(:, 2))), combinations_to_plot, 'UniformOutput', false);
              % plot_enrichment_significances = cellfun(@(x)ismember(x, significant_combinations), combinations_to_plot, 'UniformOutput', false);
              plot_enrichment_significances = cellfun(@(x)condition_sensor_combination_ismember(x, significant_combinations), combinations_to_plot, 'UniformOutput', false);
              plot_aspect_ratio = 16 / 10;
            case 4
              % Per condition pair, only sensors that change:
              % error('Not yet implemented!')
              for condition_pair_index = 1:number_condition_pairs
                condition_index1 = condition_pair_indices(condition_pair_index, 1);
                condition_index2 = condition_pair_indices(condition_pair_index, 2);
                current_condition1 = conditions{condition_index1, 1};
                current_condition2 = conditions{condition_index2, 1};
                condition_pair_distinct = condition_index1 ~= condition_index2;
                if ~condition_pair_distinct
                  continue
                end
                
                for sensor_index = 1:number_sensors
                  current_sensor = sensors{sensor_index};
                  
                  condition_sensor_combination_index1 = find(ismember(condition_sensor_combinations(:, 1), current_condition1) & ismember(condition_sensor_combinations(:, 2), current_sensor));
                  condition_sensor_combination_index2 = find(ismember(condition_sensor_combinations(:, 1), current_condition2) & ismember(condition_sensor_combinations(:, 2), current_sensor));
                  
                  if isempty(condition_sensor_combination_index1) || isempty(condition_sensor_combination_index2)
                    % No imaging runs for this combination:
                    continue
                  end
                  
                  plot_enrichment_significances{end + 1} = current_enrichment_all_significances_function(condition_sensor_combinations(condition_sensor_combination_index1, :), condition_sensor_combinations(condition_sensor_combination_index2, :));
                  
                  if ~any(plot_enrichment_significances{end})
                    plot_enrichment_significances(end) = [];
                    continue
                  end
                  
                  current_combinations_to_include = unique([condition_sensor_combination_index1, condition_sensor_combination_index2]);
                  combinations_to_plot{end + 1} = condition_sensor_combinations(current_combinations_to_include, :);
                  combinations_to_plot_groups{end + 1} = {1:size(combinations_to_plot{end}, 1)};
                  plot_suffixes{end + 1} = sprintf('_only-changed_bt%s,%s_%s', condition_name_abbreviation_function(current_condition1), condition_name_abbreviation_function(current_condition2), current_sensor);
                  plot_enrichment_counts{end + 1} = current_enrichment_counts(current_combinations_to_include, :);
                  plot_enrichment_means{end + 1} = current_enrichment_means(current_combinations_to_include, :);
                  plot_enrichment_confidence_intervals{end + 1} = current_enrichment_confidence_intervals(current_combinations_to_include, :, :);
                  
                  % plot_enrichment_significances{end + 1} = condition_sensor_combination_pair_ismember(combinations_to_plot(end), significant_combination_pairs);
                  % plot_enrichment_significances{end + 1} = combination_enrichment_tests(condition_sensor_combination_pair_ismember({combination_enrichment_tests.combinations}, combinations_to_plot(end))).;
                  % plot_enrichment_significances{end + 1} = current_enrichment_all_significances_function(condition_sensor_combinations(condition_sensor_combination_index1, :), condition_sensor_combinations(condition_sensor_combination_index2, :));
                end
                plot_show_significances_in_plot = true;
                
              end
              plot_aspect_ratio = 1 / 1;
            case 5
              % All hierarchical clusters:
              % error('Not yet implemented!')
              if master_script_options.skip_hierarchical_clustering
                error('Cannot run if master_script_options.skip_hierarchical_clustering')
              end
              cluster_assignments = all_model_clusters{model_type_index, ismember(clustering_distance_metrics, 'euclidean')};
              number_clusters = max(cluster_assignments);
              cluster_index_format_string = sprintf('%%%dd', floor(log10(number_clusters)) + 1);
              for cluster_index = 1:number_clusters
                current_combinations_to_include = cluster_assignments == cluster_index;
                % combinations_to_plot{cluster_index} = condition_sensor_combinations(current_combinations_to_include, :);
                % If combinations_to_plot{cluster_index} just has a one-column cell array, assume that it is a custom title/legend entry for this plot:
                combinations_to_plot{1}{cluster_index, 1} = sprintf(['Cluster ', cluster_index_format_string], cluster_index);
                combinations_to_plot_groups{1}{cluster_index, 1} = ones(1, size(combinations_to_plot{1}{cluster_index}, 1));
                plot_suffixes{1} = ['_all-clusters'];
                
                % Recompute enrichments, count, mean, confidence intervals (unused though):
                % combined_plot_enrichments = cell2mat(current_enrichments(current_combinations_to_include, :));
                if all(cellfun(@isempty, current_enrichments(current_combinations_to_include, :)))
                  continue
                end
                % [combined_lognormal_parameters, combined_lognormal_confidence_intervals] = arrayfun(@(given_index)lognfit(cell2mat(current_enrichments(current_combinations_to_include, given_index)), 1 - confidence_level), 1:number_all_relative_times, 'UniformOutput', false);
                combined_lognormal_parameters = cell(1, number_all_relative_times);
                combined_lognormal_confidence_intervals = cell(1, number_all_relative_times);
                for all_relative_time_index = 1:number_all_relative_times
                  % current_time_combined_enrichments = cell2mat(current_enrichments(current_combinations_to_include, all_relative_time_index));
                  current_time_combined_enrichments = cell2mat(current_enrichments(current_combinations_to_include, all_relative_time_index)')';
                  if isempty(current_time_combined_enrichments)
                    combined_lognormal_parameters{all_relative_time_index} = nan(1, 2);
                    combined_lognormal_confidence_intervals{all_relative_time_index} = nan(2, 2);
                  else
                    [combined_lognormal_parameters{all_relative_time_index}, combined_lognormal_confidence_intervals{all_relative_time_index}] = lognfit(current_time_combined_enrichments, 1 - confidence_level);
                  end
                end
                combined_lognormal_parameters = cell2mat(combined_lognormal_parameters')';
                combined_lognormal_confidence_intervals = cell2mat(combined_lognormal_confidence_intervals)';
                % zeros(number_condition_sensors, number_all_relative_times, 2, number_enrichment_types);
                % keyboard
                plot_enrichment_counts{1}(cluster_index, 1:number_all_relative_times) = sum(current_enrichment_counts(current_combinations_to_include, :), 1);
                plot_enrichment_means{1}(cluster_index, 1:number_all_relative_times) = exp(combined_lognormal_parameters(1, :));
                plot_enrichment_confidence_intervals{1}(cluster_index, 1:number_all_relative_times, 1:2) = permute(exp(combined_lognormal_confidence_intervals(1:2:end, :)), [3, 1, 2]);
              end
              % plot_show_significances_in_plot = true;
              plot_enrichment_significances = cellfun(@(x)false(size(x, 1), 1), combinations_to_plot, 'UniformOutput', false);
              plot_aspect_ratio = 16 / 10;
            case 6
              % Per hierarchical cluster:
              % error('Not yet implemented!')
              if master_script_options.skip_hierarchical_clustering
                error('Cannot run if master_script_options.skip_hierarchical_clustering')
              end
              cluster_assignments = all_model_clusters{model_type_index, ismember(clustering_distance_metrics, 'euclidean')};
              number_clusters = max(cluster_assignments);
              cluster_index_format_string = sprintf('%%%dd', floor(log10(number_clusters)) + 1);
              for cluster_index = 1:number_clusters
                current_combinations_to_include = cluster_assignments == cluster_index;
                % combinations_to_plot{cluster_index} = condition_sensor_combinations(current_combinations_to_include, :);
                % If combinations_to_plot{cluster_index} just has a one-column cell array, assume that it is a custom title/legend entry for this plot:
                combinations_to_plot{cluster_index} = {sprintf(['Cluster ', cluster_index_format_string], cluster_index)};
                combinations_to_plot_groups{cluster_index} = {ones(1, size(combinations_to_plot{cluster_index}, 1))};
                plot_suffixes{cluster_index} = ['_', sprintf(['cluster', cluster_index_format_string], cluster_index)];
                
                % Recompute enrichments, count, mean, confidence intervals (unused though):
                % combined_plot_enrichments = cell2mat(current_enrichments(current_combinations_to_include, :));
                if all(cellfun(@isempty, current_enrichments(current_combinations_to_include, :)))
                  continue
                end
                % [combined_lognormal_parameters, combined_lognormal_confidence_intervals] = arrayfun(@(given_index)lognfit(cell2mat(current_enrichments(current_combinations_to_include, given_index)), 1 - confidence_level), 1:number_all_relative_times, 'UniformOutput', false);
                combined_lognormal_parameters = cell(1, number_all_relative_times);
                combined_lognormal_confidence_intervals = cell(1, number_all_relative_times);
                for all_relative_time_index = 1:number_all_relative_times
                  % current_time_combined_enrichments = cell2mat(current_enrichments(current_combinations_to_include, all_relative_time_index));
                  current_time_combined_enrichments = cell2mat(current_enrichments(current_combinations_to_include, all_relative_time_index)')';
                  if isempty(current_time_combined_enrichments)
                    combined_lognormal_parameters{all_relative_time_index} = nan(1, 2);
                    combined_lognormal_confidence_intervals{all_relative_time_index} = nan(2, 2);
                  else
                    [combined_lognormal_parameters{all_relative_time_index}, combined_lognormal_confidence_intervals{all_relative_time_index}] = lognfit(current_time_combined_enrichments, 1 - confidence_level);
                  end
                end
                combined_lognormal_parameters = cell2mat(combined_lognormal_parameters')';
                combined_lognormal_confidence_intervals = cell2mat(combined_lognormal_confidence_intervals)';
                % zeros(number_condition_sensors, number_all_relative_times, 2, number_enrichment_types);
                % keyboard
                plot_enrichment_counts{cluster_index} = sum(current_enrichment_counts(current_combinations_to_include, :), 1);
                plot_enrichment_means{cluster_index} = exp(combined_lognormal_parameters(1, :));
                plot_enrichment_confidence_intervals{cluster_index} = permute(exp(combined_lognormal_confidence_intervals(1:2:end, :)), [3, 1, 2]);
              end
              % plot_show_significances_in_plot = true;
              plot_enrichment_significances = cellfun(@(x)false(size(x, 1), 1), combinations_to_plot, 'UniformOutput', false);
              plot_aspect_ratio = 16 / 10;
          end
          
          if false
            combinations_to_plot
            combinations_to_plot{:}
            plot_enrichment_significances
            plot_enrichment_significances{:}
            % % significant_test_combinations = reshape(arrayfun(@(x)x.combinations, combination_enrichment_tests(significant_difference_indices), 'UniformOutput', false), [], 1);
            % % significant_test_combinations = reshape(arrayfun(@(x)x.combinations, combination_enrichment_tests(significant_difference_indices), 'UniformOutput', false), [], 1);
            % % significant_test_combinations = [significant_test_combinations.combinations].'
            % significant_test_combinations = combination_enrichment_tests(significant_difference_indices);
            % significant_test_combinations2 = cellfun(@(x)x.', {significant_test_combinations.combinations}.', 'UniformOutput', false)
            % [significant_test_combinations.combinations] = significant_test_combinations2{:};
            % significant_test_combinations = [significant_test_combinations.combinations].'
            significant_test_combinations = {combination_enrichment_tests(significant_difference_indices).combinations}
            significant_test_combinations{:}
            
            significant_sensors, significant_sensor_indices, significant_combinations, significant_combination_pairs, significant_combination_pairs{:}
            
            dbstack, beep, keyboard
          end
          
          % for plot_index = 1:size(combinations_to_plot, 1)
          for plot_index = 1:length(combinations_to_plot)
            % current_condition_sensor_combinations = combinations_to_plot(plot_index, :);
            % If current_condition_sensor_combinations just has a one-column cell array, assume that it is a custom title/legend entry for this plot:
            current_condition_sensor_combinations = combinations_to_plot{plot_index};
            current_plot_suffix = plot_suffixes{plot_index};
            current_plot_enrichment_means = plot_enrichment_means{plot_index};
            current_plot_enrichment_significances = plot_enrichment_significances{plot_index};
            if isempty(current_plot_enrichment_means)
              continue
            end
            current_plot_enrichment_confidence_intervals = plot_enrichment_confidence_intervals{plot_index};
            
            % combinations_to_plot_groups
            % error
            
            % current_condition_sensor_combinations, current_plot_suffix
            
            % Display:
            close all
            set(gcf, 'Visible', 'off')
            % set(gcf, 'Visible', 'on')
            figure_position = get(gcf, 'Position');
            % aspect_ratio = size(current_view_image_matrix, 2) / size(current_view_image_matrix, 1);
            panels_two_to_a_row = false;
            % panels_two_to_a_row = true;
            if panels_two_to_a_row
              % aspect_ratio = 10 / 16;
              aspect_ratio = plot_aspect_ratio / 2;
              figure_height_scale = .5;
            else
              % aspect_ratio = 1 / 1;
              % aspect_ratio = 16 / 10;
              % aspect_ratio = 2.35 / 1;
              aspect_ratio = plot_aspect_ratio;
              figure_height_scale = 1;
              figure_height_scale = figure_height_scale * 975 / 847;
            end
            figure_height = round(master_script_options.figure_maximum_width / aspect_ratio * figure_height_scale);
            % figure_width = round(figure_height * aspect_ratio);
            figure_width = round(master_script_options.figure_maximum_width * figure_height_scale);
            set(gcf, 'Position', [1, 1, figure_width, figure_height])
            dpi = master_script_options.figure_dpi;
            
            current_legend_handles = [];
            current_legend_entries = {};
            current_legend_significance_handle = [];
            
            % plot_significances = false(length(current_condition_sensor_combinations), number_all_relative_times);
            % for test_index = 1:length(combination_enrichment_tests)
              % current_test = combination_enrichment_tests(test_index);
              % if ~strcmp(current_test.enrichment_type, current_enrichment_type) || ~current_test.t_test_p_value_significant
                % continue
              % end
              % current_test_relative_combination_indices = find(ismember(current_test.combinations(:, 1), current_condition_sensor_combinations(:, 1)) & ismember(current_test.combinations(:, 2), current_condition_sensor_combinations(:, 2)));
              % if false
                % beep, keyboard
              % end
              % plot_significances(current_test_relative_combination_indices, current_test.all_relative_time_index) = true;
            % end
            warning('Check significances in plots!')
            
            hold on
            for combination_index = 1:size(current_condition_sensor_combinations, 1)
              % Retrieve relevant data:
              current_combination = current_condition_sensor_combinations(combination_index, :);
              % If current_condition_sensor_combinations just has a one-column cell array, assume that it is a custom title/legend entry for this plot:
              if size(current_combination, 2) > 1
                current_condition = current_combination{1};
                current_sensor = current_combination{2};
              end
              current_combination_enrichment_means = current_plot_enrichment_means(combination_index, :);
              current_combination_enrichment_confidence_intervals = current_plot_enrichment_confidence_intervals(combination_index, :, :);
              if size(current_combination, 2) > 1
              % if size(current_combination, 2) == 2
                % current_combination_enrichment_test_significances = combination_enrichment_test_significances(ismember(sensors, current_sensor), :);
                % current_combination_enrichment_test_significances = [combination_enrichment_tests(strcmp({combination_enrichment_tests.sensor}, current_sensor)).t_test_p_value_significant];
                % error('Which pair is being tested?')
                % Go through all tests and mark a combination at a time as significant if a test shows a change and has both combinations in this figure (not backwards-compatible...):
                % current_combination_enrichment_test_significances = [combination_enrichment_tests(strcmp({combination_enrichment_tests.sensor}, current_sensor)).t_test_p_value_significant];
                % if all(~isfinite(current_combination_enrichment_test_significances))
                  % continue
                % end
              end
              
              % Plot line series:
              if size(current_combination, 2) > 1
                current_color = safe_colors(ismember(sensors, current_sensor), :);
                current_marker = marker_styles(mod(find(ismember(sensors, current_sensor)) - 1, length(marker_styles)) + 1);
                current_line_style = line_styles{mod(find(ismember(conditions, current_condition)) - 1, length(line_styles)) + 1};
              else
                current_color = safe_colors(mod(combination_index - 1, size(safe_colors, 1)) + 1, :);
                current_marker = marker_styles(mod(combination_index - 1, size(marker_styles, 1)) + 1);
                current_line_style = line_styles{mod(plot_index - 1, size(plot_index, 1)) + 1};
              end
              plot_parameters = {'Color', current_color, 'MarkerFaceColor', current_color, 'LineStyle', current_line_style, 'Marker', current_marker};
              % current_legend_handles(end + 1) = plot(all_relative_times_seconds_rounded, current_combination_enrichment_means, plot_parameters{:});
              current_legend_handles(end + 1) = errorbar(all_relative_times_seconds_rounded, current_combination_enrichment_means, current_combination_enrichment_means - current_combination_enrichment_confidence_intervals(:, :, 1), current_combination_enrichment_confidence_intervals(:, :, 2) - current_combination_enrichment_means, plot_parameters{:});
              if size(current_combination, 2) > 1
                current_legend_entries{end + 1} = [current_condition, ' - ', current_sensor];
              else
                current_legend_entries{end + 1} = [current_combination{1}];
              end
              
              if size(current_combination, 2) > 1
                current_both_enrichment_means = current_enrichment_means(ismember(condition_sensor_combinations(:, 2), current_sensor), :);
                % if size(current_plot_enrichment_significances, 1) == 1 && size(current_plot_enrichment_significances, 1) == number_all_relative_times
                if plot_show_significances_in_plot
                  % Just a single time series of significances for plotting asterisks on the axes:
                  significant_time_indices = current_plot_enrichment_significances;
                else
                  % Single or time series of significances for plotting asterisks in the legend:
                  significant_time_indices = current_plot_enrichment_significances(combination_index, :);
                end
                % if any(current_combination_enrichment_test_significances)
                if any(significant_time_indices)
                  if plot_show_significances_in_plot
                    % significant_time_indices = any(plot_significances(current_test_relative_combination_indices, :), 1);
                    % significant_times_seconds = all_relative_times_seconds_rounded(current_combination_enrichment_test_significances);
                    significant_times_seconds = all_relative_times_seconds_rounded(significant_time_indices);
                    % Mark significant differences with asterisks:
                    % % Mean:
                    % significant_enrichment_centers = mean(current_both_enrichment_means(:, current_combination_enrichment_test_significances), 1);
                    % Above lines:
                    maximum_combination_enrichment = max(current_both_enrichment_means(:));
                    % significant_enrichment_centers = mean([maximum_combination_enrichment, enrichment_limits(2)]) + zeros(1, sum(current_combination_enrichment_test_significances));
                    significant_enrichment_centers = mean([maximum_combination_enrichment, enrichment_limits(2)]) + zeros(length(significant_times_seconds), 1);
                    current_legend_significance_handle = scatter(significant_times_seconds, significant_enrichment_centers, 'k*');
                    % keyboard
                  else
                    current_legend_entries{end} = [current_legend_entries{end}, ' *'];
                    if false
                      combination_index, current_combination
                      plot_significances
                      beep, keyboard
                    end
                  end
                end
              end
            end
            hold off
            ylim(enrichment_limits)
            xlim([all_relative_times_seconds_rounded(1), all_relative_times_seconds_rounded(end)])
            % legend_entries = arrayfun(@(x)[current_condition_sensor_combinations{x, 1}, ' - ', current_condition_sensor_combinations{x, 2}], 1:size(current_condition_sensor_combinations, 1), 'UniformOutput', false);
            if ~isempty(current_legend_significance_handle)
              current_legend_handles(end + 1) = current_legend_significance_handle;
              current_legend_entries{end + 1} = 'Significant differences';
            end
            % legend_handle = legend(current_legend_handles, current_legend_entries, 'Interpreter', 'none');
            legend_handle = legend(current_legend_handles, current_legend_entries, 'Interpreter', 'none', 'Location', 'best');
            legend_handle_children = get(legend_handle, 'child');
            
            % Make sure everything has the same LineWidth property value:
            % set(legend_handle_children(2:3:end), 'LineWidth', get(0, 'DefaultLineLineWidth'))
            set(findobj(legend_handle_children, '-property', 'LineWidth'), 'LineWidth', get(0, 'DefaultLineLineWidth'))
            set(findobj(gca, '-property', 'LineWidth'), 'LineWidth', get(0, 'DefaultLineLineWidth'))
            
            xlabel('Time (seconds)', 'Interpreter', 'none')
            set(gca, 'XTick', all_relative_times_seconds_rounded)
            ylabel('Enrichment', 'Interpreter', 'none')
            
            % Write to disk:
            set(gcf, 'Color', 'w')
            image_filename = sprintf('%senrichments_mdl-type_%s_enr-type_%s%s', master_script_options.model_prefix, model_type, current_enrichment_type, current_plot_suffix);
            image_full_filename = [figures_location, image_filename, '.png'];
            % imwrite(image_to_show, [figures_location, image_filename, '.png'])
            % export_fig(image_full_filename, '-zbuffer', '-png', '-a1', '-nocrop', ['-r', num2str(dpi)])
            export_fig(image_full_filename, '-zbuffer', '-png', '-a1', ['-r', num2str(dpi)])
            close gcf; clear gcf;
          end
          
        end
        
      end
      
    end

    
    
    % number_tests_expected, number_tests_performed


    
    if ~false
      % Save the above statistical results in a Microsoft Office-friendly format:
      
      % error('Broken!')
      
      enrichment_statistics = load([models_location, sprintf('%senrichment_statistics.mat', master_script_options.model_prefix)]);
      
      %
      % get a table of corrected p-values:
      % enrichment_types_to_write = {'max', 'meantop090'};
      enrichment_types_to_write = {'meantop090'};
      for enrichment_type_index = 1:length(enrichment_types_to_write)
      
        current_enrichment_type = enrichment_types_to_write{enrichment_type_index};
        
        % a = load([models_location, 'enrichment_statistics_mdl-type_standardized_voxels_enr-type_', current_enrichment_type, '.mat']);
        % combination_enrichment_tests(test_index).t_test_p_value = corrected_p_values(test_index);
        % combination_enrichment_tests(test_index).t_test_p_value_significant = corrected_p_value_significances(test_index);
        % Enrichments are only computed for model_type standardized_voxels:
        enrichment_type_indices = strcmp(current_enrichment_type, {combination_enrichment_tests.enrichment_type});
        % % Assumes all tests ran (maybe should put nan values instead?):
        % current_t_test_p_values = [enrichment_statistics.combination_enrichment_tests(enrichment_type_indices).t_test_p_value];
        % current_t_test_p_values = reshape(current_t_test_p_values, number_all_relative_times, []).';
        % current_t_test_all_relative_time_indices = [enrichment_statistics.combination_enrichment_tests(enrichment_type_indices).all_relative_time_index];
        % current_t_test_all_relative_time_indices = reshape(current_t_test_all_relative_time_indices, number_all_relative_times, []).';
        % current_t_test_descriptions = cellfun(@(x)[sprintf('%s - %s vs. %s - %s', x{1, 1}, x{1, 2}, x{2, 1}, x{2, 2})], {enrichment_statistics.combination_enrichment_tests.combinations}, 'UniformOutput', false);
        % current_t_test_descriptions = reshape(current_t_test_descriptions, number_all_relative_times, []).';
        % Doesn't:
        current_t_test_p_values = zeros(number_condition_pairs, number_all_relative_times);
        current_t_test_all_relative_time_indices = zeros(size(current_t_test_p_values));
        current_t_test_all_relative_time_indices = zeros(size(current_t_test_p_values));
        % current_t_test_descriptions = cell(size(current_t_test_p_values));
        current_t_test_descriptions = cell(number_condition_pairs, 1);
        for test_index = 1:length(combination_enrichment_tests)
          test_info = combination_enrichment_tests(test_index);
          test_condition_pair_index = find(ismember(condition_pair_indices, sort([find(ismember(conditions, test_info.combinations{1, 1}), 1), find(ismember(conditions, test_info.combinations{2, 1}), 1)], 'ascend'), 'rows'), 1);
          % test_enrichment_type_index = find(ismember(test_info.enrichment_type, enrichment_types), 1);
          test_all_relative_time_index = test_info.all_relative_time_index;
          current_t_test_p_values(test_condition_pair_index, test_all_relative_time_index) = test_info.t_test_p_value;
          current_t_test_all_relative_time_indices(test_condition_pair_index, test_all_relative_time_index) = test_info.all_relative_time_index;
          % current_t_test_descriptions{test_condition_pair_index, test_all_relative_time_index} = sprintf('%s - %s vs. %s - %s', test_info.combinations{[1, 3, 2, 4]});
          current_t_test_descriptions{test_condition_pair_index} = sprintf('%s - %s vs. %s - %s', test_info.combinations{[1, 3, 2, 4]});
        end
        warning('>>>> HACK, didn''t fix sanity check here, just commented!')
        % if any(reshape(diff(current_t_test_all_relative_time_indices, [], 1) ~= 0, [], 1)) || any(reshape(~ismember(current_t_test_descriptions, current_t_test_descriptions(:, 1)), [], 1))
          % error('Tests not in correct order or indexed incorrectly!')
        % end
        if false
          % Debug info:
          % Show that reshaping is correct:
          current_t_test_p_values
          current_t_test_all_relative_time_indices
          beep, keyboard
        end
        
        % Collect available statistics into CSV and SYLK formatted files:
        
        % csv_field_matrix = [{''}, arrayfun(@(x)num2str(x), reshape(round(enrichment_statistics.all_relative_times_seconds), 1, []), 'UniformOutput', false)];
        csv_field_matrix = [{''}, reshape(num2cell(round(enrichment_statistics.all_relative_times_seconds)), 1, [])];
        sylk_field_matrix = [{[]}, reshape(num2cell(round(enrichment_statistics.all_relative_times_seconds)), 1, [])];
        
        for condition_pair_index = 1:number_condition_pairs
          condition_index1 = condition_pair_indices(condition_pair_index, 1);
          condition_index2 = condition_pair_indices(condition_pair_index, 2);
          current_condition1 = conditions{condition_index1, 1};
          current_condition2 = conditions{condition_index2, 1};
          condition_pair_distinct = condition_index1 ~= condition_index2;
          if ~condition_pair_distinct
            continue
          end
          
          % reshaped_all_relative_times_seconds = arrayfun(@(x)num2str(x), reshape(round(enrichment_statistics.all_relative_times_seconds), 1, []), 'UniformOutput', false);
          % first_nonempty_current_t_test_description_index = find(~cellfun(@isempty, current_t_test_descriptions), 1);
          % first_nonempty_current_t_test_description = '';
          % if ~isempty(first_nonempty_current_t_test_description_index)
            % first_nonempty_current_t_test_description = current_t_test_descriptions{first_nonempty_current_t_test_description_index};
          % end
          current_pair_t_test_description = current_t_test_descriptions{condition_pair_index};
          
          csv_field_matrix = [csv_field_matrix; {current_pair_t_test_description}, arrayfun(@(x)num2str(x), current_t_test_p_values(condition_pair_index, :), 'UniformOutput', false)];
          % sylk_field_matrix = [sylk_field_matrix; current_t_test_descriptions(condition_pair_index, :), num2cell(current_t_test_p_values)];
          sylk_field_matrix = [sylk_field_matrix; {current_pair_t_test_description}, num2cell(current_t_test_p_values(condition_pair_index, :))];
        end
          
        % fid = fopen([models_location, 'enrichment_statistics_mdl-type_standardized_voxels_enr-type_', current_enrichment_type, '_p_values.csv'], 'wt');
        fid = fopen([models_location, sprintf('%senrichment_statistics_mdl-type_standardized_voxels_enr-type_%s_p_values.csv', master_script_options.model_prefix, current_enrichment_type)], 'wt');
        for i = 1:size(csv_field_matrix, 1)
            for j = 1:size(csv_field_matrix, 2)
                if j > 1
                    fprintf(fid, ',');
                end
                if i == 1
                    %fprintf(fid, '<b>');
                    fprintf(fid, '*');
                end
                if ischar(csv_field_matrix{i, j})
                  fwrite(fid, csv_field_matrix{i, j});
                else
                  fprintf(fid, '%f', csv_field_matrix{i, j});
                end
                if i == 1
                    %fprintf(fid, '</b>');
                    fprintf(fid, '*');
                end
            end
            fprintf(fid, '\n');
        end
        fclose(fid);

        % SYLK format table, hacked together by running Excel 2007 experiments:
        % fid = fopen([models_location, 'enrichment_statistics_mdl-type_standardized_voxels_enr-type_', current_enrichment_type, '_p_values.slk'], 'wt');
        fid = fopen([models_location, sprintf('%senrichment_statistics_mdl-type_standardized_voxels_enr-type_%s_p_values.slk', master_script_options.model_prefix, current_enrichment_type)], 'wt');
        fprintf(fid, 'ID;P\n');
        fprintf(fid, 'P;PGeneral\n');
        fprintf(fid, 'P;EHelvetica\n');
        fprintf(fid, 'P;EHelvetica;SB\n');
        for i = 1:size(sylk_field_matrix, 1)
            for j = 1:size(sylk_field_matrix, 2)
                if isempty(sylk_field_matrix{i, j})
                    continue
                end
                if i == 1 || j == 1 || (i > 1 && j > 1 && sylk_field_matrix{i, j} < .05)
                    % Bold formatting:
                    fprintf(fid, 'F;Y%d;X%d;SD\n', i, j);
                end
                if i > 1 && j > 1
                    fprintf(fid, 'F;FE2D\n', i, j);
                end
                
                fprintf(fid, 'C;Y%d;X%d', i, j);
                if ischar(sylk_field_matrix{i, j})
                    fprintf(fid, ';K"%s"', sylk_field_matrix{i, j});
                else
                    fprintf(fid, ';K%d', sylk_field_matrix{i, j});
                end
                fprintf(fid, '\n');
                
                if i > 1 && j > 1
                    fprintf(fid, 'F;FE2D\n', i, j);
                end
            end
        end
        fprintf(fid, 'E\n');
        fclose(fid);
        
      end
    
    end
  
  end
  
  
  
  % beep, keyboard
  % error('Implementation yet unfinished below this line!')

  % t_cell_info.figures_info = struct();
  % t_cell_info.figures_info.registration_options = registration_options;
  
  
  
end  

