function [t_cell_info] = master_script_generate_model_maps_and_csv_files(t_cell_info, options)
  % convert model files to csv
  %
  % 2015-08-18 xruan: Copied from master_script_generate_model_spatiotemporal_figure3b.
  % 
  %  Also count cell frames. 
  % 
  % 
  % 
  % % Dependencies:
  % % From the File Exchange: export_fig, pmkmp, toolbox_graph, toolbox_fast_marching, affine, Snake3D, Mesh_voxelisation, dirr, inhull
  % % From tebuck: modified version of Jieyue Li's HPA_lib, rasterize_mesh

%
% Author: Taraz E. Buck and Xiongtao Ruan
%
% Copyright (C) 2012-2016 Murphy Lab
% Computational Biology Department
% School of Computer Science
% Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu



  
  
  
  debug = ~true
  if debug
    addpath(genpath(pwd));
  end
  
  
  master_script_options = t_cell_info.options;
  regions_location = t_cell_info.path_info.regions_location;
  segmentations_filtered_location = t_cell_info.path_info.segmentations_filtered_location;
  alignments_location = t_cell_info.path_info.alignments_location;
  deformations_location = t_cell_info.path_info.deformations_location;
  models_location = t_cell_info.path_info.models_location;
  figures_location = t_cell_info.path_info.figures_location;
  videos_location = t_cell_info.path_info.videos_location;
  
  model_map_directory = 'model_maps/'
  
  if master_script_options.use_two_point_synapses
      model_map_directory = ['two-point_', model_map_directory];
  end
  
  model_map_location = [t_cell_info.path_info.figures_location, model_map_directory];
  
  if ~exist(model_map_location, 'dir')
      mkdir(model_map_location);
  end  

  
  illustration_colormap_to_use = t_cell_info.plotting_info.illustration_colormap_to_use;
  
  condition_name_abbreviation_function = t_cell_info.abbreviation_info.condition_name_abbreviation_function;
  
  model_types = t_cell_info.model_type_info.model_types;
  number_model_types = t_cell_info.model_type_info.number_model_types;
  model_representation_functions = t_cell_info.model_type_info.model_representation_functions;
  model_reconstruction_functions = t_cell_info.model_type_info.model_reconstruction_functions;
  model_dimensionality_reduction_methods = t_cell_info.model_type_info.model_dimensionality_reduction_methods;
  model_dimensionality_reduction_method_parameters = t_cell_info.model_type_info.model_dimensionality_reduction_method_parameters;
  
  
  run_relative_path_list = t_cell_info.synapse_info.run_relative_path_list;
  run_key_list = t_cell_info.synapse_info.run_key_list;
  % run_data_list = t_cell_info.synapse_info.run_data_list;
  run_images_list = t_cell_info.synapse_info.run_images_list;
  run_brightfield_images_list = t_cell_info.synapse_info.run_brightfield_images_list;
  run_image_frame_indices_list = t_cell_info.synapse_info.run_image_frame_indices_list;
  run_number_cells = t_cell_info.synapse_info.run_number_cells;
  run_number_frames = t_cell_info.synapse_info.run_number_frames;
  run_point_tracks = t_cell_info.synapse_info.run_point_tracks;
  run_relative_time_tracks = t_cell_info.synapse_info.run_relative_time_tracks;
  run_relative_times = t_cell_info.synapse_info.run_relative_times;
  run_number_relative_times = t_cell_info.synapse_info.run_number_relative_times;
  all_relative_times = t_cell_info.synapse_info.all_relative_times;
  number_all_relative_times = t_cell_info.synapse_info.number_all_relative_times;
  all_relative_times_seconds_data = t_cell_info.synapse_info.all_relative_times_seconds_data;
  all_relative_times_seconds = t_cell_info.synapse_info.all_relative_times_seconds;
  all_relative_times_seconds_rounded = t_cell_info.synapse_info.all_relative_times_seconds_rounded;
  number_runs = t_cell_info.synapse_info.number_runs;
  condition_sensor_combinations = t_cell_info.synapse_info.condition_sensor_combinations;
  number_condition_sensors = t_cell_info.synapse_info.number_condition_sensors;
  conditions = t_cell_info.synapse_info.conditions;
  sensors = t_cell_info.synapse_info.sensors;
  number_conditions = t_cell_info.synapse_info.number_conditions;
  number_sensors = t_cell_info.synapse_info.number_sensors;
  unique_condition_sensor_combination_function = t_cell_info.synapse_info.unique_condition_sensor_combination_function;
  unique_condition_sensor_combination_pair_function = t_cell_info.synapse_info.unique_condition_sensor_combination_pair_function;
  condition_sensor_combination_ismember = t_cell_info.synapse_info.condition_sensor_combination_ismember;
  condition_sensor_combination_pair_ismember = t_cell_info.synapse_info.condition_sensor_combination_pair_ismember;
  
  template_centroid = t_cell_info.template_info.template_centroid;
  template_synapse = t_cell_info.template_info.template_synapse;
  template_image = t_cell_info.template_info.template_image;
  template_crop_function = t_cell_info.template_info.template_crop_function;

  window_size_2d = t_cell_info.preprocessing_info.window_size_2d;
  window_center_2d = t_cell_info.preprocessing_info.window_center_2d;
  cropped_size = t_cell_info.preprocessing_info.cropped_size;
  
  
  [sub1, sub2, sub3] = ind2sub(size(template_image), find(template_image > 0));
  
  subscripts = [sub1, sub2, sub3];
  
  cell_frame_count = {};
  
  for model_type_index = 1:number_model_types
    model_type = model_types{model_type_index};
    
    if ~strcmp(model_type, 'standardized_voxels')
        continue;
    end
    
     
    model_type
    
    
 
    % Collect mean models:
    all_means_cropped = cell(number_condition_sensors, number_all_relative_times);
    all_means_cropped_zy = cell(number_condition_sensors, number_all_relative_times);
    all_means_cropped_nonempty = false(number_condition_sensors, number_all_relative_times);
    
    % collect standard deviration of models
    all_stds_cropped = cell(number_condition_sensors, number_all_relative_times);
    all_stds_cropped_zy = cell(number_condition_sensors, number_all_relative_times);
    all_stds_cropped_nonempty = false(number_condition_sensors, number_all_relative_times);

    
    should_interpolate_empty_panels = false;
    % should_interpolate_empty_panels = true;
    
    for condition_sensor_combination_index = 1:number_condition_sensors
      current_condition = condition_sensor_combinations{condition_sensor_combination_index, 1};
      current_sensor = condition_sensor_combinations{condition_sensor_combination_index, 2};
      
      % all_means_cropped = cell(number_all_relative_times, 1);
      % all_means_cropped_zy = cell(number_all_relative_times, 1);
      current_cell_frame_count = {};
      current_cell_frame_count{1} = current_condition;
      current_cell_frame_count{2} = current_sensor;
      current_cell_frame_count{3} = 0;
      accumute_count = 0;
      for all_relative_time_index = 1:number_all_relative_times
        relative_time = all_relative_times(all_relative_time_index);

        % Read all files sequentially:
        current_model_filename = [models_location, sprintf('%s%s_%s_reltime% 3d_%s.mat', master_script_options.model_prefix, current_condition, current_sensor, relative_time, model_type)];
        % current_model_filename = [models_location, sprintf('%s_%s_frame%03d_reltime% 3d_%s.mat', current_condition, current_sensor, all_relative_time_index, relative_time, model_type)];
        % current_model_filename
        if ~exist(current_model_filename, 'file')
          continue
        end
        % current_mean = [];
        % current_standard_deviation = [];
        % current_covariance = [];
        current_data = [];
        current_data_relevant_run_keys = [];
        current_data_relevant_run_key_indices = [];
        load(current_model_filename)
        % keyboard
                        
        % Make model illustrations for this sensor:

        current_count = size(current_data, 1);
        
        % Used to be < 2:
        if current_count < 1
          continue
        end
        
        csv_filename = sprintf('%s%s_%s_frame%03d_reltime% 3d_%s', master_script_options.model_prefix, condition_name_abbreviation_function(current_condition), current_sensor, all_relative_time_index, relative_time, model_type);
        
        csv_full_filename = [model_map_location, csv_filename, '.csv'];
        
        std_csv_filename = sprintf('%s%s_%s_frame%03d_reltime% 3d_%s_std', master_script_options.model_prefix, condition_name_abbreviation_function(current_condition), current_sensor, all_relative_time_index, relative_time, model_type);
        
        std_csv_full_filename = [model_map_location, std_csv_filename, '.csv'];

	
        if exist(csv_full_filename, 'file') && exist(std_csv_full_filename, 'file')
          %continue
        end
        

        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        current_mean = mean(current_data, 1);

        current_subscripts_mean = [subscripts, current_mean'];
        
        csvwrite(csv_full_filename, current_subscripts_mean);

	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

        current_std = std(current_data, [], 1);
        
        current_subscripts_std = [subscripts, current_std'];
        
        csvwrite(std_csv_full_filename, current_subscripts_std);


        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
        accumute_count = accumute_count + current_count;        
        
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      end
      current_cell_frame_count{3} = accumute_count;
      current_condition
      current_sensor
      accumute_count
      pause(1)
      cell_frame_count{end + 1} = current_cell_frame_count;
    end
  end
  
    csv_field_matrix = cell_frame_count'
    fid = fopen([models_location, 'model_cell_frame_count.csv'], 'wt');
    for i = 1:size(csv_field_matrix, 1)
        csv_field_matrix_i = csv_field_matrix{i}
        for j = 1:size(csv_field_matrix_i, 2)
            if j > 1
                fprintf(fid, ',');
            end
            if ischar(csv_field_matrix{i}{j})
              fwrite(fid, csv_field_matrix{i}{j});
            else
              fprintf(fid, '%f', csv_field_matrix{i}{j});
            end        
        end
        fprintf(fid, '\n');
    end
    fclose(fid);
  save([models_location, 'model_cell_frame_count.mat'], 'cell_frame_count');
end  

