function [t_cell_info] = master_script_generate_model_spatiotemporal_figure3b(t_cell_info, options)
  % Produce spatiotemporal models of three sensors ( cofilin, MRLC, and
  % WAVE2) 
  % for figure 3B in the paper.
  % xruan 07/29/2015 copied from master_script_generate_model_figures.m
  % 
  % 
  % 
  % % Dependencies:
  % % From the File Exchange: export_fig, pmkmp, toolbox_graph, toolbox_fast_marching, affine, Snake3D, Mesh_voxelisation, dirr, inhull
  % % From tebuck: modified version of Jieyue Li's HPA_lib, rasterize_mesh

%
% Author: Taraz E. Buck and Xiongtao Ruan
%
% Copyright (C) 2012-2016 Murphy Lab
% Computational Biology Department
% School of Computer Science
% Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu



  
  
  
  % Variables referenced more than once can be copied to local variables:
  
  debug = true
  if debug
    addpath(genpath(pwd));
  end
  
  
  master_script_options = t_cell_info.options;
  regions_location = t_cell_info.path_info.regions_location;
  segmentations_filtered_location = t_cell_info.path_info.segmentations_filtered_location;
  alignments_location = t_cell_info.path_info.alignments_location;
  deformations_location = t_cell_info.path_info.deformations_location;
  models_location = t_cell_info.path_info.models_location;
  figures_location = t_cell_info.path_info.figures_location;
  videos_location = t_cell_info.path_info.videos_location;
  
  model_map_directory = 'model_maps/'
  
  if master_script_options.use_two_point_synapses
      model_map_directory = ['two-point_', model_map_directory];
  end
  
  model_map_location = [t_cell_info.path_info.figures_location, model_map_directory];
  
  if ~exist(model_map_location, 'dir')
      mkdir(model_map_location);
  end  
    
  figure3b_directory = 'figure3b/';
  
  figure3b_location = [t_cell_info.path_info.figures_for_paper_location, figure3b_directory];
  
  if ~exist(figure3b_location, 'dir')
      mkdir(figure3b_location);
       t_cell_info.path_info.figure3b_location = figure3b_location;
  end
  
  illustration_colormap_to_use = t_cell_info.plotting_info.illustration_colormap_to_use;
  
  condition_name_abbreviation_function = t_cell_info.abbreviation_info.condition_name_abbreviation_function;
  
  model_types = t_cell_info.model_type_info.model_types;
  number_model_types = t_cell_info.model_type_info.number_model_types;
  % model_representation_functions = t_cell_info.model_type_info.model_representation_functions;
  % model_reconstruction_functions = t_cell_info.model_type_info.model_reconstruction_functions;
  model_dimensionality_reduction_methods = t_cell_info.model_type_info.model_dimensionality_reduction_methods;
  model_dimensionality_reduction_method_parameters = t_cell_info.model_type_info.model_dimensionality_reduction_method_parameters;
  
  model_original_representation_functions = t_cell_info.model_info.model_original_representation_functions;
  model_original_reconstruction_functions = t_cell_info.model_info.model_original_reconstruction_functions;
  model_reduction_representation_functions = t_cell_info.model_info.model_reduction_representation_functions;
  model_reduction_reconstruction_functions = t_cell_info.model_info.model_reduction_reconstruction_functions;
  model_representation_functions = t_cell_info.model_info.model_representation_functions;
  model_reconstruction_functions = t_cell_info.model_info.model_reconstruction_functions;
  
  run_relative_path_list = t_cell_info.synapse_info.run_relative_path_list;
  run_key_list = t_cell_info.synapse_info.run_key_list;
  % run_data_list = t_cell_info.synapse_info.run_data_list;
  run_images_list = t_cell_info.synapse_info.run_images_list;
  run_brightfield_images_list = t_cell_info.synapse_info.run_brightfield_images_list;
  run_image_frame_indices_list = t_cell_info.synapse_info.run_image_frame_indices_list;
  run_number_cells = t_cell_info.synapse_info.run_number_cells;
  run_number_frames = t_cell_info.synapse_info.run_number_frames;
  run_point_tracks = t_cell_info.synapse_info.run_point_tracks;
  run_relative_time_tracks = t_cell_info.synapse_info.run_relative_time_tracks;
  run_relative_times = t_cell_info.synapse_info.run_relative_times;
  run_number_relative_times = t_cell_info.synapse_info.run_number_relative_times;
  all_relative_times = t_cell_info.synapse_info.all_relative_times;
  number_all_relative_times = t_cell_info.synapse_info.number_all_relative_times;
  all_relative_times_seconds_data = t_cell_info.synapse_info.all_relative_times_seconds_data;
  all_relative_times_seconds = t_cell_info.synapse_info.all_relative_times_seconds;
  all_relative_times_seconds_rounded = t_cell_info.synapse_info.all_relative_times_seconds_rounded;
  number_runs = t_cell_info.synapse_info.number_runs;
  condition_sensor_combinations = t_cell_info.synapse_info.condition_sensor_combinations;
  number_condition_sensors = t_cell_info.synapse_info.number_condition_sensors;
  conditions = t_cell_info.synapse_info.conditions;
  sensors = t_cell_info.synapse_info.sensors;
  number_conditions = t_cell_info.synapse_info.number_conditions;
  number_sensors = t_cell_info.synapse_info.number_sensors;
  unique_condition_sensor_combination_function = t_cell_info.synapse_info.unique_condition_sensor_combination_function;
  unique_condition_sensor_combination_pair_function = t_cell_info.synapse_info.unique_condition_sensor_combination_pair_function;
  condition_sensor_combination_ismember = t_cell_info.synapse_info.condition_sensor_combination_ismember;
  condition_sensor_combination_pair_ismember = t_cell_info.synapse_info.condition_sensor_combination_pair_ismember;
  
  template_centroid = t_cell_info.template_info.template_centroid;
  template_synapse = t_cell_info.template_info.template_synapse;
  template_image = t_cell_info.template_info.template_image;
  template_crop_function = t_cell_info.template_info.template_crop_function;

  window_size_2d = t_cell_info.preprocessing_info.window_size_2d;
  window_center_2d = t_cell_info.preprocessing_info.window_center_2d;
  cropped_size = t_cell_info.preprocessing_info.cropped_size;

  segmentation_rasterization_function = t_cell_info.segmentation_info.segmentation_rasterization_function;
  
  model_original_representation_functions = t_cell_info.model_info.model_original_representation_functions;
  model_original_reconstruction_functions = t_cell_info.model_info.model_original_reconstruction_functions;
  model_reduction_representation_functions = t_cell_info.model_info.model_reduction_representation_functions;
  model_reduction_reconstruction_functions = t_cell_info.model_info.model_reduction_reconstruction_functions;
  
%   all_model_distances = t_cell_info.clustering_info.all_model_distances;
%   all_model_trees = t_cell_info.clustering_info.all_model_trees;
%   all_model_clusters = t_cell_info.clustering_info.all_model_clusters;
%   clustering_distance_metrics = t_cell_info.clustering_info.clustering_distance_metrics;



  
  confidence_level = master_script_options.confidence_level;
  
  
  % warning('Hard-coding two major conditions!')
  % major_conditions = {'B7 Blockade', 'Full Stimulus'}.'
  
  
  should_compute_individual_frame_illustrations = false;
  % should_compute_individual_frame_illustrations = true;
  
  should_skip_completed_images = false;
  % should_skip_completed_images = true;
  
  % warning('>>>> HACK, performing statistical tests of enrichment under master_script_options.skip_model_figure_generation!')
  
  current_mean_intensity_limit_excluded_size = 0;
  
  current_standard_deviation_intensity_limit_excluded_size = 0;
  
  % current_mean_intensity_limit_excluded_size = 1e-3;
  % current_mean_intensity_limit_excluded_size = 5e-3;
  intensity_limit_function = @(given_image, given_limits)given_image .* (given_image < given_limits(2)) .* (given_image > given_limits(1)) + given_limits(1) .* (given_image <= given_limits(1)) + given_limits(2) .* (given_image >= given_limits(2));
  
  if master_script_options.illustration_uses_histogram_equalization
    % histogram_equalization_function = @(given_image)masked_histogram_equalization(given_image, template_image > .5);
    histogram_equalization_function = @(given_image, given_limits)masked_histogram_equalization(given_image, (given_image > given_limits(1)) & (given_image < given_limits(2)));
  else
    histogram_equalization_function = @(given_image, given_limits)given_image;
  end
  
  if master_script_options.illustration_uses_logarithmic_scale
    intensity_warping_function = @(given_image, given_limits)log(given_image);
  else
    intensity_warping_function = @(given_image, given_limits)given_image;
  end

  

  % Plot formatting:
  use_fixed_safe_colors = false;
  % use_fixed_safe_colors = true;
  if use_fixed_safe_colors
    % http://safecolours.rigdenage.com/colourchoice.html#examples
    safe_colors = [.4, 1, .2; 1., .6, .2; 1., .2, .2; .2, .4, 0; .8, .6, .6; .8, .8, .8; 0, 0, 0; 0, .8, 1; .6, 0, .8; 0, .6, .6];
    Attempted modification, not good:
    % safe_colors = [.4, .8, .4; 1., .6, .2; 1., .2, .2; .2, .4, 0; .8, .6, .6; .7, .7, .7; 0, 0, 0; 0, .8, 1; .6, 0, .8; 0, .6, .6];
  else
    safe_colors = distinguishable_colors(number_sensors, {'w','k'});
  end
  % This should eventually cycle through combinations of markers and colors or plot things separately... Could also use filled vs. not markers.
  % marker_styles = '.o+xsd^v<>ph';
  % marker_styles = '.o+xsd^vph';
  marker_styles = 'o.x*sd^vph';
  % line_styles = {'-', '--'};
  % % Hack because B7 sorts before Full:
  % % line_styles = {'--', '-'};
  % line_styles = {':', '-'};
  % Now have three conditions:
  line_styles = {':', '--', '-'};
  
  
%   % Enrichment computation parameters:
%   % enrichment_region_percentiles = [98, 94, 90]
%   enrichment_region_percentiles = [90]
%   should_use_per_cell_max_enrichment = false
%   % should_use_per_cell_max_enrichment = true
%   % should_use_global_enrichment_region = false
%   should_use_global_enrichment_region = true
%   should_use_per_cell_enrichment_region = false
%   % should_use_per_cell_enrichment_region = true
  
  % Create illustrations, compute enrichments, compute statistics:
  
  
  % if strcmp(model_type, 'standardized_voxels')
  combination_enrichment_tests = [];
  % combination_enrichment_tests = struct(zeros(0, 1));
  % end
  
  for model_type_index = 1:number_model_types
    model_type = model_types{model_type_index};
    
    if ~strcmp(model_type, 'standardized_voxels')
        continue;
    end
    
    representation_function = model_representation_functions{model_type_index};
    reconstruction_function = model_reconstruction_functions{model_type_index};
    original_representation_function = model_original_representation_functions{model_type_index};
    original_reconstruction_function = model_original_reconstruction_functions{model_type_index};
    reduction_representation_function = model_reduction_representation_functions{model_type_index};
    reduction_reconstruction_function = model_reduction_reconstruction_functions{model_type_index};
    dimensionality_reduction_method = model_dimensionality_reduction_methods{model_type_index};
    dimensionality_reduction_method_parameters = model_dimensionality_reduction_method_parameters{model_type_index};
    
    model_type
    
    model_dimensionality_reduction_full_filename = [models_location, sprintf('%s%s_%s', master_script_options.model_prefix, model_type, dimensionality_reduction_method), '.mat'];
    
    if ~exist(model_dimensionality_reduction_full_filename, 'file')
      continue
    end
    
    
 
    % Collect mean models:
    all_means_cropped = cell(number_condition_sensors, number_all_relative_times);
    all_means_cropped_zy = cell(number_condition_sensors, number_all_relative_times);
    all_means_cropped_nonempty = false(number_condition_sensors, number_all_relative_times);
    
    % collect standard deviration of models
    all_stds_cropped = cell(number_condition_sensors, number_all_relative_times);
    all_stds_cropped_zy = cell(number_condition_sensors, number_all_relative_times);
    all_stds_cropped_nonempty = false(number_condition_sensors, number_all_relative_times);

    
    should_interpolate_empty_panels = false;
    % should_interpolate_empty_panels = true;
    
    for condition_sensor_combination_index = 1:number_condition_sensors
      current_condition = condition_sensor_combinations{condition_sensor_combination_index, 1};
      current_sensor = condition_sensor_combinations{condition_sensor_combination_index, 2};
      
      % all_means_cropped = cell(number_all_relative_times, 1);
      % all_means_cropped_zy = cell(number_all_relative_times, 1);

      for all_relative_time_index = 1:number_all_relative_times
        relative_time = all_relative_times(all_relative_time_index);

        % Read all files sequentially:
        current_model_filename = [models_location, sprintf('%s%s_%s_reltime% 3d_%s.mat', master_script_options.model_prefix, current_condition, current_sensor, relative_time, model_type)];
        % current_model_filename = [models_location, sprintf('%s_%s_frame%03d_reltime% 3d_%s.mat', current_condition, current_sensor, all_relative_time_index, relative_time, model_type)];
        % current_model_filename
        if ~exist(current_model_filename, 'file')
          continue
        end
        % current_mean = [];
        % current_standard_deviation = [];
        % current_covariance = [];
        current_data = [];
        current_data_relevant_run_keys = [];
        current_data_relevant_run_key_indices = [];
        load(current_model_filename)
        % keyboard
        
        current_data = reduction_representation_function(current_data);
                
        % Make model illustrations for this sensor:

        current_count = size(current_data, 1);
        
        % Used to be < 2:
        if current_count < 1
          continue
        end
        
        title_string = {sprintf('Condition: %s, sensor: %s, relative time: %d, number cells: %d', current_condition, current_sensor, relative_time, current_count), sprintf('model type: %s', model_type)};
        image_filename = sprintf('%s%s_%s_frame%03d_reltime% 3d_%s', master_script_options.model_prefix, condition_name_abbreviation_function(current_condition), current_sensor, all_relative_time_index, relative_time, model_type);
        
        image_full_filename = [figures_location, image_filename, '.png'];
        
        if should_skip_completed_images && exist(image_full_filename, 'file')
          continue
        end
        

        current_mean = mean(current_data, 1);
        current_standard_deviation = std(current_data, [], 1);
      
        % Reconstruct images in the template:
        
        current_mean_reconstructed = reconstruction_function(current_mean);

        % Produce the image for this condition-sensor combination:
        
        current_mean_intensity_limits = [0, quantile(current_mean_reconstructed(template_image > .5), 1 - current_mean_intensity_limit_excluded_size)];
        
        
        current_mean_cropped = template_crop_function(current_mean_reconstructed);
        
        
        current_standard_deviation_reconstructed = reconstruction_function(current_standard_deviation);

        % Produce the image for this condition-sensor combination:
        
        current_standard_deviation_intensity_limits = [0, quantile(current_standard_deviation_reconstructed(template_image > .5), 1 - current_standard_deviation_intensity_limit_excluded_size)];
        
        
        current_standard_deviation_cropped = template_crop_function(current_standard_deviation_reconstructed);
        
        
        % Compute and store the enrichments:
                % if true
        
        % keyboard
        current_mean_cropped = intensity_limit_function(current_mean_cropped, current_mean_intensity_limits);
        current_mean_cropped = histogram_equalization_function(current_mean_cropped, current_mean_intensity_limits);
        current_mean_cropped = intensity_warping_function(current_mean_cropped, current_mean_intensity_limits);
        
        
        % current_mean_cropped_zy = permute(current_mean_cropped, [2, 3, 1]);
        current_mean_cropped_zy = permute(current_mean_cropped, [3, 2, 1]);
        
        
        % keyboard
        current_standard_deviation_cropped = intensity_limit_function(current_standard_deviation_cropped, current_standard_deviation_intensity_limits);
        current_standard_deviation_cropped = histogram_equalization_function(current_standard_deviation_cropped, current_standard_deviation_intensity_limits);
        current_standard_deviation_cropped = intensity_warping_function(current_standard_deviation_cropped, current_standard_deviation_intensity_limits);
        
        
        % current_standard_deviation_cropped_zy = permute(current_standard_deviation_cropped, [2, 3, 1]);
        current_standard_deviation_cropped_zy = permute(current_standard_deviation_cropped, [3, 2, 1]);
        
        
        
        
        % Add to spatiotemporal illustration:
        % Do not contrast stretch:
        % all_means_cropped{all_relative_time_index} = reshape_2d(current_mean_cropped, 1);
        % all_means_cropped_zy{all_relative_time_index} = reshape_2d(current_mean_cropped_zy, 1);
        all_means_cropped{condition_sensor_combination_index, all_relative_time_index} = current_mean_cropped;
        all_means_cropped_zy{condition_sensor_combination_index, all_relative_time_index} = current_mean_cropped_zy;
        all_means_cropped_nonempty(condition_sensor_combination_index, all_relative_time_index) = true;

        all_standard_deviation_cropped{condition_sensor_combination_index, all_relative_time_index} = current_mean_cropped;
        all_standard_deviation_cropped_zy{condition_sensor_combination_index, all_relative_time_index} = current_mean_cropped_zy;
        all_standard_deviation_cropped_nonempty(condition_sensor_combination_index, all_relative_time_index) = true;


        if should_compute_individual_frame_illustrations
          close, set(gcf, 'Visible', 'off'), clf, set(gcf, 'Position', [1, 1, 3, 4] * 200)

          title_color = [0, 0, 0];
          set(gcf, 'Color', 'w')
          % title_color = [1, 1, 1];
          % set(gcf, 'Color', 'k')
          
          subplot(2, 1, 1)
          imagesc(reshape_contrast(current_mean_cropped, -1)), axis image, set(gca, 'XTick', [], 'YTick', [])
          colormap(illustration_colormap_to_use)
          title([title_string(1:end - 1), {[title_string{end}, ' (bottom to top slices)']}], 'Interpreter', 'none', 'Color', title_color)
          
          subplot(2, 1, 2)
          imagesc(reshape_contrast(current_mean_cropped_zy, -1)), axis image, set(gca, 'XTick', [], 'YTick', [])
          colormap(illustration_colormap_to_use)
          title([title_string(1:end - 1), {[title_string{end}, ' (synapse to end slices)']}], 'Interpreter', 'none', 'Color', title_color)
          
          dpi = master_script_options.figure_dpi;
          export_fig(image_full_filename, '-opengl', '-png', '-a1', '-nocrop', ['-r', num2str(dpi)])
        end
        
        % Now save grayscale images:
        image_to_show = [reshape_contrast(current_mean_cropped, -1); reshape_contrast(current_mean_cropped_zy, -1)];
        % grayscale_image_full_filename = [figures_location, image_filename, '_grayscale.png'];
        grayscale_image_full_filename = [videos_location, image_filename, '_grayscale.png'];
        imwrite(image_to_show, grayscale_image_full_filename)       
      end
      
 
      if false
        % Debug info:
        
        beep, keyboard
      end
      % error('Remove slices in all dimensions to make same as smallest one!')
      
      % Fill in empty models:
      if ~any(all_means_cropped_nonempty(condition_sensor_combination_index, :))
        continue
      end
      all_means_cropped_nonempty_panel = all_means_cropped{condition_sensor_combination_index, find(all_means_cropped_nonempty(condition_sensor_combination_index, :), 1)};
      all_means_cropped_zy_nonempty_panel = all_means_cropped_zy{condition_sensor_combination_index, find(all_means_cropped_nonempty(condition_sensor_combination_index, :), 1)};
      % Set to nan:
      all_means_cropped_nonempty_panel = nan(size(all_means_cropped_nonempty_panel));
      all_means_cropped_zy_nonempty_panel = nan(size(all_means_cropped_zy_nonempty_panel));
      for nonempty_index = reshape(find(~all_means_cropped_nonempty(condition_sensor_combination_index, :)), 1, [])
        all_means_cropped{condition_sensor_combination_index, nonempty_index} = all_means_cropped_nonempty_panel;
        all_means_cropped_zy{condition_sensor_combination_index, nonempty_index} = all_means_cropped_zy_nonempty_panel;
      end
      
    end
    
      %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      %% start here
      
      all_means_cropped 
      all_standard_deviation_cropped
      condition_sensor_combinations
      
      %% Figure 3B
      
      % sensors = {'Cofilin', 'MRLC', 'WAVE2'};
      figure3B_sensors = {'LAT', 'LAT', 'LAT'}
      figure3B_time_points = [0, 180];
      figure3B_relative_time_index = [3, 5];
      figure3B_conditions = {'Full Stimulus', 'B7 Blockade', 'B7 Blockade active Rac Cofilin'};
      
      condition_sensor_combinations_strings = cellfun(@(x, y) [x, y], condition_sensor_combinations(:, 1), condition_sensor_combinations(:, 2), 'UniformOutput', false)
      needed_condition_sensor_combinations_strings = cellfun(@(x, y) [x, y], figure3B_conditions', figure3B_sensors', 'UniformOutput', false)
      
      row_ind_logical = cellfun(@(x) strcmp(condition_sensor_combinations_strings, x), needed_condition_sensor_combinations_strings, 'UniformOutput', false);       
      
      row_inds = cell2mat(cellfun(@(x) find (x == 1), row_ind_logical, 'UniformOutput', false))
      
      column_inds = figure3B_relative_time_index;
      
      [condition_sensor_time_sub_x, condition_sensor_time_sub_y] = meshgrid(column_inds, row_inds);
      
      condition_sensor_time_inds = sub2ind(size(all_means_cropped), condition_sensor_time_sub_y(:), condition_sensor_time_sub_x(:));
      
      figure3B_mean_cropped = all_means_cropped(condition_sensor_time_inds);
      
      figure3B_standard_deviation_cropped = all_standard_deviation_cropped(condition_sensor_time_inds);
      
      % only need slice 7 : 18
      figure3B_images_reshape_function = @(image) [reshape_2d(image(:, :, 7 : 12)); reshape_2d(image(:, :, 13 : 18))];
      
      figure3B_mean_reshape = cellfun(@(x) figure3B_images_reshape_function(x), figure3B_mean_cropped, 'UniformOutput', false);
      figure3B_standard_deviation_reshape = cellfun(@(x) figure3B_images_reshape_function(x), figure3B_mean_cropped, 'UniformOutput', false);
      figure3B_mean_reshape = reshape(figure3B_mean_reshape, [2, 3])';
      figure3B_standard_deviation_reshape = reshape(figure3B_standard_deviation_reshape, [2, 3])';
      
      
      
      max_mean_intensity = max(cellfun(@(x) max(x(:)), reshape(figure3B_mean_reshape, [], 1)));
      
      figure3B_mean_reshape_norm = cellfun(@(x) x / max_mean_intensity, figure3B_mean_reshape, 'UniformOutput', false);
      
      max_standard_deviation_intensity = max(cellfun(@(x) max(x(:)), reshape(figure3B_standard_deviation_reshape, [], 1)));
      
      figure3B_standard_deviation_norm = cellfun(@(x) x / max_standard_deviation_intensity, figure3B_standard_deviation_reshape, 'UniformOutput', false);
      close all 
      figure,
      figure_width = 1200;
      figure_height = 600;
      set(gcf, 'Position', [1, 1, figure_width, figure_height])
  
      positions = [
      0.12, 0.6, 0.36, 0.23;   
      0.52, 0.6, 0.36, 0.23; 
      0.12, 0.35, 0.36, 0.23; 
      0.52, 0.35, 0.36, 0.23; 
      0.12, 0.1, 0.36, 0.23; 
      0.52, 0.1, 0.36, 0.23; 
      ];
            
      [m, n] = size(figure3B_mean_reshape);
      for i = 1 : m
          for j = 1 : n
              ind = sub2ind([n , m], j, i);           
              axes('Position', positions(ind, :))
              imagesc(figure3B_mean_reshape_norm{i, j});
              axis off, colormap(illustration_colormap_to_use)
              caxis([0, 1]);
              
              if ind == 1
                  title('0 seconds')
                  text('Position', [-5, 27], 'String', figure3B_sensors{1}, 'Rotation',90)
              end
              if ind == 2 
                  title('180 seconds')
              end
              
              if ind == 3
                  text('Position', [-5, 27], 'String' , figure3B_sensors{2}, 'Rotation',90)
              end
              
              if ind == 5
                  text('Position', [-5, 27], 'String' , figure3B_sensors{3}, 'Rotation',90)
              end
              hold on    
          end
      end

      h = colorbar;
      set(h, 'Position', [.9 .1 .04 .73]);
      ylabel(h, 'Relative probability (per model)');
      axis off
      hold off;
      dpi = 300;
      figure3B_name = sprintf('%s%s_spatiotemporal_model_mean', master_script_options.model_prefix, model_type);
      
      export_fig([figure3b_location, figure3B_name], '-opengl', '-png', '-a1', ['-r', num2str(dpi)]);

      %% Figure S2
      
            figureS2_sensors = {'LAT', 'LAT', 'LAT'}
      figureS2_time_points = [0, 180];
      figureS2_relative_time_index = [3, 5];
      figureS2_conditions = {'Full Stimulus', 'B7 Blockade', 'B7 Blockade active Rac Cofilin'};
      
      condition_sensor_combinations_strings = cellfun(@(x, y) [x, y], condition_sensor_combinations(:, 1), condition_sensor_combinations(:, 2), 'UniformOutput', false)
      needed_condition_sensor_combinations_strings = cellfun(@(x, y) [x, y], figureS2_conditions', figureS2_sensors', 'UniformOutput', false)
      
      row_ind_logical = cellfun(@(x) strcmp(condition_sensor_combinations_strings, x), needed_condition_sensor_combinations_strings, 'UniformOutput', false);       
      
      row_inds = cell2mat(cellfun(@(x) find (x == 1), row_ind_logical, 'UniformOutput', false))
      
      column_inds = figureS2_relative_time_index;
      
      [condition_sensor_time_sub_x, condition_sensor_time_sub_y] = meshgrid(column_inds, row_inds);
      
      condition_sensor_time_inds = sub2ind(size(all_standard_deviation_cropped), condition_sensor_time_sub_y(:), condition_sensor_time_sub_x(:));
      
      figureS2_standard_deviation_cropped = all_standard_deviation_cropped(condition_sensor_time_inds);
      
      figureS2_standard_deviation_cropped = all_standard_deviation_cropped(condition_sensor_time_inds);
      
      % only need slice 7 : 18
      figureS2_images_reshape_function = @(image) [reshape_2d(image(:, :, 7 : 12)); reshape_2d(image(:, :, 13 : 18))];
      
      figureS2_standard_deviation_reshape = cellfun(@(x) figureS2_images_reshape_function(x), figureS2_standard_deviation_cropped, 'UniformOutput', false);
      figureS2_standard_deviation_reshape = cellfun(@(x) figureS2_images_reshape_function(x), figureS2_standard_deviation_cropped, 'UniformOutput', false);
      figureS2_standard_deviation_reshape = reshape(figureS2_standard_deviation_reshape, [2, 3])';
      figureS2_standard_deviation_reshape = reshape(figureS2_standard_deviation_reshape, [2, 3])';
      
      
      
      max_standard_deviation_intensity = max(cellfun(@(x) max(x(:)), reshape(figureS2_standard_deviation_reshape, [], 1)));
      
      figureS2_standard_deviation_reshape_norm = cellfun(@(x) x / max_standard_deviation_intensity, figureS2_standard_deviation_reshape, 'UniformOutput', false);
      
      max_standard_deviation_intensity = max(cellfun(@(x) max(x(:)), reshape(figureS2_standard_deviation_reshape, [], 1)));
      
      figureS2_standard_deviation_norm = cellfun(@(x) x / max_standard_deviation_intensity, figureS2_standard_deviation_reshape, 'UniformOutput', false);
      close all 
      figure,
      figure_width = 1200;
      figure_height = 600;
      set(gcf, 'Position', [1, 1, figure_width, figure_height])
  
      positions = [
      0.12, 0.6, 0.36, 0.23;   
      0.52, 0.6, 0.36, 0.23; 
      0.12, 0.35, 0.36, 0.23; 
      0.52, 0.35, 0.36, 0.23; 
      0.12, 0.1, 0.36, 0.23; 
      0.52, 0.1, 0.36, 0.23; 
      ];
            
      [m, n] = size(figureS2_standard_deviation_reshape);
      for i = 1 : m
          for j = 1 : n
              ind = sub2ind([n , m], j, i);           
              axes('Position', positions(ind, :))
              imagesc(figureS2_standard_deviation_reshape_norm{i, j});
              axis off, colormap(illustration_colormap_to_use), caxis([0, 1]);
              
              if ind == 1
                  title('0 seconds')
                  text('Position', [-5, 27], 'String', figureS2_sensors{1}, 'Rotation',90)
              end
              if ind == 2 
                  title('180 seconds')
              end
              
              if ind == 3
                  text('Position', [-5, 27], 'String' , figureS2_sensors{2}, 'Rotation',90)
              end
              
              if ind == 5
                  text('Position', [-5, 27], 'String' , figureS2_sensors{3}, 'Rotation',90)
              end
              hold on    
          end
      end

      h = colorbar;
      set(h, 'Position', [.9 .1 .04 .73]);
      ylabel(h, 'Relative probability (per model)');
      axis off
      hold off;
      dpi = 300;
      figureS2_name = sprintf('%s%s_spatiotemporal_model_standard_deviation', master_script_options.model_prefix, model_type);
      
      export_fig([figure3b_location, figureS2_name], '-opengl', '-png', '-a1', ['-r', num2str(dpi)]);
      
      
      %% model maps (one point and two point synapse)
      all_means_cropped 
      all_standard_deviation_cropped
      condition_sensor_combinations
      
      [m, n] = size(all_means_cropped);
      max_mean_intensity = max(cellfun(@(x) max(x(:)), reshape(all_means_cropped, [], 1)));
     
      for i = 1 : m,
          for j = 1 : n
              current_sensor = condition_sensor_combinations{i, 1}
              current_condition = condition_sensor_combinations{i, 2}
              current_relative_time = all_relative_times(j);
              figure_name =  sprintf('%s%s_%s_%s_reltime %d_model_mean_map', ...
                  master_script_options.model_prefix, model_type, current_sensor, current_condition, current_relative_time);
              [can_start, final_name, final_exists] = chunk_start(model_map_location, figure_name);
              
              if ~can_start
                  continue;
              end
              
              close all; 
              figure;
              set(gcf, 'Visible', 'off')
              set(gcf, 'Visible', 'on')
              set(gcf, 'Renderer', 'opengl');
              axes('Position', [0.1, 0.1, 0.75, 0.8]);
              dpi = master_script_options.figure_dpi;
              figure_height = 60;
              figure_width = 1500;
              set(gcf, 'Position', [1, 1, figure_width, figure_height]);
              
              current_model = all_means_cropped{i, j};
              current_model_norm = current_model / max_mean_intensity;

              imagesc(reshape_2d(current_model_norm))
              axis off, colormap(illustration_colormap_to_use), caxis([0, 1]);
              
              h = colorbar;
              set(h, 'Position', [.86 .1 .02 .8]);            
              axis off
              
              export_fig([model_map_location, figure_name], '-opengl', '-png', '-a1', ['-r', num2str(dpi)]);
              chunk_finish(model_map_location, figure_name)
          end
      end   
   end
end
