function [t_cell_info] = master_script_generate_model_videos(t_cell_info, options)
  % Produce videos of constructed models for visual evaluation.
  %
  % 2014-01-11 tebuck: Copied from master_script_generate_model_figures.m.
  % 
  % 
  % 
  % 
  % 
  % % Dependencies:
  % % From the File Exchange: export_fig, pmkmp, toolbox_graph, toolbox_fast_marching, affine, Snake3D, Mesh_voxelisation, dirr, inhull
  % % From tebuck: modified version of Jieyue Li's HPA_lib, rasterize_mesh

%
% Author: Taraz E. Buck and Xiongtao Ruan
%
% Copyright (C) 2012-2016 Murphy Lab
% Computational Biology Department
% School of Computer Science
% Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu



  
  
  
  % Variables referenced more than once can be copied to local variables:
  master_script_options = t_cell_info.options;
  regions_location = t_cell_info.path_info.regions_location;
  segmentations_filtered_location = t_cell_info.path_info.segmentations_filtered_location;
  alignments_location = t_cell_info.path_info.alignments_location;
  deformations_location = t_cell_info.path_info.deformations_location;
  models_location = t_cell_info.path_info.models_location;
  figures_location = t_cell_info.path_info.figures_location;
  videos_location = t_cell_info.path_info.videos_location;

  illustration_colormap_to_use = t_cell_info.plotting_info.illustration_colormap_to_use;
  
  % xruan 08/17/2015
  % add the usage of condition name abbreviation
  condition_name_abbreviation_function = t_cell_info.abbreviation_info.condition_name_abbreviation_function;

  model_types = t_cell_info.model_type_info.model_types;
  number_model_types = t_cell_info.model_type_info.number_model_types;
  model_representation_functions = t_cell_info.model_type_info.model_representation_functions;
  model_reconstruction_functions = t_cell_info.model_type_info.model_reconstruction_functions;
  model_dimensionality_reduction_methods = t_cell_info.model_type_info.model_dimensionality_reduction_methods;
  model_dimensionality_reduction_method_parameters = t_cell_info.model_type_info.model_dimensionality_reduction_method_parameters;

  run_relative_path_list = t_cell_info.synapse_info.run_relative_path_list;
  run_key_list = t_cell_info.synapse_info.run_key_list;
  % run_data_list = t_cell_info.synapse_info.run_data_list;
  run_images_list = t_cell_info.synapse_info.run_images_list;
  run_brightfield_images_list = t_cell_info.synapse_info.run_brightfield_images_list;
  run_image_frame_indices_list = t_cell_info.synapse_info.run_image_frame_indices_list;
  run_number_cells = t_cell_info.synapse_info.run_number_cells;
  run_number_frames = t_cell_info.synapse_info.run_number_frames;
  run_point_tracks = t_cell_info.synapse_info.run_point_tracks;
  run_relative_time_tracks = t_cell_info.synapse_info.run_relative_time_tracks;
  run_relative_times = t_cell_info.synapse_info.run_relative_times;
  run_number_relative_times = t_cell_info.synapse_info.run_number_relative_times;
  all_relative_times = t_cell_info.synapse_info.all_relative_times;
  number_all_relative_times = t_cell_info.synapse_info.number_all_relative_times;
  all_relative_times_seconds_data = t_cell_info.synapse_info.all_relative_times_seconds_data;
  all_relative_times_seconds = t_cell_info.synapse_info.all_relative_times_seconds;
  all_relative_times_seconds_rounded = t_cell_info.synapse_info.all_relative_times_seconds_rounded;
  number_runs = t_cell_info.synapse_info.number_runs;
  condition_sensor_combinations = t_cell_info.synapse_info.condition_sensor_combinations;
  number_condition_sensors = t_cell_info.synapse_info.number_condition_sensors;
  conditions = t_cell_info.synapse_info.conditions;
  sensors = t_cell_info.synapse_info.sensors;
  number_conditions = t_cell_info.synapse_info.number_conditions;
  number_sensors = t_cell_info.synapse_info.number_sensors;
  
  template_centroid = t_cell_info.template_info.template_centroid;
  template_synapse = t_cell_info.template_info.template_synapse;
  template_image = t_cell_info.template_info.template_image;
  template_crop_function = t_cell_info.template_info.template_crop_function;

  window_size_2d = t_cell_info.preprocessing_info.window_size_2d;
  window_center_2d = t_cell_info.preprocessing_info.window_center_2d;
  cropped_size = t_cell_info.preprocessing_info.cropped_size;

  segmentation_rasterization_function = t_cell_info.segmentation_info.segmentation_rasterization_function;
  
  model_original_representation_functions = t_cell_info.model_info.model_original_representation_functions;
  model_original_reconstruction_functions = t_cell_info.model_info.model_original_reconstruction_functions;
  model_reduction_representation_functions = t_cell_info.model_info.model_reduction_representation_functions;
  model_reduction_reconstruction_functions = t_cell_info.model_info.model_reduction_reconstruction_functions;
  
  all_model_distances = t_cell_info.clustering_info.all_model_distances;
  all_model_trees = t_cell_info.clustering_info.all_model_trees;
  all_model_clusters = t_cell_info.clustering_info.all_model_clusters;
  clustering_distance_metrics = t_cell_info.clustering_info.clustering_distance_metrics;


  
  
  for model_type_index = 1:number_model_types
    model_type = model_types{model_type_index};
    representation_function = model_representation_functions{model_type_index};
    reconstruction_function = model_reconstruction_functions{model_type_index};
    dimensionality_reduction_method = model_dimensionality_reduction_methods{model_type_index};
    dimensionality_reduction_method_parameters = model_dimensionality_reduction_method_parameters{model_type_index};
    
    model_type
    
    % model_dimensionality_reduction_full_filename = [models_location, sprintf('%s_%s', model_type, dimensionality_reduction_method), '.mat'];
    
    % if ~exist(model_dimensionality_reduction_full_filename, 'file')
      % continue
    % end
    
    % Combine some sensors into multi-channel videos:
    % Each entry in this cell array is a 3x2 cell array containing condition as the first column and sensor as the second (condition is assumed constant across channels, and 3 channels assumed, for the code below):
    % video_channels = {...
      % {'Full Stimulus', 'ARP3'; 'Full Stimulus', 'CPalpha1'; 'Full Stimulus', 'WASP'}; ...
      % {'Full Stimulus', 'Cofilin'; 'Full Stimulus', 'MRLC'; 'Full Stimulus', 'WASP'}; ...
      % };
    video_channels = {};
    video_titles = {};
    
    should_videos_be_colorblind_friendly = false;
    % should_videos_be_colorblind_friendly = true;
    
    % Create channels by hierarchical clusters:

    % % Inconsistency-based thresholding does not work here, it just gives many clusters or one, so hard coding three clusters:
    % warning('Inconsistency-based thresholding does not work here, it just gives many clusters or one, so hard coding three clusters')
    % cluster_assignments = cluster(model_trees{ismember(a.model_types, model_type), ismember(a.clustering_distance_metrics, 'euclidean')}, 'MaxClust', 3);
    % % [num2cell(cluster_assignments), condition_sensor_combinations]
    
    cluster_assignments = all_model_clusters{ismember(model_types, model_type), ismember(clustering_distance_metrics, 'euclidean')};
    number_clusters = max(cluster_assignments);
    
    % First go through each member of each cluster:
    for cluster_index = 1:number_clusters
      cluster_members = find(cluster_assignments == cluster_index);
      number_cluster_members = length(cluster_members);
      current_channel_index = 1;
      current_channels = cell(1, 3);
      for member_index = 1:number_cluster_members
        % current_channels{current_channel_index} = condition_sensor_combinations(member_index, :);
        % current_channels(current_channel_index, 1:2) = condition_sensor_combinations(member_index, :);
        current_channels(current_channel_index, 1:2) = condition_sensor_combinations(cluster_members(member_index), :);
        video_channels{end + 1} = current_channels;
        % video_titles{end + 1} = sprintf('Cluster %d', cluster_index);
        video_titles{end + 1} = sprintf('cluster %d', cluster_index);
        if should_videos_be_colorblind_friendly
          current_channel_index = mod(current_channel_index, 2) + 1;
        else
          current_channel_index = mod(current_channel_index, 3) + 1;
        end
      end
    end
    
    % Then go through each set of two (should_videos_be_colorblind_friendly) or three clusters:
    current_channel_index = 1;
    current_channels = cell(1, 3);
    current_clusters = zeros(1, 3);
    
    if false
      % condition_sensor_combinations, cluster_assignments
      [condition_sensor_combinations, num2cell(cluster_assignments)]
      whos condition_sensor_combinations cluster_assignments
      beep, keyboard
    end
    
    for cluster_index = 1:number_clusters
      cluster_members = find(cluster_assignments == cluster_index);
      number_cluster_members = length(cluster_members);
      % current_channels{current_channel_index} = condition_sensor_combinations(cluster_members(1), :);
      current_channels(current_channel_index, 1:2) = condition_sensor_combinations(cluster_members(1), :);
      current_clusters(current_channel_index) = cluster_index;
      video_channels{end + 1} = current_channels;
      % video_titles{end + 1} = sprintf('Clusters %d', current_clusters(1));
      video_titles{end + 1} = sprintf('clusters %d', current_clusters(1));
      if should_videos_be_colorblind_friendly
        current_channel_index = mod(current_channel_index, 2) + 1;
        if current_clusters(2) > 0
          video_titles{end} = sprintf('%s, %d', video_titles{end}, current_clusters(2));
        end
      else
        current_channel_index = mod(current_channel_index, 3) + 1;
        if current_clusters(2) > 0
          video_titles{end} = sprintf('%s, %d', video_titles{end}, current_clusters(2));
        end
        if current_clusters(3) > 0
          video_titles{end} = sprintf('%s, %d', video_titles{end}, current_clusters(3));
        end
      end
    end
    
    % video_channels
    % video_titles
    % beep, keyboard
    
    % xruan 08/18/2015 
    % self-defined video channel pairs for video S11 in the paper.
    video_channels = {...
       {'Full Stimulus', 'ARP3', []; 'Full Stimulus', 'MRLC', []; 'Full Stimulus', 'Cofilin', []}; ...
       {'Full Stimulus', 'WAVE2', []; 'Full Stimulus', 'MRLC', []; 'Full Stimulus', 'Cofilin', []}; ...
      };
     
    % current_video_frame_index = 1;
    for video_index = 1:length(video_channels)
      current_video_channels = video_channels{video_index};
      current_video_title = video_titles{video_index};
      
      for all_relative_time_index = 1:number_all_relative_times
        relative_time = all_relative_times(all_relative_time_index);
        
        relative_time_image_rgb = [];
        current_video_frame_index = (video_index - 1) * number_all_relative_times + (all_relative_time_index - 1) + 1;

        for channel_index = 1:size(current_video_channels, 1)
          current_condition = current_video_channels{channel_index, 1};
          current_sensor = current_video_channels{channel_index, 2};
           
          % xruan 08/17/2015 
          % use condition abbreviation name as did in
          % master_script_generate_model_figures
          
          % image_filename = sprintf('%s_%s_reltime% 3d_%s', current_condition, current_sensor, relative_time, model_type);
          % image_filename = sprintf('%s%s_%s_frame%03d_reltime% 3d_%s', master_script_options.model_prefix, current_condition, current_sensor, all_relative_time_index, relative_time, model_type);
          image_filename = sprintf('%s%s_%s_frame%03d_reltime% 3d_%s', master_script_options.model_prefix, condition_name_abbreviation_function(current_condition), current_sensor, all_relative_time_index, relative_time, model_type);
          
          grayscale_image_full_filename = [videos_location, image_filename, '_grayscale.png'];
          if ~exist(grayscale_image_full_filename, 'file')
            % beep, keyboard
            continue
          end
          grayscale_image = double(imread(grayscale_image_full_filename));
          grayscale_image = contrast_stretch(grayscale_image);
          % grayscale_image = imresize(grayscale_image, master_script_options.videos_scale, 'bilinear');
          grayscale_image = imresize(grayscale_image, master_script_options.videos_scale, 'nearest');
          
          master_script_options.videos_contrast_enhancement_method = 'top_intensity25';
          current_video_title = '';
          
          switch master_script_options.videos_contrast_enhancement_method
            case 'exp-scale'
              grayscale_image = exp(grayscale_image);
              grayscale_image(~isfinite(grayscale_image)) = min(grayscale_image(isfinite(grayscale_image)));
              grayscale_image = contrast_stretch(grayscale_image);
              
            case 'log-scale'
              grayscale_image = log(grayscale_image);
              grayscale_image(~isfinite(grayscale_image)) = min(grayscale_image(isfinite(grayscale_image)));
              grayscale_image = contrast_stretch(grayscale_image);
              
            case 'threshold'
              if ~false
                % Debug info:
                close all
                figure,
                imshow([grayscale_image; grayscale_image >= graythresh(grayscale_image(grayscale_image > min(grayscale_image(:))))], [])
                 beep
                pause(1)
                % keyboard
              end
              % grayscale_image = grayscale_image >= graythresh(grayscale_image(template_image >= .5));
              % Hacky, should use a properly reshaped template_crop_function of template_image:        
              grayscale_image = grayscale_image >= graythresh(grayscale_image(grayscale_image > min(grayscale_image(:))));
              grayscale_image = double(grayscale_image);
           
           case 'top_intensity25'
             percetile_val = 0.75;
             grayscale_image(145 : 192, 231 : 276) = 0;
             intensity_val = grayscale_image(grayscale_image > min(grayscale_image(:)));
             intensity_val_sorted = sort(intensity_val);
             intensity_cdf = cumsum(intensity_val_sorted) / sum(intensity_val_sorted);
             [~, ind] = min(abs(intensity_cdf - percetile_val));
             threshold = intensity_val_sorted(ind);
             if false
                % Debug info:
                graythresh(grayscale_image(grayscale_image > min(grayscale_image(:))));
                threshold
                close all
                figure,
                imshow([grayscale_image; double(grayscale_image > threshold)], []);
                beep
                pause(1)
                % keyboard
             end
             
             grayscale_image = double(grayscale_image > threshold);            
            case 'top_intensity35'
             percetile_val = 0.65;
             grayscale_image(145 : 192, 231 : 276) = 0;
             intensity_val = grayscale_image(grayscale_image > min(grayscale_image(:)));
             intensity_val_sorted = sort(intensity_val);
             intensity_cdf = cumsum(intensity_val_sorted) / sum(intensity_val_sorted);
             [~, ind] = min(abs(intensity_cdf - percetile_val));
             threshold = intensity_val_sorted(ind);
             if false
                % Debug info:
                graythresh(grayscale_image(grayscale_image > min(grayscale_image(:))));
                threshold
                close all
                figure,
                imshow([grayscale_image; double(grayscale_image > threshold)], []);
                beep
                pause(1)
                % keyboard
              end
                grayscale_image = double(grayscale_image > threshold);
            case 'top_intensity40'
             percetile_val = 0.6;
             grayscale_image(145 : 192, 231 : 276) = 0;
             intensity_val = grayscale_image(grayscale_image > min(grayscale_image(:)));
             intensity_val_sorted = sort(intensity_val);
             intensity_cdf = cumsum(intensity_val_sorted) / sum(intensity_val_sorted);
             [~, ind] = min(abs(intensity_cdf - percetile_val));
             threshold = intensity_val_sorted(ind);
             if ~false
                % Debug info:
                threshold
                sum(grayscale_image(grayscale_image > threshold)) / sum(grayscale_image(grayscale_image > 0))
                sum(grayscale_image(grayscale_image < threshold)) / sum(grayscale_image(grayscale_image > 0))
                length(grayscale_image > threshold) / length(grayscale_image > 0)
                length(grayscale_image < threshold) / length(grayscale_image > 0)
                close all
                figure,
                imshow([grayscale_image; double(grayscale_image > threshold)], []);
                beep
                pause(1)
                % keyboard
              end
                grayscale_image = double(grayscale_image > threshold);
                
            case 'top_intensity10'
             percetile_val = 0.9;
             grayscale_image(145 : 192, 231 : 276) = 0;
             intensity_val = grayscale_image(grayscale_image > min(grayscale_image(:)));
             intensity_val_sorted = sort(intensity_val);
             intensity_cdf = cumsum(intensity_val_sorted) / sum(intensity_val_sorted);
             [~, ind] = min(abs(intensity_cdf - percetile_val));
             threshold = intensity_val_sorted(ind);
             if false
                % Debug info:
                graythresh(grayscale_image(grayscale_image > min(grayscale_image(:))));
                threshold
                close all
                figure,
                imshow([grayscale_image; double(grayscale_image > threshold)], []);
                beep
                pause(1)
                % keyboard
              end
                grayscale_image = double(grayscale_image > threshold);
                
            case 'top_intensity5'
             percetile_val = 0.95;
             grayscale_image(145 : 192, 231 : 276) = 0;
             intensity_val = grayscale_image(grayscale_image > min(grayscale_image(:)));
             intensity_val_sorted = sort(intensity_val);
             intensity_cdf = cumsum(intensity_val_sorted) / sum(intensity_val_sorted);
             [~, ind] = min(abs(intensity_cdf - percetile_val));
             threshold = intensity_val_sorted(ind);
             if false
                % Debug info:
                graythresh(grayscale_image(grayscale_image > min(grayscale_image(:))));
                threshold
                close all
                figure,
                imshow([grayscale_image; double(grayscale_image > threshold)], []);
                beep
                pause(1)
                % keyboard
              end
                grayscale_image = double(grayscale_image > threshold);                
                
            case 'none'
              
            % case ''
              
            otherwise
              error
          end
          
          if isempty(relative_time_image_rgb)
            relative_time_image_rgb = zeros([size(grayscale_image), 3 * ones(1, 3 - ndims(grayscale_image))]);
          end
          if should_videos_be_colorblind_friendly
            relative_time_image_rgb(:, :, channel_index) = grayscale_image(:, :, 1);
            if channel_index == 1
              relative_time_image_rgb(:, :, 3) = grayscale_image(:, :, 1);
            end
          else
            relative_time_image_rgb(:, :, channel_index) = grayscale_image(:, :, 1);
          end
          
        end
        
        % whos relative_time_image_rgb
        
        if master_script_options.videos_show_all_channels
          % Now put all three channels separately, then together:
          % relative_time_image_rgb_with_separate_images = [repmat(relative_time_image_rgb(:, : 1), [1, 1, 3]), repmat(relative_time_image_rgb(:, : 2), [1, 1, 3]), repmat(relative_time_image_rgb(:, : 3), [1, 1, 3]), relative_time_image_rgb];
          relative_time_image_rgb_with_separate_images = zeros(size(relative_time_image_rgb) .* [1, 4, 1]);
          relative_time_image_rgb_with_separate_images(:, size(relative_time_image_rgb, 2) * 0 + (1:size(relative_time_image_rgb, 2)), 1) = relative_time_image_rgb(:, :, 1);
          relative_time_image_rgb_with_separate_images(:, size(relative_time_image_rgb, 2) * 1 + (1:size(relative_time_image_rgb, 2)), 2) = relative_time_image_rgb(:, :, 2);
          relative_time_image_rgb_with_separate_images(:, size(relative_time_image_rgb, 2) * 2 + (1:size(relative_time_image_rgb, 2)), 3) = relative_time_image_rgb(:, :, 3);
          relative_time_image_rgb_with_separate_images(:, size(relative_time_image_rgb, 2) * 3 + (1:size(relative_time_image_rgb, 2)), :) = relative_time_image_rgb;
        else
          relative_time_image_rgb_with_separate_images = relative_time_image_rgb;
        end
          
        
        % Draw headings:
        
        draw_text_font_size = 1; pad_size = 15;
        draw_text_font_size = 3; pad_size = 20;
        top_heading_lines = 1;
        left_heading_lines = 3;
        
        % Allow for titles:
        top_heading_prefix = '';
        if ~isempty(video_titles)
          top_heading_lines = top_heading_lines + 1;
          top_heading_prefix = current_video_title;
          % relative_time_prefix = sprintf('Relative time %d', relative_time)
          % relative_time_prefix = sprintf('Relative time %+ 4d', all_relative_times_seconds_rounded(all_relative_time_index));
          relative_time_prefix = sprintf('Time %+ 4d', all_relative_times_seconds_rounded(all_relative_time_index));
          if ~isempty(top_heading_prefix)
            top_heading_prefix = sprintf('%s; %s', relative_time_prefix, top_heading_prefix);
          else
            top_heading_prefix = relative_time_prefix;
          end
        else
            % xruan remove cluster X in the title
            relative_time_prefix = sprintf('Time %+ 4d', all_relative_times_seconds_rounded(all_relative_time_index));
            top_heading_prefix = relative_time_prefix;
        end
        
        % Pad to make room for headings:
        relative_time_image_rgb_with_separate_images = padarray(relative_time_image_rgb_with_separate_images, pad_size * [top_heading_lines, left_heading_lines], 'pre');
        
        % Draw sensor headings:
        draw_text_options = struct('FontSize', draw_text_font_size);
        if master_script_options.videos_show_all_channels
          if ~isempty(video_titles)
            % Title:
            relative_time_image_rgb_with_separate_images(1:pad_size * 1, :, 1:3) = repmat(draw_text(top_heading_prefix, relative_time_image_rgb_with_separate_images(1:pad_size * 1, :, 1), [1 + pad_size * left_heading_lines + size(relative_time_image_rgb, 2) * 0, 1], draw_text_options), [1, 1, 3]);
            % Headings:
            relative_time_image_rgb_with_separate_images(:, :, 1) = draw_text({'', current_video_channels{1, 2}}, relative_time_image_rgb_with_separate_images(:, :, 1), [1 + pad_size * left_heading_lines + size(relative_time_image_rgb, 2) * 0, 1], draw_text_options);
            relative_time_image_rgb_with_separate_images(:, :, 1) = draw_text({'', current_video_channels{2, 2}}, relative_time_image_rgb_with_separate_images(:, :, 1), [1 + pad_size * left_heading_lines + size(relative_time_image_rgb, 2) * 1, 1], draw_text_options);
            relative_time_image_rgb_with_separate_images(:, :, 1) = draw_text({'', current_video_channels{3, 2}}, relative_time_image_rgb_with_separate_images(:, :, 1), [1 + pad_size * left_heading_lines + size(relative_time_image_rgb, 2) * 2, 1], draw_text_options);
            relative_time_image_rgb_with_separate_images(:, :, 1) = draw_text({'', 'Overlaid'}, relative_time_image_rgb_with_separate_images(:, :, 1), [1 + pad_size * left_heading_lines + size(relative_time_image_rgb, 2) * 3, 1], draw_text_options);
          else
            relative_time_image_rgb_with_separate_images(:, :, 1) = draw_text(current_video_channels{1, 2}, relative_time_image_rgb_with_separate_images(:, :, 1), [1 + pad_size * left_heading_lines + size(relative_time_image_rgb, 2) * 0, 1], draw_text_options);
            relative_time_image_rgb_with_separate_images(:, :, 1) = draw_text(current_video_channels{2, 2}, relative_time_image_rgb_with_separate_images(:, :, 1), [1 + pad_size * left_heading_lines + size(relative_time_image_rgb, 2) * 1, 1], draw_text_options);
            relative_time_image_rgb_with_separate_images(:, :, 1) = draw_text(current_video_channels{3, 2}, relative_time_image_rgb_with_separate_images(:, :, 1), [1 + pad_size * left_heading_lines + size(relative_time_image_rgb, 2) * 2, 1], draw_text_options);
            relative_time_image_rgb_with_separate_images(:, :, 1) = draw_text('Overlaid', relative_time_image_rgb_with_separate_images(:, :, 1), [1 + pad_size * left_heading_lines + size(relative_time_image_rgb, 2) * 3, 1], draw_text_options);
          end
        else
          draw_text_centered_options = draw_text_options;
          draw_text_right_options = draw_text_options;
          draw_text_centered_options.alignment = 0;
          draw_text_right_options.alignment = 1;
          if ~isempty(video_titles)
            % Title:
            % beep, keyboard
            relative_time_image_rgb_with_separate_images(1:pad_size * 1, :, 1:3) = repmat(draw_text(top_heading_prefix, relative_time_image_rgb_with_separate_images(1:pad_size * 1, :, 1), [1 + pad_size * left_heading_lines + size(relative_time_image_rgb, 2) * 0, 1], draw_text_options), [1, 1, 3]);
            % Headings:
            relative_time_image_rgb_with_separate_images(:, :, 1) = draw_text({'', current_video_channels{1, 2}}, relative_time_image_rgb_with_separate_images(:, :, 1), [1 + pad_size * left_heading_lines + size(relative_time_image_rgb, 2) * 0, 1], draw_text_options);
            % whos relative_time_image_rgb_with_separate_images
            if size(current_video_channels, 1) >= 2
              relative_time_image_rgb_with_separate_images(:, :, 2) = draw_text({'', current_video_channels{2, 2}}, relative_time_image_rgb_with_separate_images(:, :, 2), [1 + pad_size * left_heading_lines + floor(size(relative_time_image_rgb, 2) * .5), 1], draw_text_centered_options);
            end
            if size(current_video_channels, 1) >= 3
              relative_time_image_rgb_with_separate_images(:, :, 3) = draw_text({'', current_video_channels{3, 2}}, relative_time_image_rgb_with_separate_images(:, :, 3), [1 + pad_size * left_heading_lines + size(relative_time_image_rgb, 2) * 1, 1], draw_text_right_options);
            end
          else
            relative_time_image_rgb_with_separate_images(:, :, 1) = draw_text(current_video_channels{1, 2}, relative_time_image_rgb_with_separate_images(:, :, 1), [1 + pad_size * left_heading_lines + size(relative_time_image_rgb, 2) * 0, 1], draw_text_options);
            % whos relative_time_image_rgb_with_separate_images
            if size(current_video_channels, 1) >= 2
              relative_time_image_rgb_with_separate_images(:, :, 2) = draw_text(current_video_channels{2, 2}, relative_time_image_rgb_with_separate_images(:, :, 2), [1 + pad_size * left_heading_lines + floor(size(relative_time_image_rgb, 2) * .5), 1], draw_text_centered_options);
            end
            if size(current_video_channels, 1) >= 3
              relative_time_image_rgb_with_separate_images(:, :, 3) = draw_text(current_video_channels{3, 2}, relative_time_image_rgb_with_separate_images(:, :, 3), [1 + pad_size * left_heading_lines + size(relative_time_image_rgb, 2) * 1, 1], draw_text_right_options);
            end
          end
        end
        if ~master_script_options.videos_color_headings
          relative_time_image_rgb_with_separate_images(1:pad_size * top_heading_lines, :, 2) = max(relative_time_image_rgb_with_separate_images(1:pad_size * top_heading_lines, :, :), [], 3);
          relative_time_image_rgb_with_separate_images(1:pad_size * top_heading_lines, :, 3) = max(relative_time_image_rgb_with_separate_images(1:pad_size * top_heading_lines, :, :), [], 3);
        end
        
        % Make the text more visible:
        relative_time_image_rgb_with_separate_images(1:pad_size * top_heading_lines, :, :) = relative_time_image_rgb_with_separate_images(1:pad_size * top_heading_lines, :, :) * 3;
        
        % Draw view headings (origin is below text, not above, and -3 is due to slight alignment issue):
        draw_text_options = struct('FontSize', draw_text_font_size, 'direction', 1);
        relative_time_image_rgb_with_separate_images(:, :, 1) = draw_text({'Slices', 'parallel to', 'top'}, relative_time_image_rgb_with_separate_images(:, :, 1), [1, 1 + pad_size * top_heading_lines + floor(size(relative_time_image_rgb, 1) * .5 * 1) - 3], draw_text_options);
        relative_time_image_rgb_with_separate_images(:, :, 1) = draw_text({'Slices', 'parallel to', 'synapse'}, relative_time_image_rgb_with_separate_images(:, :, 1), [1, 1 + pad_size * top_heading_lines + floor(size(relative_time_image_rgb, 1) * .5 * 2) - 3], draw_text_options);
        relative_time_image_rgb_with_separate_images(:, 1:pad_size * left_heading_lines, 2) = relative_time_image_rgb_with_separate_images(:, 1:pad_size * left_heading_lines, 1);
        relative_time_image_rgb_with_separate_images(:, 1:pad_size * left_heading_lines, 3) = relative_time_image_rgb_with_separate_images(:, 1:pad_size * left_heading_lines, 1);
        % Make the text more visible:
        relative_time_image_rgb_with_separate_images(:, 1:pad_size * left_heading_lines, :) = relative_time_image_rgb_with_separate_images(:, 1:pad_size * left_heading_lines, :) * 3;
        
        % Save a video frame for RGB and two-channel colorblind-friendly combinations in green and magenta as suggested at <http://jfly.iam.u-tokyo.ac.jp/color/index.html#stain>:
        
        current_sensors = current_video_channels(:, 2);
        % % video_frame_filename = sprintf('%s_%s_%s_%s_frame%03d_%s', current_video_channels{1, 1}, current_sensors{:}, all_relative_time_index, model_type);
        % video_frame_filename = sprintf('%s_%s_%s_%s_frame%03d_%s_%s', current_video_channels{1, 1}, current_sensors{:}, all_relative_time_index, model_type, master_script_options.videos_contrast_enhancement_method);
        video_frame_filename = sprintf('%shierarchical_clustering_%s_frame%03d_%s', master_script_options.model_prefix, model_type, current_video_frame_index, master_script_options.videos_contrast_enhancement_method);
        video_frame_full_filename = [videos_location, video_frame_filename, '.png'];
        % imwrite(relative_time_image_rgb, video_frame_full_filename)
        imwrite(relative_time_image_rgb_with_separate_images, video_frame_full_filename)
        
        % % whos relative_time_image_rgb
        
        % for channel_to_exclude = 1:3
          % % channel_to_exclude = 1, set 1,2,3 = 2,3,2; 2, 1,2,3 = 1,3,1; 3, 1,2,3 = 1,2,1:
          % remaining_channels = mod(channel_to_exclude + (1:2) - 1, 3) + 1;
          % magenta_channel = min(remaining_channels);
          % green_channel = max(remaining_channels);
          
          % % relative_time_image_mg = relative_time_image_rgb;
          % relative_time_image_mg = relative_time_image_rgb_with_separate_images;
          
          % relative_time_image_mg = relative_time_image_mg(:, :, [magenta_channel, green_channel, magenta_channel]);
          % if master_script_options.videos_show_all_channels
            % relative_time_image_mg(:, pad_size * left_heading_lines + (1:size(relative_time_image_rgb, 2)) + size(relative_time_image_rgb, 2) * (channel_to_exclude - 1), :) = [];
          % end
          
          % excluded_video_frame_filename = [video_frame_filename, sprintf('_channels%d%d', magenta_channel, green_channel)];
          % excluded_video_frame_full_filename = [videos_location, excluded_video_frame_filename, '.png'];
          % imwrite(relative_time_image_mg, excluded_video_frame_full_filename)
        % end
        
      end
      
    end
    
  end
  
  
  
  % beep, keyboard
  % error('Implementation yet unfinished below this line!')

  % t_cell_info.figures_info = struct();
  % t_cell_info.figures_info.registration_options = registration_options;
  
  
  
end  

