function [t_cell_info] = master_script_generate_pipeline_illustrations_figureS1_new(t_cell_info, options)
  % Produce the illustration of the image analysis pipeline for an
  % individual cell, which is figure 3A.
  %
  % 2015-07-12 xruan: Copied from  master_script_generate_pipeline_illustrations.m.
  % 2016-03-11 xruan: put compact code in to compact images in the same
  % function. 
  % 2016-03-20 xruan: add central slice projection method and fix a bug for
  % using aligned image
  % 
  % 
  % 
  % % Dependencies:
  % % From the File Exchange: export_fig, pmkmp, toolbox_graph, toolbox_fast_marching, affine, Snake3D, Mesh_voxelisation, dirr, inhull
  % % From tebuck: modified version of Jieyue Li's HPA_lib, rasterize_mesh

%
% Author: Taraz E. Buck and Xiongtao Ruan
%
% Copyright (C) 2012-2016 Murphy Lab
% Computational Biology Department
% School of Computer Science
% Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu



  
  
  
  % Variables referenced more than once can be copied to local variables:
  master_script_options = t_cell_info.options;
  regions_location = t_cell_info.path_info.regions_location;
  segmentations_filtered_location = t_cell_info.path_info.segmentations_filtered_location;
  alignments_location = t_cell_info.path_info.alignments_location;
  deformations_location = t_cell_info.path_info.deformations_location;
  models_location = t_cell_info.path_info.models_location;
  figures_location = t_cell_info.path_info.figures_location;
  videos_location = t_cell_info.path_info.videos_location;
  illustrations_location = t_cell_info.path_info.illustrations_location;
  originals_location = t_cell_info.path_info.originals_location;
  originals_with_landmarks_location = t_cell_info.path_info.originals_with_landmarks_location;
  figure3a_location = t_cell_info.path_info.figure3a_location;
  figureS1_location = t_cell_info.path_info.figureS1_location;
  
  illustration_colormap_to_use = t_cell_info.plotting_info.illustration_colormap_to_use;
  
%   model_types = t_cell_info.model_type_info.model_types;
%   number_model_types = t_cell_info.model_type_info.number_model_types;
%   model_representation_functions = t_cell_info.model_type_info.model_representation_functions;
%   model_reconstruction_functions = t_cell_info.model_type_info.model_reconstruction_functions;
%   model_dimensionality_reduction_methods = t_cell_info.model_type_info.model_dimensionality_reduction_methods;
%   model_dimensionality_reduction_method_parameters = t_cell_info.model_type_info.model_dimensionality_reduction_method_parameters;

  run_relative_path_list = t_cell_info.synapse_info.run_relative_path_list;
  run_key_list = t_cell_info.synapse_info.run_key_list;
  % run_data_list = t_cell_info.synapse_info.run_data_list;
  run_images_list = t_cell_info.synapse_info.run_images_list;
  run_brightfield_images_list = t_cell_info.synapse_info.run_brightfield_images_list;
  run_image_frame_indices_list = t_cell_info.synapse_info.run_image_frame_indices_list;
  run_number_cells = t_cell_info.synapse_info.run_number_cells;
  run_number_frames = t_cell_info.synapse_info.run_number_frames;
  run_point_tracks = t_cell_info.synapse_info.run_point_tracks;
  run_relative_time_tracks = t_cell_info.synapse_info.run_relative_time_tracks;
  run_relative_times = t_cell_info.synapse_info.run_relative_times;
  run_number_relative_times = t_cell_info.synapse_info.run_number_relative_times;
  all_relative_times = t_cell_info.synapse_info.all_relative_times;
  number_all_relative_times = t_cell_info.synapse_info.number_all_relative_times;
  all_relative_times_seconds_data = t_cell_info.synapse_info.all_relative_times_seconds_data;
  all_relative_times_seconds = t_cell_info.synapse_info.all_relative_times_seconds;
  all_relative_times_seconds_rounded = t_cell_info.synapse_info.all_relative_times_seconds_rounded;
  number_runs = t_cell_info.synapse_info.number_runs;
  condition_sensor_combinations = t_cell_info.synapse_info.condition_sensor_combinations;
  number_condition_sensors = t_cell_info.synapse_info.number_condition_sensors;
  conditions = t_cell_info.synapse_info.conditions;
  sensors = t_cell_info.synapse_info.sensors;
  number_conditions = t_cell_info.synapse_info.number_conditions;
  number_sensors = t_cell_info.synapse_info.number_sensors;
  
  template_centroid = t_cell_info.template_info.template_centroid;
  template_synapse = t_cell_info.template_info.template_synapse;
  template_image = t_cell_info.template_info.template_image;
  template_crop_function = t_cell_info.template_info.template_crop_function;

  window_size_2d = t_cell_info.preprocessing_info.window_size_2d;
  window_center_2d = t_cell_info.preprocessing_info.window_center_2d;
  cropped_size = t_cell_info.preprocessing_info.cropped_size;

  segmentation_rasterization_function = t_cell_info.segmentation_info.segmentation_rasterization_function;
  
%   model_original_representation_functions = t_cell_info.model_info.model_original_representation_functions;
%   model_original_reconstruction_functions = t_cell_info.model_info.model_original_reconstruction_functions;
%   model_reduction_representation_functions = t_cell_info.model_info.model_reduction_representation_functions;
%   model_reduction_reconstruction_functions = t_cell_info.model_info.model_reduction_reconstruction_functions;
  
%   all_model_distances = t_cell_info.clustering_info.all_model_distances;
%   all_model_trees = t_cell_info.clustering_info.all_model_trees;
%   all_model_clusters = t_cell_info.clustering_info.all_model_clusters;
%   clustering_distance_metrics = t_cell_info.clustering_info.clustering_distance_metrics;


  function [result] = crop_slices_or_pad(given_image_to_modify, given_image_for_comparison)
    if size(given_image_to_modify, 3) > size(given_image_for_comparison, 3)
      result = given_image_to_modify(:, :, floor((end - size(given_image_for_comparison, 3)) / 2) + (1:size(given_image_for_comparison, 3)));
    else
      result = padarray(given_image_to_modify, [0, 0, floor((size(given_image_for_comparison, 3) - size(given_image_to_modify, 3)) / 2)], 'pre');
      result = padarray(result, [0, 0, size(given_image_for_comparison, 3) - size(result, 3)], 'post');
    end
    if false
      % Debug info:
      whos given_image_to_modify given_image_for_comparison result, beep, keyboard
    end
  end
  
  
  
  % Go through frames which have regions, segmentations, alignments, and deformations and show the entire process in one image:
  
  
  % relative_times_to_illustrate = all_relative_times;
  relative_times_to_illustrate = 2;
  
  condition_sensor_combinations = t_cell_info.synapse_info.condition_sensor_combinations;
  combinations_to_illustrate = condition_sensor_combinations;
  % combinations_to_illustrate = condition_sensor_combinations(strcmp(condition_sensor_combinations(:, 2), 'Actin'), :);
  
  
  s = RandStream('mt19937ar', 'Seed', 691551);
  %RandStream.setDefaultStream(s);
  RandStream.setGlobalStream(s);
  
  number_cell_frames_to_illustrate = 200;
  number_cell_frames_illustrated = 0;
  
  % deformation_list = dir([deformations_location, '*.mat']);
  % deformation_list = {deformation_list.name}.';
  % deformation_list = strcat(deformations_location, deformation_list);
  % cell_frames_to_illustrate = deformation_list(randperm(length(deformation_list)) <= number_cell_frames_to_illustrate);
  % In case of deformation failure for a method, this list should stay the same for the purpose of comparison, e.g., of alignment methods:
  region_list = dir([regions_location, '*.mat']);
  region_list = {region_list.name}.';
  region_list = strcat(regions_location, region_list);
  % cell_frames_to_illustrate = region_list(randperm(length(region_list)) <= number_cell_frames_to_illustrate);
  cell_frames_to_illustrate_indices = randperm(length(region_list));
  % cell_frames_to_illustrate_indices = cell_frames_to_illustrate_indices(cell_frames_to_illustrate_indices <= number_cell_frames_to_illustrate);
  cell_frames_to_illustrate = region_list(cell_frames_to_illustrate_indices);
  
  % keyboard

  
  close
  
  
  % Write just the template's shape:
  image_to_show = t_cell_info.template_info.template_image;
  % image_to_show = reshape_2d(image_to_show, -1);
  image_to_show = reshape_2d(image_to_show, 1);
  % Recolor the image:
  image_to_show = ind2rgb(floor(contrast_stretch(image_to_show) .* (size(illustration_colormap_to_use, 1) - 1)) + 1, illustration_colormap_to_use);
  
  image_filename = [figureS1_location, sprintf('%stemplate', master_script_options.model_prefix)];
  imwrite(image_to_show, [image_filename, '.png'])
  fprintf('Wrote illustration template.png\n')
  
  
  
  % for cell_frame_index = 1:length(cell_frames_to_illustrate)
  cell_frame_index = 0;
  while number_cell_frames_illustrated < number_cell_frames_to_illustrate && cell_frame_index < size(cell_frames_to_illustrate, 1)
    % Always increment cell_frame_index:
    cell_frame_index = cell_frame_index + 1;
    
    cell_illustration_filename = strrep(cell_frames_to_illustrate{cell_frame_index}(1:end - 4), regions_location, '');
    cell_illustration_filename_without_model_prefix = cell_illustration_filename;
    cell_illustration_filename = [master_script_options.model_prefix, cell_illustration_filename];
    
    current_condition = t_cell_info.synapse_info.condition_retrieval_function({cell_illustration_filename});
    current_sensor = t_cell_info.synapse_info.sensor_retrieval_function({cell_illustration_filename});
    if ~t_cell_info.synapse_info.condition_sensor_combination_ismember([current_condition, current_sensor], combinations_to_illustrate)
      continue
    end
    
    % cell_deformation_aligned_filename = [deformations_aligned_location, cell_illustration_filename_without_model_prefix];
    cell_deformation_filename = [deformations_location, cell_illustration_filename_without_model_prefix];
    cell_alignment_filename = [alignments_location, cell_illustration_filename_without_model_prefix];
    cell_segmentation_filename = [segmentations_filtered_location, cell_illustration_filename_without_model_prefix];
    cell_region_filename = [regions_location, cell_illustration_filename_without_model_prefix];
    % cell_original_filename = [originals_location, sprintf('%s%s', master_script_options.model_prefix, cell_illustration_filename_without_model_prefix)];
    cell_original_filename = [originals_location, cell_illustration_filename];
    % cell_original_with_landmarks_filename = [originals_with_landmarks_location, sprintf('%s%s', master_script_options.model_prefix, cell_illustration_filename_without_model_prefix)];
    cell_original_with_landmarks_filename = [originals_with_landmarks_location, cell_illustration_filename];
    
    pipeline_illustration_filename = [figureS1_location, cell_illustration_filename];
    % cell_illustration_filename = [illustrations_location, sprintf('%s%s', master_script_options.model_prefix, cell_illustration_filename_without_model_prefix)];
    cell_illustration_filename = [illustrations_location, cell_illustration_filename];
    
    % cell_illustration_filename = [figure3a_location, cell_illustration_filename];
    
    
    % % existences = [exist([cell_deformation_aligned_filename, '.mat'], 'file'), exist([cell_deformation_filename, '.mat'], 'file'), exist([cell_alignment_filename, '.mat'], 'file'), exist([cell_segmentation_filename, '.mat'], 'file'), exist([cell_region_filename, '.mat'], 'file')];
    % This implies all of the above:
    % existences = [exist([cell_deformation_aligned_filename, '.mat'], 'file')];
    existences = [exist([cell_deformation_filename, '.mat'], 'file')];
    
    if false
      % Debug info:
      fprintf('cell_deformation_filename = "%s"\n', cell_deformation_filename)
      beep, keyboard
    end
    
    if ~all(existences)
      continue
    end
    
    if false
      % Debug info:
      fprintf('cell_deformation_filename = "%s"\n', cell_deformation_filename)
      beep, keyboard
    end
    
    a = load([cell_region_filename, '.mat']);
    current_window_image = a.current_window_image;
    current_window_dic_image = a.current_window_dic_image;
    current_relative_time = a.current_relative_time;
    if false
      % Debug info:
      fprintf('current_relative_time = %d\n', current_relative_time)
    end
    if ~ismember(current_relative_time, relative_times_to_illustrate)
      continue
    end
    
    if numel(current_window_dic_image) == 0
      continue
    end

    a = load([cell_segmentation_filename, '.mat']);
    if isempty(fieldnames(a))
      continue
    end
    segmentation_structure = a.segmentation_structure;
    window_number_slices = a.window_number_slices;

    a = load([cell_alignment_filename, '.mat']);
    if isempty(fieldnames(a))
      continue
    end
    current_transform = a.current_transform;
    current_transform_homogeneous = a.current_transform_homogeneous;
    current_landmarks = a.current_landmarks;
    cropped_segmentation_image = a.cropped_segmentation_image;
    cropped_raw_image = a.cropped_raw_image;
    % a
    % keyboard

    a = load([cell_deformation_filename, '.mat']);
    if isempty(fieldnames(a))
      continue
    end
    cropped_raw_image_deformed = a.cropped_raw_image_deformed;
    cropped_segmentation_image_deformed = a.cropped_segmentation_image_deformed;
    
    if sum(cropped_segmentation_image_deformed(:) >= .5) == 0
      continue
    end
    
    %  xruan 03/20/2016
    if ~false
        % if ~ any(cellfun(@isempty, a.r.source_deformation))
        a_x = a.r.source_deformation{1}.image{1};
        a_y = a.r.source_deformation{2}.image{1};
        a_z = a.r.source_deformation{3}.image{1};
        
        [x, y, z] = meshgrid([1 : 71], 1 : 71, 1 : 35);
        vec_x = a_x;
        vec_y = a_y;
        vec_z = a_z;
        cropped_segmentation_image_central = cropped_segmentation_image(:, :, 18);
        vec_x_cen = vec_x(:, :, 18);
        vec_y_cen = vec_y(:, :, 18);
        vec_z_cen = vec_z(:, :, 18) - 18;
        vec_x_cen_use = vec_x_cen(cropped_segmentation_image_central > 0.5);
        vec_y_cen_use = vec_y_cen(cropped_segmentation_image_central > 0.5);
        vec_z_cen_use = vec_z_cen(cropped_segmentation_image_central > 0.5);
        
        XY = [vec_x_cen_use, vec_y_cen_use, ones(size(vec_y_cen_use))];
        w = (XY' * XY) \ (XY' * vec_z_cen_use);
        v_vec = [-w(1), -w(2), 1]';
        v_vec = v_vec / norm(v_vec, 2);
        b_vec = [0, 0, w(3)]';
        XYZ = [vec_x_cen_use, vec_y_cen_use, vec_z_cen_use];
        % project all points to the average plane
        pos_new = XYZ - (bsxfun(@minus, XYZ, b_vec') * v_vec) * v_vec';
        V = cropped_raw_image(:, :, 18);
        V_use = V(cropped_segmentation_image_central > 0.5);
        
        % rotation back to z = c plan
        
        v1 = [0, 0, 1]';
        k = cross(v_vec, v1);
        sin_theta = norm(k, 2);
        cos_theta = v_vec' * v1;
        k = k / norm(k, 2);
        K = [0, -k(3), k(2); k(3), 0, - k(1); -k(2), k(1), 0];
        pos_new_back = pos_new + sin_theta * pos_new * K' + (1 - cos_theta) * pos_new * K' * K';
        center = [36, 36];
        z_center = w' * [center';1];
        center_new = [center, z_center] + sin_theta * [center, z_center] * K' + (1 - cos_theta) * [center, z_center] * K' * K';
        % keep the back projection is translation-invariant
        pos_new_back = bsxfun(@plus, pos_new_back, [center, z_center] - center_new);
        
        [xq, yq] = meshgrid([1 : 71], 1 : 71);
        % x_unq = sort(unique(pos_new_back(:, 1)));
        % y_unq = sort(unique(pos_new_back(:, 2)));
        % [x_mesh, y_mesh] = meshgrid(x_unq, y_unq);
        
        % V = zeros(size(x_mesh));
        % x_sub = arrayfun(@(x) find(x_unq == x), pos_new_back(:, 1));
        % y_sub = arrayfun(@(y) find(y_unq == y), pos_new_back(:, 2));
        % V(sub2ind(size(V), y_sub, x_sub)) = V_use;
        
        Vq = griddata(pos_new_back(:, 1),pos_new_back(:, 2),V_use, xq, yq, 'linear');
        Vq(isnan(Vq)) = min(V_use);
        
        % [~, ~, current_transform] = procrustes(template_landmarks, current_landmarks, 'Reflection', false, 'Scaling', false);
        
        aligned_raw_center_projection = Vq;
        
        % figure, imshow(reshape_2d(round(a.r.source_deformation{3}.image{1}) == 18), []);

    end      
    
    
    [~, cell_illustration_filename_without_path, ~] = fileparts(cell_deformation_filename);
    [can_start, final_name, final_exists] = chunk_start(figureS1_location, cell_illustration_filename_without_path);
    if can_start
        
      % error
      
      should_show_aligned_steps = false;
      % should_show_aligned_steps = true;

      image_to_show = [];
      % Unaligned intensity image:
      if false
        % Debug info:
        imshow(reshape_contrast(current_window_image, -1))
        imshow(reshape_contrast(cropped_raw_image, -1))
      end
      current_window_image_central_slices = crop_slices_or_pad(current_window_image, cropped_raw_image);
      image_to_show = [image_to_show; contrast_stretch(current_window_image_central_slices)];
      % xruan 06/24/2015
      % padding the cropped_raw_image and cropped_segmentation_image in both side rather than the post
      % side in original case
%       if window_number_slices < size(template_image, 3)
%           cropped_raw_image = crop_slices_or_pad(cropped_raw_image(:, :, 1 : window_number_slices), current_window_image_central_slices);
%           cropped_segmentation_image = crop_slices_or_pad(cropped_segmentation_image(:, :, 1 : window_number_slices), current_window_image_central_slices);
%       end
      if should_show_aligned_steps
        % Aligned intensity image:
        image_to_show = [image_to_show; contrast_stretch(cropped_raw_image)];
      else
        % Unaligned segmentation image:
        cropped_segmentation_image_without_transformation = segmentation_rasterization_function(segmentation_structure, [t_cell_info.preprocessing_info.window_size_2d, window_number_slices]);
        % % cropped_segmentation_image_without_transformation = segmentation_rasterization_function(segmentation_structure);
        % cropped_segmentation_image_without_transformation = cropped_segmentation_image_without_transformation(:, :, floor((end - size(cropped_raw_image, 3)) / 2) + (1:size(cropped_raw_image, 3)));
        cropped_segmentation_image_without_transformation = crop_slices_or_pad(cropped_segmentation_image_without_transformation, cropped_raw_image);
        image_to_show = [image_to_show; cropped_segmentation_image_without_transformation];
        % Unaligned segmented intensity image:
        image_to_show = [image_to_show; contrast_stretch(current_window_image_central_slices .* (cropped_segmentation_image_without_transformation >= .5) + min(current_window_image_central_slices(cropped_segmentation_image_without_transformation >= .5)) .* (cropped_segmentation_image_without_transformation < .5))];
      end
      % Aligned segmentation image:
%       if draw_landmarks_on_illustrations
%         image_to_show = [image_to_show; cropped_segmentation_image_with_landmarks];
%       else
%         image_to_show = [image_to_show; cropped_segmentation_image];
%       end
      image_to_show = [image_to_show; cropped_segmentation_image];
      % Aligned segmented intensity image:
      image_to_show = [image_to_show; contrast_stretch(cropped_raw_image .* (cropped_segmentation_image >= .5) + min(cropped_raw_image(cropped_segmentation_image >= .5)) .* (cropped_segmentation_image < .5))];
      % Standardized segmented intensity image:
      image_to_show = [image_to_show; contrast_stretch(cropped_raw_image_deformed .* (cropped_segmentation_image_deformed >= .5) + min(cropped_raw_image_deformed(cropped_segmentation_image_deformed >= .5)) .* (cropped_segmentation_image_deformed < .5))];
      % image_to_show = reshape_2d(image_to_show, -1);
      image_to_show = reshape_2d(image_to_show, 1);
      
      
      
      %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      % needed images
      % current_window_dic_image: DIC 1
      % current_window_image: fluorescent image 2
      % segmented_raw_image: 4
      % cropped_segmentation_image_without_transformation: 3 ?
      % cropped_raw_image: segmenated image with intensity aligned image
      % cropped_segmentation_image: segmenated image 5
      % template_image: 7
      % cropped_raw_image_deformed: 8
      
      rand_slice_number = randi(window_number_slices) + 7;
      rand_slice_number = 18;
      segmented_raw_image = cropped_segmentation_image_without_transformation .* current_window_image_central_slices;
      segmented_raw_image_aligned = (cropped_segmentation_image > 0.5) .* cropped_raw_image +  min( cropped_raw_image((cropped_segmentation_image > 0.5))) * (cropped_segmentation_image < 0.5) ;
      segmented_raw_image_aligned = contrast_stretch(segmented_raw_image_aligned);
      segmented_raw_image_deformed = cropped_segmentation_image_deformed .* cropped_raw_image_deformed;
      contrast_stretch_current_window_image_central_slices = contrast_stretch(current_window_image_central_slices);
      contrast_stretch_segmented_raw_image = contrast_stretch(segmented_raw_image);
      contrast_stretch_segmented_raw_image_deformed = contrast_stretch(segmented_raw_image_deformed);
              
      max_intensity = max(current_window_image_central_slices(:));
      min_intensity = min(current_window_image_central_slices(current_window_image_central_slices > 0));
      
      image_contrast_stretch_function = @(img) (img - min_intensity) / ( max_intensity - min_intensity);
      
      current_window_image_central_slices = image_contrast_stretch_function(current_window_image_central_slices);
      segmented_raw_image = image_contrast_stretch_function(segmented_raw_image);
      % segmented_raw_image_aligned = image_contrast_stretch_function(segmented_raw_image_aligned);
      segmented_raw_image_deformed = image_contrast_stretch_function(segmented_raw_image_deformed); 
      
      
      image_1 = current_window_dic_image;
      image_2 = current_window_image_central_slices(:, :, rand_slice_number);
      image_3 = cropped_segmentation_image_without_transformation(:, :, rand_slice_number);
      image_4 = segmented_raw_image(:, :, rand_slice_number);
      image_5 = cropped_segmentation_image(:, :, rand_slice_number);
      image_6 = segmented_raw_image_aligned(:, :, rand_slice_number);
      image_7 = template_image(:, :, rand_slice_number);
      image_8 = segmented_raw_image_deformed(:, :, rand_slice_number);
      
      
      % image_final = [image_2; image_3; image_4; image_8];
      image_final = [image_2; image_3; image_6; contrast_stretch(aligned_raw_center_projection)];
      
      positions = [
      0.1, 0.76, 0.8, 0.2;   
      0.1, 0.52, 0.8, 0.2; 
      0.1, 0.28, 0.8, 0.2;   
      0.1, 0.04, 0.8, 0.2;   
      ];
            
      
      close all; 
      figure('Color', illustration_colormap_to_use(1, :))
      set(gcf, 'Visible', 'off')
      % set(gcf, 'Visible', 'on')
      set(gcf, 'Renderer', 'opengl');

      dpi = master_script_options.figure_dpi;
      figure_height = 500; %round(master_script_options.figure_maximum_width / aspect_ratio * figure_height_scale);
      figure_width = 150; %round(master_script_options.figure_maximum_width * figure_height_scale);
      set(gcf, 'Position', [1, 1, figure_width, figure_height])
      
      %matrix_plot_handle = gca;
%       subplot(4, 1, 1)
%       imshow(image_1, []);
%       freezeColors;
       
      axes('Position', positions(1, :))      
      imagesc(image_2);
      axis off, colormap(illustration_colormap_to_use)
      caxis([0, 1]);
      hold on

      axes('Position', positions(2, :))      
      imagesc(image_3);
      axis off, colormap(illustration_colormap_to_use)
      caxis([0, 1]);
      hold on

      axes('Position', positions(3, :))      
      imagesc(image_6);
      axis off, colormap(illustration_colormap_to_use)
      caxis([0, 1]);
      hold on
      
      axes('Position', positions(4, :))      
      imagesc(image_8);
      axis off, colormap(illustration_colormap_to_use)
      caxis([0, 1]);
      hold off     

      export_fig(pipeline_illustration_filename, '-opengl', '-png', '-a1', ['-r', num2str(dpi)])
      %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      
%       close all; 
%       figure('Color',  illustration_colormap_to_use(1, :))
%       set(gcf, 'Visible', 'off')
%       set(gcf, 'Visible', 'on')
%       set(gcf, 'Renderer', 'opengl');
%       figure_position = get(gcf, 'Position');
% 
%       dpi = master_script_options.figure_dpi;
%       figure_height = 500; %round(master_script_options.figure_maximum_width / aspect_ratio * figure_height_scale);
%       figure_width = 150; %round(master_script_options.figure_maximum_width * figure_height_scale);
%       set(gcf, 'Position', [1, 1, figure_width, figure_height])
%       imagesc(image_final)
%       axis off
%       colormap(illustration_colormap_to_use)
%       export_fig([pipeline_illustration_filename, '_integrate'], '-opengl', '-png', '-a1', ['-r', num2str(dpi)]);
      % imwrite(image_final, [pipeline_illustration_filename, '.tiff']);
      %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      
      chunk_finish(figureS1_location, cell_illustration_filename_without_path)
      
    end
    
    number_cell_frames_illustrated = number_cell_frames_illustrated + 1;
  end
  
  
  
  % compact the images
  
illustration_colormap_to_use = t_cell_info.plotting_info.illustration_colormap_to_use;

illustration_colormap_to_use = pmkmp(1024, 'CubicL');

figures_path = figureS1_location;

figures_info = dirr(figures_path);

figures_name = {figures_info.name}';
%03/12/2016
regex_pattern = 'Actin.*png';

file_mat = cellfun(@(x) regexp(x, regex_pattern), figures_name, 'UniformOutput', false);

file_names_filtered = figures_name(cell2mat(cellfun(@(x) ~isempty(x), file_mat, 'UniformOutput', false)));

s = RandStream('mt19937ar', 'Seed', 691551);
%RandStream.setDefaultStream(s);
RandStream.setGlobalStream(s);

figures_shown_index = randperm(length(file_names_filtered));

figures_name_shown = file_names_filtered(figures_shown_index(1 : 200));


positions = zeros(10, 4);
positions(:, 3) = 0.164;
positions(:, 4) = 0.47;
positions(1 : 5, 1) = 0.01 : 0.184 : 0.8;
positions(6 : 10, 1) = 0.01 : 0.184 : 0.8;
positions(1 : 5, 2) = 0.52;
positions(6 : 10, 2) = 0.01;



for i = 1 : 20
    for j = 1 : 10
        curr_ind = j + 10 * (i - 1);
        current_figure_name = figures_name_shown{curr_ind};

        current_figure = imread([figures_path, current_figure_name]);
        ind = j;
        if ind == 1

            close all 
            figure('Color', illustration_colormap_to_use(1, :))
            figure_width = 900;
            figure_height = 1200;
            set(gcf, 'Position', [1, 1, figure_width, figure_height])
            set(gcf, 'Visible', 'off')
            % set(gcf, 'Visible', 'on')
            set(gcf, 'Renderer', 'painters');
        end
        %iptsetpref('ImshowBorder','tight')
        axes('Position', positions(ind, :))
        imshow(current_figure);
        hold on;

        if ind == 5
          h = colorbar;
          set(h, 'Position', [.93 0.52 .05 .47]);
          axis off
          caxis([0, 1]);
          set(h, 'Ticks', []);
          colormap(illustration_colormap_to_use);
          hold off;
        end
        if ind == 10
          h = colorbar;
          set(h, 'Position', [.93 0.01 .05 .47]);
          axis off
          set(h, 'Ticks', [])
          caxis([0, 1]);
          colormap(illustration_colormap_to_use);
          hold off;
          % change to 150dpi 03/12/2016
          dpi = 150;
          final_figure_name = sprintf('figure_%s-%s', num2str(curr_ind - 9), num2str(curr_ind));
          export_fig([figures_path, final_figure_name], '-opengl',  '-nocrop', '-png', '-a1', ['-r', num2str(dpi)]);
        end
    end
end

  
end