function [t_cell_info] = master_script_generate_segmentation_illustrations(t_cell_info, options)
  % Produce videos of constructed models for visual evaluation.
  %
  % 2014-06-25 tebuck: Copied from master_script_generate_model_videos.m.
  % 
  % 
  % 
  % 
  % 
  % % Dependencies:
  % % From the File Exchange: export_fig, pmkmp, toolbox_graph, toolbox_fast_marching, affine, Snake3D, Mesh_voxelisation, dirr, inhull
  % % From tebuck: modified version of Jieyue Li's HPA_lib, rasterize_mesh

%
% Author: Taraz E. Buck and Xiongtao Ruan
%
% Copyright (C) 2012-2016 Murphy Lab
% Computational Biology Department
% School of Computer Science
% Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu



  
  
  
  % Variables referenced more than once can be copied to local variables:
  master_script_options = t_cell_info.options;
  regions_location = t_cell_info.path_info.regions_location;
  segmentations_filtered_location = t_cell_info.path_info.segmentations_filtered_location;
  alignments_location = t_cell_info.path_info.alignments_location;
  deformations_location = t_cell_info.path_info.deformations_location;
  models_location = t_cell_info.path_info.models_location;
  figures_location = t_cell_info.path_info.figures_location;
  videos_location = t_cell_info.path_info.videos_location;
  illustrations_location = t_cell_info.path_info.illustrations_location;
  segmentation_illustrations_location = [t_cell_info.options.results_location, 'segmentation_illustrations/'];
  originals_location = t_cell_info.path_info.originals_location;
  originals_with_landmarks_location = t_cell_info.path_info.originals_with_landmarks_location;

  illustration_colormap_to_use = t_cell_info.plotting_info.illustration_colormap_to_use;
  
%   model_types = t_cell_info.model_type_info.model_types;
%   number_model_types = t_cell_info.model_type_info.number_model_types;
%   model_representation_functions = t_cell_info.model_type_info.model_representation_functions;
%   model_reconstruction_functions = t_cell_info.model_type_info.model_reconstruction_functions;
%   model_dimensionality_reduction_methods = t_cell_info.model_type_info.model_dimensionality_reduction_methods;
%   model_dimensionality_reduction_method_parameters = t_cell_info.model_type_info.model_dimensionality_reduction_method_parameters;

  run_relative_path_list = t_cell_info.synapse_info.run_relative_path_list;
  run_key_list = t_cell_info.synapse_info.run_key_list;
  % run_data_list = t_cell_info.synapse_info.run_data_list;
  run_images_list = t_cell_info.synapse_info.run_images_list;
  run_brightfield_images_list = t_cell_info.synapse_info.run_brightfield_images_list;
  run_image_frame_indices_list = t_cell_info.synapse_info.run_image_frame_indices_list;
  run_number_cells = t_cell_info.synapse_info.run_number_cells;
  run_number_frames = t_cell_info.synapse_info.run_number_frames;
  run_point_tracks = t_cell_info.synapse_info.run_point_tracks;
  run_relative_time_tracks = t_cell_info.synapse_info.run_relative_time_tracks;
  run_relative_times = t_cell_info.synapse_info.run_relative_times;
  run_number_relative_times = t_cell_info.synapse_info.run_number_relative_times;
  all_relative_times = t_cell_info.synapse_info.all_relative_times;
  number_all_relative_times = t_cell_info.synapse_info.number_all_relative_times;
  all_relative_times_seconds_data = t_cell_info.synapse_info.all_relative_times_seconds_data;
  all_relative_times_seconds = t_cell_info.synapse_info.all_relative_times_seconds;
  all_relative_times_seconds_rounded = t_cell_info.synapse_info.all_relative_times_seconds_rounded;
  number_runs = t_cell_info.synapse_info.number_runs;
  condition_sensor_combinations = t_cell_info.synapse_info.condition_sensor_combinations;
  number_condition_sensors = t_cell_info.synapse_info.number_condition_sensors;
  conditions = t_cell_info.synapse_info.conditions;
  sensors = t_cell_info.synapse_info.sensors;
  number_conditions = t_cell_info.synapse_info.number_conditions;
  number_sensors = t_cell_info.synapse_info.number_sensors;
  
  template_centroid = t_cell_info.template_info.template_centroid;
  template_synapse = t_cell_info.template_info.template_synapse;
  template_image = t_cell_info.template_info.template_image;
  template_crop_function = t_cell_info.template_info.template_crop_function;

  window_size_2d = t_cell_info.preprocessing_info.window_size_2d;
  window_center_2d = t_cell_info.preprocessing_info.window_center_2d;
  cropped_size = t_cell_info.preprocessing_info.cropped_size;

  segmentation_rasterization_function = t_cell_info.segmentation_info.segmentation_rasterization_function;
  
  % segmentation_illustrations_location = [t_cell_info.options.results_location, 'segmentation_illustrations/'];
  
  if ~exist(segmentation_illustrations_location, 'dir')
      mkdir(segmentation_illustrations_location);
  end
  
  
%   
%   model_original_representation_functions = t_cell_info.model_info.model_original_representation_functions;
%   model_original_reconstruction_functions = t_cell_info.model_info.model_original_reconstruction_functions;
%   model_reduction_representation_functions = t_cell_info.model_info.model_reduction_representation_functions;
%   model_reduction_reconstruction_functions = t_cell_info.model_info.model_reduction_reconstruction_functions;
  
%   all_model_distances = t_cell_info.clustering_info.all_model_distances;
%   all_model_trees = t_cell_info.clustering_info.all_model_trees;
%   all_model_clusters = t_cell_info.clustering_info.all_model_clusters;
%   clustering_distance_metrics = t_cell_info.clustering_info.clustering_distance_metrics;


  function [result] = crop_slices_or_pad(given_image_to_modify, given_image_for_comparison)
    if size(given_image_to_modify, 3) > size(given_image_for_comparison, 3)
      result = given_image_to_modify(:, :, floor((end - size(given_image_for_comparison, 3)) / 2) + (1:size(given_image_for_comparison, 3)));
    else
      result = padarray(given_image_to_modify, [0, 0, floor((size(given_image_for_comparison, 3) - size(given_image_to_modify, 3)) / 2)], 'pre');
      result = padarray(result, [0, 0, size(given_image_for_comparison, 3) - size(result, 3)], 'post');
    end
    if false
      % Debug info:
      whos given_image_to_modify given_image_for_comparison result, beep, keyboard
    end
  end
  
  
  
  % Go through frames which have regions, segmentations, alignments, and deformations and show the entire process in one image:
  
  
  % relative_times_to_illustrate = all_relative_times;
  relative_times_to_illustrate = 2;
  
  condition_sensor_combinations = t_cell_info.synapse_info.condition_sensor_combinations;
  combinations_to_illustrate = condition_sensor_combinations;
  % combinations_to_illustrate = condition_sensor_combinations(strcmp(condition_sensor_combinations(:, 2), 'Actin'), :);
  
  
  s = RandStream('mt19937ar', 'Seed', 691551);
  %RandStream.setDefaultStream(s);
  RandStream.setGlobalStream(s);
  
  number_cell_frames_to_illustrate = 100;
  number_cell_frames_illustrated = 0;
  
  % deformation_list = dir([deformations_location, '*.mat']);
  % deformation_list = {deformation_list.name}.';
  % deformation_list = strcat(deformations_location, deformation_list);
  % cell_frames_to_illustrate = deformation_list(randperm(length(deformation_list)) <= number_cell_frames_to_illustrate);
  % In case of deformation failure for a method, this list should stay the same for the purpose of comparison, e.g., of alignment methods:
  region_list = dir([regions_location, '*.mat']);
  region_list = {region_list.name}.';
  region_list = strcat(regions_location, region_list);
  % cell_frames_to_illustrate = region_list(randperm(length(region_list)) <= number_cell_frames_to_illustrate);
  cell_frames_to_illustrate_indices = randperm(length(region_list));
  % cell_frames_to_illustrate_indices = cell_frames_to_illustrate_indices(cell_frames_to_illustrate_indices <= number_cell_frames_to_illustrate);
  cell_frames_to_illustrate = region_list(cell_frames_to_illustrate_indices);
  
  % keyboard

  
  close
  
  
%   % Write just the template's shape:
%   image_to_show = t_cell_info.template_info.template_image;
%   % image_to_show = reshape_2d(image_to_show, -1);
%   image_to_show = reshape_2d(image_to_show, 1);
%   % Recolor the image:
%   image_to_show = ind2rgb(floor(contrast_stretch(image_to_show) .* (size(illustration_colormap_to_use, 1) - 1)) + 1, illustration_colormap_to_use);
%   
%   image_filename = [illustrations_location, sprintf('%stemplate', master_script_options.model_prefix)];
%   imwrite(image_to_show, [image_filename, '.png'])
%   fprintf('Wrote illustration template.png\n')
%   
  
  
  % for cell_frame_index = 1:length(cell_frames_to_illustrate)
  cell_frame_index = 0;
  while number_cell_frames_illustrated < number_cell_frames_to_illustrate && cell_frame_index < size(cell_frames_to_illustrate, 1)
    % Always increment cell_frame_index:
    cell_frame_index = cell_frame_index + 1;
    
    cell_illustration_filename = strrep(cell_frames_to_illustrate{cell_frame_index}(1:end - 4), regions_location, '');
    cell_illustration_filename_without_model_prefix = cell_illustration_filename;
    cell_illustration_filename = [master_script_options.model_prefix, cell_illustration_filename];
    
    current_condition = t_cell_info.synapse_info.condition_retrieval_function({cell_illustration_filename});
    current_sensor = t_cell_info.synapse_info.sensor_retrieval_function({cell_illustration_filename});
    if ~t_cell_info.synapse_info.condition_sensor_combination_ismember([current_condition, current_sensor], combinations_to_illustrate)
      continue
    end
    
    % cell_deformation_aligned_filename = [deformations_aligned_location, cell_illustration_filename_without_model_prefix];
    cell_deformation_filename = [deformations_location, cell_illustration_filename_without_model_prefix];
    cell_alignment_filename = [alignments_location, cell_illustration_filename_without_model_prefix];
    cell_segmentation_filename = [segmentations_filtered_location, cell_illustration_filename_without_model_prefix];
    cell_region_filename = [regions_location, cell_illustration_filename_without_model_prefix];
    % cell_original_filename = [originals_location, sprintf('%s%s', master_script_options.model_prefix, cell_illustration_filename_without_model_prefix)];
    cell_original_filename = [originals_location, cell_illustration_filename];
    % cell_original_with_landmarks_filename = [originals_with_landmarks_location, sprintf('%s%s', master_script_options.model_prefix, cell_illustration_filename_without_model_prefix)];
    cell_original_with_landmarks_filename = [originals_with_landmarks_location, cell_illustration_filename];
    % cell_illustration_filename = [illustrations_location, sprintf('%s%s', master_script_options.model_prefix, cell_illustration_filename_without_model_prefix)];
    cell_illustration_filename = [illustrations_location, cell_illustration_filename];
    
    % % existences = [exist([cell_deformation_aligned_filename, '.mat'], 'file'), exist([cell_deformation_filename, '.mat'], 'file'), exist([cell_alignment_filename, '.mat'], 'file'), exist([cell_segmentation_filename, '.mat'], 'file'), exist([cell_region_filename, '.mat'], 'file')];
    % This implies all of the above:
    % existences = [exist([cell_deformation_aligned_filename, '.mat'], 'file')];
    existences = [exist([cell_deformation_filename, '.mat'], 'file')];
    
    if false
      % Debug info:
      fprintf('cell_deformation_filename = "%s"\n', cell_deformation_filename)
      beep, keyboard
    end
    
    if ~all(existences)
      continue
    end
    
    if false
      % Debug info:
      fprintf('cell_deformation_filename = "%s"\n', cell_deformation_filename)
      beep, keyboard
    end
    
    a = load([cell_region_filename, '.mat']);
    current_window_image = a.current_window_image;
    current_window_dic_image = a.current_window_dic_image;
    current_relative_time = a.current_relative_time;
    if false
      % Debug info:
      fprintf('current_relative_time = %d\n', current_relative_time)
    end
    if ~ismember(current_relative_time, relative_times_to_illustrate)
      continue
    end
    
    if numel(current_window_dic_image) == 0
      continue
    end

    a = load([cell_segmentation_filename, '.mat']);
    if isempty(fieldnames(a))
      continue
    end
    segmentation_structure = a.segmentation_structure;
    window_number_slices = a.window_number_slices;

    a = load([cell_alignment_filename, '.mat']);
    if isempty(fieldnames(a))
      continue
    end
    current_transform = a.current_transform;
    current_transform_homogeneous = a.current_transform_homogeneous;
    current_landmarks = a.current_landmarks;
    cropped_segmentation_image = a.cropped_segmentation_image;
    cropped_raw_image = a.cropped_raw_image;
    % a
    % keyboard

    a = load([cell_deformation_filename, '.mat']);
    if isempty(fieldnames(a))
      continue
    end
    cropped_raw_image_deformed = a.cropped_raw_image_deformed;
    cropped_segmentation_image_deformed = a.cropped_segmentation_image_deformed;
    
    if sum(cropped_segmentation_image_deformed(:) >= .5) == 0
      continue
    end

    [~, cell_illustration_filename_without_path, ~] = fileparts(cell_deformation_filename);
    [can_start, final_name, final_exists] = chunk_start(illustrations_location, cell_illustration_filename_without_path);
    if can_start
      
      draw_landmarks_on_illustrations = false;
      % draw_landmarks_on_illustrations = true;
      if draw_landmarks_on_illustrations
        % Some of these end up with negative Z values for some reason:
        
        cropped_landmarks = current_transform_homogeneous * [current_landmarks, ones(size(current_landmarks, 1), 1)]';
        cropped_landmarks = cropped_landmarks(1:3, :)';
      
        % Draw the landmarks and a line connecting them:
        cropped_segmentation_image_with_landmarks = cropped_segmentation_image;
        [line_x, line_y, line_z] = bresenham_line3d(cropped_landmarks(1, :), cropped_landmarks(2, :));
        line_coordinates = [line_x', line_y', line_z'];
        % keyboard
        current_transform_homogeneous, current_landmarks, cropped_landmarks, line_coordinates, whos cropped_segmentation_image_with_landmarks
        
        if false
          % Debug info:
          
          % any(cropped_landmarks(:) < 0)
          % any(line_coordinates(:) < 0)
          if any(cropped_landmarks(:) < 0) || any(line_coordinates(:) < 0)
            keyboard
          end
        end
        
        % Line:
        line_image = false(size(cropped_segmentation_image_with_landmarks));
        line_image(sub2ind(size(cropped_segmentation_image_with_landmarks), line_coordinates(:, 2), line_coordinates(:, 1), line_coordinates(:, 3))) = true;
        % Landmarks (synapse and centroid):
        line_image(round(cropped_landmarks(1, 2)) + (-1:1), round(cropped_landmarks(1, 1)) + (-1:1), round(cropped_landmarks(1, 3)) + (-1:1)) = true;
        % line_image(round(cropped_landmarks(2, 2)) + (-1:1), round(cropped_landmarks(2, 1)) + (-1:1), round(cropped_landmarks(2, 3)) + (-1:1)) = true;
        % cropped_segmentation_image_binary = double(cropped_segmentation_image >= .5);
        % [gx, gy, gz] = gradient(cropped_segmentation_image_binary);
        % cropped_segmentation_image_edges = (gx ~= 0) | (gy ~= 0) | (gz ~= 0);
        % line_image(cropped_segmentation_image_edges > 0) = true;

        line_dilating_element = zeros(3, 3, 3);
        line_dilating_element(2, 2, 2) = 1;
        line_dilating_element(1, 2, 2) = 1;
        line_dilating_element(3, 2, 2) = 1;
        line_dilating_element(2, 1, 2) = 1;
        line_dilating_element(2, 3, 2) = 1;
        line_dilating_element(2, 2, 1) = 1;
        line_dilating_element(2, 2, 3) = 1;
        line_image_dilated = imdilate(line_image, line_dilating_element);
        cropped_segmentation_image_with_landmarks(line_image_dilated) = 0;
        cropped_segmentation_image_with_landmarks(line_image) = 1;

        % cropped_segmentation_image = cropped_segmentation_image_with_landmarks;
      end
      
      
      % error
      
%       should_show_aligned_steps = false;
%       % should_show_aligned_steps = true;
% 
%       image_to_show = [];
%       % Unaligned intensity image:
%       if false
%         % Debug info:
%         imshow(reshape_contrast(current_window_image, -1))
%         imshow(reshape_contrast(cropped_raw_image, -1))
%       end
      current_window_image_central_slices = crop_slices_or_pad(current_window_image, cropped_raw_image);
%       image_to_show = [image_to_show; contrast_stretch(current_window_image_central_slices)];
      % xruan 06/24/2015
      % padding the cropped_raw_image and cropped_segmentation_image in both side rather than the post
      % side in original case
%       if window_number_slices < size(template_image, 3)
          cropped_raw_image = crop_slices_or_pad(cropped_raw_image(:, :, 1 : window_number_slices), current_window_image_central_slices);
          cropped_segmentation_image = crop_slices_or_pad(cropped_segmentation_image(:, :, 1 : window_number_slices), current_window_image_central_slices);
%       end
%       if should_show_aligned_steps
%         % Aligned intensity image:
%         image_to_show = [image_to_show; contrast_stretch(cropped_raw_image)];
%       else
        % Unaligned segmentation image:
         cropped_segmentation_image_without_transformation = segmentation_rasterization_function(segmentation_structure, [t_cell_info.preprocessing_info.window_size_2d, window_number_slices]);
%         % % cropped_segmentation_image_without_transformation = segmentation_rasterization_function(segmentation_structure);
%         % cropped_segmentation_image_without_transformation = cropped_segmentation_image_without_transformation(:, :, floor((end - size(cropped_raw_image, 3)) / 2) + (1:size(cropped_raw_image, 3)));
         cropped_segmentation_image_without_transformation = crop_slices_or_pad(cropped_segmentation_image_without_transformation, cropped_raw_image);
%         image_to_show = [image_to_show; cropped_segmentation_image_without_transformation];
%         % Unaligned segmented intensity image:
%         image_to_show = [image_to_show; contrast_stretch(current_window_image_central_slices .* (cropped_segmentation_image_without_transformation >= .5) + min(current_window_image_central_slices(cropped_segmentation_image_without_transformation >= .5)) .* (cropped_segmentation_image_without_transformation < .5))];
%       end
%       % Aligned segmentation image:
%       if draw_landmarks_on_illustrations
%         image_to_show = [image_to_show; cropped_segmentation_image_with_landmarks];
%       else
%         image_to_show = [image_to_show; cropped_segmentation_image];
%       end
%       % Aligned segmented intensity image:
%       image_to_show = [image_to_show; contrast_stretch(cropped_raw_image .* (cropped_segmentation_image >= .5) + min(cropped_raw_image(cropped_segmentation_image >= .5)) .* (cropped_segmentation_image < .5))];
%       % Standardized segmented intensity image:
%       image_to_show = [image_to_show; contrast_stretch(cropped_raw_image_deformed .* (cropped_segmentation_image_deformed >= .5) + min(cropped_raw_image_deformed(cropped_segmentation_image_deformed >= .5)) .* (cropped_segmentation_image_deformed < .5))];
%       % image_to_show = reshape_2d(image_to_show, -1);
%       image_to_show = reshape_2d(image_to_show, 1);
%       
%       % Recolor the image:
%       % image_to_show = ind2rgb(round(image_to_show .* size(illustration_colormap_to_use, 1)), illustration_colormap_to_use);
%       image_to_show = ind2rgb(floor(contrast_stretch(image_to_show) .* (size(illustration_colormap_to_use, 1) - 1)) + 1, illustration_colormap_to_use);
%       
%       % Add DIC at the end to use a grayscale colormap:
%       image_to_show = [repmat(contrast_stretch(current_window_dic_image), [1, size(cropped_raw_image, 3), 3]); image_to_show];
% 
%       
%       image_filename = cell_illustration_filename;
%       imwrite(image_to_show, [image_filename, '.png'])
%       fprintf('Wrote illustration %s\n', cell_illustration_filename)

      current_window_image_shown = current_window_image_central_slices; 
      current_window_image_seg_show = current_window_image_central_slices .* ...
          (cropped_segmentation_image_without_transformation >= .5);
      aligned_image_shown = cropped_raw_image;
      aligned_image_seg_shown = cropped_raw_image .* cropped_segmentation_image;
      
        close all;
        scale = 2; set(gcf, 'Position', [100, 100, 100 * 4 * scale, 100 * 3 * scale])
        set(gcf,'render','opengl')
        set(gcf, 'Visible', 'off');
        set(gcf, 'Visible', 'on');

        subplot(3, 4, 1)
        imagesc(mean(current_window_image_shown, 3)); 
        color_limits = caxis();
        axis image
        title({'Image XY'})

        subplot(3, 4, 2)
        imagesc(squeeze(mean(current_window_image_shown, 2))); 
        caxis(color_limits)
        axis image
        title({'Image YZ'})

        subplot(3, 4, 3)
        imagesc(squeeze(mean(current_window_image_shown, 1))); 
        caxis(color_limits)
        axis image
        title({'Image XZ'})

        subplot(3, 4, 4)
        % % Intentionally blank
        % imagesc(segmentation_structure.Best_extra_data.img_seg); 
        imagesc(current_window_image_shown(:, :, 1)); 
        title({'Initial 2D segn'})


        subplot(3, 4, 5)
        % imagesc(mean(rotated_image .* ~original_rotated_segmentation, 3)); 
        imagesc(mean(current_window_image_seg_show, 3)); 
        caxis(color_limits)
        axis image
        title({'Without BK segn XY'})

        subplot(3, 4, 6)
        % imagesc(squeeze(mean(rotated_image .* ~original_rotated_segmentation, 2))); 
        imagesc(squeeze(mean(current_window_image_seg_show, 2))); 
        caxis(color_limits)
        axis image
        title({'Without BK segn YZ'})

        subplot(3, 4, 7)
        % imagesc(squeeze(mean(rotated_image .* ~original_rotated_segmentation, 1))); 
        imagesc(squeeze(mean(current_window_image_seg_show, 1))); 
        caxis(color_limits)
        axis image
        title({'Without BK segn XZ'})

        patch_axes_handle = subplot(3, 4, 8);
        m = isosurface(cropped_segmentation_image_without_transformation, .5);
        patch(m, 'EdgeColor', 'none', 'FaceColor', [.3, .9, .1]);
        view(3), axis equal, camlight, lighting phong
        axis equal
        title({'BK segn surface'})


        subplot(3, 4, 9)
        imagesc(mean(aligned_image_seg_shown, 3)); 
        caxis(color_limits)
        axis image
        title({'Without BK histeq segn XY'})

        subplot(3, 4, 10)
        imagesc(squeeze(mean(aligned_image_seg_shown, 2))); 
        caxis(color_limits)
        axis image
        title({'Without BK histeq segn YZ'})

        subplot(3, 4, 11)
        imagesc(squeeze(mean(aligned_image_seg_shown, 1))); 
        caxis(color_limits)
        axis image
        title({'Without BK histeq segn XZ'})

        patch_axes_handle2 = subplot(3, 4, 12);
        m = isosurface(cropped_segmentation_image, .5);
        patch(m, 'EdgeColor', 'none', 'FaceColor', [.3, .9, .1]);
        view(3), axis equal, camlight, lighting phong
        axis equal
        xlim(patch_axes_handle2, xlim(patch_axes_handle))
        ylim(patch_axes_handle2, ylim(patch_axes_handle))
        zlim(patch_axes_handle2, zlim(patch_axes_handle))
        title({'BK histeq segn surface'})

        image_filename_split = strsplit(cell_illustration_filename, '/');
        
        image_filename = [segmentation_illustrations_location, image_filename_split{end}];
        % ppi = 90;
        ppi = 150;
        % ppi = 300;
        % export_fig(image_filename, '-opengl', '-png', '-a1', '-nocrop', ['-r', num2str(ppi)])
        export_fig(image_filename, '-opengl', '-png', '-a1', ['-r', num2str(ppi)])
      
      
      
      
      
      % Write the window's DIC image:
%       if ~isempty(current_window_dic_image)
%         current_window_dic_image_stretched = contrast_stretch(current_window_dic_image);
%         current_window_dic_image_filename = [cell_original_filename, '_DIC'];
%         imwrite(current_window_dic_image_stretched, [current_window_dic_image_filename, '.png']);
%       end
%       
%       % Write the window as a TIF for viewing in ImageJ:
%       current_window_image_stretched = contrast_stretch(current_window_image);
%       
%       current_window_image_filename = cell_original_filename;
%       current_window_image_landmarks_filename = cell_original_with_landmarks_filename;
      % compression_mode = 'none';
%       compression_mode = 'Deflate';
      % Original:
%       imwrite(current_window_image_stretched(:, :, 1), [current_window_image_filename, '.tif'], 'Compression', compression_mode);

%       if draw_landmarks_on_originals
%         % Original with landmarks:
%         imwrite(current_window_image_stretched_with_landmarks(:, :, 1), [current_window_image_landmarks_filename, '.tif'], 'Compression', compression_mode);
%       end
%       for z_index = 2:size(current_window_image_stretched, 3)
%         % Original:
%         imwrite(current_window_image_stretched(:, :, z_index), [current_window_image_filename, '.tif'], 'WriteMode', 'append', 'Compression', compression_mode);
%         
%         if draw_landmarks_on_originals
%           % Original with landmarks:
%           imwrite(current_window_image_stretched_with_landmarks(:, :, z_index), [current_window_image_landmarks_filename, '.tif'], 'WriteMode', 'append', 'Compression', compression_mode);
%         end
%       end
      
      chunk_finish(illustrations_location, cell_illustration_filename_without_path)
      
    end
    
    % % Write an image showing the synapse point and centroid:
    % current_window_dic_image_filename = [cell_original_filename, '_DIC'];
    % current_window_image_landmarks_filename = [cell_original_filename, '_DIC'];
    % imwrite(current_window_dic_image_stretched, [current_window_dic_image_filename, '.png']);
    
    % Increment number_cell_frames_illustrated only if the following code can run:
    number_cell_frames_illustrated = number_cell_frames_illustrated + 1;

  end
  
  
  % beep, keyboard
  % error('Implementation yet unfinished below this line!')

  % t_cell_info.figures_info = struct();
  % t_cell_info.figures_info.registration_options = registration_options;
  t_cell_info.path_info.illustrations_location = segmentation_illustrations_location; 
  
  
end  

