function [t_cell_info] = master_script_morph_intensities_within_template(t_cell_info)
  % Register intensities within the template after the initial morph to the template.
  %
  % 2013-11-18 tebuck: Copied from master_script_morph_segmentations.m.
  % 
  % 
  % 
  % 
  % 
  % % Dependencies:
  % % From the File Exchange: export_fig, pmkmp, toolbox_graph, toolbox_fast_marching, affine, Snake3D, Mesh_voxelisation, dirr, inhull
  % % From tebuck: modified version of Jieyue Li's HPA_lib, rasterize_mesh

%
% Author: Taraz E. Buck and Xiongtao Ruan
%
% Copyright (C) 2012-2016 Murphy Lab
% Computational Biology Department
% School of Computer Science
% Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu



  
  
  
  % Variables referenced more than once can be copied to local variables:
  master_script_options = t_cell_info.options;
  regions_location = t_cell_info.path_info.regions_location;
  segmentations_filtered_location = t_cell_info.path_info.segmentations_filtered_location;
  alignments_location = t_cell_info.path_info.alignments_location;
  deformations_location = t_cell_info.path_info.deformations_location;
  
  run_relative_path_list = t_cell_info.synapse_info.run_relative_path_list;
  run_key_list = t_cell_info.synapse_info.run_key_list;
  run_images_list = t_cell_info.synapse_info.run_images_list;
  run_brightfield_images_list = t_cell_info.synapse_info.run_brightfield_images_list;
  run_image_frame_indices_list = t_cell_info.synapse_info.run_image_frame_indices_list;
  run_number_cells = t_cell_info.synapse_info.run_number_cells;
  run_number_frames = t_cell_info.synapse_info.run_number_frames;
  run_point_tracks = t_cell_info.synapse_info.run_point_tracks;
  run_relative_time_tracks = t_cell_info.synapse_info.run_relative_time_tracks;
  run_relative_times = t_cell_info.synapse_info.run_relative_times;
  run_number_relative_times = t_cell_info.synapse_info.run_number_relative_times;
  all_relative_times = t_cell_info.synapse_info.all_relative_times;
  number_all_relative_times = t_cell_info.synapse_info.number_all_relative_times;
  all_relative_times_seconds_data = t_cell_info.synapse_info.all_relative_times_seconds_data;
  all_relative_times_seconds = t_cell_info.synapse_info.all_relative_times_seconds;
  number_runs = t_cell_info.synapse_info.number_runs;
  
  template_centroid = t_cell_info.template_info.template_centroid;
  template_synapse = t_cell_info.template_info.template_synapse;
  template_image = t_cell_info.template_info.template_image;

  window_size_2d = t_cell_info.preprocessing_info.window_size_2d;
  window_center_2d = t_cell_info.preprocessing_info.window_center_2d;
  cropped_size = t_cell_info.preprocessing_info.cropped_size;

  segmentation_rasterization_function = t_cell_info.segmentation_info.segmentation_rasterization_function;
  
  
  % error('Implementation yet unfinished below this line!')
  
  
  function [result] = constrast_stretch_protein_in_template(protein_image)
    % protein_image_raised_background = protein_image + quantile(protein_image(protein_image > 0), .01) .* (protein_image == 0);
    protein_image_raised_background = protein_image .* (template_image == 1) + quantile(protein_image(template_image == 1), 0) .* (template_image < 1);
    % whos protein_image protein_image_raised_background template_image
    % protein_image_above_zero_quantiles = quantile(protein_image(protein_image > 0), 0:.1:1)
    % intensity_mean_max = quantile(protein_image_raised_background(template_image > 0), .99);
    % intensity_mean_max = quantile(protein_image_raised_background(template_image == 1), .99);
    intensity_mean_max = quantile(protein_image_raised_background(template_image == 1), 1);
    high_pixels = protein_image >= intensity_mean_max;
    protein_image_raised_background = protein_image_raised_background .* ~high_pixels + intensity_mean_max .* high_pixels;
    result = protein_image_raised_background;
  end

  function [result] = normalize_protein_in_template(protein_image)
    result = protein_image ./ sum(protein_image(template_image == 1));
  end

  
  
  
  if false
    fprintf('\n\n========================================\nStatistics prior to model construction:\n')
    run_number_synapses_marked = cellfun(@(x)sum(sum(~cellfun(@isempty, x))), run_relative_time_tracks);
    run_number_synapses_marked_mean = mean(run_number_synapses_marked)
    run_number_synapses_marked_quartiles = prctile(run_number_frames, [0:25:100])
    run_number_cells_mean = mean(run_number_cells)
    run_number_cells_quartiles = prctile(run_number_cells, [0:25:100])
    number_runs
    run_number_synapses_marked_sum = sum(run_number_synapses_marked)
    run_number_cells_sum = sum(run_number_cells)
    % error
  end
  
  
  
  
  
  beep, keyboard
  error('Implementation yet unfinished below this line!')

  t_cell_info.morphing_info = struct();
  t_cell_info.morphing_info.registration_options = registration_options;
  
  
  
end  

