function [t_cell_info] = master_script_morph_segmentations(t_cell_info, options)
  % Register frames of the times series of each cell to the template.
  %
  % 2013-11-17 tebuck: Copied from master_script_align_segmentations.m.
  % 
  % 
  % 
  % 
  % 
  % % Dependencies:
  % % From the File Exchange: export_fig, pmkmp, toolbox_graph, toolbox_fast_marching, affine, Snake3D, Mesh_voxelisation, dirr, inhull
  % % From tebuck: modified version of Jieyue Li's HPA_lib, rasterize_mesh

%
% Author: Taraz E. Buck and Xiongtao Ruan
%
% Copyright (C) 2012-2016 Murphy Lab
% Computational Biology Department
% School of Computer Science
% Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu



  
  
  
  % Variables referenced more than once can be copied to local variables:
  master_script_options = t_cell_info.options;
  fprintf_diary_cycle_function = t_cell_info.log_info.fprintf_diary_cycle_function;
  regions_location = t_cell_info.path_info.regions_location;
  segmentations_filtered_location = t_cell_info.path_info.segmentations_filtered_location;
  alignments_location = t_cell_info.path_info.alignments_location;
  deformations_location = t_cell_info.path_info.deformations_location;
  
  run_relative_path_list = t_cell_info.synapse_info.run_relative_path_list;
  run_key_list = t_cell_info.synapse_info.run_key_list;
  % run_data_list = t_cell_info.synapse_info.run_data_list;
  run_images_list = t_cell_info.synapse_info.run_images_list;
  run_brightfield_images_list = t_cell_info.synapse_info.run_brightfield_images_list;
  run_image_frame_indices_list = t_cell_info.synapse_info.run_image_frame_indices_list;
  run_number_cells = t_cell_info.synapse_info.run_number_cells;
  run_number_frames = t_cell_info.synapse_info.run_number_frames;
  run_point_tracks = t_cell_info.synapse_info.run_point_tracks;
  run_relative_time_tracks = t_cell_info.synapse_info.run_relative_time_tracks;
  run_relative_times = t_cell_info.synapse_info.run_relative_times;
  run_number_relative_times = t_cell_info.synapse_info.run_number_relative_times;
  all_relative_times = t_cell_info.synapse_info.all_relative_times;
  number_all_relative_times = t_cell_info.synapse_info.number_all_relative_times;
  all_relative_times_seconds_data = t_cell_info.synapse_info.all_relative_times_seconds_data;
  all_relative_times_seconds = t_cell_info.synapse_info.all_relative_times_seconds;
  number_runs = t_cell_info.synapse_info.number_runs;
  
  template_centroid = t_cell_info.template_info.template_centroid;
  template_synapse = t_cell_info.template_info.template_synapse;
  template_image = t_cell_info.template_info.template_image;

  window_size_2d = t_cell_info.preprocessing_info.window_size_2d;
  window_center_2d = t_cell_info.preprocessing_info.window_center_2d;
  cropped_size = t_cell_info.preprocessing_info.cropped_size;

  segmentation_rasterization_function = t_cell_info.segmentation_info.segmentation_rasterization_function;
  
  template_volume = t_cell_info.alignment_info.template_volume;
  segmentation_volume_threshold = t_cell_info.alignment_info.segmentation_volume_threshold;
  
  % debug_print_deformation_skip_reasons = false;
  debug_print_deformation_skip_reasons = true
  
  check_morph_file_loadility = master_script_options.check_morph_file_loadility;
  % Set options for T cell image registration:
  registration_options = struct();
  
  % registration_options.filter_radius = 4;
  % registration_options.filter_radius = 8;
  % registration_options.filter_radius = 16;
  % registration_options.filter_radius = 32;
  % registration_options.filter_radius = ceil(master_script_options.cell_diameter / 4 / 2) * 2;
  registration_options.filter_radius = ceil(master_script_options.cell_diameter / 2 / 2) * 2;
  registration_options.kernel_z_radius = registration_options.filter_radius;
  registration_options.maximum_deformation_per_step = [1, 1, 1] * .5;
  registration_options.window_radius = t_cell_info.template_info.template_options.imx;
  % registration_options.convergence_absolute_error = prod(cropped_size) ./ 2e3;
  % registration_options.convergence_absolute_error = prod(cropped_size) ./ 1e3;
  % registration_options.convergence_absolute_error = prod(cropped_size) ./ 5e2;
  registration_options.convergence_absolute_error = prod(cropped_size) ./ 2e2;
  % registration_options.convergence_absolute_error = prod(cropped_size) ./ 1e2;
  % registration_options.convergence_absolute_error = prod(cropped_size) ./ 1e1;
  registration_options.single_sided = true;
  registration_options.maximum_iterations = max(t_cell_info.template_info.template_dimensions ./ registration_options.maximum_deformation_per_step);

  registration_options.save_intermediates = false;
  registration_options.save_stages = false;
  % registration_options.save_intermediate_images = true;
  % registration_options.always_save_full_intermediates = true;
  registration_options
  
  
  
  
  % % error('Update to scale voxels to cubical, use proper param adjustments to scale!')
  % warning('Update to scale voxels to cubical, use proper param adjustments to scale!')
  
  
  % Keep running until everything is finished:
  run_cell_index_frame_indices_uncomputed = cell(number_runs, 1);
  for run_index = 1:number_runs
    run_key = run_key_list{run_index};
    number_cells = t_cell_info.synapse_info.run_number_cells(run_index);
    number_frames = run_number_frames(run_index);
    cell_synapse_point_tracks = run_point_tracks{run_index};
    run_cell_index_frame_indices_uncomputed{run_index} = false(number_cells, number_frames);
    for current_cell_index = 1:number_cells
      for frame_index = 1:number_frames
        current_synapse_center_rounded = cell_synapse_point_tracks{current_cell_index, frame_index};
        if isempty(current_synapse_center_rounded)
          continue
        end
        current_synapse_center_rounded = current_synapse_center_rounded(1, 1:2);
        
        first_frame_index = find(~cellfun(@isempty, cell_synapse_point_tracks(current_cell_index, :)), 1);
        if isempty(first_frame_index)
          run_cell_indices_uncomputed{run_index}(current_cell_index) = false;
          continue
        end
        
        cell_deformation_filename = [run_key, sprintf('_frame%05d_synapse%05d,%05d', frame_index, current_synapse_center_rounded)];
        cell_alignment_filename = [alignments_location, cell_deformation_filename];
        
        if ~exist([cell_alignment_filename, '.mat'], 'file')
          % Given modifications to master_script_align_segmentations.m, all alignment should already be complete and thus this file will never be available (should look into why it needs to be checked, though):
          continue
        end
        
        run_cell_index_frame_indices_uncomputed{run_index}(current_cell_index, frame_index) = true;
      end
    end
  end
  
  recently_did_any_work = false;
  did_any_work = false;
  
  while any(cellfun(@(x)any(x(:)), run_cell_index_frame_indices_uncomputed))
    
    recently_did_any_work = did_any_work;
    did_any_work = false;
    
    % Compute deformations:
    for run_index = 1:number_runs
      % Prepare useful variables:
      run_relative_path = run_relative_path_list{run_index};
      run_key = run_key_list{run_index};
      run_image_path = run_images_list{run_index};
      number_cells = run_number_cells(run_index);
      number_frames = run_number_frames(run_index);
      cell_synapse_point_tracks = run_point_tracks{run_index};
      cell_relative_time_tracks = run_relative_time_tracks{run_index};
      
      % fprintf_diary_cycle_function('Morphing aligned segmentations for run "%s"\n', run_key)
      printed_run_work_message = false;
      
      % Go through frames of each cell:
      for current_cell_index = 1:size(cell_synapse_point_tracks, 1)

        for frame_index = 1:number_frames
          if ~run_cell_index_frame_indices_uncomputed{run_index}(current_cell_index, frame_index)
            continue
          end
          
          current_synapse_center_rounded = cell_synapse_point_tracks{current_cell_index, frame_index};
          % if isempty(current_synapse_center_rounded)
            % continue
          % end
          current_synapse_center_rounded = current_synapse_center_rounded(1, 1:2);
          
          % % This takes a long time, so do not repeat work by checking for a file indicating this synapse file's windows are finished:
          cell_deformation_filename = [run_key, sprintf('_frame%05d_synapse%05d,%05d', frame_index, current_synapse_center_rounded)];
          cell_alignment_filename = [alignments_location, cell_deformation_filename];
          cell_segmentation_filename = [segmentations_filtered_location, cell_deformation_filename];
          cell_region_filename = [regions_location, cell_deformation_filename];
          
          % 02/24/2016 xruan
          % [can_start, final_name, final_exists] = chunk_start(deformations_location, cell_deformation_filename);
          [can_start, final_name, final_exists] = chunk_start_clean(deformations_location, cell_deformation_filename, [], check_morph_file_loadility);
          
          if final_exists
            run_cell_index_frame_indices_uncomputed{run_index}(current_cell_index, frame_index) = false;
          end
          if final_exists && false
            a = load(final_name)
            % Debug plot:
            % imshow(reshape_2d([contrast_stretch(a.cropped_raw_image); a.cropped_segmentation_image; template_image; a.cropped_segmentation_image_deformed], -1)), pause
            % keyboard
            % imshow(reshape_2d([template_image; contrast_stretch(a.cropped_raw_image); contrast_stretch(a.cropped_raw_image_deformed); a.cropped_segmentation_image; a.cropped_segmentation_image_deformed], -1)), pause
            imshow(reshape_2d([template_image; a.cropped_segmentation_image; a.cropped_segmentation_image_deformed; contrast_stretch(a.cropped_raw_image); contrast_stretch(a.cropped_raw_image_deformed)], -1)), pause
            % imshow(reshape_2d([template_image; a.cropped_segmentation_image; a.cropped_segmentation_image_deformed; contrast_stretch(a.cropped_raw_image); contrast_stretch(a.cropped_raw_image_deformed)], -1)), pause
            % imshow(reshape_contrast([a.cropped_raw_image; a.cropped_raw_image .* a.cropped_segmentation_image], -1)), pause
            % imshow(reshape_contrast([a.cropped_raw_image_deformed; a.cropped_raw_image_deformed .* a.cropped_segmentation_image_deformed], -1)), pause
            % imshow(reshape_contrast(a.cropped_raw_image_deformed, -1)), pause
            % imshow(reshape_2d([; contrast_stretch([a.cropped_raw_image_deformed; a.cropped_raw_image_deformed .* a.cropped_segmentation_image_deformed])], -1)), pause
          end
            
          if ~can_start
            continue
          end
          
          did_any_work = true;
          if printed_run_work_message
            fprintf_diary_cycle_function('Morphing aligned segmentations for run "%s"\n', run_key)
            printed_run_work_message = true;
          end;
          
          a = load([cell_segmentation_filename, '.mat']);
          % Feb. 14, 2016 xruan disable it
          % segmentation_structure = a.segmentation_structure;
          window_number_slices = a.window_number_slices;

          % try
            a = load([cell_alignment_filename, '.mat']);
          % catch raised_error
            % getReport(raised_error, 'extended')
          % end
          if isempty(fieldnames(a))
            temp = struct; save(final_name, '-struct', 'temp'); chunk_finish(deformations_location, cell_deformation_filename);
            run_cell_index_frame_indices_uncomputed{run_index}(current_cell_index, frame_index) = false;
            if debug_print_deformation_skip_reasons, fprintf_diary_cycle_function('Skipping morph %s cell %d frame %d because alignment empty\n', run_key, current_cell_index, frame_index), end
            continue
          end
          current_transform = a.current_transform;
          % xruan 08/14/2015
          % for wave2 active data, make sure it is double for deformation. 
          cropped_segmentation_image = double(a.cropped_segmentation_image);  
          % cropped_segmentation_image = a.cropped_segmentation_image;
          cropped_raw_image = a.cropped_raw_image;
          
          current_volume = sum(cropped_segmentation_image(:));
          is_segmentation_low_volume = current_volume <= segmentation_volume_threshold;
          
          if is_segmentation_low_volume
            % if debug_print_deformation_skip_reasons, fprintf_diary_cycle_function('Skipping morph %s cell %d because cropped_segmentation_image blank\n', run_key, current_cell_index), end
            if debug_print_deformation_skip_reasons, fprintf_diary_cycle_function('Skipping morph %s cell %d frame %d because cropped_segmentation_image blank\n', run_key, current_cell_index, frame_index), end
            % chunk_finish(deformations_location, cell_deformation_filename);
            temp = struct; save(final_name, '-struct', 'temp'); chunk_finish(deformations_location, cell_deformation_filename);
            run_cell_index_frame_indices_uncomputed{run_index}(current_cell_index, frame_index) = false;
            continue
          end
          
          % xruan 02/24/2016 
          fprintf_diary_cycle_function('Morphing aligned segmentations for cell "%s"\n', cell_deformation_filename)

          a = load([cell_region_filename, '.mat']);
          current_window_image = a.current_window_image;
          
          
          source = cropped_segmentation_image;
          target = template_image;
          if false
            % Debug plot:
            imshow(reshape_2d([source; target], -1)), pause
          end
          
          r = Greedy3D_lambda_pre_compressed(...
            source, target, 1, registration_options)
            
          cropped_segmentation_image_deformed = interp3(cropped_segmentation_image, r.source_deformation{1}.get_data(), r.source_deformation{2}.get_data(), r.source_deformation{3}.get_data());
          cropped_raw_image_deformed = interp3(cropped_raw_image, r.source_deformation{1}.get_data(), r.source_deformation{2}.get_data(), r.source_deformation{3}.get_data());

          save(final_name, 'cropped_segmentation_image_deformed', 'cropped_raw_image_deformed', 'registration_options', 'r')
          chunk_finish(deformations_location, cell_deformation_filename);
          run_cell_index_frame_indices_uncomputed{run_index}(current_cell_index, frame_index) = false;

        end
      end
    end
    
    pause(30)
    % xruan 02/24/2016
    out_status = chunk_lock_clean(deformations_location);
    if out_status
      fprintf_diary_cycle_function('Delete lock files whose jobs are not running!\n')
    end      
    if recently_did_any_work && ~did_any_work
      fprintf_diary_cycle_function('Apparently no more work to do, wait for other jobs')
    end

    if false
      % Debug info:
      if ~recently_did_any_work && ~did_any_work
        cellfun(@(x)any(x(:)), run_cell_index_frame_indices_uncomputed).'
        beep
        keyboard
      end
    end
    
  end



  
  % beep, keyboard
  % error('Implementation yet unfinished below this line!')

  t_cell_info.morphing_info = struct();
  t_cell_info.morphing_info.registration_options = registration_options;
  
  
  
end  

