function [t_cell_info] = master_script_process_image_resolution(t_cell_info, options)
% xruan 07/25/2015
% 15 03 23 X Y resolution is 2X2. So we need first process images and the
% annotation files. 

%
% Author: Taraz E. Buck and Xiongtao Ruan
%
% Copyright (C) 2012-2016 Murphy Lab
% Computational Biology Department
% School of Computer Science
% Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu





addpath(genpath('dirr'));
%% downsample images
image_location =  '/media/SDA3/images/5C.C7 LAT 15 03 23';
result_location = '/media/SDA3/images/processed_image';
[~, ~, image_path] = dirr(image_location, 'name', '');
image_path = image_path';

img_pattern = '.tif';
isimage = ~cellfun(@isempty, regexp(image_path, img_pattern));
image_path_true = image_path(isimage);

%length(image_path_true) 

for i = 1 : length(image_path_true) 
    image_i = image_path_true{i};
    image_path_split = regexp(image_i, '/', 'split');
    image_new_path = [result_location, filesep,  strjoin(image_path_split(5 : end - 1), '/')];
    mkdir_recursive(image_new_path);
    image_i_name = image_path_split{end};
    im_info = imfinfo(image_i);
    num_img = numel(im_info);
    I1 = imread(image_i, 1);
    [m, n, z] = size(I1);
    if z > 1
        continue;
    end
    
    im_size = [m, n, num_img];
    I = zeros(im_size);
    I_resize = zeros([round(m / 2), round(n / 2), num_img]);
    
    output_image = [ image_new_path, filesep, image_i_name];
    if exist(output_image, 'file')
        continue;
        delete(output_image);
    end
    
    for j = 1 : num_img
        I_j = double(imread(image_i, j));
        I(:, :, j) = double(I_j);
        I_j_resize = imresize(I_j, [round(m / 2), round(n / 2)], 'method', 'bilinear');
        I_resize(:, :, j) = I_j_resize;              
        imwrite(uint16(I_j_resize), output_image, 'WriteMode', 'append',  'Compression','none');     
    end
    t = Tiff(output_image, 'w');
    tagstruct.ImageLength = size(I_resize,1)
    tagstruct.ImageWidth = size(I_resize,2)
    tagstruct.Photometric = Tiff.Photometric.RGB
    tagstruct.BitsPerSample = 8
    %tagstruct.SamplesPerPixel = 3
    tagstruct.RowsPerStrip = 16
    tagstruct.PlanarConfiguration = Tiff.PlanarConfiguration.Chunky
    tagstruct.Software = 'MATLAB'
    tagstruct.SubIFD = 2  % required to create subdirectories
    t.setTag(tagstruct)
    t.write(I_resize);
end


%% deal with annotation files
synapse_location = '/projects/cellorganizer/xruan/tcell_project/tcell_annotation/All coordinates two-point 2015/Converted to CSV';

[~, ~, csv_path] = dirr(synapse_location, 'name', '');
csv_path = csv_path';

file_pattern = '15 03 23';
is_desire_files = ~cellfun(@isempty, regexp(csv_path, file_pattern));
csv_path_true = csv_path(is_desire_files)

for i = 1 : length(csv_path_true)
    csv_path_i = csv_path_true{i}
    
    csv_path_split = regexp(csv_path_i, '/', 'split');
    csv_new_path = [result_location, '/'];
    mkdir_recursive(csv_new_path);
    csv_i_name = csv_path_split{end};
    
    
    run_file_handle = fopen(csv_path_i);
    run_data_first_line = fgetl(run_file_handle);
    % Find number of columns as number of non-quoted commas plus one:
    run_data_first_line_commas = regexprep(run_data_first_line, '"[^"]*"', '');
    run_data = {};
    field_pattern = '([^,"\r\n]*|("[^"\r\n]*"))';
    run_data_number_columns = sum(run_data_first_line_commas == ',') + 1;
    fseek(run_file_handle, 0, 'bof');

    while ~feof(run_file_handle)
        % run_data{end + 1} = textscan(run_file_handle, '%q', 'Delimiter', ',');
        current_line = fgetl(run_file_handle);
        % run_data{end + 1, 1} = textscan(current_line, [repmat('%[^,"\r\n],', 1, run_data_number_columns - 1), '%[^,"\r\n]']);
        run_data{end + 1, 1} = regexp(current_line, [repmat([field_pattern, ','], 1, run_data_number_columns - 1), field_pattern], 'tokens');
        run_data{end} = run_data{end}{1};
        % current_line
        % pause
        % beep, keyboard
    end
    fclose(run_file_handle);
    run_data_resize = {};
    run_data_resize{1} = run_data{1};
    
    for j = 2 : length(run_data)
        run_data_j = run_data{j}
        run_data_j_mod( 1 : length(run_data_j)) = {''};
        if isempty(run_data_j)
            run_data_resize{j} = run_data_j;
            continue
        end
        
        %region label
        run_data_j_mod{1} = run_data_j{1};
        
        % Distance
        if ~isempty(run_data_j{2})
            run_data_j_mod{2} = num2str(str2num(run_data_j{2}) / 2);
        end
        
        % Angle
        run_data_j_mod{3} = run_data_j{3};
        
        % Left
        if ~isempty(run_data_j{4})
            run_data_j_mod{4} = num2str(str2num(run_data_j{4}) / 2);
        end     
        
        % Top
        if ~isempty(run_data_j{5})
            run_data_j_mod{5} = num2str(str2num(run_data_j{5}) / 2);
        end
        
        % Width
        if ~isempty(run_data_j{6})
            run_data_j_mod{6} = num2str(str2num(run_data_j{6}) / 2);
        end        
        
        % Heigth
        if ~isempty(run_data_j{7})
            run_data_j_mod{7} = num2str(str2num(run_data_j{7}) / 2);
        end         
        
        % blank and relative time
        run_data_j_mod(8 : 9) = run_data_j(8 : 9);
        
        % coorinates
        if ~isempty(run_data_j{10})
            coordinates = run_data_j{10};
            regex_pattern = '\d+';
            coordinates_sep = regexp(coordinates, regex_pattern, 'match');
            
            run_data_j_mod{10} = ['"', num2str(str2num(coordinates_sep{1}) / 2), ', ', num2str(str2num(coordinates_sep{2}) / 2), '"'];
        end
        
        run_data_j_mod(11 : 12) = run_data_j(11 : 12);
            
        ischar_check = cellfun(@ischar, run_data_j_mod);
        run_data_j_mod(~ischar_check) = '';
        % run_data_j_mod
        % run_data_j
        % length(run_data_j_mod)
        % length(run_data_j)
        run_data_resize{j} = run_data_j_mod;  
    end   
        csv_field_matrix = run_data_resize'
        fid = fopen([csv_new_path, filesep, csv_i_name], 'wt');
        for i = 1:size(csv_field_matrix, 1)
            csv_field_matrix_i = csv_field_matrix{i}
            for j = 1:size(csv_field_matrix_i, 2)
                if j > 1
                    fprintf(fid, ',');
                end
                if ischar(csv_field_matrix{i}{j})
                  fwrite(fid, csv_field_matrix{i}{j});
                else
                  fprintf(fid, '%f', csv_field_matrix{i}{j});
                end        
            end
            fprintf(fid, '\n');
        end
        fclose(fid);
end
