function [options] = master_script_process_options(options)
  % 2014-05-06 tebuck: Copied from master_script.m.
  % % Dependencies:
  % % From the File Exchange: export_fig, pmkmp, toolbox_graph, toolbox_fast_marching, affine, Snake3D, Mesh_voxelisation, dirr, inhull
  % % From tebuck: modified version of Jieyue Li's HPA_lib, rasterize_mesh
  % 02/23/2016 xruan: simplify code and delete unnecessary code

%
% Author: Taraz E. Buck and Xiongtao Ruan
%
% Copyright (C) 2012-2016 Murphy Lab
% Computational Biology Department
% School of Computer Science
% Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu


  
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % Set defaults and process options structure:
  
  if ~false 
        nodes_to_exclude = '2-2[0-9]|2-30';
        [stat, hostname] = system('hostname');
        hostname = deblank(hostname);
        if regexp(hostname, nodes_to_exclude)
            disp(sprintf('%s is not going to be used in the running and go to sleep until the end of the running.', hostname))
            pause(24 * 3600);
        end
  end
  
  
  default_options = master_script_get_default_options();
  
  if ~exist('options', 'var')
    options = default_options;
  else
    options_prior_to_processing = options;
    options = process_options_structure(default_options, options);
  end

  
  % Allow shortcuts to frequently used data sets for convenience of development:
  
  % First create a base set of options for some all_conditions variants:
  % Use Excel files for all conditions:
  all_conditions_base_options = options;
  % all_conditions_base_options.image_location = {'/images/wuelfing2012/KTR /', '/projects/cellorganizer/xruan/tcell_project/images/tcell_images/'};
  
  base_dir = which(mfilename);
  base_dir = strsplit(base_dir, '/');
  base_dir = strjoin(base_dir(1 : end - 2), '/');
  all_conditions_base_options.image_location = { [base_dir, '/Tcell_images/']};
  result_base_dir =  [base_dir, '/intermediate_results/'];
  synapse_base_dir =  [base_dir, '/Tcell_annotations/'];
  all_conditions_base_options.synapse_location = {[synapse_base_dir, 'one_point/'], [synapse_base_dir, 'two_point/']};
  
  % all_conditions_base_options.results_location = '/projects/cellorganizer/xruan/tcell_project/tcell_excel_results/';
  all_conditions_base_options.use_segmentation_filtering = false;
  % all_conditions_base_options.use_segmentation_filtering = true;
  all_conditions_base_options.skip_morphing_inside_template = true;
  all_conditions_base_options.sensors_to_exclude = {};
  % Ignore:
  all_conditions_base_options.sensors_to_exclude{end + 1, 1} = 'Vav1';
  % Full Stimulus HS1 has poorly segmented high magnification runs that dominate the model:
  % xruan 02/23/2016 disable warning
  % warning('2014-11-19: Add rule for HS1 images! Or delete associated .csv files!')
  % xruan 08/04/2015 
  % include HS1
  % all_conditions_base_options.sensors_to_exclude{end + 1, 1} = 'HS1';
  all_conditions_base_options.sensors_to_exclude{end + 1, 1} = 'Ezrin';
  all_conditions_base_options.sensors_to_exclude{end + 1, 1} = 'Akt';
  % all_conditions_base_options.condition_name_abbreviations = {'B7 Blockade active Rac Cofilin - ', 'B7 ac Rac Cfl - '};
  all_conditions_base_options.condition_name_abbreviations = {'B7 Blockade active Rac Cofilin', 'B7BlkActiveRacCfl'; 'B7 Blockade', 'B7Blk'; 'Full Stimulus', 'FullStim'};
  
  if isfield(options, 'named_options_set')
    fprintf('Using named_options_set "%s"\n', options.named_options_set), diary off, diary on
    switch options.named_options_set
      case 'full_set'     
        % As above
        
      case 'all_conditions'
      
        % Use Excel files for all conditions:
        options = all_conditions_base_options;
        
      case 'all_conditions_one_point'
      
        % Use Excel files for all conditions:
        options = all_conditions_base_options;
        % options.synapse_location = {'/projects/cellorganizer/xruan/tcell_project_freeze/tcell_annotation/All coordinates/Converted to CSV/', ...
        %  };             
        options.sensors_to_exclude{end + 1, 1} = 'Akt';
        options.sensors_to_exclude{end + 1, 1} = 'LAT';
        options.results_location = [result_base_dir, 'tcell_excel_results_1pt_repro_85_sn_st/'];
        
      case 'all_conditions_only_actin_one_point'      
        % Same as all_conditions but ignoring everything that isn't actin and two-point annotated:
        % Use Excel files for all conditions:
        options = all_conditions_base_options;
        options.model_prefix = 'only-actin_';
        options.synapse_location = {[synapse_base_dir, 'one_point/'], [synapse_base_dir, 'two_point/'], ...
            [synapse_base_dir, 'reconstruction/']};        
        options.results_location = [result_base_dir, 'tcell_excel_results_1pt_only-actin_repro_85_sn_st/'];

        % options.use_two_point_synapses = true;
        options.skip_model_video_saving = true;
        % No need for another directory as the prefix takes care of that.
        % Ignore:
        % Current non-actin sensors:
        options.sensors_to_exclude{end + 1, 1} = 'ARP3';
        options.sensors_to_exclude{end + 1, 1} = 'Akt';
        options.sensors_to_exclude{end + 1, 1} = 'Cofilin';
        options.sensors_to_exclude{end + 1, 1} = 'Coronin1A';
        options.sensors_to_exclude{end + 1, 1} = 'CPalpha1';
        options.sensors_to_exclude{end + 1, 1} = 'HS1';        
        options.sensors_to_exclude{end + 1, 1} = 'MRLC';
        options.sensors_to_exclude{end + 1, 1} = 'WASP';
        options.sensors_to_exclude{end + 1, 1} = 'WAVE2';
        options.sensors_to_exclude{end + 1, 1} = 'LAT';
        
      case 'all_conditions_only_actin_two_point'
      
        % Same as all_conditions but ignoring everything that isn't actin and two-point annotated:
        % Use Excel files for all conditions:
        options = all_conditions_base_options;
        options.model_prefix = 'only-actin_2pt_';
        options.synapse_location = {[synapse_base_dir, 'two_point/'], ...
            [synapse_base_dir, 'reconstruction/']};
        options.results_location = [result_base_dir, 'tcell_excel_results_2pt_only-actin_repro_85_sn_st/'];

        options.use_two_point_synapses = true;
        options.skip_model_video_saving = true;
        % No need for another directory as the prefix takes care of that.
        % Ignore:
        % Current non-actin sensors:
        options.sensors_to_exclude{end + 1, 1} = 'ARP3';
        options.sensors_to_exclude{end + 1, 1} = 'Akt';
        options.sensors_to_exclude{end + 1, 1} = 'Cofilin';
        options.sensors_to_exclude{end + 1, 1} = 'Coronin1A';
        options.sensors_to_exclude{end + 1, 1} = 'CPalpha1';
        options.sensors_to_exclude{end + 1, 1} = 'MRLC';
        options.sensors_to_exclude{end + 1, 1} = 'WASP';
        options.sensors_to_exclude{end + 1, 1} = 'WAVE2';
        options.sensors_to_exclude{end + 1, 1} = 'LAT';
        
      case 'all_conditions_only_LAT_two_point'
      
        % Same as all_conditions but ignoring everything that isn't actin and two-point annotated:
        % Use Excel files for all conditions:
        options = all_conditions_base_options;
        % options.image_location = { '/media/SDA3/images/wuelfing2015_organized/'};
        options.model_prefix = 'only-lat_';
        options.synapse_location = {[synapse_base_dir, 'two_point/'], ...
            [synapse_base_dir, 'reconstruction/']};
        options.results_location = [result_base_dir, 'tcell_excel_results_2pt_only-lat_repro_85_sn_st/'];
        
        options.use_two_point_synapses = true;
        % No need for another directory as the prefix takes care of that.
        % Ignore:
        % Current non-actin sensors:
        options.sensors_to_exclude{end + 1, 1} = 'ARP3';
        options.sensors_to_exclude{end + 1, 1} = 'Akt';
        options.sensors_to_exclude{end + 1, 1} = 'Actin';
        options.sensors_to_exclude{end + 1, 1} = 'Cofilin';
        options.sensors_to_exclude{end + 1, 1} = 'Coronin1A';
        options.sensors_to_exclude{end + 1, 1} = 'CPalpha1';
        options.sensors_to_exclude{end + 1, 1} = 'HS1';          
        options.sensors_to_exclude{end + 1, 1} = 'MRLC';
        options.sensors_to_exclude{end + 1, 1} = 'WASP';
        options.sensors_to_exclude{end + 1, 1} = 'WAVE2';    
        
      case 'all_conditions_only_WAVE2_one_point'
        options = all_conditions_base_options;
        %options.image_location = {'/projects/cellorganizer/xruan/tcell_project/images/tcell_images/'};
        options.synapse_location = {[synapse_base_dir, 'one_point/'], ...
            [synapse_base_dir, 'reconstruction/']};
        options.results_location = [result_base_dir, 'tcell_excel_results_1pt_WAVE2_repro_85_sn_st/'];
        
        options.model_prefix = 'one_point_WAVE2_';
        options.use_WAVE2_manual_seg = false;
        % the segmentation of reconstruction is different so don't run
        % illustration for this setting. 
        options.skip_illustration_saving = true;
        % options.skip_window_saving = true;
        % options.skip_hierarchical_clustering = true;
        options.skip_model_video_saving = true;
        
        options.sensors_to_exclude{end + 1, 1} = 'ARP3';
        options.sensors_to_exclude{end + 1, 1} = 'Actin';
        options.sensors_to_exclude{end + 1, 1} = 'Cofilin';
        options.sensors_to_exclude{end + 1, 1} = 'Coronin1A';
        options.sensors_to_exclude{end + 1, 1} = 'CPalpha1';
        options.sensors_to_exclude{end + 1, 1} = 'MRLC';
        options.sensors_to_exclude{end + 1, 1} = 'Akt';
        options.sensors_to_exclude{end + 1, 1} = 'WASP';               
        options.sensors_to_exclude{end + 1, 1} = 'HS1';  
        options.sensors_to_exclude{end + 1, 1} = 'LAT'; 
        
      case 'all_conditions_only_WAVE2_active_1pt'
        options = all_conditions_base_options;
        % options.image_location = {'/projects/cellorganizer/xruan/tcell_project/images_organized/tcell_paper_images/'};
        %options.image_location = {'/projects/cellorganizer/xruan/tcell_project/images/tcell_images/'};
        options.synapse_location = {[synapse_base_dir, 'reconstruction/']};
        options.results_location = [result_base_dir, 'tcell_excel_results_1pt_WAVE2_repro_85_sn_st/'];
        options.model_prefix = 'one_point_WAVE2_';
        options.use_WAVE2_manual_seg = true;
        
        options.skip_window_saving = true;
        options.skip_segmentation = true;
        options.WAVE2_active_root_dir = [options.image_location{1}];
        
        options.sensors_to_exclude{end + 1, 1} = 'ARP3';
        options.sensors_to_exclude{end + 1, 1} = 'Actin';
        options.sensors_to_exclude{end + 1, 1} = 'Cofilin';
        options.sensors_to_exclude{end + 1, 1} = 'Coronin1A';
        options.sensors_to_exclude{end + 1, 1} = 'CPalpha1';
        options.sensors_to_exclude{end + 1, 1} = 'MRLC';
        options.sensors_to_exclude{end + 1, 1} = 'Akt';
        options.sensors_to_exclude{end + 1, 1} = 'WASP';               
        options.sensors_to_exclude{end + 1, 1} = 'HS1';                  
        options.sensors_to_exclude{end + 1, 1} = 'LAT';
        
      case 'none'
      
        % Nothing here
        
      otherwise
        error('Unknown named_options_set value ''%s''', options.named_options_set)
    end
  end
  
  % options.debug_use_debug_switch_values_in_master_script = true;
  if options.debug_use_debug_switch_values_in_master_script
    % For debugging, skip everything that is already (partially) done:
    options.skip_window_saving = true;
    options.skip_segmentation = true;
    options.skip_alignment = true;
    options.skip_morphing = true;
    % % Takes maybe a minute, run every so often:
    options.skip_morphing_inside_template = true;
    options.skip_model_building = true;
    options.skip_statistical_testing_across_conditions = true;
    % options.skip_hierarchical_clustering = true;
    options.skip_model_figure_generation = true;
    options.skip_model_video_saving = true;
    % options.skip_illustration_saving = true;
    % options.skip_create_blender_figures = true;
  end
  
  if ~options.skip_postprocess
      options.check_morph_file_loadility = false;
      options.skip_hierarchical_clustering = false;
  end
  % options, pause
  
end

