function [t_cell_info] = master_script_produce_windows(t_cell_info, options)
  % 2013-11-07 tebuck: Copied from master_script_get_template_info.m.
  % 2016-02 xruan: change job scheduling method and add the check of tmp
  % files in the end. If there is any tmp files, delete them and run them. 
  % 
  % 
  % 
  % 
  % % Dependencies:
  % % From the File Exchange: export_fig, pmkmp, toolbox_graph, toolbox_fast_marching, affine, Snake3D, Mesh_voxelisation, dirr, inhull
  % % From tebuck: modified version of Jieyue Li's HPA_lib, rasterize_mesh

%
% Author: Taraz E. Buck and Xiongtao Ruan
%
% Copyright (C) 2012-2016 Murphy Lab
% Computational Biology Department
% School of Computer Science
% Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu



  
  
  
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % Produce windows from images corresponding to individual synapses:
  
  
  % Variables referenced more than once can be copied to local variables:
  master_script_options = t_cell_info.options;
  fprintf_diary_cycle_function = t_cell_info.log_info.fprintf_diary_cycle_function;
  regions_location = t_cell_info.path_info.regions_location;
  room = t_cell_info.preprocessing_info.room;
  window_size_2d = t_cell_info.preprocessing_info.window_size_2d;
  window_center_2d = t_cell_info.preprocessing_info.window_center_2d;
  
  desired_voxel_size = t_cell_info.synapse_info.desired_voxel_size;
  design_voxel_size = t_cell_info.synapse_info.design_voxel_size;
  
  
  % save_run_finished_files = false
  % Necessary to save image loading for now:
  save_run_finished_files = true
  
  
  % error('Update to scale voxels to cubical, use proper param adjustments to scale!')
  warning('Update to scale voxels to cubical, use proper param adjustments to scale!')
  
  
  % Keep running until everything is finished:
  % Work is divided up by run:
  runs_uncomputed = 1:t_cell_info.synapse_info.number_runs;
  
  recently_did_any_work = false;
  did_any_work = false;
  
  while ~isempty(runs_uncomputed)
    
    recently_did_any_work = did_any_work;
    did_any_work = false;
    
    % warning('>>>> HACK, randomized window production order'), for run_index = randperm(t_cell_info.synapse_info.number_runs)
    for run_index = runs_uncomputed
      % run = t_cell_info.synapse_info.run_list{run_index};
      % Prepare useful variables:
      run_relative_path = t_cell_info.synapse_info.run_relative_path_list{run_index};
      run_key = t_cell_info.synapse_info.run_key_list{run_index};
      run_image_path = t_cell_info.synapse_info.run_images_list{run_index};
      run_brightfield_images = t_cell_info.synapse_info.run_brightfield_images_list{run_index};
      number_cells = t_cell_info.synapse_info.run_number_cells(run_index);
      number_frames = t_cell_info.synapse_info.run_number_frames(run_index);
      cell_synapse_point_tracks = t_cell_info.synapse_info.run_point_tracks{run_index};
      cell_relative_time_tracks = t_cell_info.synapse_info.run_relative_time_tracks{run_index};
      
      run_image_size = t_cell_info.synapse_info.run_image_size_list(run_index, :);
      run_image_voxel_size = t_cell_info.synapse_info.run_image_voxel_size_list(run_index, :);
      % current_room = room;
      % Assume room is in XY and X and Y sizes are equal:
      % current_room = room * design_voxel_size(1) / desired_voxel_size(1);
      % current_room = room * run_image_voxel_size(1) / desired_voxel_size(1);
      % current_room changes with the inverse with respect to current_z_desired_scale because, while using imtransform, we want to pack more voxels from the input image into the same space as the desired_voxel_size expects:
      current_room = room * desired_voxel_size(1) / run_image_voxel_size(1);
      % current_z_desired_scale = run_image_size(3) * run_image_voxel_size(3) / desired_voxel_size(3);
      % But, with imresize, we want to scale up voxels that are larger:
      current_z_desired_scale = run_image_voxel_size(3) / desired_voxel_size(3);
      
      % fprintf_diary_cycle_function('run_image_voxel_size %f x %f x %f, run_image_size %f x %f x %f, current_room %f, current_z_desired_scale %f\n', run_image_voxel_size, run_image_size, current_room, current_z_desired_scale)
      printed_run_work_message = false;

      if save_run_finished_files
        % This takes a long time, so do not repeat work by checking for a file indicating this synapse file's windows are finished:
        synapse_region_saving_finished_filename = [run_key, '_finished'];
        % xruan 02/24/2016
        % [can_start, final_name, final_exists] = chunk_start(regions_location, synapse_region_saving_finished_filename);
        [can_start, final_name, final_exists] = chunk_start_clean(regions_location, synapse_region_saving_finished_filename);
        if final_exists
          % Don't try to do this run again:
          runs_uncomputed(runs_uncomputed == run_index) = [];
        end
      else
        can_start = true;
      end

      % if final_exists && ~false
        % f = dir([regions_location, run_key, '_frame*_synapse*.mat']);
        % for ind = 1:length(f)
          % a = load([regions_location, f(ind).name]);
          % % Debug plot:
          % % imshow(reshape_contrast(current_window_image, -1))
          % imshow(contrast_stretch(mean(a.current_window_image, 3)))
          % colormap(pmkmp)
          % pause
        % end
      % end

      if ~can_start
        continue
      end
      
      did_any_work = true;
      if ~printed_run_work_message
        fprintf_diary_cycle_function('run_image_voxel_size %f x %f x %f, run_image_size %f x %f x %f, current_room %f, current_z_desired_scale %f\n', run_image_voxel_size, run_image_size, current_room, current_z_desired_scale)
        printed_run_work_message = true;
      end
          
      fprintf_diary_cycle_function('Creating window images for run "%s"\n', run_key)
      
      number_windows_written = 0;

      run_dic_image = [];
      if ~isempty(run_brightfield_images)
        try
          run_dic_image = robust_read_stack(run_brightfield_images);
        catch raised_error
          % getReport(raised_error, 'extended')
          warning('Cannot read DIC time series!')
          % fprintf('Error during statistic computation: %s\n', getReport(raised_error, 'basic'))
          % fprintf('Data sizes: %d, %d, %d, %d\n', size(current_condition1_reduced_data), size(current_condition2_reduced_data))
          % continue
          % warning('Setting run_dic_image = []')
          run_dic_image = [];
        end
      end
      
      for frame_index = 1:number_frames
        current_frame_image_filename = run_image_path{frame_index};
        if isempty(current_frame_image_filename)
          continue
        end
        
        current_frame_image = robust_read_stack(current_frame_image_filename);
        
        if false
          whos run_dic_image
          current_frame_image_filename
          frame_index
          run_brightfield_images
        end
        
        if ~isempty(run_dic_image)
          current_frame_dic_image = run_dic_image(:, :, frame_index);
        end

        cell_indices_to_output = 1:size(cell_synapse_point_tracks, 1);
        % Produce windows for only one cell for each run (so maximum number of windows processed is (number_runs * number_all_relative_times):
        if master_script_options.debug_produce_windows_one_cell_per_run
          % cell_indices_to_output = 1;
          cell_indices_to_output = cell_indices_to_output(1:min(master_script_options.debug_produce_windows_number_cells_per_run, end));
        end
        
        for current_cell_index = cell_indices_to_output
        
          % Check if this cell appears in this frame:
          current_synapse_center_rounded = cell_synapse_point_tracks{current_cell_index, frame_index};
          if isempty(current_synapse_center_rounded)
            continue
          end
          current_synapse_annotations = cell_synapse_point_tracks{current_cell_index, frame_index};
          current_synapse_center_rounded = current_synapse_center_rounded(1, 1:2);
          current_relative_time = cell_relative_time_tracks{current_cell_index, frame_index};
          % % Continue if there is no synapse or it is outside of the image (no longer necessary due to change in master_script_get_synapse_info.m):
          % if current_synapse_center_rounded(1) < 1 || current_synapse_center_rounded(2) < 1 || current_synapse_center_rounded(1) > size(current_frame_image, 2) || current_synapse_center_rounded(2) > size(current_frame_image, 1)
            % continue
          % end
          % Write a window from this image to regions_location so further processing is just of those files and derivatives with similar names:
          current_window_image_filename = [regions_location, run_key, sprintf('_frame%05d_synapse%05d,%05d', frame_index, current_synapse_center_rounded)];
          
          current_window_image = imtransform(current_frame_image, maketform('affine', eye(3)), 'bilinear', 'UData', [1, size(current_frame_image, 2)], 'VData', [1, size(current_frame_image, 1)], 'Size', [window_size_2d, size(current_frame_image, 3)], 'XData', current_synapse_center_rounded(1) + current_room * [-1, 1], 'YData', current_synapse_center_rounded(2) + current_room * [-1, 1], 'FillValues', nan);
          % Rescale 
          current_window_image = image_resize_nd(current_window_image, [1, 1, current_z_desired_scale], 'bilinear', true);
            
          current_window_dic_image = [];
          if ~isempty(run_dic_image)
            current_window_dic_image = imtransform(current_frame_dic_image, maketform('affine', eye(3)), 'UData', [1, size(current_frame_dic_image, 2)], 'VData', [1, size(current_frame_dic_image, 1)], 'Size', window_size_2d, 'XData', current_synapse_center_rounded(1) + current_room * [-1, 1], 'YData', current_synapse_center_rounded(2) + current_room * [-1, 1], 'FillValues', nan);
          end
          
          % Transform annotations for alignment convenience:
          
          % current_synapse_center_rounded_transformed = window_center_2d;
          % N x 2:
          current_synapse_annotations_transformed = reshape(current_synapse_annotations, 2, []).';
          % Translate and scale:
          current_synapse_annotations_transform = struct('current_synapse_center_rounded', current_synapse_center_rounded, 'current_room', current_room, 'room', room);
          current_synapse_annotations_transform_function = @(given_annotations)(given_annotations - repmat(current_synapse_annotations_transform.current_synapse_center_rounded, [size(given_annotations, 1), 1]) + current_synapse_annotations_transform.current_room) .* (current_synapse_annotations_transform.room / current_synapse_annotations_transform.current_room) + 1;
          current_synapse_annotations_transformed = current_synapse_annotations_transform_function(current_synapse_annotations_transformed);
          current_synapse_center_rounded_transformed = current_synapse_annotations_transformed(1, :);
          
          
          if false
            % Debug info:
            % close
            set(gcf, 'visible', 'on')
            whos current_window_image
            % imshow(reshape_contrast(current_window_image, -1))
            % imagesc(reshape_2d(current_window_image, -1)), axis image, colormap(t_cell_info.plotting_info.illustration_colormap_to_use), colorbar
            imagesc(mean(current_window_image, 3)), axis image, colormap(t_cell_info.plotting_info.illustration_colormap_to_use), colorbar
            hold on
            scatter(current_synapse_annotations_transformed(:, 1), current_synapse_annotations_transformed(:, 2), 'rx')
            hold off
            window_center_2d, current_synapse_center_rounded_transformed
            % beep, keyboard
            beep, pause
          end
          
          % error('Update to scale voxels to cubical, use proper param adjustments to scale!')
          
          % keyboard
          current_window_image = extrapolate_nans(current_window_image);

          if false
            % Debug plot:
            % imshow(reshape_contrast(current_window_image, -1))
            imshow(contrast_stretch(mean(current_window_image, 3)))
            pause
          end
          save([current_window_image_filename, '.mat'], 'current_window_image', 'current_window_dic_image', 'current_synapse_center_rounded', 'current_synapse_annotations', 'current_synapse_annotations_transform', 'current_synapse_center_rounded_transformed', 'current_synapse_annotations_transformed', 'current_relative_time')

          number_windows_written = number_windows_written + 1;
          
        end
      end
      
      fprintf_diary_cycle_function('  Created %d window images\n', number_windows_written)
      
      if save_run_finished_files
        temp = struct; save(final_name, '-struct', 'temp');
        chunk_finish(regions_location, synapse_region_saving_finished_filename);
      end
      
      runs_uncomputed(runs_uncomputed == run_index) = [];
      
    end
    
    pause(1)
    
    % xruan 02/24/2016
    out_status = chunk_lock_clean(regions_location);
    if out_status
        fprintf_diary_cycle_function('Delete lock files whose jobs are not running!\n')
    end
            
    if recently_did_any_work && ~did_any_work
      fprintf_diary_cycle_function('Apparently no more work to do, wait for other jobs')
    end
    
  end
  
  
  t_cell_info.produce_windows_info = struct();
  t_cell_info.produce_windows_info.room = room;
  
  
  
end  

