function [t_cell_info] = master_script_segment_windows_new(t_cell_info, options)
  % 2013-11-07 tebuck: Copied from master_script_produce_windows.m.
  % 07/16/2015 xruan: set option for the segmenation of two point
  % annotation. 
  % 
  % 
  % 
  % 
  % % Dependencies:
  % % From the File Exchange: export_fig, pmkmp, toolbox_graph, toolbox_fast_marching, affine, Snake3D, Mesh_voxelisation, dirr, inhull
  % % From tebuck: modified version of Jieyue Li's HPA_lib, rasterize_mesh

%
% Author: Taraz E. Buck and Xiongtao Ruan
%
% Copyright (C) 2012-2016 Murphy Lab
% Computational Biology Department
% School of Computer Science
% Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu



  
  
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % Segment windows corresponding to individual synapses:

  
  % Variables referenced more than once can be copied to local variables:
  master_script_options = t_cell_info.options;
  fprintf_diary_cycle_function = t_cell_info.log_info.fprintf_diary_cycle_function;
  regions_location = t_cell_info.path_info.regions_location;
  segmentations_location = t_cell_info.path_info.segmentations_location;
  % condition_retrieval_function = t_cell_info.synapse_info.condition_retrieval_function;
  sensors = t_cell_info.synapse_info.sensors;
  number_sensors = t_cell_info.synapse_info.number_sensors;
  sensor_retrieval_function = t_cell_info.synapse_info.sensor_retrieval_function;
  number_runs = t_cell_info.synapse_info.number_runs;
  
  window_center_2d = t_cell_info.preprocessing_info.window_center_2d;
  window_size_2d = t_cell_info.preprocessing_info.window_size_2d;
  
  template_cell_radius = t_cell_info.template_info.template_cell_radius;
  background_subtraction_function = t_cell_info.preprocessing_info.background_subtraction_function;
  
  sensor_specific_segmentation_coarse_snake_options = master_script_options.sensor_specific_segmentation_coarse_snake_options;
  sensor_specific_segmentation_fine_snake_options = master_script_options.sensor_specific_segmentation_fine_snake_options;
  
  % Add things other code might need to segmentation_info:
  
  % rasterization_oversampling_scale = 1;
  rasterization_oversampling_scale = 2;
  % rasterization_oversampling_scale = 4;
  segmentation_rasterization_function = @(given_mesh, window_size)rasterize_mesh(given_mesh, struct('oversampling_scale', rasterization_oversampling_scale, 'cropping', [ones(3, 1), window_size']));
  % segmentation_rasterization_function = @(given_mesh)rasterize_mesh(given_mesh, struct('oversampling_scale', rasterization_oversampling_scale, 'cropping', [ones(3, 1), [window_size_2d, window_number_slices]']));
  t_cell_info.segmentation_info.rasterization_oversampling_scale = rasterization_oversampling_scale;
  t_cell_info.segmentation_info.segmentation_rasterization_function = segmentation_rasterization_function;
  t_cell_info.segmentation_info.window_preprocessing_function = @two_stage_snake_segmentation_window_preprocessing;
  t_cell_info.segmentation_info.two_stage_segmentation = @two_stage_snake_segmentation;

  
  % Segmentation-specific options:
  two_stage_segmentation_options = struct;
  two_stage_segmentation_options.window_center_2d = window_center_2d;
  two_stage_segmentation_options.seed_radius = master_script_options.seed_radius;
  two_stage_segmentation_options.cell_diameter = master_script_options.cell_diameter;
  two_stage_segmentation_options.background_subtraction_function = background_subtraction_function;
  two_stage_segmentation_options.coarse_snake_options = master_script_options.segmentation_coarse_snake_options;
  two_stage_segmentation_options.fine_snake_options = master_script_options.segmentation_fine_snake_options;
  
  if master_script_options.skip_segmentation
  
    warning('>>>> HACK, options.skip_segmentation true!')
  
  else
    
    % error('Update to scale voxels to cubical, use proper param adjustments to scale!')
    warning('Update to scale voxels to cubical, use proper param adjustments to scale!')
    
    
    % Keep running until everything is finished:
    % Work is divided up by cell and frame:
    cell_index_frame_indices_uncomputed = cell(number_runs, 1);
    for run_index = 1:number_runs
      run_image_path = t_cell_info.synapse_info.run_images_list{run_index};
      number_cells = t_cell_info.synapse_info.run_number_cells(run_index);
      number_frames = t_cell_info.synapse_info.run_number_frames(run_index);
      cell_synapse_point_tracks = t_cell_info.synapse_info.run_point_tracks{run_index};
      cell_index_frame_indices_uncomputed{run_index} = false(size(cell_synapse_point_tracks, 1), number_frames);
      for frame_index = 1:number_frames
        current_frame_image_filename = run_image_path{frame_index};
        if isempty(current_frame_image_filename)
          continue
        end
        for current_cell_index = 1:number_cells
          current_synapse_center_rounded = cell_synapse_point_tracks{current_cell_index, frame_index};
          if isempty(current_synapse_center_rounded)
            continue
          end
          
          first_frame_index = find(~cellfun(@isempty, cell_synapse_point_tracks(current_cell_index, :)), 1);
          if isempty(first_frame_index)
            run_cell_indices_uncomputed{run_index}(current_cell_index) = false;
            continue
          end
          
          cell_index_frame_indices_uncomputed{run_index}(current_cell_index, frame_index) = true;
        end
      end
    end

    recently_did_any_work = false;
    did_any_work = false;
    
    while any(cellfun(@(x)any(x(:)), cell_index_frame_indices_uncomputed))
      
      recently_did_any_work = did_any_work;
      did_any_work = false;
      
      for run_index = 1:number_runs
        % Prepare useful variables:
        run_relative_path = t_cell_info.synapse_info.run_relative_path_list{run_index};
        run_key = t_cell_info.synapse_info.run_key_list{run_index};
        run_image_path = t_cell_info.synapse_info.run_images_list{run_index};
        number_cells = t_cell_info.synapse_info.run_number_cells(run_index);
        number_frames = t_cell_info.synapse_info.run_number_frames(run_index);
        cell_synapse_point_tracks = t_cell_info.synapse_info.run_point_tracks{run_index};
        cell_relative_time_tracks = t_cell_info.synapse_info.run_relative_time_tracks{run_index};
        
        % fprintf_diary_cycle_function('Segmenting window images for run "%s"\n', run_key)
        printed_run_work_message = false;
        
        % Go through frames of each cell:
        % for current_cell_index = reshape(find(any(cell_index_frame_indices_uncomputed{run_index}, 2)), 1, [])
        for frame_index = 1:number_frames
          current_frame_image_filename = run_image_path{frame_index};
          if isempty(current_frame_image_filename)
            continue
          end
            
          for current_cell_index = 1:number_cells

            current_synapse_center_rounded = cell_synapse_point_tracks{current_cell_index, frame_index};
            if isempty(current_synapse_center_rounded)
              continue
            end
            current_synapse_center_rounded = current_synapse_center_rounded(1, 1:2);
            
            if ~cell_index_frame_indices_uncomputed{run_index}(current_cell_index, frame_index)
              continue
            end
            
            % Check for work done:
            cell_segmentation_filename = [run_key, sprintf('_frame%05d_synapse%05d,%05d', frame_index, current_synapse_center_rounded)];
            cell_region_filename = [regions_location, cell_segmentation_filename];
            % cell_segmentation_filename = [segmentations_location, cell_segmentation_filename];
            
            % Shouldn't be any additional exceptions, disabled:
            % if ~exist([cell_region_filename, '.mat'], 'file')
              % continue
            % end
            
            if false
              % Debug info:
              segmentations_location, cell_segmentation_filename
              % error
            end
            % xruan 02/24/2016
            % [can_start, final_name, final_exists] = chunk_start(segmentations_location, cell_segmentation_filename);
            [can_start, final_name, final_exists] = chunk_start_clean(segmentations_location, cell_segmentation_filename);
            
            if final_exists
              cell_index_frame_indices_uncomputed{run_index}(current_cell_index, frame_index) = false;
            end
            
            if ~can_start
              continue
            end
            
            did_any_work = true;
            if ~printed_run_work_message
              fprintf_diary_cycle_function('Segmenting window images for run "%s"\n', run_key)
              printed_run_work_message = true;
            end;
            
            a = load([cell_region_filename, '.mat']);
            current_window_dic_image = a.current_window_dic_image;
            current_window_image = a.current_window_image;
            
            % Segment the 3D fluorescent image:
            window_number_slices = size(current_window_image, 3);
            current_two_stage_segmentation_options = two_stage_segmentation_options;
            current_sensor = sensor_retrieval_function({run_key});
            relevant_coarse_snake_option_indices = strcmpi(sensor_specific_segmentation_coarse_snake_options(:, 1), current_sensor);
            relevant_fine_snake_option_indices = strcmpi(sensor_specific_segmentation_fine_snake_options(:, 1), current_sensor);
            
            % xruan 07/16/2015
            % check if segment the right image
            current_two_stage_segmentation_options.current_synapse_annotations_transformed = a.current_synapse_annotations_transformed;
            % xruan 07/21/2015
            % judge one or two point annotation in two stage segmentaiton
            current_two_stage_segmentation_options.use_two_point_synapses = master_script_options.use_two_point_synapses;            
            
            % Assumes custom parameters will either exist or not exist together for coarse and fine:
            switch sum(relevant_coarse_snake_option_indices)
              case 0
                ;
              case 1
                current_two_stage_segmentation_options.coarse_snake_options = sensor_specific_segmentation_coarse_snake_options{relevant_coarse_snake_option_indices, 2};
                current_two_stage_segmentation_options.fine_snake_options = sensor_specific_segmentation_fine_snake_options{relevant_fine_snake_option_indices, 2};
              otherwise
                error('More than one relevant segmentation option set for sensor %s!', current_sensor)
            end
            [segmentation_structure] = two_stage_snake_segmentation(two_stage_snake_segmentation_window_preprocessing(current_window_image, current_two_stage_segmentation_options), current_two_stage_segmentation_options);
            
            window_filename = cell_region_filename;
            save(final_name, 'segmentation_structure', 'window_filename', 'window_number_slices')
            chunk_finish(segmentations_location, cell_segmentation_filename);
            cell_index_frame_indices_uncomputed{run_index}(current_cell_index, frame_index) = false;
          end
        end
      end
      
      pause(1)
      
      % xruan 02/24/2016
      out_status = chunk_lock_clean(segmentations_location);
      if out_status
          fprintf_diary_cycle_function('Delete lock files whose jobs are not running!\n')
      end
      
      if recently_did_any_work && ~did_any_work
        fprintf_diary_cycle_function('Apparently no more work to do, wait for other jobs')
      end
      
    end
    
    
    
    if false
      fprintf('Finished segmenting windows, now check window_sensor_index_list!\n')
      % window_sensor_index_list(window_index) = sensor_retrieval_function(window_segmentation_filename);
      sensors_with_counts = [sensors, num2cell(hist(window_sensor_index_list, 1:number_sensors)).']
      beep, keyboard
    end

  
  end
  

  % t_cell_info.segmentation_info = struct();
  
end

