function [t_cell_info] = master_script_trace_cell_info_in_models(t_cell_info, options)
  % Fit parametric distributions to the protein intensities in the template space.
  %
  % 2015-11-04 xruan: Copied from master_script_build_models.m.
  % 
  % 
  % 
  % 
  % 
  % % Dependencies:
  % % From the File Exchange: export_fig, pmkmp, toolbox_graph, toolbox_fast_marching, affine, Snake3D, Mesh_voxelisation, dirr, inhull
  % % From tebuck: modified version of Jieyue Li's HPA_lib, rasterize_mesh

%
% Author: Taraz E. Buck and Xiongtao Ruan
%
% Copyright (C) 2012-2016 Murphy Lab
% Computational Biology Department
% School of Computer Science
% Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu



  
  
  
  % Variables referenced more than once can be copied to local variables:
  master_script_options = t_cell_info.options;
  fprintf_diary_cycle_function = t_cell_info.log_info.fprintf_diary_cycle_function;
  regions_location = t_cell_info.path_info.regions_location;
  segmentations_location = t_cell_info.path_info.segmentations_location;
  segmentations_filtered_location = t_cell_info.path_info.segmentations_filtered_location;
  alignments_location = t_cell_info.path_info.alignments_location;
  deformations_location = t_cell_info.path_info.deformations_location;
  models_location = t_cell_info.path_info.models_location;
  
  run_relative_path_list = t_cell_info.synapse_info.run_relative_path_list;
  run_key_list = t_cell_info.synapse_info.run_key_list;
  % run_data_list = t_cell_info.synapse_info.run_data_list;
  run_images_list = t_cell_info.synapse_info.run_images_list;
  run_brightfield_images_list = t_cell_info.synapse_info.run_brightfield_images_list;
  run_image_frame_indices_list = t_cell_info.synapse_info.run_image_frame_indices_list;
  run_number_cells = t_cell_info.synapse_info.run_number_cells;
  run_number_frames = t_cell_info.synapse_info.run_number_frames;
  run_point_tracks = t_cell_info.synapse_info.run_point_tracks;
  run_relative_time_tracks = t_cell_info.synapse_info.run_relative_time_tracks;
  run_relative_times = t_cell_info.synapse_info.run_relative_times;
  run_number_relative_times = t_cell_info.synapse_info.run_number_relative_times;
  all_relative_times = t_cell_info.synapse_info.all_relative_times;
  number_all_relative_times = t_cell_info.synapse_info.number_all_relative_times;
  all_relative_times_seconds_data = t_cell_info.synapse_info.all_relative_times_seconds_data;
  all_relative_times_seconds = t_cell_info.synapse_info.all_relative_times_seconds;
  number_runs = t_cell_info.synapse_info.number_runs;
  
  condition_sensor_combinations = t_cell_info.synapse_info.condition_sensor_combinations;
  number_condition_sensors = t_cell_info.synapse_info.number_condition_sensors;
  conditions = t_cell_info.synapse_info.conditions;
  sensors = t_cell_info.synapse_info.sensors;
  number_conditions = t_cell_info.synapse_info.number_conditions;
  number_sensors = t_cell_info.synapse_info.number_sensors;
  condition_sensor_combination_ismember = t_cell_info.synapse_info.condition_sensor_combination_ismember;
  
  model_types = t_cell_info.model_type_info.model_types;
  number_model_types = t_cell_info.model_type_info.number_model_types;
  model_representation_functions = t_cell_info.model_type_info.model_representation_functions;
  model_reconstruction_functions = t_cell_info.model_type_info.model_reconstruction_functions;
  model_dimensionality_reduction_methods = t_cell_info.model_type_info.model_dimensionality_reduction_methods;
  model_dimensionality_reduction_method_parameters = t_cell_info.model_type_info.model_dimensionality_reduction_method_parameters;
  model_dimensionality_reduction_method_transformations = t_cell_info.model_type_info.model_dimensionality_reduction_method_transformations;
  
  template_centroid = t_cell_info.template_info.template_centroid;
  template_synapse = t_cell_info.template_info.template_synapse;
  template_image = t_cell_info.template_info.template_image;

  window_size_2d = t_cell_info.preprocessing_info.window_size_2d;
  window_center_2d = t_cell_info.preprocessing_info.window_center_2d;
  cropped_size = t_cell_info.preprocessing_info.cropped_size;
  background_level_function = t_cell_info.preprocessing_info.background_level_function;

  segmentation_rasterization_function = t_cell_info.segmentation_info.segmentation_rasterization_function;
  
  template_volume = t_cell_info.alignment_info.template_volume;
  segmentation_volume_threshold = t_cell_info.alignment_info.segmentation_volume_threshold;
  
  
  % debug_print_model_filenames = false;
  debug_print_model_filenames = true
  
  
  condition_sensor_combinations, number_condition_sensors, conditions, sensors, number_conditions, number_sensors
  model_types, number_model_types
  number_all_relative_times
  
  % error('Implementation yet unfinished below this line!')
    


  if master_script_options.skip_model_building
    
    warning('>>>> HACK, options.skip_model_building true!')
    
  else
    
    
    warning('Need to ensure all above computations are finished somehow before building models... Maybe block on every single good deformation''s file?')
    warning('Nevermind, can always rebuild!')
    % % [~, ~, deformations_list] = dir(deformations_location, 'name', '');
    % deformations_list = dir(deformations_location);
    % deformations_list = {deformations_list.name}.';
    % % deformations_list = strcat(deformations_location, deformations_list);
    % relevant_pattern = ['\.tmp$'];
    % number_temporary_deformation_files = sum(~cellfun(@isempty, regexp(deformations_list, relevant_pattern, 'start')))
    % if number_temporary_deformation_files > 0
      % error('%d temporary deformation files remain!', number_temporary_deformation_files)
    % end
    
    % deformations_list = master_script_list_relevant_results(t_cell_info, struct('given_directory', deformations_location));
    % deformations_list, beep, keyboard
    
    
    
    for condition_sensor_combination_index = 1:number_condition_sensors
      current_condition = condition_sensor_combinations{condition_sensor_combination_index, 1};
      current_sensor = condition_sensor_combinations{condition_sensor_combination_index, 2};
      % current_condition, current_sensor
      % error
      
      fprintf('condition_sensor_combination_index %d, current_condition "%s", current_condition "%s"\n', condition_sensor_combination_index, current_condition, current_sensor)

      % Find relevant runs:
      % Should this just compare .mat files for speed?:
      run_list_relevant = true(size(run_key_list));
      % relevant_pattern = ['^[^-]+ - ', current_condition, '_', current_sensor, '\.xls'];
      relevant_pattern = ['^[^-]+ - ', current_condition, '_', current_sensor, '_'];
      run_list_relevant = run_list_relevant & ~cellfun(@isempty, regexp(run_key_list, relevant_pattern, 'start'));
      relevant_runs = run_key_list(run_list_relevant);
      relevant_run_indices = reshape(find(run_list_relevant), 1, []);
      
      % relevant_runs, beep, keyboard
      
      % We want separate model files for each relative time to keep down memory usage and sooner have intermediate results:
      
      model_used_filenames = {};
      model_data_group = {};
      
      for all_relative_time_index = 1:number_all_relative_times
        relative_time = all_relative_times(all_relative_time_index);
        model_used_filenames{all_relative_time_index} = {};
        
        fprintf(' all_relative_time_index %d, relative_time % d\n', all_relative_time_index, relative_time)
        
        % We want separate model files for each model type (makes adding new model types easier):
        
        for model_type_index = 1:number_model_types
          model_type = model_types{model_type_index};
          if ~strcmp(model_type, 'standardized_voxels')
              continue;
          end
          
          representation_function = model_representation_functions{model_type_index};
          reconstruction_function = model_reconstruction_functions{model_type_index};

          % This is the .mat file, not the figure:
          current_model_filename = sprintf('%s%s_%s_reltime% 3d_%s', master_script_options.model_prefix, current_condition, current_sensor, relative_time, model_type);
          % current_model_filename = sprintf('%s_%s_frame%03d_reltime% 3d_%s', current_condition, current_sensor, all_relative_time_index, relative_time, model_type);
          % [can_start, final_name, final_exists] = chunk_start(models_location, current_model_filename);
          
          % if debug_print_model_filenames, fprintf_diary_cycle_function('Model file ''%s'': can_start %d, final_exists %d\n', current_model_filename, can_start, final_exists), end
          
          
          if false
            % Debugging:
            warning('>>>> HACK, always building models!')
            can_start = true;
            chunk_finish(models_location, current_model_filename);
          end
          
            % if ~can_start
            %     continue
            % end
          
          fprintf('  model_type_index %d, model_type "%s"\n', model_type_index, model_type)
          
          current_count = 0;
          
          % Running/accumulated model:
          current_mean = [];
          current_standard_deviation = [];
          current_covariance = [];
          
          should_compute_covariance = false;
          
          % Batch model:
          % Preallocate as much space as might be necessary:
          current_number_cells = sum(cellfun(@(x)size(x, 1), run_relative_time_tracks(relevant_run_indices)));
          current_number_features = numel(representation_function(double(template_image)));
          current_data = zeros(current_number_cells, current_number_features);
          
          current_data_relevant_run_keys = run_key_list(relevant_run_indices);
          current_data_relevant_run_key_indices = zeros(current_number_cells, 1);
          
          for run_index = relevant_run_indices
            % Prepare useful variables:
            run_relative_path = run_relative_path_list{run_index};
            run_key = run_key_list{run_index};
            run_image_path = run_images_list{run_index};
            number_cells = run_number_cells(run_index);
            number_frames = run_number_frames(run_index);
            cell_synapse_point_tracks = run_point_tracks{run_index};
            cell_relative_time_tracks = run_relative_time_tracks{run_index};
            
            cell_relative_time_matrix = cellfun(@(x)[x, nan(1, 1 - length(x))], cell_relative_time_tracks);
            
            [relative_time_cell_indices, relative_time_frame_indices] = find(cell_relative_time_matrix == relative_time);
            
            for cell_frame_pair_index = 1:length(relative_time_cell_indices)
              cell_index = relative_time_cell_indices(cell_frame_pair_index);
              frame_index = relative_time_frame_indices(cell_frame_pair_index);
              current_synapse_center_rounded = cell_synapse_point_tracks{cell_index, frame_index};
              if isempty(current_synapse_center_rounded)
                continue
              end
              current_synapse_center_rounded = current_synapse_center_rounded(1, 1:2);
              cell_deformation_filename = [run_key, sprintf('_frame%05d_synapse%05d,%05d', frame_index, current_synapse_center_rounded)];
              
              if false && all_relative_time_index == 1 && model_type_index == 1
                % Debug info:
                fprintf('   cell_frame_pair_index %d, cell_index %d, frame_index %d\n', cell_frame_pair_index, cell_index, frame_index)
                % keyboard
              end
              
              % Eventually, this should wait for all .mat files, and master_script_morph_segmentations.m should save .mat files with empty results if failed instead of leaving .tmp files. This way, running multiple jobs, others will quit when chunk_start here fails but the job that computes will get everything in the model files.
              
              if ~exist([deformations_location, cell_deformation_filename, '.mat'], 'file')
                if false && exist([segmentations_location, cell_deformation_filename, '.mat'], 'file')
                  fprintf('   "%s" does not exist, but segmentation does!\n', [deformations_location, cell_deformation_filename, '.mat'])
                  % beep, keyboard
                end
                if false && exist([segmentations_filtered_location, cell_deformation_filename, '.mat'], 'file')
                  fprintf('   "%s" does not exist, but filtered segmentation does!\n', [deformations_location, cell_deformation_filename, '.mat'])
                  % beep, keyboard
                end
                if false && all_relative_time_index == 1 && model_type_index == 1
                  % Debug info:
                  fprintf('   "%s" does not exist!\n', [deformations_location, cell_deformation_filename, '.mat'])
                  % run_existing_files = ls([deformations_location, run_key, '*.mat'])
                  % run_existing_files = arrayfun(@(x)x.name, dir([deformations_location, run_key, '*.mat']), 'UniformOutput', false)
                  % beep, keyboard
                  % pause
                end
                continue
              end

              % fprintf('   "%s" exists!\n', [deformations_location, cell_deformation_filename, '.mat'])

              a = load([regions_location, cell_deformation_filename, '.mat']);
              if isempty(fieldnames(a))
                continue
              end
              current_window_image = a.current_window_image;
              try
                a = load([deformations_location, cell_deformation_filename, '.mat']);
              catch 
                  continue
              end
              if isempty(fieldnames(a))
                continue
              end
              cropped_raw_image_deformed = a.cropped_raw_image_deformed;
              cropped_segmentation_image_deformed = a.cropped_segmentation_image_deformed;
              
              current_volume = sum(cropped_segmentation_image_deformed(:));
              is_segmentation_low_volume = current_volume <= segmentation_volume_threshold;
              
              % error('Correct if stmt')
              if is_segmentation_low_volume
                continue
              end
              
              % keyboard
              
              % current_deformed_image = cropped_raw_image_deformed .* template_image;
              % current_deformed_image(template_image > .5) = contrast_stretch(current_deformed_image(template_image > .5));
              % current_deformed_image = constrast_stretch_protein_in_template(cropped_raw_image_deformed);
              % current_deformed_image = constrast_stretch_protein_in_template(background_subtraction_function(cropped_raw_image_deformed));
              
              % Subtract the original image's estimated background, then normalize total intensity to one:
              current_deformed_image = cropped_raw_image_deformed - background_level_function(current_window_image);
              current_deformed_image(current_deformed_image < 0) = 0;
              current_deformed_image(template_image < .5) = 0;
              current_deformed_image = current_deformed_image ./ sum(current_deformed_image(:));
              
              current_parameters = representation_function(current_deformed_image);
              
              if false
                current_parameters_norm = norm(current_parameters)
                imagesc(reshape_2d(current_deformed_image, -1)), axis image, colormap(illustration_colormap_to_use), pause
              end
              model_used_filenames{all_relative_time_index}{end + 1} = cell_deformation_filename;
              % Batch model:
              current_count = current_count + 1;
              % current_data(current_count, :) = reshape(current_parameters, 1, []);
              current_data(current_count, :) = current_parameters;
              % keyboard
              current_data_relevant_run_key_indices(current_count, :) = find(ismember(current_data_relevant_run_keys, run_key));
              
            end

            if false && all_relative_time_index == 1 && model_type_index == 1
              % Debug info:
              keyboard
            end
            
          end
          
          % Batch model:
          current_data = current_data(1:current_count, :);
          current_mean = mean(current_data, 1);
          current_standard_deviation = std(current_data, [], 1);
          if should_compute_covariance
            current_covariance = cov(current_data);
          end
          current_data_relevant_run_key_indices = current_data_relevant_run_key_indices(1:current_count, :);

          % if any(isnan(current_mean))
            % beep, keyboard
          % end
          
          fprintf('   current_count %d\n', current_count)
          
          if false && current_count == 0
            % Debug info:
            beep, keyboard
          end
          
          if current_count > 0
            % save(final_name, 'current_data', 'current_mean', 'current_standard_deviation', 'current_covariance', 'current_data_relevant_run_keys', 'current_data_relevant_run_key_indices')
            % Leave the .tmp lock file so this work isn't repeated:
            % chunk_finish(models_location, current_model_filename);
          end
          model_data_group{all_relative_time_index} = current_data;
          current_model_file = [models_location, current_model_filename, '.mat'];
          if exist(current_model_file, 'file')
              a = load(current_model_file);
          end
          
          % check if the current result is the same as existing model
          
          if sum(a.current_mean - current_mean ) < 1^(-10)
              disp('the current running can reproduce the model before')
          else
              disp('the current running is different from the running before.')
          end
          
                 
          % Leave the .tmp lock file so this work isn't repeated:
          % chunk_finish(models_location, current_model_filename);
          
          current_count = 0;
          % pause
        end
      end
      
      [~, compare_time_ind] = max(cellfun(@length, model_used_filenames));
      compare_reltime_filenames = model_used_filenames{compare_time_ind};
      time_series_data = {};
      for i = 1 : length(compare_reltime_filenames)
        current_data_mat = NaN(length(all_relative_times), 6628);
        current_cell = compare_reltime_filenames{i};
        current_cell_info = regexp(current_cell, '_', 'split');
        current_reltime = all_relative_times(compare_time_ind);
        current_frame = str2num(current_cell_info{end - 1}(end - 1 : end));

        current_synapse = regexp(current_cell_info{end}, 'synapse([0-9]+,[0-9]+)', 'tokens')   ;
        current_synapse = current_synapse{1};

        run_relative_path = strjoin(current_cell_info(1 : end - 2), '_');
        run_index = find(~cellfun(@isempty, regexp(run_relative_path_list, run_relative_path)) == 1);

        run_key = run_key_list{run_index};
        run_image_path = run_images_list{run_index};
        number_cells = run_number_cells(run_index);
        number_frames = run_number_frames(run_index);
        cell_synapse_point_tracks = run_point_tracks{run_index};
        cell_relative_time_tracks = run_relative_time_tracks{run_index};


        cell_synapse_current_frame = {cell_synapse_point_tracks{:, current_frame}};
        [exist_synapse, exist_ind] = find(cellfun(@isempty, cell_synapse_current_frame) == 0);
        cell_index = exist_ind(~cellfun(@isempty, regexp(cellfun(@(x) sprintf('%05d,%05d', x(1), x(2)), cell_synapse_current_frame(exist_ind), 'UniformOutput', false), current_synapse)));
        for j = 1 : length(all_relative_times)
            reltime_j = all_relative_times(j);
            frame_index = find(cellfun(@(x) strcmp(num2str(x), num2str(reltime_j)), cell_relative_time_tracks(cell_index, :)) == 1);
            if isempty(frame_index)
                continue
            end

            current_synapse_center_rounded = cell_synapse_point_tracks{cell_index, frame_index};
            if isempty(current_synapse_center_rounded)
                continue
            end
            current_synapse_center_rounded = current_synapse_center_rounded(1, 1:2);
            cell_deformation_filename = [run_key, sprintf('_frame%05d_synapse%05d,%05d', frame_index, current_synapse_center_rounded)];
            data_ind = find(cellfun(@(x) strcmp(cell_deformation_filename, x), model_used_filenames{j}) == 1);

            if isempty(data_ind)
                continue;
            end
            current_data_mat(j , :) = model_data_group{j}(data_ind, :);
        end
        time_series_data{i} = current_data_mat;
      end
      
      save_dir = '~/projects/tcell_simulation/data/';
      mkdir(save_dir);
      save(sprintf('%s%s%s_%s_time_series_data.mat', save_dir, master_script_options.model_prefix, current_condition, current_sensor), 'time_series_data', 'model_used_filenames',  'model_data_group');
           
      
      % pause
    end
    % error

  end
  
  
  % beep, keyboard
  % error('Implementation yet unfinished below this line!')
  warning('Implementation yet unfinished below this line!')

  t_cell_info.model_info = struct();
  t_cell_info.model_info.model_representation_functions = model_representation_functions;
  t_cell_info.model_info.model_reconstruction_functions = model_reconstruction_functions;
  
  t_cell_info.model_info.model_filenames = model_filenames;
  t_cell_info.model_info.model_reduced_filenames = model_reduced_filenames;
  
  
end  

