function [t_cell_info] = master_script_view_radial_distributions(t_cell_info, options)
  % Produces figures showing radial distributions for all models and model types.
  %
  % 2014-10-03 tebuck: Copied from master_script_optimize_segmentation_parameters.m.
  % % Dependencies:
  % % From the File Exchange: export_fig, pmkmp, toolbox_graph, toolbox_fast_marching, affine, Snake3D, Mesh_voxelisation, dirr, inhull
  % % From tebuck: modified version of Jieyue Li's HPA_lib, rasterize_mesh

%
% Author: Taraz E. Buck and Xiongtao Ruan
%
% Copyright (C) 2012-2016 Murphy Lab
% Computational Biology Department
% School of Computer Science
% Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu



  
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % Initialize logging and make subdirectory for storing logs:

  
  start_time = tic;

  filename = mfilename
  base_filename = [filename, '/']; 

  warning('off', 'MATLAB:MKDIR:DirectoryExists');
  warning('off', 'Images:initSize:adjustingMag');
  warning('off', 'MATLAB:xlsread:Mode');
  mkdir(filename)

  if length(dbstack) < 2
    % This might be called by other functions, so don't start a diary in such cases in case the calling functions have already done so:
    start_job_diary(base_filename)
  end

  
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % Set defaults and process options structure:
  
  
  default_options = master_script_get_default_options;
  default_options.use_profiling = false;
  
  if ~exist('options', 'var')
    options = default_options;
  else
    options_prior_to_processing = options;
    options = master_script_process_options(options);
    options = process_options_structure(default_options, options);
  end
  
  
  % Profile code?
  if options.use_profiling
    profile('-memory','on'); profile reset; profile on;
  end
  
  
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % Run master_script part of the way so we don't have to duplicate its setup code here:
  
  
  options_for_master_script = options;
  % Skip everything:
  options_for_master_script.skip_window_saving = true;
  options_for_master_script.skip_segmentation = true;
  options_for_master_script.skip_alignment = true;
  options_for_master_script.skip_morphing = true;
  options_for_master_script.skip_morphing_inside_template = true;
  options_for_master_script.skip_model_building = true;
  options_for_master_script.skip_hierarchical_clustering = true;
  options_for_master_script.skip_model_figure_generation = true;
  options_for_master_script.skip_model_video_saving = true;
  options_for_master_script.skip_illustration_saving = true;
  options_for_master_script.skip_statistical_testing_across_conditions = true;
  options_for_master_script.skip_create_blender_figures = true;
  options_for_master_script.use_profiling = false;

  % t_cell_info = master_script(options_for_master_script);
  master_script_options = t_cell_info.options;
  % t_cell_info, dbstack, beep, keyboard, error
  
  fprintf_diary_cycle_function = t_cell_info.log_info.fprintf_diary_cycle_function;
  % regions_location = t_cell_info.path_info.regions_location;
  % segmentations_location = t_cell_info.path_info.segmentations_location;
  models_location = t_cell_info.path_info.models_location;
  figures_location = t_cell_info.path_info.figures_location;
  
  illustration_colormap_to_use = t_cell_info.plotting_info.illustration_colormap_to_use;
  
  all_relative_times = t_cell_info.synapse_info.all_relative_times;
  number_all_relative_times = t_cell_info.synapse_info.number_all_relative_times;
  condition_sensor_combinations = t_cell_info.synapse_info.condition_sensor_combinations;
  number_condition_sensors = t_cell_info.synapse_info.number_condition_sensors;
  conditions = t_cell_info.synapse_info.conditions;
  sensors = t_cell_info.synapse_info.sensors;
  number_conditions = t_cell_info.synapse_info.number_conditions;
  number_sensors = t_cell_info.synapse_info.number_sensors;
  condition_sensor_combination_ismember = t_cell_info.synapse_info.condition_sensor_combination_ismember;
  condition_retrieval_function = t_cell_info.synapse_info.condition_retrieval_function;
  sensor_retrieval_function = t_cell_info.synapse_info.sensor_retrieval_function;
  
  
  % window_center_2d = t_cell_info.preprocessing_info.window_center_2d;
  % window_size_2d = t_cell_info.preprocessing_info.window_size_2d;
  % background_subtraction_function = t_cell_info.preprocessing_info.background_subtraction_function;
  
  % segmentation_rasterization_function = t_cell_info.segmentation_info.segmentation_rasterization_function;
  
  
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % Add necessary code subdirectories to the path:

  
  % Should use help to automatically derive further dependencies from this file's dependencies...
  
  % From the Mathworks File Exchange, possibly with modifications:
  addpath(genpath('discretesample'))
  addpath(genpath('distinguishable_colors'))
  
  
  
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % Plot stuff:
  
  
  if ~false
    t_cell_info
  end
  
  model_types = t_cell_info.model_type_info.model_types;
  radial_distribution_model_types = t_cell_info.model_type_info.radial_distribution_model_types;
  radial_distribution_types = t_cell_info.model_type_info.radial_distribution_types;
  radial_model_type_indices = reshape(find(ismember(model_types, radial_distribution_model_types)), 1, []);
  if false
    dbstack, beep, keyboard
  end
  % error('What about relative times? Plot as image with density as intensity?')
  for model_type_index = radial_model_type_indices
    model_type = model_types{model_type_index};
    model_type_reduced_filenames = t_cell_info.model_info.model_reduced_filenames{model_type_index};
    if isempty(model_type_reduced_filenames)
      continue
    end
    a = load(model_type_reduced_filenames);
    current_model_files = t_cell_info.model_info.model_filenames{model_type_index};
    current_data = [];
    current_data_run_keys = {};
    current_data_run_key_indices = [];
    current_data_combination_indices = [];
    current_data_relative_times = [];
    current_model_reduction_reconstruction_function = t_cell_info.model_info.model_reduction_reconstruction_functions{model_type_index};
    current_radial_distribution_type = radial_distribution_types{find(radial_model_type_indices == model_type_index)};
    
    for model_file_index = 1:length(current_model_files)
      current_file_path = current_model_files{model_file_index};
      b = load(current_file_path);
      current_data = [current_data; b.current_data];
      current_data_run_key_indices = [current_data_run_key_indices; b.current_data_relevant_run_key_indices + length(current_data_run_keys)];
      current_data_run_keys = [current_data_run_keys; b.current_data_relevant_run_keys];
      % current_data_run_key_combination_indices = cellfun(@(x)find(condition_sensor_combination_ismember(condition_sensor_combinations, [condition_retrieval_function({x}), sensor_retrieval_function({x})])), current_data_run_keys);
      current_data_run_key_combination_indices = cellfun(@(x)find(condition_sensor_combination_ismember(condition_sensor_combinations, [condition_retrieval_function({x}), sensor_retrieval_function({x})])), b.current_data_relevant_run_keys);
      current_data_combination_indices = [current_data_combination_indices; current_data_run_key_combination_indices(b.current_data_relevant_run_key_indices)];
      % Relative time:
      [~, current_filename] = fileparts(current_file_path);
      current_datum_relative_time = regexp(current_filename, '_reltime([ \-0-9]+)_', 'tokens');
      current_datum_relative_time = str2num(strtrim(current_datum_relative_time{1}{1}));
      current_data_relative_times = [current_data_relative_times; ones(size(b.current_data_relevant_run_key_indices)) * current_datum_relative_time];
    end
    if false
      dbstack, beep, keyboard
    end
    % error('What about relative times? Plot as image with density as intensity?')
    
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Plot radial distributions:
    
    
    % Evaluate distributions for plotting:
    
    number_radial_coordinates = 100;
    radial_coordinates = (0:number_radial_coordinates) ./ number_radial_coordinates;
    
    % current_model_rasterization_function = @(x)t_cell_info.model_info.model_reconstruction_functions(x);
    current_model_rasterization_function = @(x, y)t_cell_info.model_type_info.fwd_cyto_radial_model_density(current_model_reduction_reconstruction_function(x), current_radial_distribution_type, y);
    
    % current_data_mean = mean(current_data, 1);
    % current_data_std = std(current_data, [], 1);
    
    
    % Step 1: Collect mean models for all cscombos:
    
    number_features = size(current_data, 2);
    combination_means = nan(number_condition_sensors, number_all_relative_times, number_features);
    combination_covariances = [];
    if number_features <= 100
      combination_covariances = zeros(number_condition_sensors, number_all_relative_times, number_features, number_features);
    end
    for condition_sensor_combination_index = 1:number_condition_sensors
      for all_relative_time_index = 1:number_all_relative_times
        relative_time = all_relative_times(all_relative_time_index);
        
        current_condition = condition_sensor_combinations{condition_sensor_combination_index, 1};
        current_sensor = condition_sensor_combinations{condition_sensor_combination_index, 2};
        combination_data = current_data((current_data_combination_indices == condition_sensor_combination_index) & (current_data_relative_times == relative_time), :);
        if numel(combination_data) == 0
          continue
        end
        combination_means(condition_sensor_combination_index, all_relative_time_index, :) = mean(combination_data);
        if ~isempty(combination_covariances)
          combination_covariances(condition_sensor_combination_index, all_relative_time_index, :, :) = cov(combination_data);
        end
      end
    end
    
    
    if false
      number_condition_sensors, condition_sensor_combinations
      dbstack, beep, keyboard
    end
    
    
    % Step 2: Arrange models spatially for visualization by computing PCA (in parameter space) of cscombo means (or use MDS of K-S, TV, or other distances directly from PDFs):
    
    if false
      current_data_pcaed = pca(current_data);
      
      current_data_lda = direct_lda(struct('data', current_data, 'class_assignments', current_data_combination_indices, 'minimum_r_squared', 0));
      current_data_ldaed = current_data_lda.data_sphered;
    end
      
    if false
      % Plot models' parameters:
      using_ldaed = numel(current_data_ldaed) > 0;
      if using_ldaed
        data_to_plot = current_data_ldaed;
      else
        data_to_plot = current_data;
      end
      data_to_plot = [data_to_plot, zeros(size(data_to_plot, 1), max(0, 3 - size(data_to_plot, 2)))];
      % Display:
      close all
      % set(gcf, 'Visible', 'off')
      set(gcf, 'Visible', 'on')
      figure_position = get(gcf, 'Position');
      % aspect_ratio = size(current_view_image_matrix, 2) / size(current_view_image_matrix, 1);
      aspect_ratio = 10 / 16;
      figure_height_scale = 1;
      figure_height_scale = figure_height_scale * 975 / 847;
      figure_height = round(master_script_options.figure_maximum_width / aspect_ratio * figure_height_scale);
      figure_width = round(master_script_options.figure_maximum_width * figure_height_scale);
      set(gcf, 'Position', [1, 1, figure_width, figure_height])
      dpi = master_script_options.figure_dpi;
      
      plot_colors = distinguishable_colors(number_condition_sensors, 'w');
      if false
        % Discards from below that might look nice on a fast connection:
        % scatter3(combination_data(:, 1), combination_data(:, 2), combination_data(:, 3), [], condition_sensor_combination_index + zeros(size(combination_data, 1), 1))
        % h = plot_empirical_density(struct('data', combination_data));
        % set(h, 'EdgeColor', 'none', 'FaceColor', 'interp', 'FaceLighting', 'phong', 'AmbientStrength', .4)
        % set(h, 'EdgeColor', 'none', 'FaceColor', plot_colors(condition_sensor_combination_index, :), 'FaceLighting', 'phong', 'AmbientStrength', .4, 'FaceAlpha', .3)
        % Takes a really long time to draw:
        % set(h, 'EdgeColor', plot_colors(condition_sensor_combination_index, :), 'FaceColor', 'none', 'EdgeLighting', 'phong', 'AmbientStrength', .7, 'EdgeAlpha', .3)
        % set(h, 'EdgeColor', plot_colors(condition_sensor_combination_index, :), 'FaceColor', 'none', 'EdgeLighting', 'flat', 'AmbientStrength', .7)
        % set(h, 'EdgeColor', plot_colors(condition_sensor_combination_index, :), 'FaceColor', 'none')
        % set(h, 'EdgeColor', plot_colors(condition_sensor_combination_index, :), 'FaceColor', 'none', 'LineWidth', 2)
      % light
      end
      for condition_sensor_combination_index = 1:number_condition_sensors
        current_condition = condition_sensor_combinations{condition_sensor_combination_index, 1};
        current_sensor = condition_sensor_combinations{condition_sensor_combination_index, 2};
        combination_data = data_to_plot(current_data_combination_indices == condition_sensor_combination_index, :);
        h = plot_empirical_density(struct('data', combination_data, 'plot_options', struct('gaussian_sphere_subdivisions', 15)));
        % h = plot_empirical_density(struct('data', combination_data, 'plot_options', struct('gaussian_draw_type', 'axes')));
        set(h, 'EdgeColor', plot_colors(condition_sensor_combination_index, :), 'FaceColor', 'none', 'LineWidth', 2, 'MeshStyle', 'column')
        hold all
      end
      hold off
      legend_entries = strcat(condition_sensor_combinations(:, 1), ' - ', condition_sensor_combinations(:, 2));
      legend(legend_entries, 'Interpreter', 'none', 'Location', 'EastOutside')
      title_string = sprintf('Models of type %s for all conditions', model_types{model_type_index});
      if using_ldaed
        title_string = [title_string, ', LDAed'];
      end
      title(title_string, 'Interpreter', 'none')
      % colormap(illustration_colormap_to_use)
      % colorbar
      axis equal
    
      dbstack, beep, keyboard
    end
    
    % Step 3: Plot those means with markers shaped like plots:
    
    
    % error('Not yet implemented!')
    for condition_sensor_combination_index = 1:number_condition_sensors
      current_condition = condition_sensor_combinations{condition_sensor_combination_index, 1};
      current_sensor = condition_sensor_combinations{condition_sensor_combination_index, 2};
      for all_relative_time_index = 1:number_all_relative_times
        relative_time = all_relative_times(all_relative_time_index);
        
        % Display:
        close all
        set(gcf, 'Visible', 'off')
        % set(gcf, 'Visible', 'on')
        figure_position = get(gcf, 'Position');
        % aspect_ratio = size(current_view_image_matrix, 2) / size(current_view_image_matrix, 1);
        panels_two_to_a_row = false;
        % aspect_ratio = 10 / 16;
        aspect_ratio = 16 / 10;
        figure_height_scale = 1;
        figure_height_scale = figure_height_scale * 975 / 847;
        figure_height = round(master_script_options.figure_maximum_width / aspect_ratio * figure_height_scale);
        figure_width = round(master_script_options.figure_maximum_width * figure_height_scale);
        set(gcf, 'Position', [1, 1, figure_width, figure_height])
        dpi = master_script_options.figure_dpi;
        
        % legend_entries = {};
        % Mean:
        current_combination_mean = squeeze(combination_means(condition_sensor_combination_index, all_relative_time_index, :));
        current_mean_curve = current_model_rasterization_function(current_combination_mean, radial_coordinates);
        plot(radial_coordinates, current_mean_curve, 'b')
        
        if ~isempty(combination_covariances)
          % Standard deviation:
          current_combination_covariance = squeeze(combination_covariances(condition_sensor_combination_index, all_relative_time_index, :, :));
          current_combination_std = sqrt(diag(current_combination_covariance));
          
          hold all
          current_added_std_curve = current_model_rasterization_function(current_combination_mean + current_combination_std, radial_coordinates);
          current_subtracted_std_curve = current_model_rasterization_function(current_combination_mean - current_combination_std, radial_coordinates);
          % plot(radial_coordinates, current_added_std_curve)
          % plot(radial_coordinates, current_subtracted_std_curve)
          plot(radial_coordinates, [current_added_std_curve; current_subtracted_std_curve]', 'b--')
          
          % legend_entries = {'Mean', 'Mean + sd', 'Mean - sd'};
          legend_entries = {'Mean', 'Mean +/- sd'};
          legend(legend_entries, 'Interpreter', 'none', 'Location', 'EastOutside')
          
          hold off
        end
        
        xlabel('Radial distance from center')
        ylabel('Probability density')
        if false
          legend_entries = strcat(condition_sensor_combinations(:, 1), ' - ', condition_sensor_combinations(:, 2));
          % legend(legend_entries, 'Interpreter', 'none')
          legend(legend_entries, 'Interpreter', 'none', 'Location', 'EastOutside')
        else
          title(sprintf('%s - %s - reltime % 3d - model %s', current_condition, current_sensor, relative_time, model_type), 'Interpreter', 'none')
        end
        
        if false
          dbstack, beep, keyboard
        end
        set(gcf, 'Color', 'w')
        image_filename = sprintf('%sradial_plot_%s_%s_reltime% 3d_%s', master_script_options.model_prefix, current_condition, current_sensor, relative_time, model_type);
        image_full_filename = [figures_location, image_filename, '.png'];
        export_fig(image_full_filename, '-zbuffer', '-png', '-a1', ['-r', num2str(dpi)])
        
      end
    end
    
    
  end
  
  
  
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % Find all work relevant to given options:
  
  
  
  if options.use_profiling
    p = profile('info');
    profile_filename = [optimization_directory, 'profile_results', '.mat']
    profsave(p, profile_filename);
    save([profile_filename, '.mat'], 'p');
  end
  
  
  time_so_far = toc(start_time)
  
  diary('off')
  
end
