function [p_value, degrees_of_freedom, statistic] = pearson_chi_square_histograms(given_histogram1, given_histogram2)
  % Pearsons chi-square test for histograms of arbitrary dimensionality. Histogram inputs are counts and are not normalized.
  %
  % References:
  % Gagunashvili 2007, "Pearsons Chi-Square Test Modifications for Comparison of Unweighted and Weighted Histograms and Two Weighted Histograms"
  %
  % Tests:
  % for c = 0:.1:1, geometric_mean_p_value = []; for r = 1:1000, a = rand(1, 10); a = a ./ sum(a) .* 100; b = rand(1, 10); b = b ./ sum(b) .* 100; b = a * (1 - c) + b * (c); p_value = pearson_chi_square_histograms(a, b); geometric_mean_p_value = [geometric_mean_p_value, p_value]; end, geometric_mean_p_value = geomean(geometric_mean_p_value); c, geometric_mean_p_value, end
  %
  % Dependencies:
  % File Exchange:
  % Mine:
  % 
  % 2013-08-12 tebuck: Created.

%
% Author: Taraz E. Buck and Xiongtao Ruan
%
% Copyright (C) 2012-2016 Murphy Lab
% Computational Biology Department
% School of Computer Science
% Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu


  
  number_data1 = sum(given_histogram1(:));
  number_data2 = sum(given_histogram2(:));
  
  degrees_of_freedom = numel(given_histogram1) - 1;
  
  % intermediate_histogram = (given_histogram1 + given_histogram2) ./ (number_data1 + number_data2);
  % statistic = sum((given_histogram1(:) - intermediate_histogram(:) * number_data1).^2 ./ (intermediate_histogram(:) * number_data1)) + sum((given_histogram2(:) - intermediate_histogram(:) * number_data2).^2 ./ (intermediate_histogram(:) * number_data2)) + ;
  
  statistic = sum((given_histogram1(:) * number_data2 - given_histogram2(:) * number_data1).^2 ./ (given_histogram1(:) + given_histogram2(:))) ./ (number_data1 * number_data2);
  % p_value = 1 - chi2cdf(statistic, degrees_of_freedom);
  % p_value_by_subtraction = p_value
  % Get more precision by computing without subtraction:
  p_value = gammainc(statistic, degrees_of_freedom / 2, 'upper') / gamma(degrees_of_freedom / 2);
  % % Try using logarithms to get nonzero values:
  % p_value = exp(log(gammainc(statistic, degrees_of_freedom / 2, 'upper')) - gammaln(degrees_of_freedom / 2));
  % % Get even more precision by computing without subtraction and using scaling (R2010a doesn't have 'scaledupper' option!):
  % p_value = gammainc(statistic, degrees_of_freedom / 2, 'scaledupper') / gamma(degrees_of_freedom / 2)
  % p_value = exp(log(gammainc(statistic, degrees_of_freedom / 2, 'scaledupper')) - statistic + degrees_of_freedom / 2 * log(statistic) - gammaln(degrees_of_freedom / 2))
  % p_value_by_upper = p_value
  % statistic, degrees_of_freedom, p_value
  
end


