function [result_handles] = plot_empirical_density(options)
  % Computes an empirical density using one of several models or a given model???
  % % Options for:
  % % - 3D surface with opacity
  % % - 2D outlines or filled shapes
  % % - Currently supports empirical density by normal, GMM, KDE???
  %
  % Dependencies: generate_superellipse_shape_set.m
  % File Exchange: 
  % [Finish!]
  % 
  % Other:
  % % ihler_kde
  %
  % 2014-07-07 tebuck: Copied from evaluate_segmentation_method_performance.m.
  % 2014-11-10 tebuck: Adding general-dimension KDE functionality, want to otherwise complete for convenience... nvm too much to do for now.

%
% Author: Taraz E. Buck and Xiongtao Ruan
%
% Copyright (C) 2012-2016 Murphy Lab
% Computational Biology Department
% School of Computer Science
% Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu




  % Process options:
  default_options = struct();
  default_options.data = [];
  default_options.fit_function = 'normal';
  default_options.density_function = [];
  default_options.plot_function = [];
  default_options.plot_options = struct;
  default_options.plot_options.gaussian_sphere_subdivisions = 20;
  default_options.plot_options.gaussian_draw_type = 'mesh';
  default_options.axis_subdivision = 100;
  default_options.verbose = 0;
  % default_options.verbose = 1;
  % default_options.verbose = 2;
  % For GMMs and other randomly-initialized fits:
  default_options.number_random_parameter_initializations = 10;
  
  % % Parallel-related:
  % default_options.maximum_parallel_cpus = 1;
  
    
  if ~exist('options', 'var')
    options = default_options; 
  else
    options = process_options_structure(default_options, options, {}, true);
  end
  options
  
  
  if isempty(options.data)
    % For a built-in test, generate some images if options.number_images == 0 (the default value):
    % addpath('~/4dcell/ihler_kde');
    options.data = mvnrnd([1, 1, 1], [1, .5, -.2; .5, 1, -.1; -.2, -.1, 1], 100);
    options.fit_function = 'normal';
    
    options.verbose = 2;
  end
  
  
  function [current_handle] = gaussian_plot_function(given_coordinates, given_parameters)
    dimensionality = size(given_coordinates, 2);
    given_mean = given_parameters.mean;
    given_covariance = given_parameters.covariance;
    if dimensionality >= 3
      given_mean = given_mean(1:3);
      given_covariance = given_covariance(1:3, 1:3);
    end
    [axes, squared_lengths] = spectral_decomposition(given_covariance);
    lengths = sqrt(diag(squared_lengths));
    scaled_axes = axes * diag(lengths);
    if dimensionality == 1
      half_number_coordinates = 1000;
      coordinates = (-half_number_coordinates:half_number_coordinates) ./ half_number_coordinates;
      % Sample unevenly for better drawing:
      coordinates = coordinates.^2 .* sign(coordinates);
      current_handle = plot(coordinates, normpdf(coordinates, given_mean, given_covariance));
    elseif dimensionality == 2
      error('Not yet implemented!')
    elseif dimensionality >= 3
      switch options.plot_options.gaussian_draw_type
        case 'mesh'
          [x, y, z] = sphere(options.plot_options.gaussian_sphere_subdivisions);
          % coordinates = [x, y, z] * scaled_axes(1:3, 1:3);
          final_basis = given_coordinates;
          coordinates = [x(:), y(:), z(:)];
          coordinates = coordinates * scaled_axes + repmat(given_mean, size(coordinates, 1), 1);
          current_handle = surf(reshape(coordinates(:, 1), size(x)), reshape(coordinates(:, 2), size(x)), reshape(coordinates(:, 3), size(x)));
        case 'axes'
          error('Not yet implemented!')
          % current_handle = 
      end
    end
  end
  
  
  if ischar(options.fit_function)
    switch lower(options.fit_function)
      case {'normal', 'gaussian'}
        options.fit_function = @(given_data)struct('mean', mean(given_data), 'covariance', cov(given_data));
        options.density_function = @(given_coordinates, given_parameters)mvnpdf(given_coordinates, given_parameters.mean, given_parameters.covariance);
        options.plot_function = @gaussian_plot_function;
      case 'kde'
        error('Not yet implemented!')
        % options.fit_function = @(given_data)kde(data', 'HALL', [], 'Gaussian');
        % options.density_function = @(given_coordinates, given_parameters)mvnpdf(given_coordinates, given_parameters.mean, given_parameters.covariance);
      otherwise
        error('Not yet implemented!')
    end
  end
  
  
  data = options.data;
  
  
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % :
  
  
  result_handles = [];
  if ~isempty(options.plot_function)
    model = options.fit_function(options.data);
    current_handle = options.plot_function(options.data, model);
    result_handles = [result_handles, current_handle];
  else
    error('Not yet implemented!')
  end
  
  
end


