function [processed_vertex_values] = process_mesh_neighborhoods(single_mesh, vertex_values, compute_function)
  % Apply operations on the discrete neighborhood of each vertex (like convolution or morphological image processing, but on meshes).
  %
  % 2013-02-27 tebuck: created.
  %
  % Dependencies:
  % From the File Exchange: toolbox_graph

%
% Author: Taraz E. Buck and Xiongtao Ruan
%
% Copyright (C) 2012-2016 Murphy Lab
% Computational Biology Department
% School of Computer Science
% Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu




  adjacency = logical(triangulation2adjacency(single_mesh.faces, single_mesh.vertices));
  processed_vertex_values = zeros(size(adjacency, 1), 1);
  for vertex_index = 1:length(processed_vertex_values)
    processed_vertex_values(vertex_index) = compute_function(vertex_values(vertex_index), vertex_values(adjacency(vertex_index, :)));
  end
end
