function [a] = reshape_2d(a, r)
  % Reshape a 3D image a so that its Z slices (along the third dimension) become arranged left to right in a 2D image. If r > 1, wrap the slices like left-to-right text into r rows. If r == -1, automatically determine the number of slices to get an approximately square image.

%
% Author: Taraz E. Buck and Xiongtao Ruan
%
% Copyright (C) 2012-2016 Murphy Lab
% Computational Biology Department
% School of Computer Science
% Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu




  if ~exist('r', 'var') || isempty(r)
    r = 1;
  elseif r == -1
    r = max(floor(sqrt(size(a, 3) * size(a, 2) / size(a, 1))), 1);
  end
  s = size(a);
  c = round(s(3) / r);
  b = zeros(s(1) * r, s(2) * c) + mean(a(:));
  for row_index = 1:r
    for column_index = 1:c
      if (row_index - 1) * c + column_index > s(3)
        break
      end
      rs = s(1) * (row_index - 1) + 1;
      rf = s(1) * (row_index);
      cs = s(2) * (column_index - 1) + 1;
      cf = s(2) * (column_index);
      b(rs:rf, cs:cf) = a(:, :, (row_index - 1) * c + column_index);
    end
  end
  a = b;
  
