function [t] = rotation_matrix3h(...
  axis_index, angle_degrees...
  )
  % 3D rotation matrix in homogenous coordinates.
  % 2011-05-18 tebuck
  % 2013-04-01 tebuck: If axis_index is not scalar, it is interpreted as the axis about which to rotate.

%
% Author: Taraz E. Buck and Xiongtao Ruan
%
% Copyright (C) 2012-2016 Murphy Lab
% Computational Biology Department
% School of Computer Science
% Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu




  degrees_to_radians = pi / 180.; 
  if numel(axis_index) == 1
    t = [...
      cos(angle_degrees * degrees_to_radians), ...
      -sin(angle_degrees * degrees_to_radians), 0, 0; ...
      sin(angle_degrees * degrees_to_radians), ...
      cos(angle_degrees * degrees_to_radians), 0, 0; ...
      0, 0, 1, 0; ...
      0, 0, 0, 1; ...
        ]; 
    t(1:3, 1:3) = circshift(...
      t(1:3, 1:3), [-1, -1] * (3 - axis_index));
  else
    % error('not finished')
    x = axis_index(1);
    y = axis_index(2);
    z = axis_index(3);
    c = cos(angle_degrees * degrees_to_radians);
    s = sin(angle_degrees * degrees_to_radians);
    omc = 1 - c;
    t = [...
      x*x*omc+c, x*y*omc-z*s, x*z*omc+y*s, 0; ...
      y*x*omc+z*s, y*y*omc+c, y*z*omc-x*s, 0; ...
      z*x*omc-y*s, z*y*omc+x*s, z*z*omc+c, 0; ...
      0, 0, 0, 1; ...
      ];
  end
  
  
  
