function [result_eigenvectors, result_eigenvalues] = spectral_decomposition(given_matrix, given_sort_by_absolute_eigenvalues)

%
% Author: Taraz E. Buck and Xiongtao Ruan
%
% Copyright (C) 2012-2016 Murphy Lab
% Computational Biology Department
% School of Computer Science
% Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu




  if ~exist('given_sort_by_absolute_eigenvalues', 'var') || isempty(given_sort_by_absolute_eigenvalues)
    given_sort_by_absolute_eigenvalues = false;
  end
  % if options.use_schur_decomposition
    % [result_eigenvectors, result_eigenvalues] = schur(given_matrix);
  % else
    [result_eigenvectors, result_eigenvalues] = eig(given_matrix);
  % end
  % Because schur sometimes returns non-diagonal second output:
  result_eigenvalues = diag(result_eigenvalues);
  if given_sort_by_absolute_eigenvalues
    % Sort by decreasing absolute eigenvalue:
    [~, result_eigenvalue_order] = sort(abs(result_eigenvalues), 1, 'descend');
  else
    % Sort by decreasing eigenvalue:
    [~, result_eigenvalue_order] = sort(result_eigenvalues, 1, 'descend');
  end
  result_eigenvalues = result_eigenvalues(result_eigenvalue_order);
  result_eigenvectors = result_eigenvectors(:, result_eigenvalue_order);
  % % Because this can be sparse:
  % result_eigenvalues = spdiags(result_eigenvalues, 0, number_landmarks, number_landmarks);
  % No sparsity for now for convenience:
  result_eigenvalues = diag(result_eigenvalues);
end
