function [M] = transform_matrix_generation(alpha, beta, gamma, lambda, translation, x_shift)
% created by xruan 07/14/2015
% alpha: rotation by positive x axis 
% beta: rotation by positive y axis
% gamma: rotation by positive z axis
% lambad: rescale factor
% x_shift: unchanged point in transformation, 3 by 1

%
% Author: Taraz E. Buck and Xiongtao Ruan
%
% Copyright (C) 2012-2016 Murphy Lab
% Computational Biology Department
% School of Computer Science
% Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu



M1 = [eye(3), - x_shift; zeros(1, 3), 1];

M2 = [lambda * eye(3), zeros(3, 1); zeros(1, 3), 1];

M3 = eye(4);
M3(1, 1) = cos(gamma);
M3(1, 2) = sin(gamma);
M3(2, 1) = - sin(gamma);
M3(2, 2) = cos(gamma);

M4 = eye(4);
M4(1, 1) = cos(beta);
M4(1, 3) = - sin(beta);
M4(3, 1) =  sin(beta);
M4(3, 3) = cos(beta);

M5 = eye(4);
M5(2, 2) = cos(alpha);
M5(2, 3) = sin(alpha);
M5(3, 2) = -sin(alpha);
M5(3, 3) = cos(alpha);

M6 = [eye(3), x_shift; zeros(1, 3), 1];

M7 = eye(4);
M7(1 : 3, 4) = translation;

M = M7 * M6 * M5 * M4 * M3 * M2 * M1;
end

