function [result_mean] = update_running_mean(given_preceding_mean, given_parameters, given_index)
  % Update given_preceding_mean computed from (given_index - 1) data with given_parameters. Initial call should be with given_preceding_mean = zeros, given_index = 1.
  %
  % References:
  % Knuth, "The Art of Computer Programming, Volume 2: Seminumerical Algorithms", section 4.2.2
  % Welford 1962, "Note on a Method for Calculating Corrected Sums of Squares and Products"
  %
  % Tests:
  % d = 5;
  % a = randn(50, d) * randn(d, d);
  % am = mean(a, 1)
  % as = std(a, [], 1)
  % ac = cov(a)
  % arm = zeros(1, d);
  % ars = 0;
  % arc = zeros(d, d);
  % for index = 1:size(a, 1)
  %   pm = arm;
  %   cm = update_running_mean(pm, a(index, :), index);
  %   ps = ars;
  %   cs = update_running_standard_deviation(pm, cm, ps, a(index, :), index);
  %   pc = arc;
  %   cc = update_running_covariance(pm, cm, pc, a(index, :), index);
  %   arm = cm;
  %   ars = cs;
  %   arc = cc;
  % end
  % ars = finalize_running_standard_deviation(ars, size(a, 1));
  % arc = finalize_running_covariance(arc, size(a, 1));
  % arm
  % ars
  % arc
  % mean_error = norm(am - arm)
  % mean_standard_deviation_error = norm(as - ars)
  % mean_covariance_error = norm(ac - arc)
  %
  % Dependencies:
  % File Exchange:
  % Mine:
  % 
  % 2013-08-16 tebuck: Created.

%
% Author: Taraz E. Buck and Xiongtao Ruan
%
% Copyright (C) 2012-2016 Murphy Lab
% Computational Biology Department
% School of Computer Science
% Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu


  
  % whos
  result_mean = given_preceding_mean + (given_parameters - given_preceding_mean) ./ given_index;
  
end


